#ifndef _CICON_VIEWER_
#define _CICON_VIEWER_

#include <QtGui>
#include "TextEdit.h"
#include "LineEdit.h"
#include "PixmapsCache.h"
#include "ThreadIcon.h"
#include "ThreadAreaIcon.h"
#include "ReserveCache.h"

class CIconViewer : public QListWidget {
  Q_OBJECT
  private:
    QSettings settings;
    QSize size;
    QString themes, ext;
    PixmapsCache *pxCache;
    ReserveCache *rCache1, *rCache2;
    QPixmap pixmap_link, pix;
    QPainter painter;

    QDir dir;
    QFileInfo fileinfo, CurrentFileInfo;
    QFileInfoList *fileinfolist;

    QListWidgetItem *item;
    CTextEdit *TextEditor;
    CLineEdit *LineEditor;

    QTimer  timer;
    CThreadIcon *ThreadIcon;
    CThreadAreaIcon * ThreadAreaIcon;

    int mousePosX, mousePosY;
    QBitArray *bitIcon;
    int startElement, endElement;
    QList<QListWidgetItem*> DragListItem;
    QDrag *Drag;
    QTimer DragTimer, ScrollDownTimer, ScrollUpTimer;
    QPoint DragPos;

  public:
    QSize SizeHint;
    bool ShowHidden, EditElementText, EditElementLine, StartTimer, BlockList, *ExcludeDirsMime, *ExcludeDirsTemplate;
    QFont * font;
    QList<QListWidgetItem*> itemsList;
    bool * image;
    QList<QString>* TemplateFilter;
    QList<QRegExp>* MimeFilter;

  protected:
    virtual void mousePressEvent(QMouseEvent*);
    virtual void mouseMoveEvent(QMouseEvent*);
    virtual void mouseDoubleClickEvent(QMouseEvent*);
    virtual void keyPressEvent(QKeyEvent*);
    virtual void dragMoveEvent(QDragMoveEvent*);
    virtual void dragEnterEvent(QDragEnterEvent*);
    virtual void dragLeaveEvent(QDragLeaveEvent*);
    virtual void dropEvent(QDropEvent*);
    virtual void startDrag(Qt::DropActions act = Qt::MoveAction);

  public:
    CIconViewer( QWidget * wgt = 0 );
    ~CIconViewer();

    inline void setPixmapsCache ( PixmapsCache* cache, ReserveCache *rcache1, ReserveCache *rcache2){
      pxCache = cache;
      rCache1 = rcache1;
      rCache2 = rcache2;
      setPixmapLink();
    }

    inline void setFileInfoList ( QFileInfoList * list){
      fileinfolist = list;
    }
    inline void setPixmapLink() { pixmap_link = pxCache->getIcon(111)->scaled(64, 64, Qt::KeepAspectRatio); }
    inline CThreadIcon* getThreadIcon() { return ThreadIcon; }
    inline CThreadAreaIcon * getThreadAreaIcon() { return ThreadAreaIcon; }

  public slots:
    void showDirectory ( QString, bool );
    void updateDirectory( QString);
    void findAbc(QString);
    void textChanged();
    void editElement();

    void initAreaIcon(int);

    inline void finishSubThread() {
       while ( ThreadAreaIcon->isRunning()) {
         QApplication::processEvents();
         ThreadAreaIcon->execute = false;
       }
  }

    void setPixmapElement(int, int, int, bool);
    void selectionChanged();
    void timeoutDragTimer();
    void scrollUp();
    void scrollDown();

  signals:
    void renameItem (QString, QString);
    void contextMenu ( QMouseEvent*, QString );
    void contextMenu ( QMouseEvent*, QList<QString>);
    void contextMenu ( QMouseEvent*);
    void contextAction( QString);
    void contextAction( QString, QString);
    void contextAction (QString, QList<QString>);
    void refreshWrite(QDir dir, bool);
    void refreshWrite(int);
    void refreshWrite(QFileInfo, bool);
    void refreshWrite ();
    void pressKey(QString);
    void delSymbolAbc();
    void resetSelection();
    void dropMenu(QList<QString>, QPoint,  QString, bool, int);
    void expandedItem(QListWidgetItem*);

};

#endif
