#ifndef _CICON_VIEWER_NAVI_
#define _CICON_VIEWER_NAVI_

#include <QtGui>
#include "ToolBar.h"
#include "Button.h"
#include "LineEdit.h"
#include "PixmapsCache.h"

class CIconViewerNavigator : public QFrame {
  Q_OBJECT
  private:
    int IconSize;
    QString oldPath;
    QList<QString> history, listdrives;
    QList<int> scrollPosition;
    int  posHistory;
    QList<CButton*> ActionList;
    QList<QWidget*> SeparatorList;
    QList<CPointSeparator*> PointList;
    QList<QString> ButtonsText;
    bool textpanel;
    QLineEdit * String;
    CButton           *LabelIcon,
                             *TextButton,
                             *BackButton,
                             *ForwardButton;
    QList<int> sizesButton, realSizesButton;
    int currentIndexButton, freeButtonSpace;
    PixmapsCache *pxCache;
    CExcludeEvents *ExcludeEvents;
    int Width;
    QFont *Font;
    int width3point;

  public:
     CIconViewerNavigator ( QWidget *p = 0 );
     inline QList<QString> * getListDrives (){
       return &listdrives;
     }
     inline void addHistory ( QString str ) {
       posHistory = history.count() - 1;
       QList<QString>::Iterator It = history.end(); It--;
       if ( posHistory == -1 ) {
         history << str;
         posHistory++;
       }
       else {
         if ( *It != str ) {
              if ( history.count() < 21 ) {
                history << str;
                posHistory++;
              }
              else {
                history << str;
                history.removeFirst();
              }
           }
        }
     }
     inline void setScrollPosition(int poz) {
       if (posHistory >= 0) scrollPosition[posHistory] = poz;
     }
     inline bool getBackState() {
       if ( history.count() > 1 && posHistory > 0 ) return true;
          else return false;
     }
     inline bool getNextState() {
       if ( posHistory < history.count() -1) return true;
          else return false;
     }
     inline int getPosHistory () {
       return posHistory;
     }
     inline QString getBackHistory(int* scroll) {
       if ( posHistory > 0 ) posHistory--;
       QList<QString>::Iterator It = history.begin();
       It = It + posHistory;
       *scroll = scrollPosition[posHistory];
       return *It;
     }
     inline QString getForwardHistory(int *scroll) {
       posHistory++;
       QList<QString>::Iterator It = history.begin();
       It = It + posHistory;
       *scroll = scrollPosition[posHistory];
       return *It;
     }
     inline bool getHomeState() {
       if (history[posHistory] == QDir::homePath()) return false; else return true;
     }
     inline void setPixmapsCache ( PixmapsCache *pcache) {
       pxCache = pcache;
       TextButton = new CButton(this);
       TextButton->setToolTip(tr("Text string"));
       TextButton->setIcon(*(pxCache->getIcon(35)));
       TextButton->setMaximumWidth(IconSize + 12);
       TextButton->setMinimumWidth(IconSize + 12);
       TextButton->setMinimumHeight(IconSize +  14);
       TextButton->setMaximumHeight(IconSize + 14);
       TextButton->setCheckable(true);
       TextButton->setChecked(false);

       BackButton = new CButton(this);
       BackButton->setIcon(*(pxCache->getIcon(36)));
       BackButton->setMaximumWidth(IconSize);
       BackButton->setMinimumWidth(IconSize);
       BackButton->setMinimumHeight(IconSize + 14);
       BackButton->setMaximumHeight(IconSize + 14);
       BackButton->hide();

       ForwardButton = new CButton(this);
       ForwardButton->setIcon(*(pxCache->getIcon(37)));
       ForwardButton->setMaximumWidth(IconSize);
       ForwardButton->setMinimumWidth(IconSize);
       ForwardButton->setMinimumHeight(IconSize + 14);
       ForwardButton->setMaximumHeight(IconSize + 14);
       ForwardButton->hide();

       connect ( TextButton, SIGNAL(clicked()),SLOT(changeText()));
       connect ( BackButton, SIGNAL(clicked()), SLOT(backStrPath()));
       connect ( ForwardButton, SIGNAL(clicked()), SLOT(forwardStrPath()));
     }

    inline void setFont(QFont *font){
      Font = font;
      QFontMetrics fm(*Font);
      width3point = fm.width("...");
    }
    inline void setLabelIcon(QIcon icon) {
      LabelIcon->setIcon(icon);
    }
    QString getLocation();
    void setTextOnly(bool en);
    bool isTextOnly();
    void clear();
    void showPath();

    void addAction (QString&, QString&);
    void addPoint();
    void addSeparator();
    QString catString( QString, int );

  signals:
    void changeDirectory ( QString );
    void changeLocation(QString);
    void clickedButton(CButton*);

  public slots:
    void setPath ( QString );
    void changeText();
    void changeTextPanel();
    void backStrPath();
    void forwardStrPath();
    void clickButton();
    void setBackColor (QColor);
    void setFontColor (QColor);
    void setFontSize();
    void refreshTheme();

  protected:
    virtual void resizeEvent(QResizeEvent*);

};

#endif
