#ifndef _CICON_VIEWER_PANEL_BASE_
#define _CICON_VIEWER_PANEL_BASE

#include <QtGui>
#include "IconViewer.h"
#include "TableViewer.h"
#include "IconViewerNavigator.h"
#include "IconViewerMoving.h"
#include "PixmapsCache.h"
#include "LineEdit.h"
#include  "ThreadOp.h"
#include "ThreadIcon.h"
#include "WebViewer.h"
#include "WebViewerTool.h"
#include "TextViewer.h"
#include "TextViewerTool.h"
#include "ImageViewer.h"
#include "SelectApplication.h"
#include "WindowProperites.h"

enum TypeF { Unknown, Known, Image, Web, Text };

class CIconViewerPanelBase : public QFrame {
  Q_OBJECT
  protected:
    bool Flat, ExcludeDirsMime, ExcludeDirsTemplate;
    QList<CButton*> ListMovingButton;
    QString theme, view, strfile, buffer, strdrop, runc;
    QList<QString> strlist;
    QDir dir;
    QList<QSize*> sizeList;
    QList<QString> * listdrives;
    QFileInfoList fileinfolist;
    QFileInfo fileinfo;
    QString current_dir;
    QVBoxLayout * VLayout;
    QHBoxLayout * HLayout;
    PixmapsCache *pxCache;
    ReserveCache *rCache1, *rCache2;
    QFileSystemWatcher watcher;
    QBitArray *bitState;
    QClipboard *clipboard;
    QMimeData *mimedata;
    QColor Color;
    QPixmap *NavigatorIcon;
    QList<QString> BufferCut;
    QList<QString> DropList;

    QList<QString> TemplateFilter;
    QList<QRegExp> MimeFilter;

    QStackedWidget * Stack;
    bool OtherView;
    void deleteViewers();
    QString createErrorText(QString);
    bool LocationStartsWithDrives(QString);
    void textViewerSave();

  public:
    CIconViewer * IconViewer;
    CTableViewer * TableViewer;
    CIconViewerNavigator * Navigator;
    CIconViewerMoving * Moving;
    CThreadIcon *ThreadIcon;
    CThreadAreaIcon* ThreadAreaIcon;
    CWebViewer *WebViewer;
    CWebViewerTool *WebViewerTool;
    CWebHistory *WebHistory;
    CTextViewer *TextViewer;
    CTextViewerTool *TextViewerTool;
    QLineEdit *WebViewerStatus;
    CImageViewer *ImageViewer;
    CImageViewerTool *ImageViewerTool;
    bool *showWarningRemove;

    bool TextViewerDontEdit;
    int TextViewerScale;
    qreal WebViewerScale;

  public:
    CIconViewerPanelBase( PixmapsCache *, ReserveCache*, ReserveCache*,  QWidget *wgt = 0);
    ~CIconViewerPanelBase();
    inline void setView(QString str)                                    { view = str;                                   }
    inline void setIconSize(QList<QSize*> sz)                 { sizeList = sz;                              }
    inline void setFont(QFont* fn){
      IconViewer->font = fn;
      TableViewer->font = fn;
      Navigator->setFont(fn);
    }
    inline void initShowWarningRemove(bool *flag)        { showWarningRemove = flag;   }
    inline bool otherViewer ()                                               { return OtherView;                      }
    inline void setColor(QColor color)                                { Color = color;                               }
    TypeF parseType(QFileInfo, QString*);
     void setNavigatorIcon(QFileInfo);


  signals:
    void execOperate(QList<QString>, QString, int);
    void setShowWarningRemove();
    void hideStatusBar(bool);
    void openElement(QListWidgetItem*);
    void openElement(QTreeWidgetItem*);
    void execDrag(int);
    void execDrag(QString);
    void hideFilter(bool);
    void refreshWrite(QDir, bool);
    void refreshWrite(QFileInfo, bool);
    void refreshWrite(int);
    void refreshWrite();


  public slots:
    void changeWatcher(QString);
    void changeDirectory ( QListWidgetItem * );
    void changeDirectory ( QTreeWidgetItem *);
    void changeDirectory ( CButton * );
    void slotMoveDirectory(CButton *);
    void changeLocation(QString);
    void changeView(QString, bool);
    void setShowHidden(bool);
    void setIconSize(QSize);
    void setFontSize();
    void setBackColor(QColor);
    void setFontColor(QColor);
    void setBackTexture(QPixmap);
    void setTheme();
    void setFlat(bool);

    void execContextMenu (QMouseEvent*, QString);
    void execContextMenu (QMouseEvent*, QList<QString>);
    void execContextMenu (QMouseEvent*);
    void execContextAction (QString, QString);
    void execContextAction (QString, QList<QString>);
    void execContextAction (QString);
    void execDropMenu (QList<QString>, QPoint, QString, bool, int);
    void slotDropMenu(QString);
    void slotDropMenu(int);

    void renameItem(QString, QString);
    void renameItem();
    void deleteItem();
    void createItem();
    void createItemFile();
    void createItemSymlink();

    void copyItemClipboard();
    void pasteItemClipboard();
    void cutItemClipboard();
    void selectAll();
    void fileOperation(QString, QString);
    void properites();

    void findAbc(QString);

    inline void changeShowWarningRemove(){
      emit setShowWarningRemove();
    }

    void runProgram();
    void selectProgram();

    void setMimeFilter(QString);
    void setTemplateFilter(QString);
    void resetSelection();

    void runWebViewer(QString);
    void setNavigatorUrl(bool);
    void runTextViewer(QString);
    void runImageViewer(PixmapsCache*, QFileInfo);
    void openElement();
    inline void setExcludeDirsMime(bool ex) {
      ExcludeDirsMime = ex;
      if ( view == "icons" || view == "list" ) IconViewer->showDirectory(current_dir, true);
      if ( view == "table" ) TableViewer->showDirectory(current_dir, true);
    }
    inline void setExcludeDirsTemplate(bool ex) {
      ExcludeDirsTemplate = ex;
      if ( view == "icons" || view == "list" ) IconViewer->showDirectory(current_dir, true);
      if ( view == "table" ) TableViewer->showDirectory(current_dir, true);
    }

  friend class CIconViewerPanel;

};

#endif
