#ifndef _CICON_VIEWER_SETTINGS_
#define _CICON_VIEWER_SETTINGS_

#include <QtGui>
#include "PixmapsCache.h"
#include "SettingsCache.h"
#include "ToolBar.h"
#include "Button.h"
#include "AnimateButton.h"
#include "MessageBox.h"
#include "Cvs.h"
#include "../Utilities/CleanKey/CleanKey.h"

class CIconViewerSettings : public CToolBar {
  Q_OBJECT
  private:
    CCvs *CVS;
    QSettings settings;
    QColor currentFontColor, currentBackColor;
    QSize currentListSize, currentIconSize, currentTableSize;
    QString currentTheme, currentBackTextureName, view;
    QPixmap currentBackTexture;
    QFont font;
    int currentFontSize;
    CButton             *ShowHide,
                                 *BackColor,
                                 *BackTexture,
                                 *FontColor,
                                 *FontSize,
                                 *IconSize,
                                 *Theme,
                                 *Updates,
                                 *CleanKey,
                                 *Styles;

     QString fileNameStateOp;
     CAnimateButton* stateOp;
     QPixmap *is_update, *no_update;

     SettingsCache settingsCache;

     QMenu  *back_texture,
                   *font_size,
                   *icon_size,
                   *_theme,
                   *styles;

    bool showHidden,
            backgroundColor,
            backgroundTexture,
            autoWindowUpdates,
            ifUpdates;

    PixmapsCache *pxCache;
    QList<QString> StyleList;
    int StyleNumber;

  public:
    bool SaveSettings;

  private:
    void readSettings();
    void writeSettings();

  public:
    CIconViewerSettings (PixmapsCache *, QWidget *wgt = 0);
    ~CIconViewerSettings();

    inline bool getShowHidden(){ return showHidden; }
    inline QList<QSize*> getIconSize() {
      QList<QSize*> sz;
      sz << &currentIconSize << &currentListSize << &currentTableSize;
      return sz;
    }
    inline QFont * getFont() { return &font;  }
    inline bool getBackgroundColor(){  return backgroundColor;  }
    inline QColor getBackColor(){ return currentBackColor; }
    inline QColor getFontColor(){ return currentFontColor;}
    inline QPixmap getBackTexture(){ return currentBackTexture; }
    inline void setUpdateButtonPixmap() {
      is_update = pxCache->getIcon(67);
      no_update = pxCache->getIcon(68);
    }

  signals:
    void changeShowHidden(bool);
    void changeBackTexture(QPixmap);
    void changeBackColor(QColor);
    void changeFontColor(QColor);
    void changeFontSize();
    void changeIconSize(QSize);
    void changeTheme();
    void setNotSaveSettings();

  public slots:
    void clickedShowHidden();
    void clickedBackTexture(QAction*);
    void clickedBackColor();
    void clickedFontColor();
    void clickedFontSize(QAction*);
    void clickedIconSize(QAction*);
    void clickedTheme(QAction *);
    void changeView(QString);
    void setFlat(bool);

    inline void startAnimate(){
      stateOp->start();
    }
    void stopAnimate() {
      stateOp->stop();
    }
    void clickedStyles(QAction*);
    void clickedCleanKey();
    void showWindowUpdates(QBuffer*, bool);
    void showWindowUpdates();
    inline void changeAutoShowUpdates() {
      autoWindowUpdates = !autoWindowUpdates;
    }

};

#endif
