#ifndef _IMAGE_VIEWER_
#define _IMAGE_VIEWER_

#include <QtGui>
#include <QSvgWidget>
#include <QSvgRenderer>
#include "ImageViewerTool.h"

enum TypeImg{ NotImage, NormalImage, AnimatedImage, SvgImage, AnimatedSvg };

class CImageViewer : public QFrame {
  Q_OBJECT
  private:
    TypeImg ImageType;
    int CurrentImage;
    QSize MaxSize;
    QString directory, file, fileinfo, suffix;
    QDir Directory;
    QFile File;
    QFileInfo FileInfo;
    QVector<QFileInfo> FileInfoList;
    QVector<TypeImg> ImageTypeList;
    QWidget                                     *ImageCanvas;
    QSvgWidget                              *SvgCanvas;
    QLabel                                        *AnimateCanvas;
    QImage                                       *Image;
    QMovie                                       *Animate;
    CImageViewerTool                  *ImageTool;
    int Scale;
    QSize AnimateOriginSize, SvgOriginSize;
    bool ErrorImage;
    QPalette epal;

  public:
    QFont *Font;
    QScrollArea                              *ImageViewport;

  private:
    void setBackNext();

  protected:
    virtual void resizeEvent(QResizeEvent*);

  public:
    CImageViewer(PixmapsCache*, QFileInfo, QWidget *wgt = 0);
    ~CImageViewer();

    void setCastomPalette(QPalette, QColor);

    void setDirectory(QString);
    inline CImageViewerTool * returnTool() { return ImageTool; }
    void setFontSize();
    TypeImg parseType(QFileInfo);
    void showImage();

    void errorImage(QString, QString);
    void deleteCurrentImage();
    void setEnabledTool(bool);
    void setEnabledModifiedButtons(bool);

  public slots:
    void changeScaleViewport();
    void changeScaleValue(int);
    void showPreviousImage();
    void showNextImage();

    void rotateImageLeft();
    void rotateImageRight();
    void negativeImage();
    void mirrorHorizontal();
    void mirrorVertical();
    void reloadImage();

  signals:
    void setNavigatorPath(QString);

};

#endif
