#ifndef _CMESSAGE_BOX_
#define _CMESSAGE_BOX_

#include <QtGui>
#include "PixmapsCache.h"

enum TypeWind { NoType, InputString, InputWithIcon, InputTwoString, Inform, WarningCopy, WarningCopyWithWrite,  WarningCopyOnIt, WarningRemove, Question, SaveQuestion, ResultOperate };

class CMessageBox : public QDialog {
  Q_OBJECT
  private:
    PixmapsCache *pxCache;
    TypeWind wid;
    QString Text;
    CExcludeEvents *ExcludeEvents;

  public:
    QString dir;
    QGridLayout *layout;
    QLineEdit *lineedit1;
    QLineEdit *lineedit2;
    QTextEdit *textedit;
    QLabel *inform;
    QLabel *icon;
    QLabel *detals;
    QLabel *labelname1;
    QLabel *labelicon1;
    QLabel *labeldetals1;
    QLabel *labelname2;
    QLabel *labelicon2;
    QLabel *labeldetals2;
    QCheckBox *checkbox;
    QGroupBox * box1, *box2;
    QListWidget *listwidget;
    QComboBox *combobox;
    QTreeWidget *treewidget;

    QPushButton *okButton,                                      //      1
                             *cancelButton,                              //      2
                             *renameButton,                            //       3
                             *retryButton,                                 //       4
                             *retryAllButton,                            //       5
                             *writeButton,                                //        6
                             *writeAllButton;                           //        7

  public:
    CMessageBox(PixmapsCache*, QWidget* wgt = 0);

    void createInputString();
    void createInputStringWithIcon(QPixmap);
    void createInputTwoString(QPixmap);
    void createInform(QPixmap);
    void createWarningRemove(QPixmap);
    void createWarningCopy(QPixmap, bool write = true);
    void createWarningCopyOnIt(QPixmap);
    void createQuestion(QPixmap);
    void createSaveQuestion(QPixmap);
    void createResultOperate(QPixmap);

    int execInputString(QString*);
    int execInputString(QString*, QString*);

    void setCastomPalette(QPalette, QColor);
    void setTextIntoWrite(QString);

  public slots:
    inline void returnOk() { done(1); }
    inline void returnCancel() { done(2); }
    inline void returnRename() { done(3); }
    inline void returnRetry() { done(4); }
    inline void returnRetryAll() { done(5); }
    inline void returnWrite() { done(6); }
    inline void returnWriteAll() { done(7); }

    void wasTextChanged(QString);
    void wasReturnPressed();
    void changeRenameEnabled(QString);
    void changeFocus();
    void selectFile();

};

class CCopySettings : public QDialog {
  Q_OBJECT
  public:
    QList<QGroupBox*> boxes;
    QList<QRadioButton*> radiobuttons;
    QList<QVBoxLayout*> layouts;
    QList<QLabel*> labels;
    QVBoxLayout *mainlayout;
    QLabel *headerlabel;
    QCheckBox *checkbox;
    QPushButton* close;

  public:
    CCopySettings(QBitArray*, bool*,  QWidget *wgt = 0);
    void exec(QBitArray *, bool*, bool*);

  public slots:
    void changeAll();
    void end();

};

#endif
