#ifndef _CPIXMAPS_CACHE_
#define _CPIXMAPS_CACHE_

#include <QtGui>
#include <QtXml>
#include <magic.h>
#include "ReserveCache.h"
#include "SymLink.h"
#include "Stat.h"
#include "ExcludeEvents.h"
#include "Palette.h"

class Template {
  public:
     QString FullName,
                    FileName,
                    BaseName,
                    Path,
                    Type,
                    TargetLink,
                    Size,
                    Descr,
                    DescrFromContents,
                    Mime,
                    Created,
                    LastModified,
                    LastRead;

     QPixmap Icon;
     bool SymLink;

     Template ();
};

class TemplateApps {
  public:
      QString nameApp;
      QString execApp;
      QPixmap * iconApp;

     TemplateApps();
};


class XmlHandler : public QXmlDefaultHandler {
  private:
    QString lang;
    QString default_description, lang_description, attribute, buffer;
  public:
    QList<QString> globs, mimetype;
    QString description, nameicon, pclass;

    XmlHandler(QString);

    virtual bool startElement( const QString&,  const QString&, const QString&, const QXmlAttributes&);
    virtual bool characters(const QString&);
    virtual bool endElement(const QString&, const QString&, const QString&);
    virtual bool fatalError (const QXmlParseException &);
    virtual bool endDocument();
    void reset();
};

class PixmapsCache {
  private:
    QList<QFileInfo> ListDrives;
    QString current;
    QPixmap none;
    QMap<QString, QPixmap> Icon;
    QMap<QString, QPixmap> Apps;

//             Mime-Types
    QVector<QString> globs;
    QVector<QString> mimetype;
    QVector<QString> shortmimetype;
    QVector<QString> description;
    QVector<QString> pclass;
    QVector<QPixmap*> pixmap;
    QVector<QChar> standartIcon;

//             Applications
     QVector<QString>  nameApp;
     QVector<QPixmap*> iconApp;
     QVector<QString> execApp;
     QVector<QString> mimeApp;

//           User Applications And Asscociated
     QList<QString> UserApplicationList;
     QMap<QString, QString> AssociationList;

    QList<QPixmap*> pix_list;
    QString animateStateOp;
    QString theme;
    magic_t magic;
    ReserveCache* rCache1, *rCache2, *rCache3, *rCache4;

    QList<QPixmap*> MainIc;
    uint ID_Group, ID_Owner;
    QList<QString> MountPoints;

    QPixmap Logo;
    QPixmap LogoBack;

/*

  -----------------------------------------------------------------------------------------------------------------------------------------------------------------------------
                                    folder-root                         0
                                    folder-home                      1                     Каталоги     -     directories        \\ 0 - 29
                                    folder-usually                   2
                                    drive                                  3
                                    folder-hidden                   4
                                    dragable                           5
                                    block-device                    6
                                    char-device                      7
                                    soket                                 8
                                    fifo                                     9
                                    list-object                         10
   ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
                                    arrow-back                       30
                                    arrow-next                        31                Кнопки навигации по истории     -             navigator      \\ 30 - 39
                                    arrow-up                           32
                                    refresh                              33
                                    home                                34
                                    txt                                      35
                                    navileft                             36
                                    naviright                           37
  ---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
                                    icons                                 40
                                    list                                      41
                                    table                                  42
                                    one-panel                         43               Панель инструментов                  -              control          \\ 40 - 59
                                    two-panel                          44
                                    tree-panel                         45
                                    thumbnails                       46
                                    left-to-right                        47
                                    right-to-left                        48
                                    one-one                            49
  ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
                                    show-hidden                    60
                                    background-color            61
                                    background-texture         62
                                    font-color                           63                    Кнопки панели настроек        -         settings_bar        \\ 60 - 79
                                    font-size                             64
                                    icon-size                            65
                                    themes                               66
                                    is-update                           67
                                    no-update                         68
                                    clean-key                          69
                                    styles                                 70
  -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
                                    create-new                        80
                                    create-symlink                   81
                                    cut                                       82
                                    copy                                    83
                                    paste                                   84                          Кнопки Меню            -            menus                      \\ 80 - 109
                                    rename                               85
                                    delete                                 86
                                    open                                   87
                                    open-with                           88
                                    properites                          89
                                    create-file                           90
                                    select-all                            91
  ---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
                                    chunk-link                          110
                                    symbolic-link                     111                       Вспомогательные            -        castom       \\ 110 - 119
                                    unknown                            112
                                    standart                              113
  ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
                                    arrow-left                            120
                                    arrow-right                          121                        Кнопки панели Веб          -          webtool                   \\120 - 139
                                    refresh                                122
                                    save                                    123
                                    zoom-in                               124
                                    zoom-out                             125
                                    clean-history                       126
 -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
                                    previous                            140
                                    next                                    141
                                    rotate-left                           142                    Кнопки Image Viewer                                                      \\ 140 - 159
                                    rotate-right                        143
                                    clone-horizontal              144
                                    clone-vertical                   145
                                    negative                           146
                                    scale-viewport                 147
                                    refresh                               148
                                    error                                   149
                                    warning                             150
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
                                    dont-edit                           160
                                    refresh                               161
                                    save                                   162                        Кнопки   Text Viewer                                                       \\160 - 179
                                    save-as                              163
                                    zoom-in                              164
                                    zoom-out                            165
                                    undo                                    166
                                    redo                                     167
----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
                                    information                          180
                                    warning                                181
                                    error                                       182
                                    questions                              183
                                    create-folder                          184
                                    create-file                               185
                                    create-symlink                       186
                                    rename                                   187
                      ===============================================
                                    ok                                           190                                 Кнопки Messages                                                    \\ 180 -199
                                    cancel                                    191
                                    retry                                        192
                                    write                                        193
                                    browse                                    194
                                    rename                                    195
                                    save                                          196


*/

    void loadIconsAutomat ( QList<QString>, QString, QString, int);
    void readSettings();
    void writeSettings();

  public:
    bool SaveSettings;

  public:
    PixmapsCache(char **argv, ReserveCache*, ReserveCache*, ReserveCache*, ReserveCache*);
    ~PixmapsCache();

    void fullCache( QString );
    void fullCacheApps(QString);
    void fullMapApps(QString);
    void fullMapApps();
    void setQueueIcon();
    void fullRCache();

    Template fullTemplate (QFileInfo);
    QList<TemplateApps> fullTemplateApps(QFileInfo);
    QString returnExec(QString);

    inline QPixmap* getMainIcon(int i)                                { return MainIc[i];                                                      }
    inline QPixmap* getIcon(int i)                                         { return (pix_list[i]);                                                   }
    inline QPixmap* getPixmap(int i)                                   { return pixmap[i];                                                     }
    inline QPixmap* getPixmap (QString key)                    { return &(Icon[key]);                                                }
    inline QPixmap getLogo()                                                { return Logo;                                                            }
    inline QPixmap getLogoBack()                                       { return LogoBack;                                                  }
    QPixmap * getIcon(QFileInfo);
    QPixmap * getIcon(QFileInfo, QString*);
    QPixmap * getIcon(QFileInfo, QString*, QString*);
    QString getMimeSource(QFileInfo);
    bool subClassMime(QString, QString);
    QString getDescrFromContents(QFileInfo);
    inline QString getThemeName()                                      { return theme;                                                          }
    void getIndexIcon(QFileInfo*, QString *, int*, int*);
    inline QString getAnimateFile()                                        { return animateStateOp;                                         }
    QVector<TemplateApps> getApplicationList();

    inline QList<QString>* getUserApplicationList()            { return &UserApplicationList;                                }
    void addUserApplicationItem(QString);
    void addAssociationItem(QString, QString, bool);
    bool findAssociation(QString, QString);
    QString findAssociation(QString);

    inline void clear(){
      globs.clear();
      mimetype.clear();
      shortmimetype.clear();
      description.clear();
      pclass.clear();
      pixmap.clear();
      standartIcon.clear();
      pix_list.clear();

      nameApp.clear();
      execApp.clear();
      mimeApp.clear();
      iconApp.clear();

      Icon.clear();
      Apps.clear();

      rCache1->clear();
      rCache2->clear();
      rCache3->clear();
      rCache4->clear();
    }
  inline uint getOwner() { return ID_Owner; }
  inline uint getGroup() { return ID_Group; }

  QList<QString>  fullMountPoints();
  QList<QString> getMountPoints();
  QString locateMountPoints(QString);

};

#endif
