#ifndef _CTABLE_VIEWER_
#define _CTABLE_VIEWER_

#include <QtGui>
#include <magic.h>
#include "LineEdit.h"
#include "PixmapsCache.h"
#include "ThreadIcon.h"

class CTableViewer : public QTreeWidget {
  Q_OBJECT
  private:
    QSettings settings;
    QSize size;
    QString themes, ext;
    PixmapsCache *pxCache;
    ReserveCache *rCache1, *rCache2;
    QPixmap pixmap_link, pixmap;
    QPainter painter;

    QDir dir;
    QFileInfo fileinfo, CurrentFileInfo;
    QFileInfoList *fileinfolist;

    QTreeWidgetItem *item, *bufferItem;
    CLineEdit *Editor;

    QTimer  timer;
    CThreadIcon *ThreadIcon;
    CThreadAreaIcon * ThreadAreaIcon;
    int mousePosX, mousePosY, w, posItem;
    int startElement, endElement;
    QBitArray *bitIcon;
    QList<QTreeWidgetItem*> DragListItem;
    QDrag *Drag;
    QTimer DragTimer, ScrollDownTimer, ScrollUpTimer;
    QPoint DragPos;

  public:
    bool ShowHidden, StartTimer, EditElement, *ExcludeDirsMime, *ExcludeDirsTemplate;
    QFont *font;
    QList<QTreeWidgetItem*> itemsList;
    QList<QString>* TemplateFilter;
    QList<QRegExp>* MimeFilter;

  protected:
    virtual void resizeEvent(QResizeEvent *);
    virtual void mousePressEvent(QMouseEvent*);
    virtual void mouseMoveEvent(QMouseEvent*);
    virtual void mouseDoubleClickEvent(QMouseEvent *ev);
    virtual void keyPressEvent(QKeyEvent*);
    virtual void dragMoveEvent(QDragMoveEvent*);
    virtual void dragEnterEvent(QDragEnterEvent*);
    virtual void dragLeaveEvent(QDragLeaveEvent*);
    virtual void dropEvent(QDropEvent*);
    virtual void startDrag(Qt::DropActions act = Qt::MoveAction);

  public:
    CTableViewer( QWidget * wgt = 0 );
    ~CTableViewer();

    inline void setPixmapsCache (PixmapsCache * cache, ReserveCache *rcache1, ReserveCache *rcache2) {
      pxCache = cache;
      rCache1 = rcache1;
      rCache2 = rcache2;
      setPixmapLink();
    }
    inline void setFileInfoList ( QFileInfoList * list){
      fileinfolist = list;
    }

    inline void setPixmapLink() { pixmap_link = pxCache->getIcon(111)->scaled(64, 64, Qt::KeepAspectRatio); }
    inline CThreadIcon* getThreadIcon() { return ThreadIcon; }
    inline CThreadAreaIcon * getThreadAreaIcon() { return ThreadAreaIcon; }

  public slots:
    void showDirectory ( QString, bool );
    void updateDirectory(QString);
    void findAbc(QString);
    void textChanged();
    void editElement();

    void setPixmapElement(int, int, int, QString, QString, QString, bool);

    void initAreaIcon(int);
    void selectionChanged();

    inline void finishSubThread() {
      while ( ThreadAreaIcon->isRunning()) {
        QApplication::processEvents();
        ThreadAreaIcon->execute = false;
      }
  }
    void timeoutDragTimer();
    void scrollUp();
    void scrollDown();

  signals:
    void renameItem (QString, QString);
    void contextMenu ( QMouseEvent*, QString );
    void contextMenu ( QMouseEvent*, QList<QString>);
    void contextAction( QString);
    void contextAction( QString, QString);
    void contextAction (QString, QList<QString>);
    void contextMenu ( QMouseEvent*);
    void refreshWrite (QDir, bool);
    void refreshWrite (QFileInfo, bool);
    void refreshWrite (int);
    void refreshWrite ();
    void pressKey(QString);
    void delSymbolAbc();
    void resetSelection();
    void dropMenu(QList<QString>, QPoint,  QString, bool, int);
    void expandedItem(QTreeWidgetItem*);
};

#endif
