#ifndef _CTHREAD_OP_
#define _CTHREAD_OP_

#include <QtGui>
#include <unistd.h>
#include "MessageBox.h"
#include "PixmapsCache.h"
#include "SymLink.h"

class CThreadGS : public QThread {
  Q_OBJECT
  private:
    QList<QString> listObj;
    quint64 *sizeData, *countData, *countFolders, *countFiles;
    bool Exit;

  public:
    CThreadGS(QObject *parent = 0);
    CThreadGS (QList<QString>, quint64 *sz, quint64 *sd, QObject *parent = 0);

    void setData( QList<QString>, quint64*, quint64*, quint64*, quint64*);
    void getSize(QList<QString>);
    void getSizeDirectory(QString);
    void getSizeAndCount(QList<QString>);
    void getSizeDirectoryAndCount(QString);

    inline void trueExit() { Exit = true; }
    void run();

  signals:
    void signalOfValueSizeAndCount(quint64, quint64);
    void signalOfAllCounts();

};

enum { NotCopy, NotDelete };

class CThreadOp : public QThread {
  Q_OBJECT
  protected:
    int *Errn;
    QList<QString> source;
    QString target;

    quint64 sizeData, countData, sizeComplete, countComplete;
    QString sizestring;

    QTimer *timer;

  public:
    QString newName;
    QBitArray *bitState;
    QList<QString> bufferErrorMessage;
    QList<QChar> bufferErrorStatus;
    QList<QString> bufferErrorFile;
    QList<QString> bufferErrorSystem;
    bool errprint;
    int type;

  public:
    CThreadOp (QList<QString>, QString, int, QObject *parent = 0 );

    inline int exec(){
      return QThread::exec();
    }

    void copy ( QList<QString>, QString );
    void move ( QList<QString>, QString );
    void del (QList<QString>);
    void deleteDirectory (QString);
    bool copyFile(QString, QString);
    void createLinks ( QList<QString>, QString);

    void run();

  public slots:
    void getFromTimer();

  signals:
    void existsName(QString, QString);
    void execComplete(int);
    void complete();
    void errorCriticalComplete(CThreadOp*);
    void abortOperation(CThreadOp *);

};


class CProgressW : public CThreadOp {
  Q_OBJECT
  public:
    QColor BColor, FColor;
    bool trueW;
    QLabel *labelHeader;
    QLabel *countElement;
    QLabel*detals;
    QProgressBar * progressBar;
    QPushButton *cancelButton;
    QVBoxLayout *layout;
    QHBoxLayout *buttonLayout;

    QGroupBox *frame;
    QTimer *wtimer;
    CThreadGS *threadGS;
    bool endcount;

    CProgressW(QList<QString>, QString, int, QObject *parent = 0 );

  signals:
     void showProgressW(CProgressW*);
     void endExec(CProgressW*);

   public slots:
      void setRange(int);
      void createProgressW();
      void setOfValueSizeAndCount ( quint64, quint64);
      void getComplete();
      void emitAbortOperationClicked();
};


class CThreadMonitor : public QScrollArea {
  Q_OBJECT
  private:
    QColor Color;
    QFrame *MainFrame;
    QVBoxLayout *layout;
    QSize OptimumSize;

  private:
    void setOptimumSize();
    int showWarningCopy (CMessageBox*, QFileInfo&, QFileInfo&, Template&, Template&, QString*);

  public:
    bool Lock;
    int countProcess;
    bool Visible;
    QBitArray *bitState;
    PixmapsCache *pxCache;
    inline QSize getOptimumSize()  { return OptimumSize; }

  public:
    CThreadMonitor (QWidget *wgt = 0);
    inline void setPixmapsCache(PixmapsCache* pxcache) { pxCache = pxcache;  }
    void setPalette(QPalette, QColor);
    void setFont(QFont);
    void setLock(bool);

  protected:
    virtual bool event(QEvent*);

  public slots:
    void execOperate ( QList<QString>, QString, int );
    void deleteOperate(CProgressW*);
    void showProgressW(CProgressW*);
    void ifExistsName(QString,QString);
    void abortOperation(CThreadOp*);
    void errorCritical(CThreadOp*);

  signals:
    void runningProcess();
    void noRunningProcess();
    void resizeThis();

};

#endif
