#ifndef _CTREE_VIEWER_
#define _CTREE_VIEWER_

#include <QtGui>
#include <QTreeWidget>
#include "PixmapsCache.h"


class CTreeViewer : public QTreeWidget {
  Q_OBJECT
  private:
    int i, menux, menuy;
    bool                  ShowHidden;

    QDir tmpdir;
    QPixmap                root_icon,
                                     home_icon,
                                     default_icon,
                                     drive_icon,
                                     hidden_icon,
                                     symlink_icon;
     QFont font;

    QTreeWidgetItem * item;
    QFileInfo finf;
    QStringList lst, Link;
    QFileInfoList list_drv;
    QFileSystemWatcher watcher;

    QList<QTreeWidgetItem*> DragListItem;
    QDrag *Drag;
    QTimer DragTimer, ScrollDownTimer, ScrollUpTimer;
    QPoint DragPos;

  public:
    PixmapsCache *pxCache;
    QColor Color;
    QList<QString> DropList;
    QString DropTarget, stritem, strop;
    int DropOp;

  public:
    CTreeViewer ( PixmapsCache *, QWidget * p = 0);

  private:
    void createListDeleteWatcher ( QList<QTreeWidgetItem*> );
    void linkWatched ( QTreeWidgetItem * );

  protected:
    virtual void mousePressEvent(QMouseEvent*);
    virtual void dragMoveEvent(QDragMoveEvent*);
    virtual void dragEnterEvent(QDragEnterEvent*);
    virtual void dragLeaveEvent(QDragLeaveEvent*);
    virtual void dropEvent(QDropEvent*);
    virtual void startDrag(Qt::DropActions act = Qt::MoveAction);

  private slots:
    void ifExpanded ( QTreeWidgetItem * );
    void ifCollapsed ( QTreeWidgetItem * );
    void ifWatched ( QString  );
    void contextMenu(QMouseEvent*, QTreeWidgetItem*);
    void contextMenuSlot();
    void timeoutDragTimer();
    void scrollUp();
    void scrollDown();
    void execDropMenu(QList<QString>, QPoint,  QString, bool, int);
    void slotDrag();

  public:
     void setShowHidden( bool );
     void refreshTheme();

  signals:
    void execContextMenu(QMouseEvent*, QTreeWidgetItem*);
    void execOperation(QString, QString);
    void dropMenu(QList<QString>, QPoint,  QString, bool, int);
    void execOperate(QList<QString>, QString, int);
    void execDrag();
};

#endif

