#ifndef _CTREE_VIEWER_SETTINGS_
#define _CTREE_VIEWER_SETTINGS_

#include <QtGui>
#include "PixmapsCache.h"
#include "SettingsCache.h"
#include "ToolBar.h"
#include "Button.h"

class CTreeViewerSettings : public CToolBar {
  Q_OBJECT
  private:
    QSettings settings;
    QColor currentFontColor, currentBackColor;
    QSize currentIconSize;
    QString currentBackTextureName;
    QPixmap currentBackTexture;
    QFont font;
    int currentFontSize;
    CButton              *ShowHide,
                                 *BackColor,
                                 *BackTexture,
                                 *FontColor,
                                 *FontSize,
                                 *IconSize;

     SettingsCache settingsCache;

     QMenu  *back_texture,
                    *font_size,
                    *icon_size;

    bool showHidden,
             backgroundColor;

  public:
    PixmapsCache *pxCache;
    bool SaveSettings;

  private:
    void readSettings();
    void writeSettings();

  public:
    CTreeViewerSettings (PixmapsCache *, QWidget *wgt = 0);
    ~CTreeViewerSettings();

    inline bool getShowHidden(){
      return showHidden;
    }
    inline QSize getIconSize() {
      return currentIconSize ;
    }
    inline QFont * getFont(){
      return &font;
    }
    inline bool getBackgroundColor(){
      return backgroundColor;
    }
    inline QColor getBackColor(){
      return currentBackColor;
    }
    inline QColor getFontColor(){
      return currentFontColor;
    }
    inline QPixmap getBackTexture(){
      return currentBackTexture;
    }
    void refreshTheme();

  signals:
    void changeShowHidden(bool);
    void changeBackTexture(QPixmap);
    void changeBackColor(QColor);
    void changeFontColor(QColor);
    void changeFontSize(int);
    void changeIconSize(QSize);

  public slots:
    void clickedShowHidden();
    void clickedBackTexture(QAction*);
    void clickedBackColor();
    void clickedFontColor();
    void clickedFontSize(QAction*);
    void clickedIconSize(QAction*);
    void setFlat(bool);

};

#endif
