#include "../Include/Stat.h"

QBitArray intToBitArray(int param, short countbit) {
  QBitArray result(countbit);
  short size = sizeof(int) * 8;
  short step = size - countbit;
  unsigned int var;
  for ( int i = 0; i < countbit; ++i) {
    var = param << (step + i);
    var = var >> (size -1);
    result.setBit(countbit - 1 - i, var);
  }
  return result;
}

int bitArrayToInt (QBitArray bits) {
  int result = 0;
  int sizebit = bits.count();
  for ( int i = sizebit - 1; i >=0; --i) {
    result = result + bits.testBit(i);
    result = result << 1;
  }
  result = result >> 1;
  return result;
}

int countDir (QString path) {
  QByteArray str (path.toLocal8Bit());
  DIR *d;
  int i = 0;
  struct dirent *entry;
  d = opendir(str);
  if ( d == NULL ) return 0;
  entry = readdir(d);
  while ( entry) {
    ++i;
    entry = readdir(d);
  }
  closedir(d);
  return i - 2;
}

void getFileInfoList(QString path, QFileInfoList *flist) {
  QByteArray str(path.toLocal8Bit());
  QFileInfo finf;
  DIR *d;
  QString fname;
  QChar c = QDir::separator();
  struct dirent *entry;
  d = opendir(str);
  if ( d == NULL ) return;
  entry = readdir(d);
  while ( entry ) {
    fname = QString::fromUtf8(entry->d_name);
    if ( fname != "." && fname != ".." ) {
      finf.setFile(path + c + fname);
      *flist << finf;
    }
    entry = readdir(d);
  }
  closedir(d);
}

void permissions ( QString path, QBitArray* perm, int * ID_O, int *ID_G, QBitArray *bitmask) {
  perm->resize(12);
  perm->fill(false);
  QByteArray str(path.toLocal8Bit());
  struct stat sb;
  if (stat(str, &sb) == -1) {
    if (lstat(str, &sb) == -1) {
      *ID_O = -1;
      *ID_G = -1;
      return;
    }
  }
  *perm = intToBitArray(sb.st_mode);
  *ID_O = sb.st_uid;
  *ID_G = sb.st_gid;
  if ( bitmask ) {
    switch (sb.st_mode & S_IFMT) {
      case S_IFBLK:     bitmask->setBit(3, true);
      case S_IFCHR:    bitmask->setBit(4, true);
      case S_IFDIR:      bitmask->setBit(2, true);
      case S_IFIFO:      bitmask->setBit(5, true);
      case S_IFLNK:     bitmask->setBit(0, true);
      case S_IFREG:     bitmask->setBit(1, true);
      case S_IFSOCK:   bitmask->setBit(6, true);
    }
  }
}

void setPermissions(QString path, int perm, int owner, int group) {
  QByteArray str(path.toLocal8Bit());
  if ( owner != -1 ) { int z = chown(str, owner, group); z=0; }
  chmod(str, perm);
}

void setRecursivePermissions(QString path, int perm, int owner, int group) {
  QByteArray str(path.toLocal8Bit());
  if ( owner != -1) { int z = chown(str, owner, group); z=0; }
  chmod(str, perm);
  struct stat sb;
  if ( stat(str, &sb) == -1) return;
  if ((sb.st_mode & S_IFMT) == S_IFDIR) setPermissionsForDirectory(path, perm, owner, group);
}

void setPermissionsForDirectory(QString path, int perm, int owner, int group) {
    QByteArray str(path.toLocal8Bit());
    if ( owner != -1) { int z = chown(str, owner, group); z=0; }
    chmod(str, perm);
    struct stat sb;
    QChar c = QDir::separator();
    QString fname, npath;
    DIR *d;
    struct dirent *entry;
    d = opendir(str);
    if ( d == NULL ) return;
    if ( owner == -1) {
      entry = readdir(d);
      while ( entry ) {
        fname = QString::fromUtf8(entry->d_name);
        if ( fname == "." || fname == "..") {
          entry = readdir(d);
          continue;
        }
        npath = path+c+fname;
        str = npath.toLocal8Bit();
        if ( stat(str, &sb) == -1) {
          entry = readdir(d);
          continue;
        }
        if ((sb.st_mode & S_IFMT) == S_IFDIR) setPermissionsForDirectory(npath, perm, owner, group);
        else chmod(str, perm);
        entry = readdir(d);
      }
    }
    else {
      entry = readdir(d);
      while ( entry ) {
        fname = QString::fromUtf8(entry->d_name);
        if ( fname == "." || fname == "..") {
          entry = readdir(d);
          continue;
        }
        npath = path+c+fname;
        str = npath.toLocal8Bit();
        if ( stat(str, &sb) == -1) {
          entry = readdir(d);
          continue;
        }
        if ((sb.st_mode & S_IFMT) == S_IFDIR) setPermissionsForDirectory(npath, perm, owner, group);
        else {
          int z = chown( str, owner, group); z=0;
          chmod(str, perm);
        }
       entry = readdir(d);
      }
    }
    closedir(d);
}

 QString groupName(int gid) {
   struct group *gr;
   gr = getgrgid(gid);
   QString result;
   if ( gr != NULL ) {
     result = QString::fromUtf8(gr->gr_name);
   }
   else result = "";
   return result;
 }

 int groupId(QString name) {
   struct group *gr;
   int result;
   gr = getgrnam(name.toLocal8Bit());
   if ( gr != NULL ) {
     result = gr->gr_gid;
   }
   else result = -1;
   return result;
 }


void freeSpaceFrom( QString path, quint64* total, quint64* free ) {
  QByteArray str(path.toLocal8Bit());
  quint64 a, b, c;
  struct statfs entry;
  if (statfs(str, &entry) ==  -1) {
    *total = 0;
    *free = 0;
    return;
  }
  a = entry.f_blocks;
  b = entry.f_bfree;
  c = entry.f_bsize;
  *total = a*c;
  *free = b*c;
}

QBitArray fullBitMask(QString path) {
  QByteArray str(path.toLocal8Bit());
  struct stat sb;
  QBitArray bits(7);
  bits.fill(false);
  if (stat(str, &sb) == -1) {
    if (lstat(str, &sb) == -1) {
      return bits;
    }
  }
  switch (sb.st_mode & S_IFMT) {
    case S_IFBLK:     bits.setBit(3, true);
    case S_IFCHR:    bits.setBit(4, true);
    case S_IFDIR:      bits.setBit(2, true);
    case S_IFIFO:      bits.setBit(5, true);
    case S_IFLNK:     bits.setBit(0, true);
    case S_IFREG:     bits.setBit(1, true);
    case S_IFSOCK:   bits.setBit(6, true);
  }
  return bits;
}
