#include "../Include/SymLink.h"

QString readSymLink(QString path, int *errn) {
  QByteArray bufferName(path.toLocal8Bit());
  char bufferTarget[1024];
  int sizeTarget = readlink(bufferName, bufferTarget, 1024);
  if ( sizeTarget != -1) {
    bufferTarget[sizeTarget] = 0;
    *errn = 0;
    return QString::fromUtf8(bufferTarget);
  }
  else return "";
}

bool createSymLink(QString filePath, QString newPath, int *errn) {
  QByteArray fPath(filePath.toLocal8Bit());
  QByteArray nPath(newPath.toLocal8Bit());
  int n = symlink(fPath, nPath);
  if ( n == -1) return false;
  else {
    *errn = 0;
    return true;
  }
}

bool createSymLinkFrom ( QString filePath, QString newPath, int *errn) {
  QString buffer = readSymLink(filePath, errn);
  if ( buffer == "") {
    return false;
  }
  QByteArray fPath(buffer.toLocal8Bit());
  QByteArray nPath(newPath.toLocal8Bit());
  int n = symlink(fPath, nPath);
  if ( n == -1) return false;
  else {
    *errn = 0;
    return true;
  }
}

void setErrorString(QString oldPath, QString newPath, QString * errorString, int *errn) {
  QString sp("  ");
  QString spd(" : ");
  if ( *errn == EACCES)  *errorString = newPath + spd + QObject::tr("permission denied.");
  if ( *errn == EEXIST) *errorString = newPath + sp + QObject::tr("already exists.");
  if ( *errn == EFAULT) *errorString = oldPath + sp + QObject::tr("or") + sp + newPath + sp + QObject::tr("points outside your accessible address space.");
  if ( *errn == EIO ) *errorString = oldPath + sp + "->" + sp + newPath + spd + QObject::tr("an I/O error occurred.");
  if ( *errn == ENAMETOOLONG) *errorString = oldPath + sp + QObject::tr("or") + sp + newPath + sp + QObject::tr("was too long");
  if ( *errn == ENOENT) *errorString = QObject::tr("A directory component") + sp + newPath + sp + QObject::tr("does not exist or is a dangling symbolic link, or") + sp + oldPath + sp + QObject::tr("is the empty string.");
  if ( *errn == ENOMEM) *errorString = oldPath + sp + "->" + sp + newPath + spd + QObject::tr("insufficient kernel memory was available.");
  if ( *errn == ENOSPC) *errorString = oldPath + sp + "->" + sp + newPath + spd + QObject::tr("the device containing the file has no room for the new directory entry.");
  if ( *errn == ENOTDIR) *errorString = QObject::tr("A component used as a directory in") + sp + newPath + sp + QObject::tr("is not, in fact, a directory.");
  if ( *errn == EPERM) *errorString = QObject::tr("The file system containing") + sp + newPath + sp + QObject::tr("does not support the creation of symbolic links.");
  if ( *errn == EROFS) *errorString = newPath + sp + QObject::tr("is on a read-only file system.");
  if ( *errn == EINVAL) *errorString = oldPath + sp + QObject::tr("is not a symbolic link.");
  if ( *errn == ELOOP) *errorString = oldPath + sp + "->" + sp + newPath + spd + QObject::tr("too many symbolic links were encountered in translating the pathname.");
}
