#include "../../Include/IconViewer.h"

CIconViewer :: CIconViewer ( QWidget * wgt ) : QListWidget ( wgt ) {
  setMovement (QListView::Free);
  setLayoutMode ( QListView::SinglePass);
  setResizeMode ( QListView::Adjust);
  setViewMode(QListView::IconMode);
  setSelectionMode(QAbstractItemView::ExtendedSelection);
  setEditTriggers(QAbstractItemView::NoEditTriggers);
  setWordWrap(true);

  setBatchSize(20);
  setUniformItemSizes(true);
  EditElementText = false;
  EditElementLine = false;
  StartTimer = false;
  bitIcon = NULL;
  setAcceptDrops(true);

  ThreadIcon = new CThreadIcon(rCache1, fileinfolist);
  ThreadAreaIcon = new CThreadAreaIcon(rCache2, fileinfolist);

  connect ( this, SIGNAL(itemSelectionChanged()), SLOT(selectionChanged()));
  connect ( &timer, SIGNAL(timeout()), SLOT(editElement()));
  connect ( &DragTimer, SIGNAL(timeout()), this, SLOT(timeoutDragTimer()), Qt::QueuedConnection);
  connect ( &ScrollDownTimer, SIGNAL(timeout()), this, SLOT(scrollDown()), Qt::QueuedConnection);
  connect ( &ScrollUpTimer, SIGNAL(timeout()), this, SLOT(scrollUp()), Qt::QueuedConnection);
}

void CIconViewer :: showDirectory ( QString path, bool read ) {
  if ( StartTimer ) {
    timer.stop();
    StartTimer = false;
  }

  ThreadIcon->execute = false;
  while ( ThreadIcon->isRunning()) {
    QApplication::processEvents();
    ThreadIcon->execute = false;
  }
  ThreadAreaIcon->execute = false;
  while ( ThreadAreaIcon->isRunning()) {
    QApplication::processEvents();
    ThreadAreaIcon->execute = false;
  }

  if ( read ) {
     if ( TemplateFilter->count() == 0 && MimeFilter->count() == 0 ) {
          dir.setPath ( path );
          CurrentFileInfo.setFile(path);
          fileinfolist->clear();
          QFileInfoList templist, templist_hidden;
          if ( ShowHidden ) {
            templist_hidden = dir.entryInfoList ( QDir::Hidden );
            foreach ( fileinfo, templist_hidden )  if ( fileinfo.isDir()) *fileinfolist << fileinfo;
            templist = dir.entryInfoList (QDir::Dirs | QDir::NoDotAndDotDot );
            foreach ( fileinfo, templist ) if ( fileinfo.isDir()) *fileinfolist << fileinfo;
            foreach ( fileinfo, templist_hidden) if ( fileinfo.isFile() ) *fileinfolist << fileinfo;
            *fileinfolist << dir.entryInfoList(QDir::Files | QDir::System );
          }
          else {
            templist =  dir.entryInfoList ( QDir::Dirs | QDir::NoDotAndDotDot );
            foreach ( fileinfo, templist ) if ( fileinfo.isDir()) *fileinfolist << fileinfo;
            *fileinfolist << dir.entryInfoList ( QDir::Files | QDir::System);
          }
     }
     else {
       if ( TemplateFilter->count() != 0 ) {
          dir.setPath ( path );
          fileinfolist->clear();
          QFileInfoList templist, templist_hidden;
          if ( ShowHidden ) {
            if ( *ExcludeDirsTemplate ) templist_hidden = dir.entryInfoList ( QDir::Hidden );
              else  templist_hidden = dir.entryInfoList(*TemplateFilter, QDir::Hidden);
            foreach ( fileinfo, templist_hidden )  if ( fileinfo.isDir()) *fileinfolist << fileinfo;
            if ( *ExcludeDirsTemplate ) templist = dir.entryInfoList(QDir::Dirs | QDir::NoDotAndDotDot);
              else templist = dir.entryInfoList(*TemplateFilter, QDir::Dirs | QDir::NoDotAndDotDot);
            foreach ( fileinfo, templist ) if ( fileinfo.isDir()) *fileinfolist << fileinfo;
            foreach ( fileinfo, templist_hidden ) if ( fileinfo.isFile()) *fileinfolist << fileinfo;
            *fileinfolist << dir.entryInfoList(*TemplateFilter, QDir::Files | QDir::System);
          }
          else {
            if ( *ExcludeDirsTemplate ) templist =  dir.entryInfoList ( QDir::Dirs | QDir::NoDotAndDotDot );
              else templist =  dir.entryInfoList ( *TemplateFilter, QDir::Dirs | QDir::NoDotAndDotDot );
            foreach ( fileinfo, templist ) if ( fileinfo.isDir()) *fileinfolist << fileinfo;
            *fileinfolist << dir.entryInfoList ( *TemplateFilter, QDir::Files | QDir::System);
          }
       }
       if ( MimeFilter->count() != 0 ) {
          QFileInfoList templist, templist_dir_hidden, templist_dir, templist_files_hidden, templist_files,  templist_hidden;
          dir.setPath ( path );
          fileinfolist->clear();
          if ( ShowHidden ) {
            templist_hidden = dir.entryInfoList ( QDir::Hidden);
            foreach ( fileinfo, templist_hidden )  if ( fileinfo.isDir()) templist_dir_hidden << fileinfo;
            templist_dir =  dir.entryInfoList ( QDir::Dirs | QDir::NoDotAndDotDot );
            foreach ( fileinfo, templist_hidden ) if ( fileinfo.isFile()) templist_files_hidden << fileinfo;
            templist_files = dir.entryInfoList(QDir::Files | QDir::System);
          }
          else {
            templist_dir = dir.entryInfoList ( QDir::Dirs | QDir::NoDotAndDotDot );
            templist_files =  dir.entryInfoList ( QDir::Files | QDir::System);
          }
          templist.clear();
          if ( *ExcludeDirsMime ) {
            templist << templist_dir_hidden;
            templist << templist_dir;
          }
          else {
            foreach ( QFileInfo finf, templist_dir_hidden) {
              foreach ( QRegExp rexp, *MimeFilter) {
                if ( pxCache->getMimeSource(finf).contains(rexp)) { templist << finf; break; }
              }
            }
            foreach ( QFileInfo finf, templist_dir) {
              foreach ( QRegExp rexp, *MimeFilter) {
                if ( pxCache->getMimeSource(finf).contains(rexp)) { templist << finf; break; }
              }
            }
          }
          foreach ( QFileInfo finf, templist_files_hidden) {
            foreach ( QRegExp rexp, *MimeFilter) {
              if ( pxCache->getMimeSource(finf).contains(rexp)) { templist << finf; break; }
            }
          }
          foreach ( QFileInfo finf, templist_files) {
            foreach ( QRegExp rexp, *MimeFilter) {
              if ( pxCache->getMimeSource(finf).contains(rexp)) { templist << finf; break; }
            }
          }
          *fileinfolist = templist;
       }
     }
  }

  emit refreshWrite();
  itemsList.clear();
  clear();
  if ( fileinfolist->count() == 0) return;
  this->verticalScrollBar()->setSliderPosition(0);

  delete ThreadIcon;
  delete ThreadAreaIcon;
  if ( bitIcon != NULL ) delete bitIcon;

  ThreadIcon = new CThreadIcon(rCache1, fileinfolist);
  ThreadAreaIcon = new CThreadAreaIcon(rCache2, fileinfolist);

  startElement = 0;
  endElement = 0;

  bitIcon = new QBitArray(fileinfolist->count());
  ThreadAreaIcon->setData(bitIcon, 'T', &startElement, &endElement);
  ThreadIcon->setData( bitIcon, 'T');

  if ( this->viewMode() == QListView::IconMode) {
      ThreadAreaIcon->setData( bitIcon, 'I', &startElement, &endElement);
      ThreadIcon->setData( bitIcon, 'I');
  }
  if ( this->viewMode() == QListView::ListMode) {
       ThreadAreaIcon->setData(bitIcon, 'L', &startElement, &endElement);
       ThreadIcon->setData( bitIcon, 'L');
  }

  SizeHint = this->gridSize() - QSize(15, 10);
  QList<QFileInfo> listdrives = QDir::drives();
  foreach ( fileinfo, *fileinfolist ) {
    item = new QListWidgetItem (this);
    item->setSizeHint(SizeHint);
    item->setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable |
                               Qt::ItemIsEditable | Qt::ItemIsDragEnabled );
    item->setToolTip ( fileinfo.absoluteFilePath());
    item->setText(fileinfo.fileName());
    itemsList << item;
  }
    this->verticalScrollBar()->setSliderPosition(0);
    connect ( this->verticalScrollBar(), SIGNAL(valueChanged(int)), SLOT(initAreaIcon(int)));
    connect ( ThreadIcon, SIGNAL(pixmapElement( int, int, int, bool)),
                                              SLOT(setPixmapElement(int, int, int, bool)));
    connect ( ThreadAreaIcon, SIGNAL(pixmapElement(int, int, int, bool)),
                                                      SLOT(setPixmapElement(int, int, int, bool)));
    connect (ThreadIcon, SIGNAL(quitSubThread()), SLOT(finishSubThread()));
    ThreadIcon->start(QThread::HighPriority);
    ThreadAreaIcon->start(QThread::HighPriority);
    this->setFocus();
}

void CIconViewer :: editElement() {
      timer.stop();
      StartTimer = false;
      item->setSelected(false);
      emit refreshWrite(QFileInfo(item->toolTip()), false);
      QFontMetrics fontMetric (*font);
      QRect rect = visualItemRect(item);
      if ( viewMode() == QListView::IconMode ) {
          EditElementText = true;
          TextEditor = new CTextEdit(this);
          TextEditor->setFont(*font);
          TextEditor->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
          TextEditor->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
          TextEditor->setAlignment(Qt::AlignHCenter);
          int x_center = rect.x() + rect.width()/2;
          QString text = item->text();
          TextEditor->setText("<center>" + text + "</center>");
          TextEditor->setFixedWidth(this->gridSize().width());
          TextEditor->setFixedHeight( fontMetric.height()*2 + 12);
          setItemWidget(item, TextEditor);
          TextEditor->move(x_center - TextEditor->width()/2, rect.y() + this->iconSize().height());
          TextEditor->setPalette(this->palette());
          TextEditor->setFocus();
          QObject::connect (TextEditor, SIGNAL(endOfReturn()), SLOT(textChanged()));
      }
      if ( viewMode() == QListView::ListMode ) {
        EditElementLine = true;
        LineEditor = new CLineEdit(this);
        LineEditor->setFont(*font);
        LineEditor->setText(item->text());
        int q = itemsList.indexOf(item);
        QString bname = fileinfolist->value(q).baseName();
        LineEditor->setSelection(0, bname.length());
        LineEditor->setFixedWidth(rect.width() - iconSize().width() - 10);
        if ( rect.height() > fontMetric.height() + 4 )  LineEditor->setFixedHeight ( rect.height());
        else LineEditor->setFixedHeight ( fontMetric.height() + 4);
        setItemWidget(item, LineEditor);
        LineEditor->setPalette(this->palette());
        LineEditor->setFocus();
        QObject::connect( LineEditor, SIGNAL(endOfReturn()), SLOT(textChanged()));
      }
}

void CIconViewer :: mousePressEvent( QMouseEvent *ev){
  if ( StartTimer ) {
    timer.stop();
    StartTimer = false;
    return;
  }
   if ( EditElementText || EditElementLine ) {
     textChanged();
     return;
   }
  item = itemAt(ev->x(), ev->y());
  if ( ev->button() == Qt::LeftButton) {
    if ( item ) {
      mousePosX = ev->x();
      mousePosY = ev->y();
      QList<QListWidgetItem*> lsw = selectedItems();
      if ( isItemSelected(item) && (lsw.count() == 1)) {
        StartTimer = true;
        timer.start(500);
      }
    }
    QListWidget::mousePressEvent(ev);
    return;
  }

  if ( ev->button() == Qt::RightButton) {
    if ( item ) {
      if ( isItemSelected(item) && (selectedItems().count() == 1)) {
        emit contextMenu( ev, item->toolTip() ); return;
      }
      if ( isItemSelected(item) && (selectedItems().count() > 1)) {
        QList<QString> list;
        foreach ( QListWidgetItem *iitem, selectedItems()) {
          list << iitem->toolTip();
        }
        emit contextMenu( ev, list);  return;
      }
      if ( selectedItems().count() == 0 ) {
        emit contextMenu( ev, item->toolTip() );  return;
      }
      if ( !isItemSelected(item) && (selectedItems().count() > 0)) {
        foreach ( QListWidgetItem* iitem, selectedItems()) {
          iitem->setSelected(false);
        }
        emit contextMenu( ev, item->toolTip());  return;
      }
    }
    else {
      if ( selectedItems().count() > 0) {
        foreach ( QListWidgetItem* iitem, selectedItems()) {
          iitem->setSelected(false);
        }
      }
      emit contextMenu(ev); return;
    }
  }
}

void CIconViewer :: mouseMoveEvent(QMouseEvent *ev) {
  if ( StartTimer ) {
    timer.stop();
    StartTimer = false;
  }
  if ( EditElementText || EditElementLine ) textChanged();
  QListWidget::mouseMoveEvent(ev);
}

void CIconViewer :: mouseDoubleClickEvent(QMouseEvent* ev) {
  if ( StartTimer ) {
    timer.stop();
    StartTimer = false;
  }
  if ( EditElementText || EditElementLine ) textChanged();
  QListWidget::mouseDoubleClickEvent(ev);
}

void CIconViewer :: keyPressEvent(QKeyEvent * ev) {
  QString ffstr;
  if ( ev->key() == Qt::Key_Backspace || ev->key() == Qt::Key_Delete) {
    emit delSymbolAbc();
    return;
  }
  if ( ev->modifiers() == Qt::ControlModifier ) {
    if ( ev->key() == Qt::Key_A ) {
      foreach ( QListWidgetItem *item, itemsList)  item->setSelected(!item->isSelected());
      return;
    }
    if ( ev->key() == Qt::Key_X ) {
      QList<QListWidgetItem*> itt = selectedItems();
      if ( itt.count() == 1 ) {  ffstr = itt[0]->toolTip(); emit contextAction("cut", ffstr); }
      if ( itt.count() >1 ) {
        QList<QString> actlst;
        foreach ( QListWidgetItem* item, itt ) actlst << item->toolTip();
        emit contextAction("cut", actlst);
      }
      return;
    }
    if ( ev->key() == Qt::Key_C ) {
      QList<QListWidgetItem*> itt = selectedItems();
      if ( itt.count() == 1 )  {  ffstr = itt[0]->toolTip(); emit contextAction("copy", ffstr); }
      if ( itt.count() >1 ) {
        QList<QString> actlst;
        foreach ( QListWidgetItem* item, itt ) actlst << item->toolTip();
        emit contextAction("copy", actlst);
      }
      return;
    }
    if ( ev->key() == Qt::Key_V && QApplication::clipboard()->mimeData()->hasFormat("text/uri-list")) {
      emit contextAction("paste");
      return;
    }
    if ( ev->key() == Qt::Key_F && CurrentFileInfo.isWritable()) {
      emit contextAction("created");
      return;
    }
    if ( ev->key() == Qt::Key_S && CurrentFileInfo.isWritable()) {
      emit contextAction("creates");
      return;
    }
    if ( ev->key() == Qt::Key_Delete ) {
      QList<QListWidgetItem*> itt = selectedItems();
      if ( itt.count() == 1 )  {  ffstr = itt[0]->toolTip(); emit contextAction("del", ffstr); }
      if ( itt.count() >1 ) {
        QList<QString> actlst;
        foreach ( QListWidgetItem* item, itt ) actlst << item->toolTip();
        emit contextAction("del", actlst);
      }
      return;
    }
    if ( ev->key() == Qt::Key_Enter ) {
      QList<QListWidgetItem*> itt = selectedItems();
      if ( itt.count() == 0 ) emit contextAction("prop");
      if ( itt.count() == 1 )  {  ffstr = itt[0]->toolTip(); emit contextAction("prop", ffstr); }
      if ( itt.count() >1 ) {
        QList<QString> actlst;
        foreach ( QListWidgetItem* item, itt ) actlst << item->toolTip();
        emit contextAction("prop", actlst);
      }
      return;
    }
  }
  if ( ev->key() == Qt::Key_F2 ) {
    QList<QListWidgetItem*> itt = selectedItems();
    if ( itt.count() == 1 )  {  ffstr = itt[0]->toolTip(); emit contextAction("ren", ffstr); }
    return;
  }
  emit pressKey(ev->text());
}

void CIconViewer :: textChanged() {
   if ( StartTimer) {
     timer.stop();
     StartTimer = false;
   }
   fileinfo.setFile(item->toolTip());
   QString str;
   str = fileinfo.fileName();
   if ( EditElementText ) {
      if (TextEditor->toPlainText() == str ) {
        delete TextEditor;
        EditElementText = false;
        emit refreshWrite();
        return;
      }
      else {
        str = TextEditor->toPlainText();
        delete TextEditor;
        EditElementText = false;
        emit renameItem(item->toolTip(), str);
      }
   }
   if ( EditElementLine ) {
      if (LineEditor->text() == str ) {
        delete LineEditor;
        EditElementLine = false;
        emit refreshWrite();
        return;
      }
      else {
        removeItemWidget(item);
        str = LineEditor->text();
        delete LineEditor;
        EditElementLine = false;
        emit renameItem(item->toolTip(), str);
      }
   }
}

void CIconViewer :: setPixmapElement(int index_item, int index, int flag, bool link)  {
   if ( index_item >=  itemsList.count()) return;
   QListWidgetItem * item = itemsList.value(index_item);

   if (link) {
     if ( flag == 0 ) {
       pix = *(pxCache->getIcon(index));
       painter.begin(&pix);
       painter.drawPixmap(0, 64, pixmap_link);
       painter.end();
       item->setIcon(pix);
     }
     else {
       pix = *(pxCache->getPixmap(index));
       painter.begin(&pix);
       painter.drawPixmap(0, 64, pixmap_link);
       painter.end();
       item->setIcon(pix);
     }
     font->setStyle(QFont::StyleItalic);
     item->setFont(*font);
     font->setStyle(QFont::StyleNormal);
   }
   else {
     if ( flag == 0 ) item->setIcon(*(pxCache->getIcon(index)));
     else item->setIcon(*(pxCache->getPixmap(index)));
     item->setFont(*font);
   }
 bitIcon->setBit(index_item, true);
}

void CIconViewer :: initAreaIcon(int value) {
          if ( fileinfolist->count() == 0 || itemsList.count() <=0 || value <=0 ) return;
          int numElementsInString;
          QSize sizeElement = itemsList.value(0)->sizeHint();
          QSize sizeWidget = this->viewport()->size();
          int Max = this->verticalScrollBar()->maximum();
          if ( this->viewMode() == QListView::IconMode ) numElementsInString = sizeWidget.width() / sizeElement.width(); else numElementsInString = 1;
          float step;
          if ( this->viewMode() == QListView::IconMode ) {
            step = (float) fileinfolist->count() / (float) Max / (float) numElementsInString;
            startElement = step * (float) value * (float) numElementsInString;
            endElement = startElement + (sizeWidget.height() / sizeElement.height()) * (float) numElementsInString;
            if ( startElement - numElementsInString * 5 <= 0 ) startElement = 0; else startElement = startElement - numElementsInString * 5 ;
            if ( endElement + numElementsInString  >= fileinfolist->count()) {
              endElement = fileinfolist->count() - 1;
              int n = sizeWidget.height() / sizeElement.height();
              int count = n * numElementsInString + numElementsInString;
              startElement = endElement - count;
              if (startElement < 0 ) startElement = 0;
            }
              else endElement = endElement + numElementsInString ;
          }
          else {
            step = (float) fileinfolist->count() / (float) Max;
            startElement = step * (float) value;
            endElement = startElement + sizeWidget.height() / sizeElement.height();
            if ( startElement - 15 <= 0) startElement = 0; else startElement = startElement - 15;
            if ( endElement  >= fileinfolist->count()) {
              endElement = fileinfolist->count() - 1;
              startElement = endElement - (sizeWidget.height() / sizeElement.height()) *2;
              if (startElement < 0) startElement = 0;
            }
          }
    }

void CIconViewer :: selectionChanged() {
    QList<QListWidgetItem*> list = selectedItems();
    if ( list.count() == 0 ) {
      emit refreshWrite();
      emit resetSelection();
      return;
    }
    if ( list.count() == 1) {
      QFileInfo finf(list.value(0)->toolTip());
      emit refreshWrite(finf, true);
      emit resetSelection();
      return;
    }
    if ( list.count() > 1) {
      emit refreshWrite(list.count());
      emit resetSelection();
      return;
    }
}

void CIconViewer :: findAbc(QString str) {
  QString name;
  if (str == "" ) {
    this->setCurrentItem(0);
    return;
  }
  foreach ( QListWidgetItem* item, itemsList) {
    if ( item->text().startsWith(str, Qt::CaseInsensitive)) {
      this->blockSignals(true);
      this->setCurrentItem(item);
      this->scrollToItem(item);
      this->blockSignals(false);
      emit refreshWrite(QFileInfo(item->toolTip()), false);
      emit resetSelection();
      return;
    }
  }
  this->blockSignals(true);
  this->setCurrentItem(0);
  this->blockSignals(false);
}

void CIconViewer :: updateDirectory( QString path) {
  if ( StartTimer ) {
    timer.stop();
    StartTimer = false;
  }
  while ( ThreadIcon->isRunning()) { QApplication::processEvents(); }
  while ( ThreadAreaIcon->isRunning()) { QApplication::processEvents(); }

  if ( TemplateFilter->count() == 0 && MimeFilter->count() == 0 ) {
      dir.setPath ( path );
      CurrentFileInfo.setFile(path);
      fileinfolist->clear();
      QFileInfoList templist, templist_hidden;
      if ( ShowHidden ) {
        templist_hidden = dir.entryInfoList ( QDir::Hidden );
        foreach ( fileinfo, templist_hidden )  if ( fileinfo.isDir()) *fileinfolist << fileinfo;
        templist = dir.entryInfoList (QDir::Dirs | QDir::NoDotAndDotDot );
        foreach ( fileinfo, templist ) if ( fileinfo.isDir()) *fileinfolist << fileinfo;
        foreach ( fileinfo, templist_hidden) if ( fileinfo.isFile() ) *fileinfolist << fileinfo;
        *fileinfolist << dir.entryInfoList(QDir::Files | QDir::System );
      }
      else {
        templist =  dir.entryInfoList ( QDir::Dirs | QDir::NoDotAndDotDot );
        foreach ( fileinfo, templist ) if ( fileinfo.isDir()) *fileinfolist << fileinfo;
        *fileinfolist << dir.entryInfoList ( QDir::Files | QDir::System);
      }
  }
  else {
    if ( TemplateFilter->count() != 0 ) {
      dir.setPath ( path );
      fileinfolist->clear();
      QFileInfoList templist, templist_hidden;
      if ( ShowHidden ) {
        if ( *ExcludeDirsTemplate ) templist_hidden = dir.entryInfoList ( QDir::Hidden );
          else  templist_hidden = dir.entryInfoList(*TemplateFilter, QDir::Hidden);
        foreach ( fileinfo, templist_hidden )  if ( fileinfo.isDir()) *fileinfolist << fileinfo;
        if ( *ExcludeDirsTemplate ) templist = dir.entryInfoList(QDir::Dirs | QDir::NoDotAndDotDot);
          else templist = dir.entryInfoList(*TemplateFilter, QDir::Dirs | QDir::NoDotAndDotDot);
        foreach ( fileinfo, templist ) if ( fileinfo.isDir()) *fileinfolist << fileinfo;
        foreach ( fileinfo, templist_hidden ) if ( fileinfo.isFile()) *fileinfolist << fileinfo;
        *fileinfolist << dir.entryInfoList(*TemplateFilter, QDir::Files | QDir::System);
      }
      else {
        if ( *ExcludeDirsTemplate ) templist =  dir.entryInfoList ( QDir::Dirs | QDir::NoDotAndDotDot );
          else templist =  dir.entryInfoList ( *TemplateFilter, QDir::Dirs | QDir::NoDotAndDotDot );
        foreach ( fileinfo, templist ) if ( fileinfo.isDir()) *fileinfolist << fileinfo;
        *fileinfolist << dir.entryInfoList ( *TemplateFilter, QDir::Files | QDir::System);
      }
    }
    if ( MimeFilter->count() != 0 ) {
      QFileInfoList templist, templist_dir_hidden, templist_dir, templist_files_hidden, templist_files,  templist_hidden;
      dir.setPath ( path );
      fileinfolist->clear();
      if ( ShowHidden ) {
        templist_hidden = dir.entryInfoList ( QDir::Hidden);
        foreach ( fileinfo, templist_hidden )  if ( fileinfo.isDir()) templist_dir_hidden << fileinfo;
        templist_dir =  dir.entryInfoList ( QDir::Dirs | QDir::NoDotAndDotDot );
        foreach ( fileinfo, templist_hidden ) if ( fileinfo.isFile()) templist_files_hidden << fileinfo;
        templist_files = dir.entryInfoList(QDir::Files | QDir::System);
      }
      else {
        templist_dir = dir.entryInfoList ( QDir::Dirs | QDir::NoDotAndDotDot );
        templist_files =  dir.entryInfoList ( QDir::Files | QDir::System);
      }
      templist.clear();
      if ( *ExcludeDirsMime ) {
        templist << templist_dir_hidden;
        templist << templist_dir;
      }
      else {
        foreach ( QFileInfo finf, templist_dir_hidden) {
          foreach ( QRegExp rexp, *MimeFilter) {
            if ( pxCache->getMimeSource(finf).contains(rexp)) { templist << finf; break; }
          }
        }
        foreach ( QFileInfo finf, templist_dir) {
          foreach ( QRegExp rexp, *MimeFilter) {
            if ( pxCache->getMimeSource(finf).contains(rexp)) { templist << finf; break; }
          }
        }
      }
      foreach ( QFileInfo finf, templist_files_hidden) {
        foreach ( QRegExp rexp, *MimeFilter) {
          if ( pxCache->getMimeSource(finf).contains(rexp)) { templist << finf; break; }
        }
      }
      foreach ( QFileInfo finf, templist_files) {
        foreach ( QRegExp rexp, *MimeFilter) {
          if ( pxCache->getMimeSource(finf).contains(rexp)) { templist << finf; break; }
        }
      }
      *fileinfolist = templist;
    }
  }

    int i = 0;
    QFileInfo targetlink;
    QList<QString> itemListString;
    QList<QString> fileListString;
    QFont fonti = *font;
    fonti.setItalic(true);

    foreach ( fileinfo, *fileinfolist ) fileListString << fileinfo.absoluteFilePath();
    foreach ( QListWidgetItem *item, itemsList) {
      if ( !fileListString.contains(item->toolTip())) {
        this->takeItem(i);
        itemsList.removeOne(item);
       --i;
      }
    ++i;
    }

    i = 0;
    foreach ( QListWidgetItem *item, itemsList ) itemListString << item->toolTip();
    foreach (fileinfo, *fileinfolist) {
      if ( itemListString.contains(fileinfo.absoluteFilePath())) {
        ++i;
        continue;
      }
      QListWidgetItem* newitem = new QListWidgetItem();
      newitem->setSizeHint(SizeHint);
      newitem->setToolTip(fileinfo.absoluteFilePath());
      if ( fileinfo.isSymLink()) {
        newitem->setFont(fonti);
        targetlink.setFile(fileinfo.symLinkTarget());
        while ( targetlink.isSymLink()) targetlink.setFile(targetlink.symLinkTarget());
        if (targetlink.exists()) {
          if ( targetlink.isDir()) {
            pix = *(pxCache->getIcon(2));
            if ( targetlink.absoluteFilePath() == QDir::homePath()) pix = *(pxCache->getIcon(1));
            if ( fileinfo.isHidden()) pix = *(pxCache->getIcon(4));
            painter.begin(&pix);
            painter.drawPixmap(0, 64, pixmap_link);
            painter.end();
            newitem->setIcon(pix);
          }
          else {
            pix = *(pxCache->getIcon(targetlink));
            painter.begin(&pix);
            painter.drawPixmap(0, 64, pixmap_link);
            painter.end();
            newitem->setIcon(pix);
            }
        }
        else {
          pix = *(pxCache->getIcon(113));
          painter.begin(&pix);
          painter.drawPixmap(0, 64, pixmap_link);
          painter.end();
          newitem->setIcon(pix);
        }
      }
      else {
        if ( fileinfo.isDir()) {
            newitem->setIcon(*(pxCache->getIcon(2)));
            if ( fileinfo.absoluteFilePath() == QDir::homePath()) newitem->setIcon(*(pxCache->getIcon(1)));
            if ( fileinfo.isHidden()) newitem->setIcon(*(pxCache->getIcon(4)));
        }
        else {
          newitem->setIcon(*(pxCache->getIcon(fileinfo)));
        }
      }
      newitem->setSizeHint(SizeHint);
      newitem->setFont(*font);
      this->insertItem(i, newitem);
      newitem->setText(fileinfo.fileName());
      itemsList.insert(i, newitem);
      ++i;
    }

    QList<QListWidgetItem*> lstt = this->selectedItems();
    if ( lstt.count() == 1 ) emit refreshWrite(QFileInfo(lstt[0]->toolTip()), false);
    if ( lstt.count() > 1 ) emit refreshWrite(lstt.count());
    if ( lstt.count() == 0 )   emit refreshWrite();
}


void CIconViewer :: startDrag(Qt::DropActions act) {
  DragListItem = this->selectedItems();
  QMimeData *DragMime = new QMimeData();
  QList<QUrl> Url;
  foreach ( item, DragListItem) Url << item->toolTip();
  DragMime->setUrls(Url);
  Drag = new QDrag(this);
  Drag->setMimeData(DragMime);
  if ( DragListItem.count() == 1 ) Drag->setPixmap(DragListItem[0]->icon().pixmap(128, 128).scaled(iconSize()));
    else Drag->setPixmap(pxCache->getIcon(5)->scaled(iconSize()));
  Drag->exec(act);
}

void CIconViewer :: dragMoveEvent( QDragMoveEvent *ev) {
  DragTimer.stop();
  DragPos = ev->pos();
  if ( DragPos.y() > this->height()-20) {
    DragTimer.stop();
    ScrollDownTimer.start(100);
    return;
  }
  else {
    if ( DragPos.y() < 10 ) {
      DragTimer.stop();
      ScrollUpTimer.start(100);
      return;
    }
    else {
      ScrollDownTimer.stop();
      ScrollUpTimer.stop();
    }
  }

  item = itemAt(DragPos);
  if ( item ) {
    QString filename = item->toolTip();
    int i;
    bool found  = false;
    for ( i = 0; i < fileinfolist->count(); ++i ) {
      if ( fileinfolist->value(i).absoluteFilePath() == filename){
        found = true;
        break;
      }
    }
    if ( found ) {
      fileinfo = fileinfolist->value(i);
      if ( fileinfo.isDir() && fileinfo.isWritable()) {
        ev->acceptProposedAction();
        DragTimer.start(700);
      }
      else ev->setAccepted(false);
    }
  }
  else
    if ( CurrentFileInfo.isWritable()) ev->setAccepted(true);
      else ev->setAccepted(false);
}

void CIconViewer :: dragEnterEvent(QDragEnterEvent* ev) {
  if ( ev->mimeData()->hasFormat("text/uri-list") && CurrentFileInfo.isWritable()) ev->acceptProposedAction();
}

void CIconViewer :: dragLeaveEvent(QDragLeaveEvent*) {
  DragTimer.stop();
  ScrollUpTimer.stop();
  ScrollDownTimer.stop();
}

void CIconViewer :: dropEvent(QDropEvent* ev) {
  DragTimer.stop();
  ScrollDownTimer.stop();
  ScrollUpTimer.stop();
  ev->acceptProposedAction();
  QList<QString> DropList;
  QFileInfo dropfinf;
  bool exm = false;
  DragPos = ev->pos();
  QList<QUrl> urllist = ev->mimeData()->urls();
  foreach ( QUrl url, urllist) DropList << url.path();
  if ( ev->keyboardModifiers() == Qt::NoModifier ) {
    exm = true;
    item = itemAt(DragPos);
    if ( item ) emit dropMenu( DropList, QCursor::pos(), item->toolTip(), exm, 0);
      else  emit dropMenu( DropList, QCursor::pos(), "", exm, 0);
    return;
  }
  if ( ev->keyboardModifiers() == Qt::CTRL ) {
    item = itemAt(DragPos);
    if ( item ) emit dropMenu(DropList, QCursor::pos(), item->toolTip(), exm, 1);
      else emit dropMenu(DropList, QCursor::pos(), "", exm, 1);
    return;
  }
  if ( ev->keyboardModifiers() == Qt::SHIFT) {
    item = itemAt(DragPos);
    if ( item ) emit dropMenu( DropList, QCursor::pos(), item->toolTip(), exm, 2);
      else emit dropMenu(DropList, QCursor::pos(), "", exm,  2);
    return;
  }
  if ( ev->keyboardModifiers() == Qt::CTRL + Qt::SHIFT) {
    item = itemAt(DragPos);
    if ( item ) emit dropMenu(DropList, QCursor::pos(), item->toolTip(), exm, 4);
      else emit dropMenu(DropList, QCursor::pos(), "", exm, 4);
  }
}

void CIconViewer :: timeoutDragTimer() {
  DragTimer.stop();
  item = itemAt(DragPos);
  if ( item ) {
    QFileInfo dragtarget(item->toolTip());
    if (dragtarget.isDir()) emit expandedItem(item);
  }
}

void CIconViewer :: scrollUp() {
  if ( verticalScrollBar()->value() >  verticalScrollBar()->minimum()) {
    if ( viewMode() == QListView::IconMode) verticalScrollBar()->setValue(verticalScrollBar()->value() - 20);
      else verticalScrollBar()->setValue(verticalScrollBar()->value() - 1);
  }
}

void CIconViewer :: scrollDown() {
  if ( verticalScrollBar()->value() < verticalScrollBar()->maximum()) {
    if ( viewMode() == QListView::IconMode) verticalScrollBar()->setValue(verticalScrollBar()->value() + 20);
      else verticalScrollBar()->setValue(verticalScrollBar()->value() + 1);
  }
}

CIconViewer :: ~CIconViewer() {
  while ( ThreadIcon->isRunning() || ThreadAreaIcon->isRunning()) QApplication::processEvents();
  delete ThreadIcon;
  delete ThreadAreaIcon;
  if ( bitIcon != NULL) delete bitIcon;
}
