#include "../../Include/IconViewerControl.h"

CIconViewerControl :: CIconViewerControl(PixmapsCache * pxcache, QWidget *wgt) :
                                                                                                                           CToolBar(wgt ){
  setIconSize(QSize(24,24));
  pxCache = pxcache;
  readSettings();
  setFlat(Flat);
  Icons = addAction ( *(pxCache->getIcon(40)), tr("Icons"), tr("Icons"));
  Icons->setCheckable(true);
  List = addAction ( *(pxCache->getIcon(41)), tr("List"), tr("List"));
  List->setCheckable(true);
  Table = addAction ( *(pxCache->getIcon(42)), tr("Table"), tr("Table"));
  Table->setCheckable(true);

  addSeparator();
  OnePanel = addAction(*(pxCache->getIcon(43)), tr("One"), tr("One panel"));
  OnePanel->setCheckable(true);
  TwoPanel = addAction(*(pxCache->getIcon(44)), tr("Two"), tr("Two panel"));
  TwoPanel->setCheckable(true);
  TreePanel = addAction(*(pxCache->getIcon(45)), tr("Tree"), tr("Tree and panel"));
  TreePanel->setCheckable(true);

  addSeparator();
  LeftToRight = addAction(*(pxCache->getIcon(47)), tr("Left-To-Right"), tr("Open left panel to right"));
  RightToLeft = addAction(*(pxCache->getIcon(48)), tr("Right-To-Left"), tr("Open right panel to left"));
  OneOne = addAction(*(pxCache->getIcon(49)), tr("Change"), tr("Change panels"));

  Layout.addStretch(1);
  addSeparator();
  FlatButton = addAction(tr("Flat"), tr("Flat buttons") );
  FlatButton->setFlat(!Flat);
  TextButton = addAction(tr("Text"), tr("Text with icons"));
  About = addAction(*(pxCache->getMainIcon(3)), "");
  TextButton->setCheckable(true);
  TextButton->setChecked(Text);
   setShowText(Text);

  if ( view == "icons" ) Icons->setChecked(true);
  if ( view == "list" ) List->setChecked(true);
  if ( view == "table" ) Table->setChecked(true);
  if ( panel == "one" ) OnePanel->setChecked(true);
  if ( panel == "two" ) TwoPanel->setChecked(true);
  if ( panel == "tree" ) TreePanel->setChecked(true);

  connect ( this, SIGNAL(clickedButton(CButton*)), SLOT(clickedView(CButton*)));
  connect ( FlatButton, SIGNAL(clicked()), SLOT(clickFlat()));
  connect ( TextButton, SIGNAL(clicked()), SLOT(setText()));
  connect (About, SIGNAL(clicked()), SLOT(showAboutWindow()));

  SaveSettings = true;
}

void CIconViewerControl :: clickedView (CButton *button) {
  bool read = false;
  if ( button == Icons ) {
    if (view == "table") read = true;
    Icons->setChecked(true);
    List->setChecked(false);
    Table->setChecked(false);
    view = "icons";
    emit changeView ( "icons", read  );
  }

  if ( button == List ) {
    if ( view == "table" ) read = true;
    Icons->setChecked(false);
    List->setChecked(true);
    Table->setChecked(false);
    view = "list";
    emit changeView ( "list", read );
  }

  if ( button == Table ) {
    if ( view == "icons" || view == "list" ) read = true;
    Icons->setChecked(false);
    List->setChecked(false);
    Table->setChecked(true);
    view = "table";
    emit changeView ( "table", read );
  }

  if (button == OnePanel ) {
    if (panel == "one") return;
    OnePanel->setChecked(true);
    TwoPanel->setChecked(false);
    TreePanel->setChecked(false);
    panel = "one";
    emit changePanel(panel);
  }

  if (button == TwoPanel) {
    if ( panel == "two") return;
    OnePanel->setChecked(false);
    TwoPanel->setChecked(true);
    TreePanel->setChecked(false);
    panel = "two";
    emit changePanel(panel);
  }

  if (button == TreePanel) {
    if ( panel == "tree") return;
    OnePanel->setChecked(false);
    TwoPanel->setChecked(false);
    TreePanel->setChecked(true);
    panel = "tree";
    emit changePanel(panel);
  }

  if ( button == LeftToRight )  emit openContentInOther("lefttoright");
  if ( button == RightToLeft ) emit openContentInOther("righttoleft");
  if ( button == OneOne ) emit openContentInOther("change");

}

void CIconViewerControl :: readSettings() {
  settings.beginGroup ("/Bathyscaphe/IconViewer" );
  view = settings.value ( "/View", "icons" ).toString();
  Flat = settings.value ( "/Flat", true ).toBool();
  Text = settings.value ( "/Text", false).toBool();
  Invert = settings.value ("/Invert", false).toBool();
  panel = settings.value ("/Panel", "tree").toString();
  Thumbs = settings.value("/Thumbnails", false).toBool();
  settings.endGroup();
}

void CIconViewerControl :: writeSettings() {
  if ( SaveSettings ) {
    settings.beginGroup ("/Bathyscaphe/IconViewer" );
    settings.setValue ("/View", view );
    settings.setValue ( "/Flat", Flat );
    settings.setValue ( "/Text", Text);
    settings.setValue ("/Invert", Invert);
    settings.setValue ("/Panel", panel);
    settings.setValue ("/Thumbnails", Thumbs);
    settings.endGroup();
  }
}

void CIconViewerControl :: setBackColor(QColor color){
  CToolBar :: setBackColor(darkerColor(color, 50));
}

void CIconViewerControl :: setFontColor(QColor color) {
  CToolBar :: setFontColor(color);
}

void CIconViewerControl :: refreshTheme() {
  Icons->setIcon( *(pxCache->getIcon(40)));
  List->setIcon( *(pxCache->getIcon(41)));
  Table->setIcon ( *(pxCache->getIcon(42)));
  OnePanel->setIcon(*(pxCache->getIcon(43)));
  TwoPanel->setIcon(*(pxCache->getIcon(44)));
  TreePanel->setIcon(*(pxCache->getIcon(45)));
  LeftToRight->setIcon(*(pxCache->getIcon(47)));
  RightToLeft->setIcon(*(pxCache->getIcon(48)));
  OneOne->setIcon(*(pxCache->getIcon(49)));
}

void CIconViewerControl :: clickFlat(){
  setFlat(!Flat);
  Flat = !Flat;
  FlatButton->setFlat(!Flat);
  emit changeFlat(Flat);
}

void CIconViewerControl :: setText(){
  Text = !Text;
  CToolBar::setShowText(Text);
}

CIconViewerControl :: ~CIconViewerControl() {
  writeSettings();
}
