#include "../../Include/IconViewerNavigator.h"
#include "../../Include/ToolBar.h"
#include "../../Include/ExcludeEvents.h"

CIconViewerNavigator :: CIconViewerNavigator ( QWidget*  wgt) : QFrame(wgt) {
   setAutoFillBackground(true);
   freeButtonSpace = 10;
   currentIndexButton = 0;
   posHistory = -1;
   for (int i = 0; i < 22; ++i) scrollPosition << 0;
   addSeparator();
   QFileInfoList drive = QDir::drives();
   foreach ( QFileInfo finf, drive ) {
     listdrives << finf.absoluteFilePath();
   }
   Width = this->width();
   IconSize = 16;
   setFrameStyle(QFrame::Panel | QFrame::Sunken);
   setContentsMargins(2, 2, 2, 2);
   setLineWidth(1);
   setMidLineWidth(1);
   setMaximumHeight(IconSize + 16);

  String = new QLineEdit(this);
  ExcludeEvents = new CExcludeEvents(82, String);
  String->installEventFilter(ExcludeEvents);
  textpanel = false;
  LabelIcon = new CButton(this);
  LabelIcon->setFlat(true);
  LabelIcon->setIconSize(QSize(24,24));
  LabelIcon->setMinimumHeight(IconSize + 14);
  LabelIcon->setMaximumHeight(IconSize +14);
  LabelIcon->setMinimumWidth(IconSize + 14);
  LabelIcon->setMaximumWidth(IconSize + 14);
  String->setMaximumHeight(IconSize + 14);
  String->setMinimumHeight(IconSize + 14);
  LabelIcon->move(4, 0);
  SeparatorList[0]->move(0, 0);
  LabelIcon->show();
  SeparatorList[0]->show();

  connect ( String, SIGNAL(returnPressed()), SLOT(changeTextPanel()));
}

void CIconViewerNavigator :: setPath ( QString path ) {
  oldPath = path;
  currentIndexButton = 0;
  if ( TextButton->isEnabled()) {
      QFileInfo fileinfo;
      if ( !textpanel ) {
        String->hide();
        QList<QString> list;
        QList<QString> listdir;
        clear();
        if ( listdrives.contains(path)) {
          list << path;
          listdir << path;
        }
        else {
            while ( true ) {
              fileinfo.setFile ( path );
              list << fileinfo.fileName();
              listdir << fileinfo.absoluteFilePath();
              path = fileinfo.absolutePath();
              if ( listdrives.contains (path)) {
                list << path;
                listdir << path;
                break;
              }
            }
        }
        QList<QString>::Iterator It = list.end();
        QList<QString>::Iterator Itt = listdir.end();
        for ( ; It != list.begin(); ) {
          --It;
          --Itt;
          if ( It != (list.end()-1)) addPoint();
          addAction ( *It, *Itt );
          addPoint();
        }
        showPath();
      }
      else {
        clear();
        String->setText(path);
        BackButton->hide();
        ForwardButton->hide();
        String->show();
        String->move(LabelIcon->width(), 0);
      }
  }
  else {
        clear();
        String->setText(path);
        BackButton->hide();
        ForwardButton->hide();
        String->move(LabelIcon->width() + 6, 0);
        String->show();
      }
}

void CIconViewerNavigator :: changeText() {
  if ( TextButton->isEnabled()) {
    if ( !textpanel ) {
      QList<CButton*>::Iterator It = ActionList.end();
      --It;
      CButton *butt = *It;
      QString path = butt->toolTip();
      textpanel = true;
      String->show();
      setPath(path);
      String->setFocus();
    }
    else {
      if ( String->text().isEmpty()) {
        emit changeLocation(oldPath);
        return;
      }
      else {
        QFileInfo finf(String->text());
        if ( !finf.exists() ) {
          emit changeLocation(String->text());
          return;
        }
      }
      textpanel = false;
      String->hide();
      setPath(String->text());
      if (oldPath != String->text()) emit changeLocation(String->text());
    }
  }
}

void CIconViewerNavigator :: clear() {
    for ( int i = 0; i < PointList.count(); ++i)  delete PointList[i];
    for ( int i = 0; i < ActionList.count(); ++i ) delete ActionList[i];
    PointList.clear();
    ActionList.clear();
    sizesButton.clear();
    realSizesButton.clear();
    ButtonsText.clear();
}

void CIconViewerNavigator :: setFontSize() {
  QFontMetrics fm(*Font);
  if ( TextButton->isEnabled()) {
    if ( !textpanel ) {
      sizesButton.clear();
      realSizesButton.clear();
      QFontMetrics fm(*Font);
      QString text;
      CButton *button;
      for (int i = 0; i < ActionList.count(); ++i) {
        button = ActionList[i];
        button->setFont(*Font);
        text = ButtonsText[i];
        button->setMaximumWidth(fm.width(text) + 10);
        button->setMinimumWidth(fm.width(text) + 10);
        sizesButton << button->width();
        realSizesButton << button->width();
      }
      showPath();
    }
  }
  String->setFont(*Font);
}

void CIconViewerNavigator :: changeTextPanel() {
  if ( !String->text().isEmpty()) emit changeLocation(String->text());
  else String->setText(oldPath);
}

 void CIconViewerNavigator :: setTextOnly(bool en) {
      if ( en ) {
          TextButton->setEnabled(false);
          String->show();
      }
      else {
        TextButton->setEnabled(true);
        if ( textpanel ) String->show();
        else String->hide();
     }
 }

QString CIconViewerNavigator :: getLocation() {
   if ( TextButton->isEnabled()) {
     if (!textpanel) {
       QList<CButton*>::Iterator It = ActionList.end() - 1;
       CButton* button = *It;
       return button->toolTip();
     }
     else return String->text();
   }
   else return String->text();
}

bool CIconViewerNavigator :: isTextOnly() {
      return TextButton->isEnabled();
    }

void CIconViewerNavigator :: showPath() {
  foreach (CPointSeparator *sep, PointList) sep->hide();
  foreach (CButton *butt, ActionList) butt->hide();
  int SizeHint = 0;
  bool bback = false, bforward = false;
  int numbersButton = currentIndexButton;
  sizesButton = realSizesButton;
  for ( int i = currentIndexButton -1; i >= 0; --i) {
    if ( SizeHint + realSizesButton[i] <= freeButtonSpace ) {
      --numbersButton;
      SizeHint = SizeHint + realSizesButton[numbersButton] + 8;
      continue;
    }
    else {
      if ( freeButtonSpace - SizeHint > 40 )
        --numbersButton;
        sizesButton[i] = freeButtonSpace -  SizeHint - 14;
        SizeHint = SizeHint + sizesButton[i] + 8;
        if ( i > 0 ) bback = true;
        break;
    }
  }

  SizeHint = 4 + LabelIcon->width();
  if ( bback ) {
    BackButton->show();
    BackButton->move(SizeHint, 0);
    SizeHint = SizeHint + BackButton->width() + 2;
  }
  else BackButton->hide();
  for ( int i = numbersButton ; i < currentIndexButton ; ++i) {
    ActionList[i]->setMaximumWidth(sizesButton[i]);
    ActionList[i]->setMinimumWidth(sizesButton[i]);
    ActionList[i]->move(SizeHint, 0);
    ActionList[i]->setText(catString(ButtonsText[i], ActionList[i]->width()));
    ActionList[i]->show();
    SizeHint = SizeHint + sizesButton[i] + 2;
    bforward = false;
    if ( i != ActionList.count() - 1) {
      PointList[i]->move(SizeHint, 2);
      PointList[i]->show();
      SizeHint = SizeHint + 6;
      bforward  = true;
    }
  }

  numbersButton = currentIndexButton -1;
  while ( numbersButton <  ActionList.count() -1 ) {
    ++numbersButton;
    if ( freeButtonSpace -  SizeHint >= realSizesButton[numbersButton] || freeButtonSpace  - SizeHint > 40) {
      if  ( freeButtonSpace -  SizeHint >= realSizesButton[numbersButton]) {
        ActionList[numbersButton]->show();
        ActionList[numbersButton]->move(SizeHint, 0);
        ActionList[numbersButton]->setText(catString(ButtonsText[numbersButton], ActionList[numbersButton]->width()));
        SizeHint = SizeHint + realSizesButton[numbersButton] + 2;
        bforward = false;
        if ( numbersButton != ActionList.count() -1) {
          PointList[numbersButton]->move(SizeHint, 2);
          PointList[numbersButton]->show();
          SizeHint = SizeHint + 6;
          bforward = true;
        }
      }
      else {
        if (freeButtonSpace - SizeHint > 40) {
          ActionList[numbersButton]->show();
          ActionList[numbersButton]->setMaximumWidth(freeButtonSpace - SizeHint);
          ActionList[numbersButton]->setMinimumWidth(freeButtonSpace - SizeHint);
          ActionList[numbersButton]->move(SizeHint, 0);
          ActionList[numbersButton]->setText(catString(ButtonsText[numbersButton], ActionList[numbersButton]->width()));
          SizeHint = SizeHint + ActionList[numbersButton]->width() + 2;
          bforward = false;
          if ( numbersButton != ActionList.count() -1) {
            PointList[numbersButton]->move(SizeHint, 2);
            PointList[numbersButton]->show();
            SizeHint = SizeHint + 6;
            bforward = true;
          }
        }
      }
    }
  }

  if ( bforward == true ) {
    ForwardButton->show();
    ForwardButton->move(this->width() - TextButton->width() - ForwardButton->width() - 2, 0);
  }
  else {
    ForwardButton->hide();
    int i = ActionList.count() - 1;
    if ( i >=0 ) {
      QPalette pal = this->palette();
      QColor col = pal.color(QPalette::Base);
      changeBackgroundPalette(&pal, darkerColor(col, 15));
      ActionList[i]->setPalette(pal);
      QFont fnt(*Font);
      fnt.setBold(true);
      ActionList[i]->setFont(fnt);
    }
  }
}

void CIconViewerNavigator :: backStrPath() {
  currentIndexButton--;
  showPath();
}

void CIconViewerNavigator :: forwardStrPath() {
  currentIndexButton++;
  showPath();
}

void CIconViewerNavigator :: resizeEvent(QResizeEvent*) {
  freeButtonSpace = this->width() - LabelIcon->width() - TextButton->width() - 40;
  TextButton->move (this->width() - TextButton->width() - 2, 0);
  String->setMaximumWidth(freeButtonSpace);
  String->setMinimumWidth(freeButtonSpace);
  showPath();
}

void CIconViewerNavigator :: addAction (QString &text, QString &toolTip){
  QFontMetrics fm(*Font);
  CButton *button = new CButton(this);
  button->setFlat(true);
  button->setToolTip(toolTip);
  button->setFont(*Font);
  button->setMaximumWidth(fm.width(text) + 10);
  button->setMinimumWidth(fm.width(text) + 10);
  button->setMinimumHeight(IconSize + 14);
  button->setMaximumHeight(IconSize + 14);
  button->setText(text);
  ActionList << button;
  ButtonsText << text;

  sizesButton << button->width();
  realSizesButton << button->width();
  ++currentIndexButton;
  QObject::connect ( button, SIGNAL(clicked()), SLOT(clickButton()));

}

void CIconViewerNavigator :: addSeparator() {
   QWidget * Separator = new QWidget(this);
   Separator->setAutoFillBackground(true);
   QPalette pal = this->palette();
   pal.setColor(Separator->backgroundRole(), Qt::red);
   Separator->setPalette(pal);
   Separator->setMaximumWidth(2);
   Separator->setMinimumWidth(2);
   Separator->setMaximumHeight(IconSize + 14);
   Separator->setMinimumHeight(IconSize + 14);
   SeparatorList << Separator;

}

void CIconViewerNavigator :: addPoint() {
   CPointSeparator *Point = new CPointSeparator(this);
   Point->setMaximumHeight(IconSize + 10);
   Point->setMinimumHeight(IconSize + 10);
   Point->setMinimumWidth(4);
   Point->setMaximumWidth(4);
   QPalette pal = this->palette();
   Point->setPalette(pal);
   PointList << Point;
}


void CIconViewerNavigator :: clickButton(){
  CButton* button = qobject_cast<CButton*>(sender());
  if (button == ActionList[ActionList.count() - 1]) {
    currentIndexButton = ActionList.count();
    showPath();
    return;
  }
  emit clickedButton(button);
}

void CIconViewerNavigator :: setBackColor (QColor color){
  QPalette pal = this->palette();
  changeBackgroundPalette(&pal, color);
  this->setPalette(pal);
  foreach ( CPointSeparator * ps, PointList) {
    ps->setPalette(pal);
  }
  foreach (CButton *button, ActionList) {
     if (button->existsMenu()) button->getMenu()->setPalette(pal);
  }
}

void CIconViewerNavigator :: setFontColor (QColor color) {
  QPalette pal = this->palette();
  changeTextPalette(&pal, color);
  this->setPalette(pal);

  foreach ( CPointSeparator * ps, PointList) {
    ps->setPalette(pal);
  }
}

QString CIconViewerNavigator :: catString ( QString str, int width) {
  QFontMetrics fm(*Font);
  if ( fm.width(str) <= (width - 10)) return str;
  str.append("...");
  while ( fm.width(str) - width3point > width - 26 ) {
    str.resize(str.length()-4);
    str.append("...");
  }
  return str;
}

void CIconViewerNavigator :: refreshTheme() {
  TextButton->setIcon(*(pxCache->getIcon(35)));
  BackButton->setIcon(*(pxCache->getIcon(36)));
  ForwardButton->setIcon(*(pxCache->getIcon(37)));
}
