#include "../../Include/IconViewerPanel.h"
#include "../../Include/IconViewerNavigator.h"
#include "../../Include/IconViewerMoving.h"
#include "../../Include/IconViewer.h"
#include "../../Include/TableViewer.h"
#include "../../Include/PixmapsCache.h"
#include "../../Include/MessageBox.h"

CIconViewerPanel :: CIconViewerPanel ( PixmapsCache * PCache, ReserveCache *RCache1, ReserveCache *RCache2, QWidget *wgt ) : CIconViewerPanelBase( PCache, RCache1, RCache2, wgt ) {
  Monitor = new CThreadMonitor();
  Monitor->setPixmapsCache(PCache);
  bitState = new QBitArray(13);
  showWarningRemove = new bool;
  readSettings();

   Monitor->setAutoFillBackground(true);
   Monitor->setFrameStyle(QFrame::Box | QFrame::Raised);
   Monitor->setLineWidth(2);
   setBitMonitor();
   Monitor->hide();

   connect ( this, SIGNAL(execOperate(QList<QString>, QString, int)), Monitor,
                                                             SLOT(execOperate(QList<QString>, QString, int)));
   connect ( Monitor, SIGNAL(runningProcess()), SLOT(showProgressW()));
   connect ( Monitor, SIGNAL(noRunningProcess()), SLOT(hideProgressW()));
   connect ( Monitor, SIGNAL(resizeThis()), SLOT(resizeProgressW()));

   WebHistory->loadHistory();
   SaveSettings = true;

}

void CIconViewerPanel :: readSettings() {
  QSettings settings;
  bool flag;
  settings.beginGroup ("/Bathyscaphe/IconViewer");
  for ( int i = 0; i < 13; ++i ) {
     flag = settings.value("/FlagsOfDiskOperation" + QString::number(i), false).toBool();
     bitState->setBit(i, flag);
  }
  if ( !bitState->testBit(0) && !bitState->testBit(1)) bitState->setBit(0, true);
  if ( !bitState->testBit(2) && !bitState->testBit(3) && !bitState->testBit(4)) bitState->setBit(2, true);
  if ( !bitState->testBit(5) && !bitState->testBit(6)) bitState->setBit(5, true);
  if ( !bitState->testBit(7) && !bitState->testBit(8) && !bitState->testBit(9)) bitState->setBit(7, true);
  if ( !bitState->testBit(10) && !bitState->testBit(11) && !bitState->testBit(12)) bitState->setBit(10, true);
  queryQuestions = settings.value("/QueryQuestions", false).toBool();
  if ( bitState->testBit(0))  *showWarningRemove = true;
  else *showWarningRemove = false;

  settings.endGroup();
}

void CIconViewerPanel :: writeSettings(){
  if ( SaveSettings ) {
    QSettings settings;
    settings.beginGroup("/Bathyscaphe/IconViewer");
    for ( int i = 0; i < 13; ++i )
      settings.setValue("/FlagsOfDiskOperation" + QString::number(i), bitState->testBit(i));
    settings.setValue("/QueryQuestions", queryQuestions);
    settings.endGroup();
    WebHistory->saveHistory();
  }
}

void CIconViewerPanel :: setFontSize(){
  CIconViewerPanelBase::setFontSize();
   Monitor->setFont(*(TableViewer->font));
}

void CIconViewerPanel :: setBackColor(QColor color) {
  CIconViewerPanelBase::setBackColor(color);
  QPalette pal = this->palette();
  changeBackgroundPalette(&pal, color);
  Monitor->setPalette(pal, color);
  Color = color;
}

void CIconViewerPanel :: setFontColor(QColor color) {
  QPalette pal = this->palette();
  changeTextPalette(&pal, color);
  Monitor->setPalette(pal, Color);
  CIconViewerPanelBase::setFontColor(color);
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////               THREADS

void CIconViewerPanel :: switchVisibleProgressW() {
  if ( Monitor->Lock) return;
  if ( Monitor->countProcess > 0 ) {
      if ( !Monitor->Visible ) {
        int xi, yi;
        xi = Rect.width() + Rect.x();
        yi = Rect.height() + Rect.y();
        QSize sz = Monitor->getOptimumSize();
        int xm = sz.width();
        int ym = sz.height();
        if ( xm > Rect.width()-24 )  xm = Rect.width() -24;
        if ( ym > Rect.height() -150 ) ym = Rect.height() - 150;
        Monitor->setFixedSize( xm + 40, ym + 20 );
        Monitor->move(xi - xm-40, yi - ym - 54);
        Monitor->Visible = true;
        Monitor->show();
        Monitor->setMouseTracking(true);
        Monitor->grabMouse();
      }
      else {
        Monitor->Visible = false;
        Monitor->hide();
        Monitor->setMouseTracking(false);
        Monitor->releaseMouse();
      }
  }
  else {
      showCopyParametres();
  }
}

void CIconViewerPanel :: showProgressW() {
  if ( Monitor->countProcess > 0 ) Monitor->Visible = false;
  switchVisibleProgressW();
}

void CIconViewerPanel :: hideProgressW() {
  Monitor->Visible = false;
  Monitor->hide();
  Monitor->setMouseTracking(false);
  Monitor->releaseMouse();
}

void CIconViewerPanel :: resizeProgressW(QRect rectange){
  Rect = rectange;
  Monitor->Visible = false;
  Monitor->setMouseTracking(false);
  Monitor->hide();
  Monitor->releaseMouse();
}

void CIconViewerPanel :: resizeProgressW() {
    int xi, yi;
    xi = Rect.width() + Rect.x();
    yi = Rect.height() + Rect.y();
    QSize sz = Monitor->getOptimumSize();
    int xm = sz.width();
    int ym = sz.height();
    if ( xm > Rect.width()-24 )  xm = Rect.width() -24;
    if ( ym > Rect.height() -150 ) ym = Rect.height() - 150;
    if ( xm < 0 || ym < 0 ) {
      Monitor->Visible = false;
      Monitor->hide();
      setMouseTracking(false);
      Monitor->releaseMouse();
      return;
    }
    Monitor->setFixedSize( xm + 40, ym + 20 );
    Monitor->move(xi - xm-40, yi - ym - 54);
}

void CIconViewerPanel :: setBitMonitor() {
  Monitor->bitState->setBit(0, bitState->testBit(4));
  Monitor->bitState->setBit(1, bitState->testBit(3));
  Monitor->bitState->setBit(2, bitState->testBit(6));
  Monitor->bitState->setBit(3, bitState->testBit(9));
  Monitor->bitState->setBit(4, bitState->testBit(8));
  Monitor->bitState->setBit(5, bitState->testBit(12));
  Monitor->bitState->setBit(6, bitState->testBit(11));
}

void CIconViewerPanel :: showCopyParametres() {
  CCopySettings *CopySettings = new CCopySettings(bitState, &queryQuestions);
  QPalette pal = this->palette();
  changeBackgroundPalette(&pal, darkerColor(Color, 30));
  CopySettings->setPalette(pal);
  CopySettings->setModal(true);
  CopySettings->exec(bitState, &queryQuestions, showWarningRemove);

  setBitMonitor();

  delete CopySettings;
}

CIconViewerPanel :: ~CIconViewerPanel(){
  writeSettings();
}
