#include "../../Include/IconViewerPanelBase.h"
#include "../../Include/IconViewerNavigator.h"
#include "../../Include/IconViewerMoving.h"
#include "../../Include/IconViewer.h"
#include "../../Include/TableViewer.h"
#include "../../Include/PixmapsCache.h"
#include "../../Include/MessageBox.h"
#include "../../Include/TextViewer.h"
#include "../../Include/TextViewerTool.h"

CIconViewerPanelBase :: CIconViewerPanelBase ( PixmapsCache * PCache, ReserveCache *RCache1, ReserveCache *RCache2, QWidget *wgt ) : QFrame (wgt ) {
  pxCache = PCache;
  rCache1 = RCache1;
  rCache2 = RCache2;
  VLayout = new QVBoxLayout ( this );
  HLayout = new QHBoxLayout ();
  HLayout->setSpacing(0);
  HLayout->setMargin(0);
  VLayout->setSpacing(0);
  VLayout->setMargin(0);
  Moving = new CIconViewerMoving();
  Moving->setPixmapsCache( PCache);
  ListMovingButton = Moving->getMovingButtons();
  Navigator = new CIconViewerNavigator ();
  Navigator->setPixmapsCache(pxCache);
  Navigator->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred));
  listdrives = Navigator->getListDrives();
  Stack = new QStackedWidget ();
  IconViewer = new CIconViewer();
  IconViewer->setPixmapsCache(PCache, RCache1, RCache2);
  IconViewer->setFileInfoList (&fileinfolist);
  IconViewer->TemplateFilter = &TemplateFilter;
  IconViewer->MimeFilter = &MimeFilter;
  IconViewer->ExcludeDirsMime = &ExcludeDirsMime;
  IconViewer->ExcludeDirsTemplate = &ExcludeDirsTemplate;
  TableViewer = new CTableViewer;
  TableViewer->setPixmapsCache (PCache, RCache1, RCache2);
  TableViewer->setFileInfoList (&fileinfolist);
  TableViewer->TemplateFilter = &TemplateFilter;
  TableViewer->MimeFilter = &MimeFilter;
  TableViewer->ExcludeDirsMime = &ExcludeDirsMime;
  TableViewer->ExcludeDirsTemplate = &ExcludeDirsTemplate;
  Stack->addWidget ( IconViewer );
  Stack->addWidget ( TableViewer );

   HLayout->setAlignment ( Qt::AlignLeft );
   HLayout->addWidget ( Moving );
   HLayout->addWidget ( Navigator );
   VLayout->addLayout ( HLayout );
   VLayout->addWidget ( Stack );
   if ( QDir::home().exists()) {
      current_dir = QDir::homePath();
      fileinfo.setFile(current_dir);
      if ( fileinfo.isReadable()) watcher.addPath(current_dir);
      Navigator->addHistory(current_dir);
      Moving->setHomeEnabled(false);
   }
   else {
     current_dir = QDir::rootPath();
     fileinfo.setFile(current_dir);
     if ( fileinfo.isReadable()) watcher.addPath(current_dir);
     Navigator->addHistory(current_dir);
   }

   clipboard = QApplication::clipboard();

   connect ( &watcher, SIGNAL(directoryChanged(QString)), SLOT(changeWatcher(QString)));

   connect ( IconViewer, SIGNAL(itemDoubleClicked(QListWidgetItem*)), SLOT(changeDirectory(QListWidgetItem*)));
   connect ( TableViewer, SIGNAL(itemDoubleClicked(QTreeWidgetItem*, int)), SLOT(changeDirectory(QTreeWidgetItem*)));
   connect ( this, SIGNAL(openElement(QListWidgetItem*)), SLOT(changeDirectory(QListWidgetItem*)));
   connect ( this, SIGNAL(openElement(QTreeWidgetItem*)), SLOT(changeDirectory(QTreeWidgetItem*)));

   connect ( Navigator, SIGNAL ( clickedButton(CButton*)), SLOT(changeDirectory(CButton*)));
   connect ( Navigator, SIGNAL ( changeLocation(QString)), SLOT(changeLocation(QString)));
   connect ( Moving, SIGNAL(clickedButton(CButton*)), SLOT(slotMoveDirectory(CButton *)));

   connect ( IconViewer, SIGNAL(renameItem(QString,QString)), SLOT(renameItem(QString, QString )));
   connect ( TableViewer, SIGNAL(renameItem(QString,QString)), SLOT(renameItem(QString,QString)));

//...................................................................................................................................................................              Exec Menu

   connect ( IconViewer, SIGNAL(contextMenu(QMouseEvent*, QString)),
                                            SLOT(execContextMenu(QMouseEvent*, QString)));
   connect ( IconViewer, SIGNAL(contextMenu(QMouseEvent*)),
                                            SLOT(execContextMenu(QMouseEvent*)));
   connect ( IconViewer, SIGNAL(contextMenu(QMouseEvent*, QList<QString>)),
                                            SLOT (execContextMenu(QMouseEvent*, QList<QString>)));
   connect ( IconViewer, SIGNAL(contextAction(QString, QList<QString>)), SLOT(execContextAction(QString, QList<QString>)));
   connect ( IconViewer, SIGNAL(contextAction(QString, QString)), SLOT(execContextAction(QString, QString)));
   connect ( IconViewer, SIGNAL(contextAction(QString)), SLOT(execContextAction(QString)));

   connect ( TableViewer, SIGNAL(contextMenu(QMouseEvent*, QString)),
                                            SLOT(execContextMenu(QMouseEvent*, QString)));
   connect ( TableViewer, SIGNAL(contextMenu(QMouseEvent*)),
                                            SLOT(execContextMenu(QMouseEvent*)));
   connect ( TableViewer, SIGNAL(contextMenu(QMouseEvent*, QList<QString>)),
                                            SLOT (execContextMenu(QMouseEvent*, QList<QString>)));
   connect ( TableViewer, SIGNAL(contextAction(QString, QList<QString>)), SLOT(execContextAction(QString, QList<QString>)));
   connect ( TableViewer, SIGNAL(contextAction(QString, QString)), SLOT(execContextAction(QString, QString)));
   connect ( TableViewer, SIGNAL(contextAction(QString)), SLOT(execContextAction(QString)));

   connect ( IconViewer, SIGNAL(dropMenu(QList<QString>, QPoint,  QString, bool, int)),
                                            SLOT(execDropMenu(QList<QString>, QPoint, QString, bool, int)));
   connect ( TableViewer, SIGNAL(dropMenu(QList<QString>, QPoint,  QString, bool, int)),
                                            SLOT(execDropMenu(QList<QString>, QPoint, QString, bool, int)));

    connect ( this, SIGNAL(execDrag(int)), this, SLOT(slotDropMenu(int)), Qt::QueuedConnection);
    connect ( this, SIGNAL(execDrag(QString)), this, SLOT(slotDropMenu(QString)), Qt::QueuedConnection);
    connect ( IconViewer, SIGNAL(expandedItem(QListWidgetItem*)), SLOT(changeDirectory(QListWidgetItem*)));
    connect ( TableViewer, SIGNAL(expandedItem(QTreeWidgetItem*)), SLOT(changeDirectory(QTreeWidgetItem*)));

  WebViewer = NULL;
  TextViewer = NULL;
  ImageViewer = NULL;
  OtherView = false;
  WebHistory = new CWebHistory();
  TextViewerDontEdit = true;
  TextViewerScale = 0;
  WebViewerScale = 1;
}

void CIconViewerPanelBase :: changeDirectory ( QListWidgetItem * item ) {
  QString str = item->toolTip();
  fileinfo.setFile(str);
  if ( !fileinfo.exists()) {
      CMessageBox * dial = new CMessageBox(pxCache, this);
      dial->createInform(*(pxCache->getIcon(182)));
      dial->inform->setText(createErrorText(str));
      dial->setCastomPalette(IconViewer->palette(), Color);
      dial->exec();
      delete dial;
      return;
  }
  if ( fileinfo.isDir()) {
      deleteViewers();
      if ( current_dir != str ) {
          dir.setPath ( str );
          if ( dir.exists()) {
            Navigator->setScrollPosition(IconViewer->verticalScrollBar()->value());
            IconViewer->showDirectory ( str, true );
            Navigator->setPath ( str);
            setNavigatorIcon(fileinfo);
            Navigator->addHistory(str);
            current_dir = str;
            watcher.removePaths(watcher.directories());
            if ( fileinfo.isReadable()) watcher.addPath(current_dir);
            if ( listdrives->contains(current_dir)) Moving->setUpEnabled (false);
              else Moving->setUpEnabled (true);
            if ( Navigator->getBackState()) Moving->setBackEnabled(true);
              else Moving->setBackEnabled(false);
            if ( Navigator->getNextState()) Moving->setNextEnabled(true);
              else Moving->setNextEnabled(false);
            Moving->setHomeEnabled(Navigator->getHomeState());
            Moving->setRefreshEnabled(true);
          }
      }
      return;
  }

  QString mimestring;
  if ( fileinfo.isFile() || fileinfo.isSymLink()) {
    TypeF FileType = parseType(fileinfo, &mimestring);
    if (FileType == Web) {
      Navigator->setPath(str);
      Navigator->setLabelIcon(*NavigatorIcon);
      Moving->setBackEnabled(true);
      Moving->setUpEnabled(false);
      Moving->setNextEnabled(false);
      Moving->setHomeEnabled(true);
      Moving->setRefreshEnabled(false);
      runWebViewer(fileinfo.absoluteFilePath());
      return;
    }
    if (FileType == Text) {
      Navigator->setPath(str);
      Navigator->setLabelIcon(*NavigatorIcon);
      Moving->setBackEnabled(true);
      Moving->setUpEnabled(false);
      Moving->setNextEnabled(false);
      Moving->setHomeEnabled(true);
      Moving->setRefreshEnabled(false);
      runTextViewer(fileinfo.absoluteFilePath());
      return;
    }
    if (FileType == Image) {
      Navigator->setPath(str);
      Navigator->setLabelIcon(*NavigatorIcon);
      Moving->setBackEnabled(true);
      Moving->setUpEnabled(false);
      Moving->setNextEnabled(false);
      Moving->setHomeEnabled(true);
      Moving->setRefreshEnabled(false);
      runImageViewer(pxCache, fileinfo);
      return;
    }
    if ( FileType == Known ) return;
    if ( FileType == Unknown ) {
      selectProgram();
      return;
    }
  }
}

void CIconViewerPanelBase :: changeDirectory ( QTreeWidgetItem * item ) {
  if ( TextViewer != NULL) if (TextViewer->document()->isModified()) textViewerSave();
  QString str = item->toolTip( 0 );
  fileinfo.setFile(str);
  if ( !fileinfo.exists()) {
      CMessageBox * dial = new CMessageBox(pxCache, this);
      dial->createInform(*(pxCache->getIcon(182)));
      dial->inform->setText(createErrorText(str));
      dial->setCastomPalette(IconViewer->palette(), Color);
      dial->exec();
      delete dial;
      return;
    }
    if ( fileinfo.isDir()) {
      deleteViewers();
      if ( current_dir != str ) {
          dir.setPath ( str );
          if ( dir.exists()) {
            if ( view == "icons" || view == "list" ) {
              Navigator->setScrollPosition(IconViewer->verticalScrollBar()->value());
              IconViewer->showDirectory (str, true);
            }
            else {
              Navigator->setScrollPosition(TableViewer->verticalScrollBar()->value());
              TableViewer->showDirectory (str, true);
            }
            Navigator->setPath ( str);
            setNavigatorIcon(fileinfo);
            Navigator->addHistory(str);
            current_dir = str;
            watcher.removePaths(watcher.directories());
            if ( fileinfo.isReadable()) watcher.addPath(current_dir);
            if ( listdrives->contains(current_dir)) Moving->setUpEnabled (false);
              else Moving->setUpEnabled (true);
            if ( Navigator->getBackState()) Moving->setBackEnabled(true);
              else Moving->setBackEnabled(false);
            if ( Navigator->getNextState()) Moving->setNextEnabled(true);
              else Moving->setNextEnabled(false);
            Moving->setHomeEnabled(Navigator->getHomeState());
            Moving->setRefreshEnabled(true);
            return;
          }
      }
      return;
   }
  QString mimestring;
  if ( fileinfo.isFile() || fileinfo.isSymLink()) {
    TypeF FileType = parseType(fileinfo, &mimestring);
    if (FileType == Web ) {
      Navigator->setPath(str);
      Navigator->setLabelIcon(*NavigatorIcon);
      Moving->setBackEnabled(true);
      Moving->setUpEnabled(true);
      Moving->setHomeEnabled(true);
      Moving->setRefreshEnabled(false);
      runWebViewer(fileinfo.absoluteFilePath());
      return;
    }
    if (FileType == Text) {
      Navigator->setPath(str);
      Navigator->setLabelIcon(*NavigatorIcon);
      Moving->setBackEnabled(true);
      Moving->setUpEnabled(false);
      Moving->setNextEnabled(false);
      Moving->setHomeEnabled(true);
      Moving->setRefreshEnabled(false);
      runTextViewer(fileinfo.absoluteFilePath());
      return;
    }
    if (FileType == Image) {
      Navigator->setPath(str);
      Navigator->setLabelIcon(*NavigatorIcon);
      Moving->setBackEnabled(true);
      Moving->setUpEnabled(false);
      Moving->setNextEnabled(false);
      Moving->setHomeEnabled(true);
      Moving->setRefreshEnabled(false);
      runImageViewer(pxCache, fileinfo);
      return;
    }
    if ( FileType == Known ) return;
    if ( FileType == Unknown ) {
      selectProgram();
      return;
    }
  }
}

void CIconViewerPanelBase :: changeDirectory ( CButton * action ) {
  if ( TextViewer != NULL) if (TextViewer->document()->isModified()) textViewerSave();
  QString str = action->toolTip();
  fileinfo.setFile(str);
  if ( !fileinfo.exists()) {
      CMessageBox * dial = new CMessageBox(pxCache, this);
      dial->createInform(*(pxCache->getIcon(182)));
      dial->inform->setText(createErrorText(str));
      dial->setCastomPalette(IconViewer->palette(), Color);
      dial->exec();
      delete dial;
      return;
    }
  if (fileinfo.isFile()) return;
  if ( OtherView ) {
    if (current_dir == str) {
        deleteViewers();
        Navigator->setPath(str);
        setNavigatorIcon(fileinfo);
        if ( listdrives->contains(current_dir)) Moving->setUpEnabled (false);
          else Moving->setUpEnabled (true);
        if ( Navigator->getBackState()) Moving->setBackEnabled(true);
          else Moving->setBackEnabled(false);
        Moving->setHomeEnabled(Navigator->getHomeState());
        Moving->setRefreshEnabled(true);
        return;
    }
    else {
        deleteViewers();
        dir.setPath (str );
        if ( dir.exists()) {
          if ( view == "icons" || view == "list" ){
            Navigator->setScrollPosition(IconViewer->verticalScrollBar()->value());
            IconViewer->showDirectory (str, true);
          }
          else {
            Navigator->setScrollPosition(TableViewer->verticalScrollBar()->value());
            TableViewer->showDirectory (str, true);
          }
          Navigator->setPath ( str );
          setNavigatorIcon(fileinfo);
          Navigator->addHistory(str);
          current_dir = str;
          watcher.removePaths(watcher.directories());
          if ( fileinfo.isReadable()) watcher.addPath(current_dir);
          if ( listdrives->contains(current_dir)) Moving->setUpEnabled (false);
            else Moving->setUpEnabled (true);
          if ( Navigator->getBackState()) Moving->setBackEnabled(true);
            else Moving->setBackEnabled(false);
          if ( Navigator->getNextState()) Moving->setNextEnabled(true);
            else Moving->setNextEnabled(false);
          Moving->setHomeEnabled(Navigator->getHomeState());
          Moving->setRefreshEnabled(true);
          return;
        }
     }
  }
  if ( current_dir != str ) {
      dir.setPath (str );
      if ( dir.exists()) {
        if ( view == "icons" || view == "list" ) IconViewer->showDirectory (str, true);
            else TableViewer->showDirectory (str, true);
        Navigator->setPath ( str );
        setNavigatorIcon(fileinfo);
        Navigator->addHistory(str);
        current_dir = str;
        watcher.removePaths(watcher.directories());
        if ( fileinfo.isReadable()) watcher.addPath(current_dir);
        if ( listdrives->contains(current_dir)) Moving->setUpEnabled (false);
          else Moving->setUpEnabled (true);
        if ( Navigator->getBackState()) Moving->setBackEnabled(true);
          else Moving->setBackEnabled(false);
        if ( Navigator->getNextState()) Moving->setNextEnabled(true);
          else Moving->setNextEnabled(false);
        Moving->setHomeEnabled(Navigator->getHomeState());
      }
  }
}

void CIconViewerPanelBase :: changeLocation(QString location) {
  if ( TextViewer != NULL) if (TextViewer->document()->isModified()) textViewerSave();
  if ( location.startsWith("file://") || LocationStartsWithDrives(location)) {
    if (location.startsWith("file://")) location.replace("file://", "");
    fileinfo.setFile(location);
    if ( fileinfo.exists()) {
      if ( fileinfo.isDir()) {
        deleteViewers();
        if ( view == "icons" || view == "list" ) {
          Navigator->setScrollPosition(IconViewer->verticalScrollBar()->value());
          IconViewer->showDirectory (location, true);
        }
        else {
          Navigator->setScrollPosition(TableViewer->verticalScrollBar()->value());
          TableViewer->showDirectory (location, true);
        }
        Navigator->setPath ( location );
        setNavigatorIcon(fileinfo);
        Navigator->addHistory(location);
        current_dir = location;
        watcher.removePaths(watcher.directories());
        if ( fileinfo.isReadable()) watcher.addPath(current_dir);
        if ( listdrives->contains(current_dir)) Moving->setUpEnabled (false);
          else Moving->setUpEnabled (true);
        if ( Navigator->getBackState()) Moving->setBackEnabled(true);
          else Moving->setBackEnabled(false);
        if ( Navigator->getNextState()) Moving->setNextEnabled(true);
          else Moving->setNextEnabled(false);
        Moving->setHomeEnabled(Navigator->getHomeState());
        Moving->setRefreshEnabled(true);
      }
      if ( fileinfo.isFile() || fileinfo.isSymLink()) {
        QString mimestring;
        TypeF FileType = parseType(fileinfo, &mimestring);
        if (FileType == Web ) {
          Navigator->setLabelIcon(*NavigatorIcon);
          Moving->setBackEnabled(true);
          Moving->setUpEnabled(true);
          Moving->setHomeEnabled(true);
          Moving->setRefreshEnabled(false);
          if (WebViewer != NULL) {
            WebViewer->load(QUrl(location));
            WebViewerTool->PrevButton->setEnabled(WebViewer->history()->canGoBack());
            WebViewerTool->ForwardButton->setEnabled(WebViewer->history()->canGoForward());
          }
          else {
            deleteViewers();
            runWebViewer(fileinfo.absoluteFilePath());
          }
          return;
        }
        if (FileType == Text) {
          Navigator->setLabelIcon(*NavigatorIcon);
          Moving->setBackEnabled(true);
          Moving->setUpEnabled(false);
          Moving->setNextEnabled(false);
          Moving->setHomeEnabled(true);
          Moving->setRefreshEnabled(false);
          if ( TextViewer != NULL ) {
            TextViewer->loadText(location);
            TextViewerTool->UndoButton->setEnabled(false);
            TextViewerTool->RedoButton->setEnabled(false);
          }
          else {
            deleteViewers();
            runTextViewer(fileinfo.absoluteFilePath());
          }
          Navigator->setPath(location);
          return;
        }
        if (FileType == Image) {
          Navigator->setLabelIcon(*NavigatorIcon);
          Moving->setBackEnabled(true);
          Moving->setUpEnabled(false);
          Moving->setNextEnabled(false);
          Moving->setHomeEnabled(true);
          Moving->setRefreshEnabled(false);
          deleteViewers();
          runImageViewer(pxCache, fileinfo);
          Navigator->setPath(location);
          return;
        }
        if ( FileType == Known ) return;
        if ( FileType == Unknown ) {
          selectProgram();
          return;
        }
      }
    }
    else {
      CMessageBox * dial = new CMessageBox(pxCache, this);
      dial->createInform(*(pxCache->getIcon(182)));
      dial->inform->setText(createErrorText(location));
      dial->setCastomPalette(IconViewer->palette(), Color);
      dial->exec();
      delete dial;
      return;
    }
  }

  else {
    if ( location.startsWith("http://") || location.startsWith("https://") ||
         location.startsWith("ftp://") || location.startsWith("gopher://")) {
      runWebViewer(location);
    }
    else {
      location = "http://" + location;
      runWebViewer(location);
    }
  }
}

void CIconViewerPanelBase :: slotMoveDirectory (CButton* button) {
  int scroll;
  if ( TextViewer != NULL) if (TextViewer->document()->isModified()) textViewerSave();
  if ( button == ListMovingButton[2] ) {                                                                                       //Вверх
    if ( OtherView ) {
      deleteViewers();
      Navigator->setPath(current_dir);
      Navigator->setLabelIcon(*NavigatorIcon);
      if ( Navigator->getBackState()) Moving->setBackEnabled(true);
        else Moving->setBackEnabled(false);
      if ( listdrives->contains(current_dir)) Moving->setUpEnabled (false);
        else Moving->setUpEnabled (true);
      Moving->setHomeEnabled(Navigator->getHomeState());
      Moving->setRefreshEnabled(true);
      return;
    }
    fileinfo.setFile(current_dir);
    current_dir = fileinfo.absolutePath();
    watcher.removePaths(watcher.directories());
    if ( fileinfo.isReadable()) watcher.addPath(current_dir);
    if ( view == "icons" || view == "list" ) {
      Navigator->setScrollPosition(IconViewer->verticalScrollBar()->value());
      IconViewer->showDirectory (current_dir, true);
    }
    else {
      Navigator->setScrollPosition(TableViewer->verticalScrollBar()->value());
      TableViewer->showDirectory (current_dir, true);
    }
    Navigator->setPath(current_dir);
    setNavigatorIcon(fileinfo);
    Navigator->addHistory(current_dir);
    if ( listdrives->contains(current_dir)) Moving->setUpEnabled (false);
    if ( Navigator->getBackState()) Moving->setBackEnabled(true);
       else Moving->setBackEnabled(false);
    if ( Navigator->getNextState()) Moving->setNextEnabled(true);
       else Moving->setNextEnabled(false);
    Moving->setHomeEnabled(Navigator->getHomeState());
  }

  if (button == ListMovingButton[0] ) {                                                                                             //Назад
    if ( OtherView ) {
      deleteViewers();
      Navigator->setPath(current_dir);
      setNavigatorIcon(fileinfo);
      if ( Navigator->getBackState()) Moving->setBackEnabled(true);
        else Moving->setBackEnabled(false);
      if ( listdrives->contains(current_dir)) Moving->setUpEnabled (false);
        else Moving->setUpEnabled (true);
      Moving->setHomeEnabled(Navigator->getHomeState());
      Moving->setRefreshEnabled(true);
      return;
    }
    if ( Navigator->getPosHistory() > 0 ) {
      fileinfo.setFile ( Navigator->getBackHistory(&scroll));
      if ( fileinfo.exists() && fileinfo.isDir()) {
        current_dir = fileinfo.absoluteFilePath();
        watcher.removePaths(watcher.directories());
        if ( fileinfo.isReadable()) watcher.addPath(current_dir);
        if ( view == "icons" || view == "list" ) {
          IconViewer->showDirectory (current_dir, true);
          IconViewer->verticalScrollBar()->setValue(scroll);
        }
        else {
          TableViewer->showDirectory (current_dir, true);
          TableViewer->verticalScrollBar()->setValue(scroll);
        }
        Navigator->setPath(current_dir);
        setNavigatorIcon(fileinfo);
        if ( listdrives->contains(current_dir)) Moving->setUpEnabled (false);
          else Moving->setUpEnabled(true);
        if ( Navigator->getBackState()) Moving->setBackEnabled(true);
          else Moving->setBackEnabled(false);
        if ( Navigator->getNextState()) Moving->setNextEnabled(true);
          else Moving->setNextEnabled(false);
        Moving->setHomeEnabled(Navigator->getHomeState());
      }
    }
  }

  if (button == ListMovingButton[1]) {                                                                                                        //Вперед
    fileinfo.setFile (Navigator->getForwardHistory(&scroll));
    if (fileinfo.exists() && fileinfo.isDir()) {
        current_dir = fileinfo.absoluteFilePath();
        watcher.removePaths(watcher.directories());
        if ( fileinfo.isReadable()) watcher.addPath(current_dir);
        if ( view == "icons" || view == "list" ) {
          IconViewer->showDirectory (current_dir, true);
          IconViewer->verticalScrollBar()->setValue(scroll);
        }
        else {
          TableViewer->showDirectory (current_dir, true);
          TableViewer->verticalScrollBar()->setValue(scroll);
        }
        Navigator->setPath(current_dir);
        setNavigatorIcon(fileinfo);
        if ( listdrives->contains(current_dir)) Moving->setUpEnabled (false);
          else Moving->setUpEnabled(true);
        if ( Navigator->getBackState()) Moving->setBackEnabled(true);
          else  Moving->setBackEnabled(false);
        if ( Navigator->getNextState()) Moving->setNextEnabled(true);
          else Moving->setNextEnabled(false);
        Moving->setHomeEnabled(Navigator->getHomeState());
      }
  }

  if ( button == ListMovingButton[3] ) {
    if ( OtherView ) deleteViewers();
    fileinfo.setFile(QDir::homePath());
    if ( !fileinfo.exists()) fileinfo.setFile(QDir::rootPath());
    current_dir = fileinfo.absoluteFilePath();
    watcher.removePaths(watcher.directories());
    if ( fileinfo.isReadable()) watcher.addPath(current_dir);
    if ( view == "icons" || view == "list" ) IconViewer->showDirectory (current_dir, true);
        else TableViewer->showDirectory (current_dir, true);
    Navigator->setPath(current_dir);
    Navigator->addHistory(current_dir);
    setNavigatorIcon(fileinfo);
    if ( listdrives->contains(current_dir)) Moving->setUpEnabled (false);
      else Moving->setUpEnabled(true);
    if ( Navigator->getBackState()) Moving->setBackEnabled(true);
      else  Moving->setBackEnabled(false);
    if ( Navigator->getNextState()) Moving->setNextEnabled(true);
      else Moving->setNextEnabled(false);
    Moving->setHomeEnabled(Navigator->getHomeState());
    Moving->setRefreshEnabled(true);
  }

  if ( button == ListMovingButton[4] ) {
    if ( view == "list" || view == "icons" ) IconViewer->showDirectory(current_dir, true);
    if ( view == "table" ) TableViewer->showDirectory(current_dir, true);
  }
}

void CIconViewerPanelBase :: changeView ( QString str, bool read ) {
  if ( !OtherView) {
    Navigator->setPath(current_dir);
    setNavigatorIcon(QFileInfo(current_dir));
  }
  if ( view == "list" || view == "icons" ) {
    ThreadIcon = IconViewer->getThreadIcon();
    ThreadAreaIcon = IconViewer->getThreadAreaIcon();
  }
  else {
    ThreadIcon = TableViewer->getThreadIcon();
    ThreadAreaIcon = TableViewer->getThreadAreaIcon();
  }
  ThreadIcon->execute = false;
  while ( ThreadIcon->isRunning()) {
    QApplication::processEvents();
    ThreadIcon->execute = false;
  }
  ThreadAreaIcon->execute = false;
  while ( ThreadAreaIcon->isRunning()) {
    QApplication::processEvents();
    ThreadAreaIcon->execute = false;
  }

  QFontMetrics fontMetric(*(IconViewer->font));
  if ( str == "icons" ) {
     IconViewer->setViewMode (QListView::IconMode);
     IconViewer->setFlow(QListView::LeftToRight);
     IconViewer->setIconSize(*(sizeList[0]));
     int x = fontMetric.width("aaaaaaaaaaaaaaaaa");
     int u = sizeList[0]->width();
     if ( x < u ) x = u + 5;
     IconViewer->setGridSize (QSize( x, sizeList[0]->height() + fontMetric.height() *2 + fontMetric.lineSpacing()) + QSize(15,10));
     QSize SizeHint = IconViewer->gridSize() - QSize(15, 10);
     foreach ( QListWidgetItem* item, IconViewer->itemsList)
       item->setSizeHint(SizeHint);
     if ( WebViewer == NULL && TextViewer == NULL && ImageViewer == NULL) Stack->setCurrentWidget ( IconViewer );
  }
  if ( str == "list" ) {
    IconViewer->setViewMode ( QListView::ListMode );
    IconViewer->setFlow(QListView::TopToBottom);
    IconViewer->setIconSize(*(sizeList[1]));
    IconViewer->setGridSize(QSize(0, sizeList[1]->height()+5) + QSize(15,10));
    QSize SizeHint = IconViewer->gridSize() - QSize(15,10);
    foreach ( QListWidgetItem* item, IconViewer->itemsList)
      item->setSizeHint(SizeHint);
   if ( WebViewer == NULL && TextViewer == NULL && ImageViewer == NULL) Stack->setCurrentWidget ( IconViewer );
  }
  if ( str == "table" ) {
    TableViewer->setIconSize(*(sizeList[2]));
    foreach ( QTreeWidgetItem * item, TableViewer->itemsList)
      item->setSizeHint(0, TableViewer->iconSize());
    if ( WebViewer == NULL && TextViewer == NULL && ImageViewer == NULL) Stack->setCurrentWidget ( TableViewer );
  }
  if ( view == str ){
    if (view == "icons" || view == "list") IconViewer->showDirectory(current_dir, read);
    if (view == "table") TableViewer->showDirectory(current_dir, read);
  }
  if ( (view == "icons" || view == "list") && str == "table" )  TableViewer->showDirectory(current_dir,read);
  if ( view == "table" && (str == "icons" || str == "list") ) IconViewer->showDirectory(current_dir,read);
  view = str;
}

void CIconViewerPanelBase :: setShowHidden(bool hide){
  IconViewer->ShowHidden = hide;
  TableViewer->ShowHidden = hide;
  if ( view == "list" || view == "icons" ) IconViewer->showDirectory(current_dir, true);
     else TableViewer->showDirectory(current_dir, true);
}

void CIconViewerPanelBase :: setIconSize(QSize sz) {
  QFontMetrics fontMetric(*(IconViewer->font));
  if ( view == "icons" ){
     IconViewer->setIconSize(sz);
     int x = fontMetric.width("aaaaaaaaaaaaaaaaa");
     int u = sizeList[0]->width();
     if ( x < u ) x = u + 5;
     IconViewer->setGridSize (QSize( x, sizeList[0]->height() + fontMetric.height() *2 + fontMetric.lineSpacing()) + QSize(15,10));
    *(sizeList[0]) = sz;
    QSize SizeHint = IconViewer->gridSize() - QSize(15,10);
     foreach ( QListWidgetItem* item, IconViewer->itemsList)
       item->setSizeHint(SizeHint);
  }
  if ( view == "list" ) {
    IconViewer->setIconSize(sz);
    IconViewer->setGridSize(QSize(0, sizeList[1]->height()+5) + QSize(15,10));
    QSize SizeHint = IconViewer->gridSize() - QSize(15,10);
    *(sizeList[1]) = sz;
     foreach ( QListWidgetItem* item, IconViewer->itemsList)
       item->setSizeHint(SizeHint);
  }
  if ( view == "table" ) {
    TableViewer->setIconSize(sz);
    *(sizeList[2]) = sz;
    foreach ( QTreeWidgetItem * item, TableViewer->itemsList)
      item->setSizeHint(0, TableViewer->iconSize());
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////                        SETTINGS

void CIconViewerPanelBase :: setFontSize(){
  Navigator->setFontSize();
  QFontMetrics fontMetric(*(IconViewer->font));
  if ( view == "icons" ) {
     int x = fontMetric.width("aaaaaaaaaaaaaaaaa");
     int u = sizeList[0]->width();
     if ( x < u ) x = u + 5;
    IconViewer->setGridSize (QSize( x, sizeList[0]->height() + fontMetric.height() *2 + fontMetric.lineSpacing()) + QSize(15,10));
    IconViewer->showDirectory(current_dir, false);
  }
  if ( view == "list" ) {
      IconViewer->setGridSize(QSize(0, sizeList[1]->height()+5) + QSize(15,10));
      IconViewer->showDirectory(current_dir, false);
  }
   if ( view == "table") TableViewer->showDirectory(current_dir, false);
   TableViewer->setFont(*(TableViewer->font));

  if ( OtherView ) {
    if ( WebViewer != NULL ) {
      WebViewer->setFont(*(IconViewer->font));
      WebViewerTool->setFontSize();
      WebViewerStatus->setFont(*(IconViewer->font));
    }
    if ( TextViewer != NULL ) {
      TextViewer->setFontSize();
      TextViewerTool->setFontSize();
    }
    if ( ImageViewer != NULL) ImageViewer->setFontSize();
  }
}

void CIconViewerPanelBase :: setBackColor(QColor color) {
  Color = color;
  QPalette pal = this->palette();
  changeBackgroundPalette(&pal, color);
  this->setPalette(pal);
  IconViewer->setPalette(pal);
  TableViewer->setPalette(pal);
  IconViewer->verticalScrollBar()->setPalette(pal);
  IconViewer->horizontalScrollBar()->setPalette(pal);
  TableViewer->verticalScrollBar()->setPalette(pal);
  TableViewer->horizontalScrollBar()->setPalette(pal);
  Navigator->setBackColor(darkerColor(color, 20));
  Moving->setBackColor(darkerColor(color, 60));
  if ( OtherView ) {
    if ( WebViewer != NULL ) {
      WebViewer->setCastomPalette(pal, Color);
    }
    if ( TextViewer != NULL ) {
      TextViewer->setCastomPalette(pal, Color);
    }
    if ( ImageViewer != NULL ) {
      ImageViewer->setCastomPalette(pal, Color);
    }
   }
 }

void CIconViewerPanelBase :: setFontColor(QColor color) {
  QPalette pal = IconViewer->palette();
  QPalette epal = pal;
  changeTextPalette(&pal, color);
  changeTextPalette(&epal, color);
  changeBackgroundPalette(&epal, Color);

  IconViewer->setPalette(pal);
  IconViewer->verticalScrollBar()->setPalette(epal);
  IconViewer->horizontalScrollBar()->setPalette(epal);
  TableViewer->setPalette(pal);
  TableViewer->verticalScrollBar()->setPalette(epal);
  TableViewer->horizontalScrollBar()->setPalette(epal);
  Navigator->setFontColor(color);
  Navigator->setPath(current_dir);
  if ( OtherView ) {
    if ( WebViewer != NULL ) {
      WebViewer->setCastomPalette(pal, Color);
    }
    if ( TextViewer != NULL ) {
      TextViewer->setCastomPalette(pal, Color);
    }
    if ( ImageViewer != NULL ) {
      ImageViewer->setCastomPalette(pal, Color);
    }
  }
}

void CIconViewerPanelBase :: setBackTexture(QPixmap pix) {
  QPalette pal = this->palette();
  QPalette epal = pal;
  QBrush brush(pix);
  pal.setBrush (QPalette::Base, brush);
  pal.setBrush(QPalette::Window, brush);
  changeBackgroundPalette(&epal, Color);

  IconViewer->setPalette(pal);
  IconViewer->verticalScrollBar()->setPalette(epal);
  IconViewer->horizontalScrollBar()->setPalette(epal);
  TableViewer->setPalette(pal);
  TableViewer->verticalScrollBar()->setPalette(epal);
  TableViewer->horizontalScrollBar()->setPalette(epal);
  if ( OtherView ) {
    if ( TextViewer != NULL ) {
      TextViewer->setCastomPalette(pal, Color);
    }
    if ( ImageViewer != NULL ) {
      ImageViewer->setCastomPalette(pal, Color);
    }
  }
}

void CIconViewerPanelBase :: setTheme(){
  Moving->refreshTheme();
  Navigator->refreshTheme();
  IconViewer->setPixmapLink();
  TableViewer->setPixmapLink();
  if ( view == "list" || view == "icons" ) IconViewer->showDirectory (current_dir, false);
  if ( view == "table" ) TableViewer->showDirectory(current_dir, false );
  if ( OtherView ) {
    if ( WebViewer != NULL ) WebViewerTool->refreshTheme();
    if ( TextViewer != NULL ) TextViewerTool->refreshTheme();
    if ( ImageViewer != NULL) ImageViewerTool->refreshTheme();
  }
}

void CIconViewerPanelBase :: setFlat (bool flat){
  Flat = flat;
  Moving->setFlat(flat);
  if ( OtherView ) {
    if ( WebViewer != NULL ) WebViewerTool->setFlat(flat);
    if ( TextViewer != NULL ) TextViewerTool->setFlat(flat);
    if ( ImageViewer != NULL ) ImageViewerTool->setFlat(flat);
  }
}

void CIconViewerPanelBase :: changeWatcher(QString directory) {
      if ( view == "icons" || view == "list" ) IconViewer->updateDirectory(directory);
      if ( view == "table" ) TableViewer->updateDirectory(directory);
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////        EXEC CONTEXT MENU

void CIconViewerPanelBase :: execContextMenu (QMouseEvent *ev, QString str) {
  strfile = str;
  strlist.clear();
  fileinfo.setFile(str);
  QFileInfo cur(current_dir);
  QMenu * Menu = new QMenu (this);
  Menu->setFont(*(IconViewer->font));
  QAction *act1, *act2, *act3, *act4;
  if ( fileinfo.isFile() || (fileinfo.isSymLink() && !fileinfo.isDir())) {
    act1 = Menu->addAction(*(pxCache->getIcon(91)), tr("Select/Invert all"));
    act1->setToolTip("sel");
    act1->setShortcut(Qt::CTRL + Qt::Key_A);
    Menu->addSeparator();
    act1 = Menu->addAction(*(pxCache->getIcon(82)), tr("Cut"));
    act1->setToolTip("cut");
    act1->setShortcut(Qt::CTRL + Qt::Key_X);
    if ( (!fileinfo.isWritable() && !fileinfo.isReadable() && !fileinfo.isSymLink()) || !cur.isWritable()) act1->setEnabled(false);
    act2 = Menu->addAction (*(pxCache->getIcon(83)), tr("Copy"));
    act2->setToolTip("copy");
    act2->setShortcut(Qt::CTRL + Qt::Key_C);
    if ( !fileinfo.isReadable()) act2->setEnabled(false);
    Menu->addSeparator();
    act1 = Menu->addAction (*(pxCache->getIcon(87)), tr("Open"));
    act1->setToolTip("open");
    QMenu *submenu = new QMenu(Menu);
    QList<TemplateApps> Tapps;
    Tapps = pxCache->fullTemplateApps(fileinfo);
    if ( Tapps.count() != 0 ) {
      foreach ( TemplateApps tapps, Tapps ) {
        act1 = submenu->addAction(*(tapps.iconApp), tapps.nameApp );
        act1->setToolTip("run");
      }
    }
    submenu->addSeparator();
    act1 = submenu->addAction(tr("Select program") + " ...");
    act1->setToolTip("cast");
    QAction *subaction = Menu->addMenu(submenu);
    subaction->setIcon(*(pxCache->getIcon(88)));
    subaction->setText(tr("Open with"));
    Menu->addSeparator();
    act3 = Menu->addAction (*(pxCache->getIcon(85)), tr("Rename") + " ...");
    act3->setToolTip("ren");
    act3->setShortcut(Qt::Key_F2);
    act4 = Menu->addAction (*(pxCache->getIcon(86)), tr("Delete"));
    act4->setToolTip("del");
    act4->setShortcut(Qt::CTRL + Qt::Key_Delete);
    if ( (!fileinfo.isWritable() && !fileinfo.isSymLink()) || !cur.isWritable()) {
      act3->setEnabled(false);
      act4->setEnabled(false);
    }
    Menu->addSeparator();
    act1 = Menu->addAction(*(pxCache->getIcon(89)), tr("Properites") + " ...");
    act1->setToolTip("prop");
    act1->setShortcut(Qt::CTRL + Qt::Key_Enter);
    QPalette pal = IconViewer->palette();
    pal.setColor(QPalette::Window, Color.darker(120));
    Menu->setPalette(pal);
    submenu->setFont(*(IconViewer->font));
    submenu->setPalette(pal);
    QAction *act = Menu->exec(ev->globalPos());
    if ( act ) {
      QString rc = act->toolTip();
      if ( rc == "run" ) runc = act->text();
      delete submenu;
      delete Menu;
      if ( rc == "sel" ) selectAll();
      if ( rc == "cut" ) cutItemClipboard();
      if ( rc == "copy" ) copyItemClipboard();
      if ( rc == "open" ) openElement();
      if ( rc == "run" )  runProgram();
      if ( rc == "cast" ) selectProgram();
      if ( rc == "ren" ) renameItem();
      if ( rc == "del" )  deleteItem();
      if ( rc == "prop" ) properites() ;
      return;
    }
    else {
      delete submenu;
      delete Menu;
      return;
    }
  }
  if ( fileinfo.isDir()) {
    act1 = Menu->addAction(*(pxCache->getIcon(91)), tr("Select/Invert all"));
    act1->setShortcut(Qt::CTRL + Qt::Key_A);
    act1->setToolTip("sel");
    Menu->addSeparator();
    act1 = Menu->addAction( *(pxCache->getIcon(82)), tr("Cut"));
    act1->setToolTip("cut");
    act1->setShortcut(Qt::CTRL + Qt::Key_X);
    act2 = Menu->addAction (*(pxCache->getIcon(83)), tr("Copy"));
    act2->setToolTip("copy");
    act2->setShortcut(Qt::CTRL + Qt::Key_C);
    Menu->addSeparator();
    act3 = Menu->addAction (*(pxCache->getIcon(84)), tr("Paste into this directory"));
    act3->setToolTip("paste");
    if ( !fileinfo.isReadable() || !cur.isWritable())  act1->setEnabled(false);
    if ( !fileinfo.isReadable()) act2->setEnabled(false);
    if ( !cur.isWritable()) act3->setEnabled(false);
    if ( !clipboard->mimeData()->hasFormat("text/uri-list")) act3->setEnabled(false);
    if ( !fileinfo.isWritable()) act1->setEnabled(false);
    Menu->addSeparator();
    act1 = Menu->addAction(*(pxCache->getIcon(80)), tr("Create new directory into this") + " ...");
    act1->setToolTip("created");
    act2 = Menu->addAction(*(pxCache->getIcon(90)), tr("Create new file into this") + " ...");
    act2->setToolTip("createf");
    act3 = Menu->addAction(*(pxCache->getIcon(81)), tr("Create symlink into this") + " ...");
    act3->setToolTip("creates");
    if ( !fileinfo.isWritable()) {
      act1->setEnabled(false);
      act2->setEnabled(false);
      act3->setEnabled(false);
    }
    Menu->addSeparator();
    act3 = Menu->addAction (*(pxCache->getIcon(85)), tr("Rename") + " ...");
    act3->setToolTip("ren");
    act3->setShortcut(Qt::Key_F2);
    act4 = Menu->addAction (*(pxCache->getIcon(86)), tr("Delete"));
    act4->setToolTip("del");
    act4->setShortcut(Qt::CTRL + Qt::Key_Delete);
    if ( !fileinfo.isWritable() || !cur.isWritable()) {
      act3->setEnabled(false);
      act4->setEnabled(false);
    }
    Menu->addSeparator();
    act1 = Menu->addAction(*(pxCache->getIcon(89)), tr("Properites") + " ...");
    act1->setToolTip("prop");
    act1->setShortcut(Qt::CTRL + Qt::Key_Enter);
    QPalette pal = IconViewer->palette();
    pal.setColor(QPalette::Window, Color.darker(120));
    Menu->setPalette(pal);
    QAction *act = Menu->exec(ev->globalPos());
    if ( act ) {
      QString rc = act->toolTip();
      delete Menu;
      if ( rc == "sel" ) selectAll();
      if ( rc == "cut" ) cutItemClipboard();
      if ( rc == "copy" ) copyItemClipboard();
      if ( rc == "paste" ) pasteItemClipboard();
      if ( rc == "created" ) createItem();
      if ( rc == "createf" ) createItemFile();
      if ( rc == "creates" ) createItemSymlink();
      if ( rc == "ren" ) renameItem();
      if ( rc == "del" ) deleteItem();
      if ( rc == "prop" ) properites() ;
      return;
    }
    else {
    delete Menu;
    return;
    }
  }
  act1 = Menu->addAction(*(pxCache->getIcon(91)), tr("Select/Invert all"));
  act1->setToolTip("sel");
  act1->setShortcut(Qt::CTRL + Qt::Key_A);
  Menu->addSeparator();
  act1 = Menu->addAction(*(pxCache->getIcon(82)), tr("Cut"));
  act1->setToolTip("cut");
  act1->setShortcut(Qt::CTRL + Qt::Key_X);
  if ( (!fileinfo.isWritable() && !fileinfo.isReadable() && !fileinfo.isSymLink()) || !cur.isWritable()) act1->setEnabled(false);
  act2 = Menu->addAction (*(pxCache->getIcon(83)), tr("Copy"));
  act2->setToolTip("copy");
  act2->setShortcut(Qt::CTRL + Qt::Key_C);
  if ( !fileinfo.isReadable()) act2->setEnabled(false);
  Menu->addSeparator();
  act1 = Menu->addAction (*(pxCache->getIcon(87)), tr("Open"));
  act1->setToolTip("open");
  Menu->addSeparator();
  act3 = Menu->addAction (*(pxCache->getIcon(85)), tr("Rename") + " ...");
  act3->setToolTip("ren");
  act3->setShortcut(Qt::Key_F2);
  act4 = Menu->addAction (*(pxCache->getIcon(86)), tr("Delete"));
  act4->setToolTip("del");
  act4->setShortcut(Qt::CTRL + Qt::Key_Delete);
  if ( (!fileinfo.isWritable() && !fileinfo.isSymLink()) || !cur.isWritable()) {
    act3->setEnabled(false);
    act4->setEnabled(false);
  }
  Menu->addSeparator();
  act1 = Menu->addAction(*(pxCache->getIcon(89)), tr("Properites") + " ...");
  act1->setToolTip("prop");
  act1->setShortcut(Qt::CTRL + Qt::Key_Enter);
  QPalette pal = IconViewer->palette();
  pal.setColor(QPalette::Window, Color.darker(120));
  Menu->setPalette(pal);
  QAction *act = Menu->exec(ev->globalPos());
  if ( act ) {
    QString rc = act->toolTip();
    if ( rc == "run" ) runc = act->text();
    delete Menu;
    if ( rc == "sel" ) selectAll();
    if ( rc == "cut" ) cutItemClipboard();
    if ( rc == "copy" ) copyItemClipboard();
    if ( rc == "open" ) openElement();
    if ( rc == "run" )  runProgram();
    if ( rc == "cast" ) selectProgram();
    if ( rc == "ren" ) renameItem();
    if ( rc == "del" )  deleteItem();
    if ( rc == "prop" ) properites() ;
    return;
  }
  else {
    delete Menu;
    return;
  }
}

void CIconViewerPanelBase :: execContextMenu(QMouseEvent *ev ) {
    strfile = "";
    strlist.clear();
    QMenu * Menu = new QMenu (this);
    Menu->setFont(*(IconViewer->font));
    dir.setPath(current_dir);
    fileinfo.setFile(current_dir);
    QAction *act1, *act2, *act3, *act4;
    act1 = Menu->addAction(*(pxCache->getIcon(91)), tr("Select/Invert all"));
    act1->setToolTip("sel");
    act1->setShortcut(Qt::CTRL + Qt::Key_A);
    Menu->addSeparator();
    act1 = Menu->addAction (*(pxCache->getIcon(84)), tr("Paste"));
    act1->setToolTip("paste");
    act1->setShortcut(Qt::CTRL + Qt::Key_V);
    if ( !clipboard->mimeData()->hasFormat("text/uri-list")) act1->setEnabled(false);
    Menu->addSeparator();
    act2 = Menu->addAction (*(pxCache->getIcon(80)), tr("Create new directory") + " ...");
    act2->setToolTip("created");
    act2->setShortcut(Qt::CTRL + Qt::Key_F);
    act3 = Menu->addAction (*(pxCache->getIcon(90)), tr("Create clean file") + " ...");
    act3->setToolTip("createf");
    act4 = Menu->addAction (*(pxCache->getIcon(81)), tr("Create symlink") + " ...");
    act4->setToolTip("creates");
    act4->setShortcut(Qt::CTRL + Qt::Key_S);
    if ( !fileinfo.isWritable()) {
      act1->setEnabled(false);
      act2->setEnabled(false);
      act3->setEnabled(false);
      act4->setEnabled(false);
    }
    Menu->addSeparator();
    act1 = Menu->addAction(*(pxCache->getIcon(89)), tr("Properites") + " ...");
    act1->setToolTip("prop");
    act1->setShortcut(Qt::CTRL + Qt::Key_Enter);
    QPalette pal = IconViewer->palette();
    pal.setColor(QPalette::Window, Color.darker(120));
    Menu->setPalette(pal);
    QAction *act = Menu->exec(ev->globalPos());
    if ( act ) {
      QString rc = act->toolTip();
      delete Menu;
      if ( rc == "sel" ) selectAll();
      if ( rc == "paste" ) pasteItemClipboard();
      if ( rc == "created" ) createItem();
      if ( rc == "createf" ) createItemFile();
      if ( rc == "creates" ) createItemSymlink();
      if ( rc == "prop" ) properites() ;
      return;
    }
    else {
      delete Menu;
      return;
    }
}

void CIconViewerPanelBase :: execContextMenu (QMouseEvent *ev, QList<QString> list){
    strlist = list;
    strfile ="";
    QMenu * Menu = new QMenu (this);
    Menu->setFont(*(IconViewer->font));
    fileinfo.setFile(current_dir);
    QAction *act1, *act2, *act3;
    act1 = Menu->addAction(*(pxCache->getIcon(91)), tr("Select/Invert all"));
    act1->setToolTip("sel");
    act1->setShortcut(Qt::CTRL + Qt::Key_A);
    Menu->addSeparator();
    act1 = Menu->addAction( *(pxCache->getIcon(82)), tr("Cut"));
    act1->setToolTip("cut");
    act1->setShortcut(Qt::CTRL + Qt::Key_X);
    act2 = Menu->addAction (*(pxCache->getIcon(83)), tr("Copy"));
    act2->setToolTip("copy");
    act2->setShortcut(Qt::CTRL + Qt::Key_C);
    Menu->addSeparator();
    act3 = Menu->addAction (*(pxCache->getIcon(86)), tr("Delete"));
    act3->setToolTip("del");
    act3->setShortcut(Qt::CTRL + Qt::Key_Delete);
    if ( !fileinfo.isReadable()) {
      act1->setEnabled(false);
      act2->setEnabled(false);
    }
    if ( !fileinfo.isWritable()) {
      act1->setEnabled(false);
      act3->setEnabled(false);
    }
    Menu->addSeparator();
    act1 = Menu->addAction(*(pxCache->getIcon(89)), tr("Properites") + " ...");
    act1->setToolTip("prop");
    act1->setShortcut(Qt::CTRL + Qt::Key_Enter);
    QPalette pal = IconViewer->palette();
    pal.setColor(QPalette::Window, Color.darker(120));
    Menu->setPalette(pal);
    QAction *act = Menu->exec(ev->globalPos());
    if ( act ) {
      QString rc = act->toolTip();
      delete Menu;
      if ( rc == "sel" ) selectAll();
      if ( rc == "cut" ) cutItemClipboard();
      if ( rc == "copy" ) copyItemClipboard();
      if ( rc == "del" ) deleteItem();
      if ( rc == "prop" ) properites() ;
      return;
    }
    else delete Menu;
}

void CIconViewerPanelBase :: execDropMenu(QList<QString> droplist, QPoint point, QString str, bool exm, int op) {
  DropList = droplist;
  if ( !str.isEmpty()) strdrop = str;
    else strdrop = current_dir;
  bool mvp = false;
  if ( droplist.count() > 0) {
    fileinfo.setFile(droplist[0]);
    if ( fileinfo.absolutePath() == strdrop) mvp = true;
  }
  if ( exm ) {
    QMenu *DropMenu = new QMenu(this);
    QAction *act1, *act2, *act3;
    act1 = DropMenu->addAction(*(pxCache->getIcon(83)), tr("Copy here"));
    act1->setShortcut(Qt::CTRL);
    act1->setToolTip("copy");
    if ( !mvp ) {
      act2 = DropMenu->addAction(*(pxCache->getIcon(84)), tr("Move here"));
      act2->setShortcut(QKeySequence(Qt::ShiftModifier));
      act2->setToolTip("move");
    }
    act3 = DropMenu->addAction(*(pxCache->getIcon(81)), tr("Create symlink"));
    act3->setShortcut(QKeySequence(Qt::ControlModifier + Qt::ShiftModifier));
    act3->setToolTip("sym");

    DropMenu->addSeparator();
    act3 = DropMenu->addAction(*(pxCache->getIcon(191)), tr("Cancel"));
    act3->setToolTip("can");
    QPalette pal = IconViewer->palette();
    pal.setColor(QPalette::Window, Color.darker(120));
    DropMenu->setPalette(pal);
    DropMenu->setFont(*(IconViewer->font));
    QAction *act = DropMenu->exec(point);
    if ( act ) {
      QString strop = act->toolTip();
      delete DropMenu;
      if ( strop == "can") return;
      emit execDrag(strop);
      return;
    }
    else {
      delete DropMenu;
      return;
    }
  }
  else {
    if ( op == 2 && mvp ) op = 1;
    emit execDrag(op);
    return;
  }
}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////         RUN COMMAND MENU

void CIconViewerPanelBase :: renameItem ( QString oldName, QString newName) {
    fileinfo.setFile(oldName);
    QString strr = newName;
    dir.setPath(current_dir);
    QList<QString> dirlist = dir.entryList( QDir::AllEntries | QDir::Hidden | QDir::NoDotAndDotDot | QDir::System);
    QFont fn = *(IconViewer->font);
    fn.setBold(true);
    while ( true ) {
      if ( strr == "." || strr == ".." || strr.contains("/") || strr.contains("\\") || strr == "" || dirlist.contains(strr)) {
        CMessageBox *dial = new CMessageBox(pxCache, this);
        dial->createInputStringWithIcon(pxCache->getIcon(181)->scaled(96, 96));
        dial->setFont(fn);
        dial->setModal(true);
        dial->setCastomPalette(IconViewer->palette(), Color);
        dial->okButton->setText(tr("Rename"));
        dial->setWindowTitle ( tr("Please try again") + "  - Bathyscaphe");
        dial->lineedit1->setText(strr);
        dial->box1->hide();
        dial->resize(400,170);
        if ( strr == "." || strr == ".." ) dial->inform->setText("<h3>" + tr("Warning:") + " </h3>  <br> " + tr("The name of the file or directory can not be '.' or '..' .") +  "<br>" + tr(" Please, give a different name:" ));
        else
          if ( strr.contains("/") || strr.contains("\\")) dial->inform->setText("<h3>" + tr("Warning:") + " </h3>  <br> " + tr("The name of the file or directory can not contain the characters - '/', '\\' .") + " <br> " + tr("Please, give a different name:"));
          else
             if ( strr == "") dial->inform->setText ("<h3>" + tr("Warning:") + "  </h3>  <br>" + tr("Name file or directory can not be empty.") + " <br>" + tr("Please, give a different name:"));
             else
                if ( dirlist.contains(strr)) {
                  dial->inform->setText("<h3>" +tr("Warning:") + " </h3>  <br> " + tr("This directory already contains an element with the same name.") + " <br>" + tr("Please, give a different name:"));
                  QFileInfo finf(dir.absolutePath() + QDir::separator() + strr);
                  Template templ = pxCache->fullTemplate(finf);
                  dial->labelname1->setText("<h3>" +  templ.FullName + "</h3");
                  dial->labelicon1->setPixmap(templ.Icon.scaled(96,96));
                  if (finf.isDir() && !finf.isSymLink())
                    dial->labeldetals1->setText("<table><tr><td>" + tr("Type") +":</td><td>" +  templ.Type + "</td></tr><tr><td>"
                                                                        + tr("Size") + ":</td><td>" + templ.Size   + "</td></tr><tr><td>"
                                                                        + tr("Created") + ":</td><td>" + templ.Created + "</td></tr><tr><td>"
                                                                        + tr("Modified") + ":</td><td>" + templ.LastModified + "</td></tr></table>");
                  else
                    dial->labeldetals1->setText("<table><tr><td>" + tr("Type") +":</td><td>" +  templ.Type + "</td></tr><tr><td>"
                                                                        + tr("Description (from extended)") +":</td><td>" + templ.Descr + "</td></tr><tr><td>"
                                                                        + tr("Contents") + ":</td><td>" + templ.DescrFromContents + "</td></tr><tr><td>"
                                                                        + tr("Size") + ":</td><td>" + templ.Size   + "</td></tr><tr><td>"
                                                                        + tr("Created") + ":</td><td>" + templ.Created + "</td></tr><tr><td>"
                                                                        + tr("Modified") + ":</td><td>" + templ.LastModified + "</td></tr></table>");
                  dial->box1->show();
                }
         dial->lineedit1->setText(strr);
         dial->lineedit1->setFocus();
         int n = dial->execInputString(&strr);
         delete dial;
         if ( n == 2 )  return;
      }
      else break;
    }
    bool result = false;
    if ( fileinfo.exists()) {
      if ( fileinfo.isSymLink() || fileinfo.isFile()) {
        result = QFile::rename(oldName, current_dir + QDir::separator() + strr);
      }
      if ( fileinfo.isDir()) {
        result = dir.rename(oldName, current_dir + QDir::separator() + strr);
      }
   }
   strfile = "";
   strlist.clear();
   if ( !result ) {
      CMessageBox *inform = new CMessageBox(pxCache);
      inform->createInform(pxCache->getIcon(182)->scaled(96, 96));
      inform->setWindowTitle(tr("Filed to rename an item.") + " - Bathyscaphe");
      inform->resize(400, 180);
      inform->inform->setText("<h3>" + tr("Error") + ": </h3>  <br>" + tr("Filed to rename an item.") + " <br>");
      inform->detals->setText("1. " + tr("Permission denied.") + ". <br> 2. " + tr("This element is used by another application."));
      inform->setFont(fn);
      inform->setCastomPalette(IconViewer->palette(), Color);
      inform->exec();
      delete inform;
   }
}

void CIconViewerPanelBase :: renameItem() {
  QString ppath;
  if (strfile == "") {
    dir.setPath(current_dir);
    ppath = current_dir;
  }
  else {
    QFileInfo tmpf(strfile);
    ppath = tmpf.absolutePath();
    dir.setPath(tmpf.absolutePath());
  }
  QList<QString> dirlist = dir.entryList( QDir::AllEntries | QDir::Hidden | QDir::NoDotAndDotDot | QDir::System);
  CMessageBox *dial = new CMessageBox(pxCache, this);
  QFont fn = *(IconViewer->font);
  fn.setBold(true);
  dial->createInputStringWithIcon(pxCache->getIcon(187)->scaled(96,96));
  dial->setFont(fn);
  dial->setModal(true);
  dial->inform->setText(tr("Please, give a new name") + ":" );
  dial->inform->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
  dial->setCastomPalette(IconViewer->palette(), Color);
  dial->okButton->setText(tr("Rename"));
  dial->setWindowTitle ( tr("Rename element") + " - Bathyscaphe");
  dial->resize(400, 170);
  fileinfo.setFile(strfile);
  dial->lineedit1->setText(fileinfo.fileName());
  dial->lineedit1->setSelection(0, fileinfo.baseName().length());
  QString strr;
  int n = dial->execInputString(&strr);
  delete dial;
  if ( n == 2 ) {
    strfile = "";
    strlist.clear();
    return;
  }
  if ( n == 1 ) {
    while ( true ) {
      if ( strr == "." || strr == ".." || strr.contains("/") || strr.contains("\\") || strr == "" || dirlist.contains(strr)) {
        CMessageBox *dial = new CMessageBox(pxCache, this);
        dial->createInputStringWithIcon(pxCache->getIcon(181)->scaled(96, 96));
        dial->setFont(fn);
        dial->setModal(true);
        dial->setCastomPalette(IconViewer->palette(), Color);
        dial->okButton->setText(tr("Rename"));
        dial->setWindowTitle ( tr("Please try again") + " - Bathyscaphe");
        dial->lineedit1->setText(strr);
        if ( strr == "." || strr == ".." ) dial->inform->setText("<h3>" + tr("Warning:") + " </h3>  <br> " + tr("The name of the file or directory can not be '.' or '..' .") +  "<br>" + tr(" Please, give a different name:" ));
        else
          if ( strr.contains("/") || strr.contains("\\")) dial->inform->setText("<h3>" + tr("Warning:") + " </h3>  <br> " + tr("The name of the file or directory can not contain the characters - '/', '\\' .") + " <br> " + tr("Please, give a different name:"));
          else
             if ( strr == "") dial->inform->setText ("<h3>" + tr("Warning:") + "  </h3>  <br>" + tr("Name file or directory can not be empty.") + " <br>" + tr("Please, give a different name:"));
             else
                if ( dirlist.contains(strr)) {
                  dial->inform->setText("<h3>" +tr("Warning:") + " </h3>  <br> " + tr("This directory already contains an element with the same name.") + " <br>" + tr("Please, give a different name:"));
                  QFileInfo finf(dir.absolutePath() + QDir::separator() + strr);
                  Template templ = pxCache->fullTemplate(finf);
                  dial->labelname1->setText("<h3>" +  templ.FullName + "</h3");
                  dial->labelicon1->setPixmap(templ.Icon.scaled(96,96));
                  if (finf.isDir() && !finf.isSymLink())
                    dial->labeldetals1->setText("<table><tr><td>" + tr("Type") +":</td><td>" +  templ.Type + "</td></tr><tr><td>"
                                                                        + tr("Size") + ":</td><td>" + templ.Size   + "</td></tr><tr><td>"
                                                                        + tr("Created") + ":</td><td>" + templ.Created + "</td></tr><tr><td>"
                                                                        + tr("Modified") + ":</td><td>" + templ.LastModified + "</td></tr></table>");
                  else
                    dial->labeldetals1->setText("<table><tr><td>" + tr("Type") +":</td><td>" +  templ.Type + "</td></tr><tr><td>"
                                                                        + tr("Description (from extended)") +":</td><td>" + templ.Descr + "</td></tr><tr><td>"
                                                                        + tr("Contents") + ":</td><td>" + templ.DescrFromContents + "</td></tr><tr><td>"
                                                                        + tr("Size") + ":</td><td>" + templ.Size   + "</td></tr><tr><td>"
                                                                        + tr("Created") + ":</td><td>" + templ.Created + "</td></tr><tr><td>"
                                                                        + tr("Modified") + ":</td><td>" + templ.LastModified + "</td></tr></table>");
                  dial->box1->show();
                }
         dial->lineedit1->setText(strr);
         dial->resize(400, 170);
         dial->lineedit1->setFocus();
         n = dial->execInputString(&strr);
         delete dial;
         if ( n == 2 )  return;
      }
      else break;
    }
    bool result = false;
    if ( fileinfo.exists()) {
      if ( fileinfo.isSymLink() || fileinfo.isFile()) {
        result = QFile::rename(strfile, ppath + QDir::separator() + strr);
      }
      if ( fileinfo.isDir()) {
        result = dir.rename(strfile, ppath + QDir::separator() + strr);
      }
   }
   strfile = "";
   strlist.clear();
   if ( !result ) {
      CMessageBox *inform = new CMessageBox(pxCache);
      inform->createInform(pxCache->getIcon(182)->scaled(96, 96));
      inform->resize(400, 180);
      inform->setWindowTitle(tr("Filed to rename an item.") + " - Bathyscaphe");
      inform->inform->setText("<h3>" + tr("Error") + ": </h3>  <br>" + tr("Filed to rename an item.") + " <br>");
      inform->detals->setText("1. " + tr("Permission denied.") + ". <br> 2. " + tr("This element is used by another application."));
      inform->setFont(fn);
      inform->setCastomPalette(IconViewer->palette(), Color);
      inform->exec();
      delete inform;
   }
  }
}

void CIconViewerPanelBase :: deleteItem() {
  if (* showWarningRemove ) {
    if ( strfile != "" || strlist.count()>0) {
      CMessageBox *warning = new CMessageBox(pxCache);
      warning->createWarningRemove(pxCache->getIcon(181)->scaled(96, 96));
      warning->setWindowTitle(tr("Deleting files and directories") + " - Bathyscaphe");
      warning->resize(400, 350);
      if ( strfile != "") {
        warning->textedit->setPlainText(strfile);
        warning->inform->setText("<h3>" + tr("Are you sure you want to delete this object?") + "</h3>");
      }
      else if ( strlist.count()>0) {
        warning->inform->setText("<h3>" + tr("Are you sure you want to remove the object's?") + " <br>" + QString("(") + QString::number(strlist.count()) + " " + tr("object(-s)") + " </h3>");
        foreach ( QString str, strlist) warning->textedit->append(str);
      }
      warning->setFont(*(IconViewer->font));
      warning->setCastomPalette(IconViewer->palette(), Color);
      connect ( warning->checkbox, SIGNAL(clicked()), SLOT(changeShowWarningRemove()));
      int n = warning->exec();
      delete warning;
      if ( n == 2 ) return;
    }
  }
  if ( strfile != "" ) {
    QList<QString> tmpl;
    tmpl << strfile;
    emit execOperate ( tmpl, current_dir, 3 );
  }
  else if ( strlist.count() > 0) emit execOperate(strlist, current_dir, 3);
}

void CIconViewerPanelBase :: createItem() {
  if ( strfile == "") dir.setPath(current_dir);
    else dir.setPath(strfile);
  QList<QString> dirlist = dir.entryList( QDir::AllEntries | QDir::Hidden | QDir::NoDotAndDotDot | QDir::System);
  QString newdir = tr("New Directory");
  QString string = newdir;
  for ( double i = 1; i < 1000000000; ++i ) {
    if ( dirlist.contains(string)) string = newdir + "_" + QString::number(i);
    else break;
  }
  QFont fn = *(IconViewer->font);
  fn.setBold(true);
  CMessageBox *dial = new CMessageBox(pxCache, this);
  dial->createInputStringWithIcon(pxCache->getIcon(184)->scaled(96,96));
  dial->setFont(fn);
  dial->setModal(true);
  dial->inform->setText(tr("Please, enter  name of the directory:"));
  dial->inform->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
  dial->lineedit1->setText(string);
  dial->setCastomPalette(IconViewer->palette(), Color);
  dial->okButton->setText(tr("Create"));
  dial->setWindowTitle ( tr("Create new directory") + " - Bathyscaphe");
  dial->resize(400, 170);
  QString strr;
  int n = dial->execInputString(&strr);
  delete dial;
  if ( n == 2 ) return;
  if ( n == 1 ) {
    while ( true ) {
      if ( strr == "." || strr == ".." || strr.contains("/") || strr.contains("\\") || strr == "" || dirlist.contains(strr)) {
        CMessageBox *dial = new CMessageBox(pxCache, this);
        dial->createInputStringWithIcon(pxCache->getIcon(181)->scaled(96, 96));
        dial->setFont(fn);
        dial->setModal(true);
        dial->lineedit1->setText(strr);
        dial->setCastomPalette(IconViewer->palette(), Color);
        dial->okButton->setText(tr("Create"));
        dial->setWindowTitle ( tr("Please try again") + " - Bathyscaphe");
        dial->resize(400, 170);
        if ( strr == "." || strr == ".." ) dial->inform->setText("<h3>" + tr("Warning:") + " </h3>  <br> " + tr("The name of the directory can not be '.' or '..' .") +  "<br>" + tr(" Please, give a different name:" ));
        else
          if ( strr.contains("/") || strr.contains("\\")) dial->inform->setText("<h3>" + tr("Warning:") + " </h3>  <br> " + tr("The name of the directory can not contain the characters - '/', '\\' .") + " <br> " + tr("Please, give a different name:"));
          else
             if ( strr == "") dial->inform->setText ("<h3>" + tr("Warning:") + "  </h3>  <br>" + tr("Name directory can not be empty.") + " <br>" + tr("Please, give a different name:"));
             else
                if ( dirlist.contains(strr)) {
                  dial->inform->setText("<h3>" +tr("Warning:") + " </h3>  <br> " + tr("This directory already contains an element with the same name.") + " <br>" + tr("Please, give a different name:"));
                  QFileInfo finf(dir.absolutePath() + QDir::separator() + strr);
                  Template templ = pxCache->fullTemplate(finf);
                  dial->labelname1->setText("<h3>" +  templ.FullName + "</h3");
                  dial->labelicon1->setPixmap(templ.Icon.scaled(96,96));
                  if (finf.isDir() && !finf.isSymLink())
                    dial->labeldetals1->setText("<table><tr><td>" + tr("Type") +":</td><td>" +  templ.Type + "</td></tr><tr><td>"
                                                                        + tr("Size") + ":</td><td>" + templ.Size   + "</td></tr><tr><td>"
                                                                        + tr("Created") + ":</td><td>" + templ.Created + "</td></tr><tr><td>"
                                                                        + tr("Modified") + ":</td><td>" + templ.LastModified + "</td></tr></table>");
                  else
                    dial->labeldetals1->setText("<table><tr><td>" + tr("Type") +":</td><td>" +  templ.Type + "</td></tr><tr><td>"
                                                                        + tr("Description (from extended)") +":</td><td>" + templ.Descr + "</td></tr><tr><td>"
                                                                        + tr("Contents") + ":</td><td>" + templ.DescrFromContents + "</td></tr><tr><td>"
                                                                        + tr("Size") + ":</td><td>" + templ.Size   + "</td></tr><tr><td>"
                                                                        + tr("Created") + ":</td><td>" + templ.Created + "</td></tr><tr><td>"
                                                                        + tr("Modified") + ":</td><td>" + templ.LastModified + "</td></tr></table>");
                  dial->box1->show();
                }
         dial->lineedit1->setText(strr);
         dial->lineedit1->setFocus();
         n = dial->execInputString(&strr);
         delete dial;
         if ( n == 2 )  return;
      }
      else break;
    }
    if ( n == 1 ) {
      if ( dir.mkdir(strr)) return;
      else {
        CMessageBox *inform = new CMessageBox(pxCache);
        inform->createInform(pxCache->getIcon(182)->scaled(96, 96));
        inform->resize(400,180);
        inform->setWindowTitle(tr("Error creating directory") + " - Bathyscaphe");
        inform->inform->setText("<h3>" + tr("Error:") + " </h3> <br>" + tr("Failed to create a directory named") + " <br>" + strr);
        inform->detals->setText("1. " + tr("Check the permissions") + " <br> 2. " + tr("Check free disk space."));
        inform->setFont(fn);
        inform->setCastomPalette(IconViewer->palette(), Color);
        inform->exec();
        delete inform;
      }
    }
  }
}

void CIconViewerPanelBase :: createItemFile() {
  if ( strfile == "") dir.setPath(current_dir);
    else dir.setPath(strfile);
  QList<QString> dirlist = dir.entryList( QDir::AllEntries | QDir::Hidden | QDir::NoDotAndDotDot | QDir::System);
  QString newdir = tr("File");
  QString string = newdir;
  for ( double i = 1; i < 1000000000; ++i ) {
    if ( dirlist.contains(string)) string = newdir + "_" + QString::number(i);
    else break;
  }
  QFile nfile;
  QFont fn = *(IconViewer->font);
  fn.setBold(true);
  CMessageBox *dial = new CMessageBox(pxCache, this);
  dial->createInputStringWithIcon(pxCache->getIcon(185)->scaled(96,96));
  dial->setFont(fn);
  dial->setModal(true);
  dial->inform->setText(tr("Please, enter  name of the file:"));
  dial->inform->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
  dial->lineedit1->setText(string);
  dial->setCastomPalette(IconViewer->palette(), Color);
  dial->okButton->setText(tr("Create"));
  dial->setWindowTitle ( tr("Create new file") + " - Bathyscaphe");
  dial->resize(400, 170);
  QString strr;
  int n = dial->execInputString(&strr);
  delete dial;
  if ( n == 2 ) return;
  if ( n == 1 ) {
    while ( true ) {
      if ( strr == "." || strr == ".." || strr.contains("/") || strr.contains("\\") || strr == "" || dirlist.contains(strr)) {
        CMessageBox *dial = new CMessageBox(pxCache, this);
        dial->createInputStringWithIcon(pxCache->getIcon(181)->scaled(96, 96));
        dial->setFont(fn);
        dial->setModal(true);
        dial->lineedit1->setText(strr);
        dial->setCastomPalette(IconViewer->palette(), Color);
        dial->okButton->setText(tr("Create"));
        dial->setWindowTitle ( "Please, return enter - Bathyscaphe");
        dial->resize(400, 170);
        if ( strr == "." || strr == ".." ) dial->inform->setText("<h3>" + tr("Warning:") + " </h3>  <br> " + tr("The name of the file can not be '.' or '..' .") +  "<br>" + tr(" Please, give a different name:" ));
        else
          if ( strr.contains("/") || strr.contains("\\")) dial->inform->setText("<h3>" + tr("Warning:") + " </h3>  <br> " + tr("The name of  file can not contain the characters - '/', '\\' .") + " <br> " + tr("Please, give a different name:"));
          else
             if ( strr == "") dial->inform->setText ("<h3>" + tr("Warning:") + "  </h3>  <br>" + tr("Name file can not be empty.") + " <br>" + tr("Please, give a different name:"));
             else
                if ( dirlist.contains(strr)) {
                  dial->inform->setText("<h3>" +tr("Warning:") + " </h3>  <br> " + tr("This directory already contains an element with the same name.") + " <br>" + tr("Please, give a different name:"));
                  QFileInfo finf(dir.absolutePath() + QDir::separator() + strr);
                  Template templ = pxCache->fullTemplate(finf);
                  dial->labelname1->setText("<h3>" +  templ.FullName + "</h3");
                  dial->labelicon1->setPixmap(templ.Icon.scaled(96,96));
                  if (finf.isDir() && !finf.isSymLink())
                    dial->labeldetals1->setText("<table><tr><td>" + tr("Type") +":</td><td>" +  templ.Type + "</td></tr><tr><td>"
                                                                        + tr("Size") + ":</td><td>" + templ.Size   + "</td></tr><tr><td>"
                                                                        + tr("Created") + ":</td><td>" + templ.Created + "</td></tr><tr><td>"
                                                                        + tr("Modified") + ":</td><td>" + templ.LastModified + "</td></tr></table>");
                  else
                    dial->labeldetals1->setText("<table><tr><td>" + tr("Type") +":</td><td>" +  templ.Type + "</td></tr><tr><td>"
                                                                        + tr("Description (from extended)") +":</td><td>" + templ.Descr + "</td></tr><tr><td>"
                                                                        + tr("Contents") + ":</td><td>" + templ.DescrFromContents + "</td></tr><tr><td>"
                                                                        + tr("Size") + ":</td><td>" + templ.Size   + "</td></tr><tr><td>"
                                                                        + tr("Created") + ":</td><td>" + templ.Created + "</td></tr><tr><td>"
                                                                        + tr("Modified") + ":</td><td>" + templ.LastModified + "</td></tr></table>");
                  dial->box1->show();
                }
         dial->lineedit1->setText(strr);
         dial->lineedit1->setFocus();
         n = dial->execInputString(&strr);
         delete dial;
         if ( n == 2 )  return;
      }
      else break;
    }
    if ( n == 1 ) {
      nfile.setFileName(current_dir + QDir::separator() + strr);
      if ( nfile.open(QIODevice::WriteOnly)) {
        nfile.close();
        return;
      }
    }
    else {
      CMessageBox *inform = new CMessageBox(pxCache);
      inform->createInform(pxCache->getIcon(182)->scaled(96, 96));
      inform->resize(400,180);
      inform->setWindowTitle(tr("Error createing file") + " - Bathyscaphe");
      inform->inform->setText("<h3>" + tr("Error:") + " </h3> <br> " + tr("Failed to create a file named") + " <br>" + strr);
      inform->detals->setText(tr("System message:") + " " + nfile.errorString());
      inform->setFont(fn);
      inform->setCastomPalette(IconViewer->palette(), Color);
      inform->exec();
      delete inform;
    }
  }
}

void CIconViewerPanelBase :: createItemSymlink() {
  if ( strfile == "") dir.setPath(current_dir);
  else dir.setPath(strfile);
  QList<QString> dirlist = dir.entryList( QDir::AllEntries | QDir::Hidden | QDir::NoDotAndDotDot | QDir::System);
  int errn;
  QFont fn = *(IconViewer->font);
  fn.setBold(true);
  CMessageBox *dial = new CMessageBox(pxCache, this);
  dial->createInputTwoString(pxCache->getIcon(186)->scaled(96,96));
  dial->setFont(fn);
  dial->setModal(true);
  dial->dir = current_dir;
  dial->inform->setText(tr("Please, enter  name and target of symbolic link:"));
  dial->inform->setAlignment(Qt::AlignLeft | Qt::AlignBottom);
  dial->setCastomPalette(IconViewer->palette(), Color);
  dial->okButton->setText(tr("Create"));
  dial->setWindowTitle ( tr("Create new link") + " - Bathyscaphe");
  dial->resize(500, 170);
  dial->labelname2->setText(tr("Name:"));
  dial->labeldetals2->setText(tr("Target:"));
  QString strr, strrpath;
  int n = dial->execInputString(&strr, &strrpath);
  delete dial;
  if ( n == 2 ) return;
  if ( n == 1 ) {
    while ( true ) {
      if ( strr == "." || strr == ".." || strr.contains("/") || strr.contains("\\") || strr == "" || dirlist.contains(strr)) {
        CMessageBox *dial = new CMessageBox(pxCache, this);
        dial->createInputTwoString(pxCache->getIcon(181)->scaled(96, 96));
        dial->setFont(fn);
        dial->setModal(true);
        dial->dir = current_dir;
        dial->lineedit1->setText(strr);
        dial->lineedit2->setText(strrpath);
        dial->labelname2->setText(tr("Name:"));
        dial->labeldetals2->setText(tr("Target:"));
        dial->setCastomPalette(IconViewer->palette(), Color);
        dial->okButton->setText(tr("Create"));
        dial->setWindowTitle ( tr("Please try again") + " - Bathyscaphe");
        dial->resize(400, 170);
        if ( strr == "." || strr == ".." ) dial->inform->setText("<h3>" + tr("Warning:") + " </h3>  <br> " + tr("The name of the symbolic link can not be '.' or '..' .") +  "<br>" + tr(" Please, give a different name:" ));
        else
          if ( strr.contains("/") || strr.contains("\\")) dial->inform->setText("<h3>" + tr("Warning:") + " </h3>  <br> " + tr("The name of the symbolic link can not contain the characters - '/', '\\' .") + " <br> " + tr("Please, give a different name:"));
          else
             if ( strr == "") dial->inform->setText ("<h3>" + tr("Warning:") + "  </h3>  <br>" + tr("Name symbolic link can not be empty.") + " <br>" + tr("Please, give a different name:"));
             else
                if ( dirlist.contains(strr)) {
                  dial->inform->setText("<h3>" +tr("Warning:") + " </h3>  <br> " + tr("This directory already contains an element with the same name.") + " <br>" + tr("Please, give a different name:"));
                  QFileInfo finf(dir.absolutePath() + QDir::separator() + strr);
                  Template templ = pxCache->fullTemplate(finf);
                  dial->labelname1->setText("<h3>" +  templ.FullName + "</h3");
                  dial->labelicon1->setPixmap(templ.Icon.scaled(96,96));
                  if (finf.isDir() && !finf.isSymLink())
                    dial->labeldetals1->setText("<table><tr><td>" + tr("Type") +":</td><td>" +  templ.Type + "</td></tr><tr><td>"
                                                                        + tr("Size") + ":</td><td>" + templ.Size   + "</td></tr><tr><td>"
                                                                        + tr("Created") + ":</td><td>" + templ.Created + "</td></tr><tr><td>"
                                                                        + tr("Modified") + ":</td><td>" + templ.LastModified + "</td></tr></table>");
                  else
                    dial->labeldetals1->setText("<table><tr><td>" + tr("Type") +":</td><td>" +  templ.Type + "</td></tr><tr><td>"
                                                                        + tr("Description (from extended)") +":</td><td>" + templ.Descr + "</td></tr><tr><td>"
                                                                        + tr("Contents") + ":</td><td>" + templ.DescrFromContents + "</td></tr><tr><td>"
                                                                        + tr("Size") + ":</td><td>" + templ.Size   + "</td></tr><tr><td>"
                                                                        + tr("Created") + ":</td><td>" + templ.Created + "</td></tr><tr><td>"
                                                                        + tr("Modified") + ":</td><td>" + templ.LastModified + "</td></tr></table>");
                  dial->box1->show();
                }
         dial->lineedit1->setText(strr);
         dial->lineedit1->setFocus();
         n = dial->execInputString(&strr, &strrpath);
         delete dial;
         if ( n == 2 )  return;
      }
      else break;
    }
    if ( n == 1 ) {
      if (createSymLink(strrpath, current_dir + QDir::separator() + strr, &errn)) return;
    }
    else {
      QString errString;
      CMessageBox *inform = new CMessageBox(pxCache);
      inform->createInform(pxCache->getIcon(182)->scaled(96, 96));
      inform->resize(400,180);
      inform->setWindowTitle(tr("Error creating symbolic link") + " - Bathyscaphe");
      inform->inform->setText("<h3>" + tr("Error:") + " </h3> <br>" + tr("Filed to create a symbolic link named") + " <br>" + strr);
      setErrorString(strrpath, current_dir + QDir::separator() + strr, &errString, &errn);
      inform->detals->setText(tr("System message:") + " " + errString);
      inform->setFont(fn);
      inform->setCastomPalette(IconViewer->palette(), Color);
      inform->exec();
      delete inform;
    }
  }
}

void CIconViewerPanelBase :: execContextAction(QString oper, QString fname) {
  strfile = fname;
  strlist.clear();
  if ( oper == "cut") {
    cutItemClipboard();
    return;
  }
  if ( oper == "copy" ) {
    copyItemClipboard();
    return;
  }
  if ( oper == "del" ) {
    deleteItem();
    return;
  }
  if ( oper == "ren" ) {
    renameItem();
    return;
  }
  if ( oper == "prop") {
    properites();
    return;
  }
}

void CIconViewerPanelBase :: execContextAction(QString oper, QList<QString> flist) {
  strfile.clear();
  strlist = flist;
  if ( oper == "cut") {
    cutItemClipboard();
    return;
  }
  if ( oper == "copy" ) {
    copyItemClipboard();
    return;
  }
  if ( oper == "del" ) {
    deleteItem();
    return;
  }
  if ( oper == "prop") {
    properites();
    return;
  }
}

void CIconViewerPanelBase :: execContextAction (QString oper) {
  strfile.clear();
  strlist.clear();
  if ( oper == "created" ) {
    createItem();
    return;
  }
  if (oper == "creates" ) {
    createItemSymlink();
    return;
  }
  if ( oper == "paste" ) {
    pasteItemClipboard();
    return;
  }
}

void CIconViewerPanelBase :: slotDropMenu(QString op) {
  if ( op == "copy" ) emit execOperate(DropList, strdrop, 1);
  if ( op == "move") emit execOperate(DropList, strdrop, 2);
  if ( op == "sym")  emit execOperate(DropList, strdrop, 4);
}

void CIconViewerPanelBase :: slotDropMenu(int op) {
  emit execOperate(DropList, strdrop, op);
}

//...................................................................................................................................................................ACTIONS WITH CLIPBOARD
void CIconViewerPanelBase :: copyItemClipboard() {
  BufferCut.clear();
  if ( strfile != "" ) {
    clipboard->clear();
    QList<QUrl> url;
    url << strfile;
    mimedata = new QMimeData();
    mimedata->setUrls(url);
    clipboard->setMimeData(mimedata);
  }
  if ( !strlist.isEmpty()) {
    clipboard->clear();
    mimedata = new QMimeData();
    QList<QUrl> url;
    foreach ( QString str, strlist ) {
      url << str;
    }
    mimedata->setUrls(url);
    clipboard->setMimeData(mimedata);
  }
}

void CIconViewerPanelBase :: pasteItemClipboard() {
 if ( clipboard->mimeData()->hasFormat("text/uri-list")) {
     QString path;
     QList<QUrl> listurls = clipboard->mimeData()->urls();
     QList<QString> sourcelist;
     foreach ( QUrl url, listurls ) {
       sourcelist <<  url.path();
     }
     if ( BufferCut.count() > 0 && BufferCut == sourcelist ) {
       BufferCut.clear();
       if ( strfile == "") execOperate(sourcelist, current_dir, 2);
       else execOperate(sourcelist, strfile, 2);
     }
     else {
       if ( strfile == "") emit execOperate ( sourcelist, current_dir, 1 );
         else emit execOperate(sourcelist, strfile, 1);
     }
 }
}

void CIconViewerPanelBase :: cutItemClipboard() {
  copyItemClipboard();
  if ( clipboard->mimeData()->hasFormat("text/uri-list")) {
    QList<QUrl> listurls = clipboard->mimeData()->urls();
      foreach ( QUrl url, listurls )  BufferCut <<  url.path();
  }
}

void CIconViewerPanelBase :: fileOperation(QString str, QString op ) {
  strfile = str;
  strlist.clear();
  if ( op == "cut" ) cutItemClipboard();
  if ( op == "copy" ) copyItemClipboard();
  if ( op == "paste" ) pasteItemClipboard();
  if ( op == "del" ) deleteItem();
  if ( op == "create" ) createItem();
  if ( op == "ren" ) renameItem();
  if ( op == "prop") properites();
}

void CIconViewerPanelBase :: selectAll() {
  if ( view == "list" || view == "icons" )
    foreach ( QListWidgetItem *item, IconViewer->itemsList)  item->setSelected(!item->isSelected());
   if ( view == "table" )
    foreach ( QTreeWidgetItem *item, TableViewer->itemsList)  item->setSelected(!item->isSelected());
}

void CIconViewerPanelBase :: properites() {
  if ( strfile == "" && strlist.count() == 0 ) {
    QFileInfo ff(current_dir);
    CWindowProperites *Wnd = new CWindowProperites(pxCache, ff, IconViewer->palette(), Color, *(IconViewer->font), this);
    Wnd->show();
    return;
  }
  if ( strfile != "" ) {
    QFileInfo ff(strfile);
    CWindowProperites *Wnd = new CWindowProperites(pxCache, ff, IconViewer->palette(), Color, *(IconViewer->font), this);
    Wnd->show();
    return;
  }
  else if (strlist.count() > 0) {
    QList<QFileInfo> flist;
    QFileInfo ff;
    foreach ( QString str, strlist) {
      ff.setFile(str);
      flist << ff;
    }
    CWindowProperites *Wnd = new CWindowProperites(pxCache, flist, IconViewer->palette(), Color, *(IconViewer->font), this);
    Wnd->show();
    return;
  }
}

void CIconViewerPanelBase :: runProgram() {
  QString buffstr = strfile;
  QByteArray byte;
  buffstr = " \"" + buffstr + "\"";
  QString exec = pxCache->returnExec(runc);
   QProcess *process = new QProcess;
   process->start(exec + buffstr);
}

void CIconViewerPanelBase :: selectProgram() {
  CSelectApplicationWindow *execBox = new CSelectApplicationWindow(pxCache, fileinfo);
  QPalette pal;
  pal = IconViewer->palette();
  execBox->setPalette(pal, Color);
  QString Command = execBox->exec();
  delete execBox;
  if ( Command != "" ) {
   QProcess *process = new QProcess;
   process->start(Command);
  }
}

void CIconViewerPanelBase :: findAbc(QString str) {
  if ( view == "list" || view == "icons") IconViewer->findAbc(str);
  if ( view == "table" ) TableViewer->findAbc(str);
}

CIconViewerPanelBase :: ~CIconViewerPanelBase(){
  delete WebHistory;
}

void CIconViewerPanelBase :: setTemplateFilter(QString str) {
  if ( str == "*") {
    TemplateFilter.clear();
    MimeFilter.clear();
    if ( view == "list" || view == "icons" ) IconViewer->showDirectory(current_dir, true);
    if ( view == "table" ) TableViewer->showDirectory(current_dir, true);
    return;
  }
  TemplateFilter = str.split(" ");
  MimeFilter.clear();
  if ( view == "list" || view == "icons" ) IconViewer->showDirectory(current_dir, true);
  if ( view == "table" ) TableViewer->showDirectory(current_dir, true);
}

void CIconViewerPanelBase :: setMimeFilter(QString str) {
  if ( str == "*") {
    TemplateFilter.clear();
    MimeFilter.clear();
    if ( view == "list" || view == "icons" ) IconViewer->showDirectory(current_dir, true);
    if ( view == "table" ) TableViewer->showDirectory(current_dir, true);
    return;
  }
  QStringList lst = str.split(" ");
  MimeFilter.clear();
  foreach ( QString ss, lst) {
    MimeFilter << QRegExp(ss);
  }
  TemplateFilter.clear();
  if ( view == "list" || view == "icons" ) IconViewer->showDirectory(current_dir, true);
  if ( view == "table" ) TableViewer->showDirectory(current_dir, true);
}

void CIconViewerPanelBase :: resetSelection() {
  if ( view == "icons"  || view == "list" ) {
    IconViewer->blockSignals(true);
    IconViewer->setCurrentItem(0);
    IconViewer->blockSignals(false);
  }
  if ( view == "table") {
    TableViewer->blockSignals(true);
    TableViewer->setCurrentItem(0);
    TableViewer->blockSignals(false);
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////     RUN VIEWERS

void CIconViewerPanelBase :: runWebViewer(QString url) {
  if ( WebViewer == NULL ) {
    WebViewer = new CWebViewer(this);
    WebViewerTool = new CWebViewerTool(pxCache);
    WebViewerTool->setFlat(Flat);
    WebViewerStatus = new QLineEdit(this);
    WebViewerStatus->setReadOnly(true);
    WebViewer->setFont(*(IconViewer->font));
    WebViewerTool->setFont(IconViewer->font);
    WebViewerTool->setFontSize();
    WebViewerStatus->setFont(*(IconViewer->font));
    Stack->addWidget(WebViewer);
    Stack->setCurrentWidget(WebViewer);
    VLayout->insertWidget(1, WebViewerTool);
    VLayout->insertWidget(3, WebViewerStatus);
    WebViewer->Scale = WebViewerScale;
    WebViewer->setToolAndHistory(WebViewerTool, WebHistory, WebViewerStatus);
    QPalette pal = this->palette();
    WebViewer->setCastomPalette(pal, Color);
    WebViewerStatus->setPalette(pal);
    OtherView = true;
    Navigator->setTextOnly(true);
    Navigator->setPath(url);

    emit hideStatusBar(true);
    emit hideFilter(true);
    emit refreshWrite();
    connect ( WebViewer, SIGNAL(loadFinished(bool)), SLOT(setNavigatorUrl(bool)));
  }

  WebViewer->goAdress(QUrl(url));
  WebViewerTool->PrevButton->setEnabled(WebViewer->history()->canGoBack());
  WebViewerTool->ForwardButton->setEnabled(WebViewer->history()->canGoForward());
}

void CIconViewerPanelBase :: runImageViewer(PixmapsCache *pxcache, QFileInfo fname) {
  ImageViewer = new CImageViewer(pxcache, fname);
  ImageViewerTool = ImageViewer->returnTool();
  ImageViewerTool->setFlat(Flat);
  QPalette pal = IconViewer->palette();
  ImageViewer->setCastomPalette(pal, Color);
  ImageViewer->Font = IconViewer->font;
  ImageViewerTool->setFont(IconViewer->font);
  ImageViewer->setFontSize();
  Stack->addWidget(ImageViewer);
  Stack->setCurrentWidget(ImageViewer);
  ImageViewer->resize(ImageViewer->size() - QSize(3, 3));
  OtherView = true;

  emit hideStatusBar(true);
  emit hideFilter(true);
  emit refreshWrite();
  connect ( ImageViewer, SIGNAL(setNavigatorPath(QString)), Navigator, SLOT(setPath(QString)));
}

void CIconViewerPanelBase :: deleteViewers() {
  int szz;
  if ( OtherView) {
      if ( WebViewer != NULL ) {
        WebViewerScale = WebViewer->Scale;
        Stack->removeWidget(WebViewer);
        delete WebViewer;
        VLayout->removeWidget(WebViewerTool);
        delete WebViewerTool;
        VLayout->removeWidget(WebViewerStatus);
        delete WebViewerStatus;
        WebViewer = NULL;
        Navigator->setTextOnly(false);
        Navigator->setPath(current_dir);
        if ( view == "icons" || view == "list" ) {
          Stack->setCurrentWidget(IconViewer);
          QList<QListWidgetItem*> lstt = IconViewer->selectedItems();
          szz = lstt.count();
          if ( szz == 1 ) emit refreshWrite(QFileInfo(lstt[0]->toolTip()), true);
        }
        if ( view == "table") {
          Stack->setCurrentWidget(TableViewer);
          QList<QTreeWidgetItem*> lstt = TableViewer->selectedItems();
          szz = lstt.count();
          if ( szz == 1 ) emit refreshWrite(QFileInfo(lstt[0]->toolTip(0)), true);
        }
        OtherView = false;
        emit hideStatusBar(false);
        emit hideFilter(false);
        return;
      }
      if ( TextViewer != NULL ) {
        TextViewerDontEdit = TextViewerTool->DontEditButton->isChecked();
        TextViewerScale = TextViewer->Scale;
        Stack->removeWidget(TextViewer);
        delete TextViewer;
        VLayout->removeWidget(TextViewerTool);
        delete TextViewerTool;
        TextViewer = NULL;
        Navigator->setPath(current_dir);
        if ( view == "icons" || view == "list" ) {
          Stack->setCurrentWidget(IconViewer);
          QList<QListWidgetItem*> lstt = IconViewer->selectedItems();
          szz = lstt.count();
          if ( szz == 1 ) emit refreshWrite(QFileInfo(lstt[0]->toolTip()), true);
        }
        if ( view == "table") {
          Stack->setCurrentWidget(TableViewer);
          QList<QTreeWidgetItem*> lstt = TableViewer->selectedItems();
          szz = lstt.count();
          if ( szz == 1 ) emit refreshWrite(QFileInfo(lstt[0]->toolTip(0)), true);
        }
        OtherView = false;
        emit hideStatusBar(false);
        emit hideFilter(false);
        return;
      }
      if ( ImageViewer != NULL ) {
        Stack->removeWidget(ImageViewer);
        delete ImageViewer;
        ImageViewer = NULL;
        Navigator->setPath(current_dir);
        if ( view == "icons" || view == "list" ) {
          Stack->setCurrentWidget(IconViewer);
          QList<QListWidgetItem*> lstt = IconViewer->selectedItems();
          szz = lstt.count();
          if ( szz == 1 ) emit refreshWrite(QFileInfo(lstt[0]->toolTip()), true);
        }
        if ( view == "table") {
          Stack->setCurrentWidget(TableViewer);
          QList<QTreeWidgetItem*> lstt = TableViewer->selectedItems();
          szz = lstt.count();
          if ( szz == 1 ) emit refreshWrite(QFileInfo(lstt[0]->toolTip(0)), true);
        }
        OtherView = false;
        emit hideStatusBar(false);
        emit hideFilter(false);
        return;
      }
    }
}

void CIconViewerPanelBase :: setNavigatorUrl(bool ok) {
  if (ok) {
    Navigator->setPath(WebViewer->url().toString());
    Navigator->setLabelIcon(WebViewer->icon());
  }
}

void CIconViewerPanelBase :: runTextViewer(QString fname) {
  if ( TextViewer == NULL ) {
    TextViewer = new CTextViewer();
    TextViewerTool = new CTextViewerTool(pxCache);
    TextViewer->Font = IconViewer->font;
    TextViewer->Scale = TextViewerScale;
    TextViewerTool->setFont(IconViewer->font);
    TextViewerTool->setFontSize();
    TextViewer->setTextViewerTool (TextViewerTool);
    TextViewerTool->setFlat(Flat);
    QPalette pal = IconViewer->palette();
    TextViewer->setCastomPalette(pal, Color);
    Stack->addWidget(TextViewer);
    Stack->setCurrentWidget(TextViewer);
    VLayout->insertWidget(1, TextViewerTool);
    TextViewerTool->DontEditButton->setChecked(TextViewerDontEdit);
    TextViewer->setReadOnly(TextViewerDontEdit);
    TextViewerTool->UndoButton->setEnabled(false);
    TextViewerTool->RedoButton->setEnabled(false);

    OtherView = true;
    emit hideStatusBar(true);
    emit hideFilter(true);
    emit refreshWrite();
    connect ( TextViewer, SIGNAL(changeNewPath(QString)), Navigator, SLOT(setPath(QString)));
  }

  TextViewer->loadText(fname);
}

TypeF CIconViewerPanelBase :: parseType(QFileInfo finf, QString *mim) {
  QString descr;
  QFileInfo TargetFileInfo;
  if ( finf.isSymLink()) {
    TargetFileInfo.setFile(finf.symLinkTarget());
    if ( TargetFileInfo.exists())
       while(TargetFileInfo.isSymLink()) TargetFileInfo.setFile(TargetFileInfo.symLinkTarget());
    if ( TargetFileInfo.exists()) NavigatorIcon = pxCache->getIcon(TargetFileInfo, &descr, mim);
       else NavigatorIcon = pxCache->getIcon(finf, &descr, mim);
  }
  else NavigatorIcon = pxCache->getIcon(finf, &descr, mim);

  QString command = pxCache->findAssociation(*mim);
  if ( !command.isEmpty()) {
    QString str = " \"" + finf.absoluteFilePath() + "\"";
    QProcess *process = new QProcess();
    process->start(command +" " + str);
    return Known;
  }

  QStringList listt = mim->split("/");
  QString mime;
  if (listt.count() == 2) mime = listt[1];
  if ( !mime.isEmpty()) {
     if ( mime == "bmp" || mime == "x-bmp" || mime == "x-MS-bmp" ||
          mime == "jpeg" || mime == "pjpeg" || mime == "png" ||
          mime == "tiff" || mime == "x-portable-bitmap" || mime == "x-portable-anymap" ||
          mime == "x-portable-graymap" || mime == "x-portable-pixmap" ||
          mime == "x-xbitmap" || mime == "x-xpixmap"  || mime == "x-ico" || mime == "x-icon" ||
          mime == "vnd.microsoft.icon" || mime == "ico" || mime == "icon" ||
          mime == "gif" || mime == "x-mng" || mime == "svg+xml" ||
          mime == "svg+xml-compressed")                                                                                                                  return Image;
  }

  if ( *mim == "text/html" || *mim == "text/htmlh" || *mim == "text/sgml" ||
        *mim == "application/xhtml+xml" )                                                                                                                   return Web;

  if ( pxCache->subClassMime(*mim, "text/html"))                                                                                                 return  Web;

  if ( pxCache->subClassMime(*mim, "text/plain"))                                                                                                return Text;

  if ( *mim == "application/x-executable") {
    QProcess *process = new QProcess();
    process->start(finf.absoluteFilePath());
    return Known;
  }

  command = pxCache->findAssociation(*mim);
  if ( !command.isEmpty()) {
    QString str = " \"" + finf.absoluteFilePath() + "\"";
    QProcess *process = new QProcess();
    process->start(command +" " + str);
    return Known;
  }
  else {
    QList<TemplateApps> Tapps;
    Tapps = pxCache->fullTemplateApps(finf);
    if ( Tapps.count() >=1) {
      QString str = " \"" + finf.absoluteFilePath() + "\"";
      QProcess *process = new QProcess();
      process->start(Tapps[0].execApp +" " + str);
      return Known;
    }
  }

  return Unknown;
}

QString CIconViewerPanelBase :: createErrorText(QString filename) {
  QString str;
  str = "<b> <font size=3>";
  str.append(tr("File or directory"));
  str.append("<br>  ");
  str.append(filename + "<br> ");
  str.append(tr("does not open."));
  str.append("<br> </font> </b>");
  str.append(" <b> <font size=2> <ul> <li>");
  str.append(tr("Not valid name."));
  str.append("<li>");
  str.append("This is broken link.");
  str.append("</ul> </font> </b>");

  return str;
}

bool CIconViewerPanelBase :: LocationStartsWithDrives(QString location) {
  foreach( QString str, *listdrives) {
    if ( location.startsWith(str)) return true;
  }
  return false;
}

void CIconViewerPanelBase :: textViewerSave() {
    CMessageBox *dial = new CMessageBox(pxCache, this);
    dial->createSaveQuestion(*(pxCache->getIcon(181)));
    dial->setCastomPalette(IconViewer->palette(), Color);
    dial->inform->setText(QString("<h4>" + tr("This text was modified") + "</h4><h4>") +
                                          QString(tr("Save this text?") + "</h4>"));
    int n = dial->exec();
    if ( n == 1) TextViewer->saveText();
    if ( n == 3) TextViewer->saveTextAs();
    delete dial;
}

void CIconViewerPanelBase :: setNavigatorIcon( QFileInfo finf) {
  NavigatorIcon = pxCache->getIcon(2);
  if (finf.absoluteFilePath() == QDir::homePath()) NavigatorIcon = pxCache->getIcon(1);
  if (QDir::drives().contains(finf)) NavigatorIcon = pxCache->getIcon(3);
  if ( finf.isRoot()) NavigatorIcon = pxCache->getIcon(0);
  if (finf.isHidden()) NavigatorIcon = pxCache->getIcon(4);
  Navigator->setLabelIcon(*NavigatorIcon);
}


void CIconViewerPanelBase :: openElement() {
  if ( view == "icons" || view == "list")
    foreach ( QListWidgetItem *itt, IconViewer->itemsList) {
      if ( itt->toolTip() == strfile ) {
	emit openElement (itt);
	return;
      }
    }
  if ( view == "table" )
    foreach ( QTreeWidgetItem *itt, TableViewer->itemsList) {
      if ( itt->toolTip(0) == strfile) {
	emit openElement(itt);
	return;
      }
    }
}








