#include "../../Include/IconViewerSettings.h"
//#include "../../Include/Palette.h"

CIconViewerSettings :: CIconViewerSettings(PixmapsCache *pxcache, QWidget *wgt) : CToolBar(wgt){
  readSettings();
  pxCache = pxcache;
  CVS = new CCvs(this);
  currentTheme = pxcache->getThemeName();
  fileNameStateOp = pxcache->getAnimateFile();
  setUpdateButtonPixmap();
  ShowHide = addAction ( *(pxCache->getIcon(60)), tr("Show/Hide hidden files") );
  ShowHide->setCheckable(true);
  if ( showHidden ) ShowHide->setChecked(true); else ShowHide->setChecked(false);
  addSeparator();
  BackColor = addAction ( *(pxCache->getIcon(61)), tr("Set background color") );
  BackTexture = addAction ( *(pxCache->getIcon(62)), tr("Set background texture") );
  FontColor = addAction ( *(pxCache->getIcon(63)), tr("Set font color") );
  FontSize = addAction ( *(pxCache->getIcon(64)), tr("Set font size") );
  IconSize = addAction ( *(pxCache->getIcon(65)), tr("Set icon size") );
  Theme = addAction ( *(pxCache->getIcon(66)), tr("Set theme icon") );

//-------------------------------------------------------------------------- create Menu
  QAction *actt;
  back_texture = new QMenu(this);
  QList<QPixmap> TexturePix = settingsCache.getTexturePixmaps();
  QList<QString> TextureName = settingsCache.getTextureName();
  QList<QPixmap>::Iterator It = TexturePix.begin();
  QList<QString>::Iterator Itt = TextureName.begin();
  for ( ; It != TexturePix.end(); ++It ){
    actt = back_texture->addAction ( *It,  *Itt );
    if ( *Itt == currentBackTextureName ) {
      actt->setEnabled(false);
      back_texture->setDefaultAction(actt);
      currentBackTexture = *It;
    }
    ++Itt;
  }
  BackTexture->includeMenu (back_texture);

  font_size = new QMenu(this);
  QString sizeitem;
  for ( int sz = 6; sz != 21; ) {
    sizeitem = QString::number(sz);
    actt = font_size->addAction(sizeitem);
    if (sz == currentFontSize) {
      actt->setEnabled(false);
      font_size->setDefaultAction(actt);
    }
    sz = sz + 1;
  }
  FontSize->includeMenu (font_size);

  icon_size = new QMenu(this);
  QList<int> sizeicon;
  sizeicon << 8 << 16 << 24 << 32 << 48 << 64 << 96 <<128;
  foreach ( int sz, sizeicon ){
    actt = icon_size->addAction ( QString::number(sz) + " x " + QString::number(sz));
    if ( view == "icons" )
      if ( currentIconSize == QSize(sz,sz)) {
        icon_size->setDefaultAction(actt);
        actt->setEnabled(false);
      }
    if ( view == "list" )
      if ( currentListSize == QSize(sz,sz)) {
        icon_size->setDefaultAction(actt);
        actt->setEnabled(false);
      }
    if ( view == "table" )
      if ( currentTableSize == QSize(sz,sz)) {
        icon_size->setDefaultAction(actt);
        actt->setEnabled(false);
      }
  }
  IconSize->includeMenu ( icon_size );

  _theme = new QMenu (this);
  TextureName = settingsCache.getThemes();
  foreach ( QString str, TextureName) {
    actt = _theme->addAction(str);
    if ( str == currentTheme ) {
      actt->setEnabled(false);
      _theme->setDefaultAction(actt);
    }
  }
  actt = _theme->addAction("default");
  if ( currentTheme == "default" ){
    actt->setEnabled(false);
    _theme->setDefaultAction(actt);
  }
  Theme->includeMenu(_theme);

  this->Layout.addStretch(1);
  addSeparator();
  stateOp = addAction(fileNameStateOp);
  stateOp->stop();
  addSeparator();

  CleanKey = addAction(*(pxCache->getIcon(70)), tr("Clean all settings"));
  Styles = addAction(*(pxCache->getIcon(69)), tr("Change style"));
  Updates = addAction(*no_update, tr("No information about new releases"));
  ifUpdates = false;

  StyleList << QStyleFactory::keys();
  if ( StyleNumber > StyleList.count()-1) StyleNumber = 0;
  styles = new QMenu(this);
  for ( int i = 0; i < StyleList.count(); ++i) {
    actt = styles->addAction(StyleList[i] + " " + QString("style"));
    if ( i == StyleNumber) {
      actt->setEnabled(false);
      styles->setDefaultAction(actt);
    }
    actt->setToolTip(QString::number(i));
    styles->addSeparator();
  }
  Styles->includeMenu(styles);
  QApplication::setStyle(QStyleFactory::create(StyleList[StyleNumber]));

  setBackColor(currentBackColor.darker(150));
  setFontColor(currentFontColor);

  connect ( back_texture, SIGNAL(triggered(QAction*)), SLOT(clickedBackTexture(QAction*)));
  connect ( font_size, SIGNAL(triggered(QAction*)), SLOT(clickedFontSize(QAction*)));
  connect ( icon_size, SIGNAL(triggered(QAction*)), SLOT(clickedIconSize(QAction*)));
  connect ( _theme, SIGNAL(triggered(QAction*)), SLOT(clickedTheme(QAction*)));
  connect ( ShowHide, SIGNAL(clicked()), SLOT(clickedShowHidden()));
  connect ( BackColor, SIGNAL(clicked()), SLOT(clickedBackColor()));
  connect ( FontColor, SIGNAL(clicked()), SLOT(clickedFontColor()));
  connect ( styles, SIGNAL(triggered(QAction*)), SLOT(clickedStyles(QAction*)));
  connect ( CleanKey, SIGNAL(clicked()), SLOT(clickedCleanKey()));
  connect ( CVS, SIGNAL(finishedUpdates(QBuffer*, bool)), SLOT(showWindowUpdates(QBuffer*, bool)));
  connect ( Updates, SIGNAL(clicked()), SLOT(showWindowUpdates()));

  SaveSettings = true;
  CVS->starting();

}

void CIconViewerSettings :: clickedShowHidden(){
    if ( showHidden ) {
      ShowHide->setChecked(false);
      showHidden = false;
      emit changeShowHidden(false);
    }
    else {
      ShowHide->setChecked(true);
      showHidden = true;
      emit changeShowHidden(true);
    }
}

void CIconViewerSettings::clickedBackColor(){                                    //action
  QColor color = QColorDialog::getColor ( currentBackColor );
  if ( color.isValid()) {
    backgroundColor = true;
    currentBackColor = color;
    QAction *actt;
    actt = back_texture->defaultAction();
    if ( actt ) actt->setEnabled(true);
    setBackColor(color.darker(150));
    emit changeBackColor(color);
  }
}

void CIconViewerSettings::clickedFontColor(){                                       //action
  QColor color = QColorDialog::getColor(currentFontColor);
  if ( color.isValid()) {
    currentFontColor = color;
    setFontColor(color);
    emit changeFontColor(color);
  }
}

void CIconViewerSettings::clickedBackTexture(QAction* action){         //menu
  backgroundColor = false;
  QAction *actt = back_texture->defaultAction();
  if ( actt ) actt->setEnabled(true);
  back_texture->setDefaultAction(action);
  action->setEnabled(false);
  currentBackTextureName = action->text();
  currentBackTexture = action->icon().pixmap(128,128);
  emit changeBackTexture(currentBackTexture);
}

void CIconViewerSettings::clickedFontSize(QAction* action) {            //menu
  QAction *actt = font_size->defaultAction();
  if ( actt ) actt->setEnabled(true);
  font_size->setDefaultAction(action);
  action->setEnabled(false);
  currentFontSize = action->text().toInt();
  font.setPixelSize(currentFontSize);
  emit changeFontSize( );
}

void CIconViewerSettings::clickedIconSize(QAction* action){                   //menu
  QAction *actt = icon_size->defaultAction();
  if ( actt ) actt->setEnabled(true);
  icon_size->setDefaultAction(action);
  action->setEnabled(false);
  QStringList lst = action->text().split(" x ");
  int sz = lst[0].toInt();
  if (view == "icons") currentIconSize = QSize(sz,sz);
  if (view == "list") currentListSize = QSize(sz,sz);
  if (view == "table") currentTableSize = QSize(sz,sz);
  emit changeIconSize(QSize(sz, sz));
}

void CIconViewerSettings::clickedTheme( QAction* action){                    //menu
  QAction *actt = _theme->defaultAction();
  if (actt) actt->setEnabled(true);
  _theme->setDefaultAction(action);
  action->setEnabled(false);
  currentTheme = action->text();
  pxCache->clear();
  pxCache->fullCache(currentTheme);
  pxCache->fullMapApps("/usr/share/icons");
  pxCache->fullMapApps();
  pxCache->fullCacheApps("/usr/share/applications");
  pxCache->setQueueIcon();
  pxCache->fullRCache();

  fileNameStateOp = pxCache->getAnimateFile();
  stateOp->setFileNameAnimation(fileNameStateOp);
  setUpdateButtonPixmap();
  if ( ifUpdates ) Updates->setIcon(*is_update);
    else Updates->setIcon(*no_update);
  ShowHide->setIcon( *(pxCache->getIcon(60)));
  BackColor->setIcon( *(pxCache->getIcon(61)));
  BackTexture->setIcon( *(pxCache->getIcon(62)));
  FontColor->setIcon( *(pxCache->getIcon(63)));
  FontSize->setIcon ( *(pxCache->getIcon(64)));
  IconSize->setIcon ( *(pxCache->getIcon(65)));
  Theme->setIcon( *(pxCache->getIcon(66)));

  emit changeTheme ();
}

void CIconViewerSettings :: changeView (QString str) {
  view = str;
  icon_size->clear();
  QList<int> sizeicon;
  QAction *actt;
  sizeicon << 8 << 16 << 24 << 32 << 48 << 64 << 96 <<128;
  foreach ( int sz, sizeicon ){
    actt = icon_size->addAction ( QString::number(sz) + " x " + QString::number(sz));
    if ( str == "icons" )
      if ( currentIconSize == QSize(sz,sz)) {
        icon_size->setDefaultAction(actt);
        actt->setEnabled(false);
      }
    if ( str == "list" )
      if ( currentListSize == QSize(sz,sz)) {
        icon_size->setDefaultAction(actt);
        actt->setEnabled(false);
      }
    if ( str == "table" )
      if ( currentTableSize == QSize(sz,sz)) {
        icon_size->setDefaultAction(actt);
        actt->setEnabled(false);
      }
  }
}

void CIconViewerSettings :: readSettings(){
  settings.beginGroup("/Bathyscaphe/IconViewer");
  showHidden = settings.value ("/ShowHide", false).toBool();
  backgroundColor = settings.value("/bbackColor", true).toBool();
  currentBackColor = settings.value("/BackColor", QColor(255,255,255)).value<QColor>();
  if ( !backgroundColor ) {
    currentBackTexture = settings.value("/BackTexture", 0).value<QPixmap>();
    currentBackTextureName = settings.value("/TextureName", "none").toString();
  }
  currentFontColor = settings.value("/FontColor", QColor(0,0,0)).value<QColor>();
  currentListSize = settings.value("/ListSize", QSize(24,24)).value<QSize>();
  currentIconSize = settings.value("/IconSize", QSize(48,48)).value<QSize>();
  currentTableSize = settings.value("/TableSize", QSize(24,24)).value<QSize>();
  currentFontSize = settings.value("/FontSize", 12).toInt();
  font.setPixelSize(currentFontSize);

  view = settings.value ("/View", "icons").toString();
  StyleNumber = settings.value("/StyleNumber", 0).toInt();
  autoWindowUpdates = settings.value("/AutoWindowUpdates", true).toBool();
  settings.endGroup();
}

void CIconViewerSettings :: writeSettings(){
  if ( SaveSettings ) {
    settings.beginGroup("/Bathyscaphe/IconViewer");
    settings.setValue ("/ShowHide", showHidden);
    settings.setValue("/bbackColor", backgroundColor);
    settings.setValue("/BackColor", currentBackColor);
    if ( !backgroundColor ) {
      settings.setValue ("/BackTexture", currentBackTexture);
      settings.setValue("/TextureName", currentBackTextureName);
    }
    settings.setValue("/FontColor", currentFontColor);
    settings.setValue ("/ListSize", currentListSize);
    settings.setValue("/IconSize", currentIconSize);
    settings.setValue("/TableSize", currentTableSize);
    settings.setValue("/FontSize", currentFontSize);
    settings.setValue("/StyleNumber", StyleNumber);
    settings.setValue("/AutoWindowUpdates", autoWindowUpdates);

    settings.endGroup();
  }
}

void CIconViewerSettings :: setFlat(bool flat){
  CToolBar::setFlat(flat);
}

void CIconViewerSettings :: clickedStyles(QAction* action) {
  int n = action->toolTip().toInt();
  QApplication::setStyle(QStyleFactory::create(StyleList[n]));
  StyleNumber = n;
  QAction *actt = styles->defaultAction();
  actt->setEnabled(true);
  action->setEnabled(false);
  styles->setDefaultAction(action);
}

void CIconViewerSettings :: clickedCleanKey() {
  CMessageBox *dialog = new CMessageBox(pxCache);
  dialog->createQuestion(*(pxCache->getIcon(181)));
  dialog->inform->setText("<b> <font size=4>" + tr("Warhing:") + " " + tr("As a result of this operation") + "<br>" +
                                            tr("will erase all custom settings.") + "<br>" + tr("You have thought?"));
  dialog->detals->setText(tr("Note: restart programm for apply changed"));
  dialog->setPalette(this->palette());
  int i = dialog->exec();
  delete dialog;
  if ( i == 1 ) {
    SaveSettings = false;
    CCleanKey clean;
    clean.deleteKeys();
    emit setNotSaveSettings();
  }
}

void CIconViewerSettings :: showWindowUpdates(QBuffer *buffer, bool ok) {
  if ( ok ) {
    QBuffer * bufferUpdates = buffer;
    bufferUpdates->setParent(this);
    if ( buffer->open(QIODevice::ReadOnly)) {
      QString content_text, append_text;
      content_text = QString(buffer->readLine());
      append_text = QString(buffer->readLine());
      QStringList tlst = append_text.split("=");
      if (tlst.count() > 1) {
        if (tlst[1].toInt() > 0 ) {
          ifUpdates = true;
          Updates->setIcon(*is_update);
          Updates->setToolTip(tr("Access to new versions..."));
        }
      }
      buffer->close();
      if ( autoWindowUpdates && ifUpdates) {
        showWindowUpdates();
      }
    }
  }
}

void CIconViewerSettings :: showWindowUpdates() {
  QBuffer *bufferUpdates = CVS->getBuffer();
  QDialog *dialog_updates = new QDialog(this);
  dialog_updates->setWindowIcon(*(pxCache->getMainIcon(2)));
  QVBoxLayout *vlay = new QVBoxLayout(dialog_updates);
  QPushButton *clbutton = new QPushButton(*(pxCache->getIcon(191)), tr("Close"));
  dialog_updates->setModal(true);
  QCheckBox *checkAuto = new QCheckBox(tr("Automatically show this window when new versions are available"));
  checkAuto->setChecked(autoWindowUpdates);
  connect ( checkAuto, SIGNAL(clicked()), SLOT(changeAutoShowUpdates()));
  connect ( clbutton, SIGNAL(clicked()), dialog_updates, SLOT(accept()));

  if ( ifUpdates ) {
    bool openlist = false;
    QString content_text, append_text;
    dialog_updates->setWindowTitle(tr("New version available") + " - Bathyscaphe");
    QTextBrowser *textarea = new QTextBrowser();
    textarea->setOpenExternalLinks(true);
    vlay->addWidget(textarea);
    vlay->addWidget(checkAuto);
    vlay->addWidget(clbutton);

    if ( bufferUpdates->open(QIODevice::ReadOnly)) {
      QStringList lstt;
      while (!bufferUpdates->atEnd()) {
        append_text = QString::fromUtf8(bufferUpdates->readLine());
        append_text.remove('\n');
        if ( append_text == "") continue;
        lstt = append_text.split("=");
        if ( lstt.count() > 0) {
          if ( lstt[0] == "COUNT_UPDATES" ) {
            content_text.append("<h3 align=center>" + tr("Your version th program is") + " " + QString::number(CVS->getVersion()) + "<br>");
            content_text.append(tr("Available on the website <a href = 'http://www.bathyscaphe.ru'> www.bathyscaphe.ru</a>  new versions") + "  (" + lstt[1] + ")</h3>");
            continue;
          }
          if ( lstt[0] == "VERSION" ) {
            if ( openlist ) {
              content_text.append("</ul>");
              openlist = false;
            }
            content_text.append("<font size=4> <b><u><center>" + tr("Version:") + " " + lstt[1] + "</center></u> </b> </font> <br>");
            continue;
          }
          if ( append_text == "CHANGES:" ) {
            if ( openlist ) {
                content_text.append("</ul>");
                openlist = false;
            }
            content_text.append("<font size=3 align=left><b>" + tr("Changes and mores:") + "</b> </font> ");
            continue;
          }
          if ( append_text == "BUGS:" ) {
            if ( openlist ) {
              content_text.append("</ul>");
              openlist = false;
            }
            content_text.append("<font size=3 align=left> <b>" + tr("Bugs and errors:") + "</b> </font> ");
            continue;
          }
          if ( !openlist ) {
            content_text.append("<ul>");
            openlist = true;
          }
          content_text.append("<li>");
          content_text.append( append_text );
          content_text.append("</li>");
        }
      }
    }
    content_text.append("</ul>");

    bufferUpdates->close();
    textarea->setHtml(content_text);
    textarea->setReadOnly(true);
    QPalette pal = this->palette();
    if (backgroundColor) pal.setBrush(QPalette::Window, QBrush(currentBackTexture));
    dialog_updates->setPalette(this->palette());
    textarea->setPalette(pal);
    dialog_updates->resize(600, 600);
    dialog_updates->exec();
    delete dialog_updates;
  }
  else {
    dialog_updates->setWindowTitle(tr("No new version") + " - Bathyscaphe");
    QLabel *Info = new QLabel();
    vlay->addWidget(Info);
    vlay->addWidget(checkAuto);
    vlay->addWidget(clbutton);
    Info->setText("<h3 align=center>" + tr("Not information about new version") + "</h3>");
    dialog_updates->setPalette(this->palette());
    dialog_updates->resize(600, 250);
    dialog_updates->exec();
    delete dialog_updates;
  }
}


CIconViewerSettings :: ~CIconViewerSettings(){
  writeSettings();
}