#include "../../Include/ImageViewer.h"

CImageViewer :: CImageViewer(PixmapsCache *pxcache, QFileInfo finf, QWidget *wgt) : QFrame(wgt) {
  FileInfo = finf;
  QFileInfoList finflist = finf.absoluteDir().entryInfoList(QDir::Files | QDir::Hidden | QDir::NoDotAndDotDot);
  int i = 0;
  ImageTool = new CImageViewerTool(pxcache, this);
  foreach ( QFileInfo felement, finflist) {
    if (!felement.isFile()) continue;
    if ( felement.isSymLink()) {
      QString str = felement.symLinkTarget();
      QFileInfo BufferFile(str);
      while (BufferFile.isSymLink()) {
        str = BufferFile.symLinkTarget();
        BufferFile.setFile(str);
      }
      if ( BufferFile.isFile()) {
        ImageType = parseType(BufferFile);
        if ( ImageType != NotImage) {
          ImageTypeList << ImageType;
          FileInfoList << felement;
          if ( FileInfo == felement ) CurrentImage = i;
          ++i;
          continue;
        }
      }
      else continue;
    }

    ImageType = parseType(felement);
    if ( ImageType != NotImage) {
      FileInfoList << felement;
      ImageTypeList << ImageType;
      if ( FileInfo == felement ) CurrentImage = i;
      ++i;
    }
  }

  Scale = 100;
  ErrorImage = false;
  ImageCanvas = new QWidget();
  AnimateCanvas = new QLabel();
  SvgCanvas = new QSvgWidget();
  Image = new QImage();
  Animate = new QMovie();
  ImageViewport = new QScrollArea(this);
  setFrameStyle(QFrame::Box | QFrame::Sunken);
  setLineWidth(2);
  this->setAutoFillBackground(true);
  ImageCanvas->setAutoFillBackground(true);

  this->setMinimumWidth(ImageTool->width());
  this->setMinimumHeight(ImageTool->width());
  MaxSize.setWidth(this->width());
  MaxSize.setHeight(this->height() - ImageTool->height());

  ImageType = ImageTypeList.value(CurrentImage);
  FileInfo = FileInfoList.value(CurrentImage);
  if ( ImageType == NormalImage ) {
    delete Image;
    Image = new QImage(FileInfo.absoluteFilePath());
    if ( Image->isNull()) {
      errorImage(tr("Warning"), tr("This file is not an image, or this format is not supported"));
      ImageTypeList[CurrentImage] = NotImage;
      ImageType = NotImage;
    }
  }
  if ( ImageType == AnimatedImage ) {
    delete Animate;
    Animate = new QMovie (FileInfo.absoluteFilePath());
    if ( !Animate->isValid()) {
      errorImage(tr("Warning"), tr("This file is not an image, or this format is not supported"));
      ImageTypeList[CurrentImage] = NotImage;
      ImageType = NotImage;
    }
    if ( FileInfo.suffix().toLower() == "gif" || ImageTool->pxCache->getMimeSource(FileInfo) == "image/gif") {
      if (Animate->frameCount() == 1) {
        ImageType = NormalImage;
        ImageTypeList[CurrentImage] = NormalImage;
        Image->load(FileInfo.absoluteFilePath());
      }
    }
    if ( ImageType == AnimatedImage) {
      AnimateCanvas->setMovie(Animate);
      Animate->start();
      AnimateOriginSize = AnimateCanvas->sizeHint();
    }
  }
  if ( ImageType == SvgImage ) {
    delete SvgCanvas;
    SvgCanvas = new QSvgWidget (FileInfo.absoluteFilePath());
    if ( !SvgCanvas->renderer()->isValid()) {
      errorImage(tr("Warning"), tr("This file is not an image, or this format is not supported"));
      ImageTypeList[CurrentImage] = NotImage;
      ImageType = NotImage;
    }
    SvgOriginSize = SvgCanvas->sizeHint();
    if ( SvgCanvas->renderer()->animated()) {
      ImageType = AnimatedSvg;
      ImageTypeList[CurrentImage] = AnimatedSvg;
      connect ( SvgCanvas->renderer(), SIGNAL(repaintNeeded()), SvgCanvas, SLOT(repaint()));
    }
  }

  setBackNext();

  connect ( ImageTool->ScaleViewportButton, SIGNAL(clicked()), SLOT(changeScaleViewport()));
  connect ( ImageTool->ScaleSlider, SIGNAL(valueChanged(int)), SLOT(changeScaleValue(int)));
  connect ( ImageTool->PreviousButton, SIGNAL(clicked()), SLOT(showPreviousImage()));
  connect ( ImageTool->NextButton, SIGNAL(clicked()), SLOT(showNextImage()));
  connect ( ImageTool->RotateLeftButton, SIGNAL(clicked()), SLOT(rotateImageLeft()));
  connect ( ImageTool->RotateRightButton, SIGNAL(clicked()), SLOT(rotateImageRight()));
  connect ( ImageTool->CloneHorizontalButton, SIGNAL(clicked()), SLOT(mirrorHorizontal()));
  connect ( ImageTool->CloneVerticalButton, SIGNAL(clicked()), SLOT(mirrorVertical()));
  connect ( ImageTool->NegativeButton, SIGNAL(clicked()), SLOT(negativeImage()));
  connect ( ImageTool->RefreshButton, SIGNAL(clicked()), SLOT(reloadImage()));

}

void CImageViewer :: setDirectory ( QString dir) {
  Directory = dir;
}

void CImageViewer :: resizeEvent(QResizeEvent *event) {
  MaxSize.setWidth(event->size().width());
  MaxSize.setHeight(event->size().height() - ImageTool->size().height());
  ImageTool->move( (this->size().width() - ImageTool->size().width())/2,
                                     this->size().height() - ImageTool->size().height());
  showImage();
}

void CImageViewer :: setFontSize() {
  this->setFont(*Font);
  ImageTool->setFontSize();
}

TypeImg CImageViewer :: parseType (QFileInfo finf) {
  QString suffix = finf.suffix().toLower();
  if ( suffix == "png" || suffix == "bpm" || suffix == "jpg" || suffix == "jpeg" ||
       suffix == "jpe" || suffix == "pjpeg" ||
       suffix == "png" || suffix == "pbm" || suffix == "pgm" || suffix == "ppm" ||
       suffix == "tiff" || suffix == "xpm" || suffix == "xbm" || suffix == "ico" )                    return NormalImage;
  if ( suffix == "mng"  || suffix == "gif" )                                                                            return AnimatedImage;
  if ( suffix == "svg" || suffix == "svgz")                                                                            return SvgImage;

  QString mit = ImageTool->pxCache->getMimeSource(finf);
  if ( mit == "image/bmp" || mit =="image/jpeg" || mit == "image/png" ||
       mit == "image/tiff" || mit == "image/x-portable-bitmap" ||
       mit == "image/x-portable-graymap" || mit == "image/x-portable-pixmap" ||
       mit == "image/x-bitmap" || mit == "image/x-pixmap"  ||
       mit == "image/vnd.microsoft.icon")                                                                           return NormalImage;
  if ( mit == "image/gif" || mit == "video/x-mng" )                                                             return AnimatedImage;
  if ( mit == "image/svg+xml" || mit == "image/svg+xml-compressed")                       return SvgImage;

  return NotImage;
}

void CImageViewer :: showImage() {
  ImageTool->ScaleSlider->blockSignals(true);
  if ( ImageType == NotImage ) {
      errorImage(tr("Warning"), tr("This file is not an image, or this format is not supported"));
      setBackNext();
      return;
  }
  if ( ErrorImage) {
    ErrorImage = false;
    setEnabledTool(true);
  }
  ImageViewport->takeWidget();
  if ( ImageType == NormalImage ) {
            QBrush brush;
            bool HScroll = false, VScroll = false;
            QSize ImgSize = Image->size();
            if ( ImageTool->ScaleViewportButton->isChecked()) {
              float k;
              int Width = ImgSize.width();
              int Height = ImgSize.height();
              if ( Width > MaxSize.width() - 6) {
                k = ((float)(MaxSize.width()-6)) / (float)Width;
                Width = (float)Width*k;
                Height = (float)Height*k;
              }
              if ( Height > MaxSize.height() - 6) {
                k = ((float)(MaxSize.height()-6)) / (float)Height;
                Width = (float)Width*k;
                Height = (float)Height*k;
              }
              ImageCanvas->setFixedSize(Width, Height);
              ImageViewport->setFixedSize(Width + 6, Height + 6);
              brush.setTextureImage(Image->scaled(Width, Height, Qt::KeepAspectRatio));
              Width = ((float)Width / (float)ImgSize.width()) * 100;
              ImageTool->RightLabel->setText(QString::number(Width) + "%");
              ImageTool->ScaleSlider->setValue(Width);
              ImageTool->ScaleSlider->setEnabled(false);
            }
            else {
              if ( Scale != 100 ) {
                ImgSize.setWidth(ImgSize.width() * Scale /100);
                ImgSize.setHeight(ImgSize.height() *Scale/100);
                brush.setTextureImage(Image->scaled(ImgSize, Qt::KeepAspectRatio));
              }
              else brush.setTextureImage(*Image);
              ImageCanvas->setFixedSize(ImgSize);

              if ( ImageCanvas->size().width() > MaxSize.width() - 6 ) {
                ImageViewport->setFixedWidth(MaxSize.width());
                VScroll = true;
              }
              else ImageViewport->setFixedWidth(ImageCanvas->size().width() + 6);
              if ( ImageCanvas->size().height() > MaxSize.height() - 6 ) {
                ImageViewport->setFixedHeight(MaxSize.height());
                HScroll = true;
              }
              else ImageViewport->setFixedHeight(ImageCanvas->size().height() + 6);

              if ( VScroll )
                if ( ImageCanvas->size().height() + ImageViewport->horizontalScrollBar()->size().height() + 7 < MaxSize.height() - 6)
                  ImageViewport->setFixedHeight(ImageCanvas->size().height() + ImageViewport->horizontalScrollBar()->size().height()+7);
              if ( HScroll )
                if ( ImageCanvas->size().width() + ImageViewport->verticalScrollBar()->size().width() + 7 < MaxSize.width() - 6)
                  ImageViewport->setFixedWidth(ImageCanvas->size().width() + ImageViewport->verticalScrollBar()->size().width() + 7);
              ImageTool->ScaleSlider->setValue(Scale);
              ImageTool->RightLabel->setText(QString::number(Scale) + "%");
              ImageTool->ScaleSlider->setEnabled(true);
            }
            QPalette pal;
            pal.setBrush(ImageCanvas->backgroundRole(), brush);
            ImageCanvas->setPalette(pal);
            ImageViewport->setWidget(ImageCanvas);
            ImageViewport->move( (this->size().width() - ImageViewport->size().width())/2,
                                                      (this->size().height() - ImageViewport->size().height() - ImageTool->size().height())/2 );
           setEnabledTool(true);
           }

  if ( ImageType == AnimatedImage ) {
            ImageViewport->setWidget(AnimateCanvas);
            bool HScroll = false, VScroll = false;
            QSize ImgSize = AnimateOriginSize;
            if ( ImageTool->ScaleViewportButton->isChecked()) {
              float k;
              int Width = ImgSize.width();
              int Height = ImgSize.height();
              if ( Width > MaxSize.width() - 6) {
                k = ((float)(MaxSize.width()-6)) / (float)Width;
                Width = (float)Width*k;
                Height = (float)Height*k;
              }
              if ( Height > MaxSize.height() - 6) {
                k = ((float)(MaxSize.height()-6)) / (float)Height;
                Width = (float)Width*k;
                Height = (float)Height*k;
              }
              AnimateCanvas->setFixedSize(Width, Height);
              ImageViewport->setFixedSize(Width + 6, Height + 6);
              Animate->setScaledSize(QSize(Width, Height));
              Width = ((float)Width / (float)ImgSize.width()) * 100;
              ImageTool->RightLabel->setText(QString::number(Width) + "%");
              ImageTool->ScaleSlider->setValue(Width);
              ImageTool->ScaleSlider->setEnabled(false);
            }
            else {
              if ( Scale != 100 ) {
                ImgSize.setWidth(ImgSize.width() * Scale /100);
                ImgSize.setHeight(ImgSize.height() *Scale/100);
                AnimateCanvas->setFixedSize(ImgSize);
                Animate->setScaledSize(ImgSize);
              }
              else {
                Animate->setScaledSize(AnimateOriginSize);
                AnimateCanvas->setFixedSize(AnimateOriginSize);
              }

              if ( AnimateCanvas->size().width() > MaxSize.width() - 6 ) {
                ImageViewport->setFixedWidth(MaxSize.width());
                VScroll = true;
              }
              else ImageViewport->setFixedWidth(AnimateCanvas->size().width() + 6);
              if ( AnimateCanvas->size().height() > MaxSize.height() - 6 ) {
                ImageViewport->setFixedHeight(MaxSize.height());
                HScroll = true;
              }
              else ImageViewport->setFixedHeight(AnimateCanvas->size().height() + 6);

              if ( VScroll )
                if ( AnimateCanvas->size().height() + ImageViewport->horizontalScrollBar()->size().height() + 7 < MaxSize.height() - 6)
                  ImageViewport->setFixedHeight(AnimateCanvas->size().height() + ImageViewport->horizontalScrollBar()->size().height()+7);
              if ( HScroll )
                if ( AnimateCanvas->size().width() + ImageViewport->verticalScrollBar()->size().width() + 7 < MaxSize.width() - 6)
                  ImageViewport->setFixedWidth(AnimateCanvas->size().width() + ImageViewport->verticalScrollBar()->size().width() + 7);
              ImageTool->ScaleSlider->setValue(Scale);
              ImageTool->RightLabel->setText(QString::number(Scale) + "%");
              ImageTool->ScaleSlider->setEnabled(true);
            }
            ImageViewport->setWidget(AnimateCanvas);
            ImageViewport->move( (this->size().width() - ImageViewport->size().width())/2,
                                                      (this->size().height() - ImageViewport->size().height() - ImageTool->size().height())/2 );

            setEnabledModifiedButtons(false);
          }

          if ( ImageType == SvgImage || ImageType == AnimatedSvg) {
            ImageViewport->setWidget(SvgCanvas);
            bool HScroll = false, VScroll = false;
            QSize ImgSize = SvgOriginSize;
            if ( ImageTool->ScaleViewportButton->isChecked()) {
              float k;
              int Width = ImgSize.width();
              int Height = ImgSize.height();
              if ( Width > MaxSize.width() - 6) {
                k = ((float)(MaxSize.width()-6)) / (float)Width;
                Width = (float)Width*k;
                Height = (float)Height*k;
              }
              if ( Height > MaxSize.height() - 6) {
                k = ((float)(MaxSize.height()-6)) / (float)Height;
                Width = (float)Width*k;
                Height = (float)Height*k;
              }
              SvgCanvas->setFixedSize(Width, Height);
              ImageViewport->setFixedSize(Width + 6, Height + 6);
              Width = ((float)Width / (float)ImgSize.width()) * 100;
              ImageTool->RightLabel->setText(QString::number(Width) + "%");
              ImageTool->ScaleSlider->setValue(Width);
              ImageTool->ScaleSlider->setEnabled(false);
            }
            else {
              if ( Scale != 100 ) {
                ImgSize.setWidth(ImgSize.width() * Scale /100);
                ImgSize.setHeight(ImgSize.height() *Scale/100);
                SvgCanvas->setFixedSize(ImgSize);

              }
              else {
                SvgCanvas->setFixedSize(SvgOriginSize);
              }

              if ( SvgCanvas->size().width() > MaxSize.width() - 6 ) {
                ImageViewport->setFixedWidth(MaxSize.width());
                VScroll = true;
              }
              else ImageViewport->setFixedWidth(SvgCanvas->size().width() + 6);
              if ( SvgCanvas->size().height() > MaxSize.height() - 6 ) {
                ImageViewport->setFixedHeight(MaxSize.height());
                HScroll = true;
              }
              else ImageViewport->setFixedHeight(SvgCanvas->size().height() + 6);

              if ( VScroll )
                if ( SvgCanvas->size().height() + ImageViewport->horizontalScrollBar()->size().height() + 7 < MaxSize.height() - 6)
                  ImageViewport->setFixedHeight(SvgCanvas->size().height() + ImageViewport->horizontalScrollBar()->size().height()+7);
              if ( HScroll )
                if ( SvgCanvas->size().width() + ImageViewport->verticalScrollBar()->size().width() + 7 < MaxSize.width() - 6)
                  ImageViewport->setFixedWidth(SvgCanvas->size().width() + ImageViewport->verticalScrollBar()->size().width() + 7);
              ImageTool->ScaleSlider->setValue(Scale);
              ImageTool->RightLabel->setText(QString::number(Scale) + "%");
              ImageTool->ScaleSlider->setEnabled(true);
            }
            ImageViewport->setWidget(SvgCanvas);
            ImageViewport->move( (this->size().width() - ImageViewport->size().width())/2,
                                                      (this->size().height() - ImageViewport->size().height() - ImageTool->size().height())/2 );

            setEnabledModifiedButtons(false);
          }

  ImageTool->ScaleSlider->blockSignals(false);
}

void CImageViewer :: changeScaleViewport() {
  showImage();
}

void CImageViewer :: changeScaleValue(int scale) {
  Scale = scale;
  showImage();
}

void CImageViewer :: showPreviousImage() {
  if ( CurrentImage > 0 ) CurrentImage--;
    else return;
  ImageType = ImageTypeList.value(CurrentImage);
  if ( ImageType == NotImage) {
      errorImage(tr("Warning"), tr("This file is not an image, or this format is not supported"));
      emit setNavigatorPath(FileInfoList.value(CurrentImage).absoluteFilePath());
      setBackNext();
      return;
    }
  if ( ImageType == NormalImage ) {
    delete Image;
    Image = new QImage(FileInfoList.value(CurrentImage).absoluteFilePath());
    if (Image->isNull()) {
      errorImage(tr("Warning"), tr("This file is not an image, or this format is not supported"));
      ImageType = NotImage;
      ImageTypeList[CurrentImage] = NotImage;
      emit setNavigatorPath(FileInfoList.value(CurrentImage).absoluteFilePath());
      setBackNext();
      return;
    }
    showImage();
    emit setNavigatorPath(FileInfoList.value(CurrentImage).absoluteFilePath());
  }

  if ( ImageType == AnimatedImage) {
    delete Animate;
    delete AnimateCanvas;
    AnimateCanvas = new QLabel();
    FileInfo = FileInfoList.value(CurrentImage);
    Animate = new QMovie(FileInfo.absoluteFilePath());
    if (!Animate->isValid()) {
      errorImage(tr("Warning"), tr("This file is not an image, or this format is not supported"));
      ImageTypeList[CurrentImage] = NotImage;
      ImageType = NotImage;
      emit setNavigatorPath(FileInfoList.value(CurrentImage).absoluteFilePath());
      setBackNext();
      return;
    }
    if ( FileInfo.suffix().toLower() == "gif" || ImageTool->pxCache->getMimeSource(FileInfo) == "image/gif") {
      if (Animate->frameCount() == 1) {
        ImageType = NormalImage;
        ImageTypeList[CurrentImage] = NormalImage;
        Image->load(FileInfo.absoluteFilePath());
        showImage();
        emit setNavigatorPath(FileInfoList.value(CurrentImage).absoluteFilePath());
        setBackNext();
        return;
      }
    }
    AnimateCanvas->setMovie(Animate);
    Animate->start();
    AnimateOriginSize = AnimateCanvas->sizeHint();
    showImage();
    emit setNavigatorPath(FileInfoList.value(CurrentImage).absoluteFilePath());
  }

  if ( ImageType == SvgImage) {
    delete SvgCanvas;
    SvgCanvas = new QSvgWidget(FileInfoList.value(CurrentImage).absoluteFilePath());
    if ( !SvgCanvas->renderer()->isValid()) {
      errorImage(tr("Warning"), tr("This file is not an image, or this format is not supported"));
      ImageType = NotImage;
      ImageTypeList[CurrentImage] = NotImage;
      emit setNavigatorPath(FileInfoList.value(CurrentImage).absoluteFilePath());
      setBackNext();
      return;
    }
    SvgOriginSize = SvgCanvas->sizeHint();
    if ( SvgCanvas->renderer()->animated()) {
      ImageType = AnimatedSvg;
      ImageTypeList[CurrentImage] = AnimatedSvg;
      connect (SvgCanvas->renderer(), SIGNAL(repaintNeeded()), SvgCanvas, SLOT(repaint()));
    }
    showImage();
    emit setNavigatorPath(FileInfoList.value(CurrentImage).absoluteFilePath());
    setBackNext();
    return;
  }

  if ( ImageType == AnimatedSvg ) {
    SvgCanvas->load(FileInfoList.value(CurrentImage).absoluteFilePath());
    SvgOriginSize = SvgCanvas->sizeHint();
    connect (SvgCanvas->renderer(), SIGNAL(repaintNeeded()), SvgCanvas, SLOT(repaint()));
    showImage();
    emit setNavigatorPath(FileInfoList.value(CurrentImage).absoluteFilePath());
  }

  setBackNext();
}

void CImageViewer :: showNextImage() {
  if ( CurrentImage < FileInfoList.count() - 1) CurrentImage++;
    else return;
  ImageType = ImageTypeList.value(CurrentImage);
  if ( ImageType == NotImage) {
      errorImage(tr("Warning"), tr("This file is not an image, or this format is not supported"));
      emit setNavigatorPath(FileInfoList.value(CurrentImage).absoluteFilePath());
      setBackNext();
      return;
    }
  if ( ImageType == NormalImage ) {
    delete Image;
    Image = new QImage(FileInfoList.value(CurrentImage).absoluteFilePath());
    if (Image->isNull()) {
      errorImage(tr("Warning"), tr("This file is not an image, or this format is not supported"));
      ImageType = NotImage;
      ImageTypeList[CurrentImage] = NotImage;
      emit setNavigatorPath(FileInfoList.value(CurrentImage).absoluteFilePath());
      setBackNext();
      return;
    }
    showImage();
    emit setNavigatorPath(FileInfoList.value(CurrentImage).absoluteFilePath());
  }

  if ( ImageType == AnimatedImage) {
    delete Animate;
    delete AnimateCanvas;
    AnimateCanvas = new QLabel();
    FileInfo = FileInfoList.value(CurrentImage);
    Animate = new QMovie(FileInfo.absoluteFilePath());
    if (!Animate->isValid()) {
      errorImage(tr("Warning"), tr("This file is not an image, or this format is not supported"));
      ImageType = NotImage;
      ImageTypeList[CurrentImage] = NotImage;
      emit setNavigatorPath(FileInfoList.value(CurrentImage).absoluteFilePath());
      setBackNext();
      return;
    }
    if ( FileInfo.suffix().toLower() == "gif" || ImageTool->pxCache->getMimeSource(FileInfo) == "image/gif") {
      if (Animate->frameCount() == 1) {
        ImageType = NormalImage;
        ImageTypeList[CurrentImage] = NormalImage;
        Image->load(FileInfo.absoluteFilePath());
        showImage();
        emit setNavigatorPath(FileInfoList.value(CurrentImage).absoluteFilePath());
        setBackNext();
        return;
      }
    }
    AnimateCanvas->setMovie(Animate);
    Animate->start();
    AnimateOriginSize = AnimateCanvas->sizeHint();
    showImage();
    emit setNavigatorPath(FileInfoList.value(CurrentImage).absoluteFilePath());
  }

  if ( ImageType == SvgImage) {
    delete SvgCanvas;
    SvgCanvas = new QSvgWidget(FileInfoList.value(CurrentImage).absoluteFilePath());
    if ( !SvgCanvas->renderer()->isValid()) {
      errorImage(tr("Warning"), tr("This file is not an image, or this format is not supported"));
      ImageType = NotImage;
      ImageTypeList[CurrentImage] = NotImage;
      emit setNavigatorPath(FileInfoList.value(CurrentImage).absoluteFilePath());
      setBackNext();
      return;
    }
    SvgOriginSize = SvgCanvas->sizeHint();
    if ( SvgCanvas->renderer()->animated()) {
      ImageType = AnimatedSvg;
      ImageTypeList[CurrentImage] = AnimatedSvg;
      connect (SvgCanvas->renderer(), SIGNAL(repaintNeeded()), SvgCanvas, SLOT(repaint()));
    }
    showImage();
    emit setNavigatorPath(FileInfoList.value(CurrentImage).absoluteFilePath());
    setBackNext();
    return;
  }

  if ( ImageType == AnimatedSvg ) {
    SvgCanvas->load(FileInfoList.value(CurrentImage).absoluteFilePath());
    SvgOriginSize = SvgCanvas->sizeHint();
    connect (SvgCanvas->renderer(), SIGNAL(repaintNeeded()), SvgCanvas, SLOT(repaint()));
    showImage();
    emit setNavigatorPath(FileInfoList.value(CurrentImage).absoluteFilePath());
  }

  setBackNext();
}


void CImageViewer :: setBackNext() {
  if ( CurrentImage > 0 ) {
    ImageTool->PreviousButton->setToolTip(FileInfoList.value(CurrentImage-1).fileName());
    ImageTool->PreviousButton->setEnabled(true);
  }
  else {
    ImageTool->PreviousButton->setToolTip(tr("No image"));
    ImageTool->PreviousButton->setEnabled(false);
  }
  if ( CurrentImage < FileInfoList.count() - 1) {
    ImageTool->NextButton->setToolTip(FileInfoList.value(CurrentImage+1).fileName());
    ImageTool->NextButton->setEnabled(true);
  }
  else {
    ImageTool->NextButton->setToolTip(tr("No image"));
    ImageTool->NextButton->setEnabled(false);
  }
}

void CImageViewer :: rotateImageLeft() {
  if (ImageType == NormalImage) {
    QImage *NewImage = new QImage ( Image->height(), Image->width(), Image->format());
    NewImage->fill(Qt::gray);
    QPainter painter;
    painter.begin(NewImage);
    painter.rotate(-90.);
    painter.drawImage(-Image->width(), 0, *Image);
    painter.end();
    delete Image;
    Image = NewImage;
    showImage();
  }
}

void CImageViewer :: rotateImageRight() {
  if ( ImageType == NormalImage) {
    QImage *NewImage = new QImage (Image->height(),  Image->width(), Image->format());
    NewImage->fill(Qt::gray);
    QPainter painter;
    painter.begin(NewImage);
    painter.rotate(90.);
    painter.drawImage(0, -Image->height(), *Image);
    painter.end();
    delete Image;
    Image = NewImage;
    showImage();
  }
}

void CImageViewer :: mirrorHorizontal() {
  if (ImageType == NormalImage) {
    QImage *BufferImage = new QImage(Image->mirrored(true, false));
    delete Image;
    Image = BufferImage;
    showImage();
  }
}


void CImageViewer :: mirrorVertical() {
  if (ImageType == NormalImage) {
    QImage *BufferImage = new QImage(Image->mirrored(false, true));
    delete Image;
    Image = BufferImage;
    showImage();
  }
}

void CImageViewer :: negativeImage() {
  if (ImageType == NormalImage) {
    Image->invertPixels(QImage::InvertRgb);
    showImage();
  }
}

void CImageViewer :: reloadImage() {
  FileInfo = FileInfoList.value(CurrentImage);
  if (!FileInfo.isSymLink()) {
    if ( FileInfo.exists() && FileInfo.isFile()) {
      ImageTypeList[CurrentImage] = parseType(FileInfo);
      FileInfoList[CurrentImage] = FileInfo;
    }
  }
  else {
    QString str = FileInfo.symLinkTarget();
    QFileInfo BufferFile(str);
    while (BufferFile.isSymLink()) {
      str = BufferFile.symLinkTarget();
      BufferFile.setFile(str);
    }
    if ( BufferFile.isFile()) {
      ImageType = parseType(BufferFile);
      if ( ImageType != NotImage) {
        ImageTypeList[CurrentImage] = ImageType;
        FileInfoList[CurrentImage] = FileInfo;
      }
      else {
        errorImage(tr("Warning"), tr("This file is not an image, or this format is not supported"));
        ImageTypeList[CurrentImage] = NotImage;
        setEnabledTool(false);
        setBackNext();
        return;
      }
    }
    else {
      if ( BufferFile.isDir()) {
        errorImage(tr("Error"), tr("This is catalog"));
        deleteCurrentImage();
        setEnabledTool(false);
        setBackNext();
        return;
      }
      if ( !BufferFile.exists()) {
        errorImage(tr("Error"), tr("This file is not exists"));
        deleteCurrentImage();
        setEnabledTool(false);
        setBackNext();
        return;
      }
    }
  }

  ImageType = ImageTypeList.value(CurrentImage);
  if ( ImageType == NormalImage ) {
    delete Image;
    Image = new QImage(FileInfoList.value(CurrentImage).absoluteFilePath());
    if ( Image->isNull()) {
      errorImage(tr("Warning"), tr("This file is not an image, or this format is not supported"));
      ImageType = NotImage;
      ImageTypeList[CurrentImage] = NotImage;
      setBackNext();
      return;
    }
    showImage();
  }

  if ( ImageType == AnimatedImage) {
    delete Animate;
    delete AnimateCanvas;
    AnimateCanvas = new QLabel();
    FileInfo = FileInfoList.value(CurrentImage);
    Animate = new QMovie(FileInfo.absoluteFilePath());
    if ( !Animate->isValid()) {
      errorImage(tr("Warning"), tr("This file is not an image, or this format is not supported"));
      ImageType = NotImage;
      ImageTypeList[CurrentImage] = NotImage;
      setBackNext();
      return;
    }
    if ( FileInfo.suffix().toLower() == "gif" || ImageTool->pxCache->getMimeSource(FileInfo) == "image/gif") {
      if (Animate->frameCount() == 1) {
        ImageType = NormalImage;
        ImageTypeList[CurrentImage] = NormalImage;
        Image->load(FileInfo.absoluteFilePath());
        showImage();
        setBackNext();
        return;
      }
    }
    AnimateCanvas->setMovie(Animate);
    Animate->start();
    AnimateOriginSize = AnimateCanvas->sizeHint();
    showImage();
  }

  if ( ImageType == SvgImage) {
    delete SvgCanvas;
    SvgCanvas = new QSvgWidget (FileInfoList.value(CurrentImage).absoluteFilePath());
    if ( !SvgCanvas->renderer()->isValid()) {
      errorImage(tr("Warning"), tr("This file is not an image, or this format is not supported"));
      ImageType = NotImage;
      ImageTypeList[CurrentImage] = NotImage;
      setBackNext();
      return;
    }
    SvgOriginSize = SvgCanvas->sizeHint();
    if ( SvgCanvas->renderer()->animated()) {
      ImageType = AnimatedSvg;
      ImageTypeList[CurrentImage] = AnimatedSvg;
      connect (SvgCanvas->renderer(), SIGNAL(repaintNeeded()), SvgCanvas, SLOT(repaint()));
    }
    showImage();
    setBackNext();
    return;
  }

  if ( ImageType == AnimatedSvg ) {
    SvgCanvas->load(FileInfoList.value(CurrentImage).absoluteFilePath());
    SvgOriginSize = SvgCanvas->sizeHint();
    connect (SvgCanvas->renderer(), SIGNAL(repaintNeeded()), SvgCanvas, SLOT(repaint()));
    showImage();
    emit setNavigatorPath(FileInfoList.value(CurrentImage).absoluteFilePath());
  }

  setBackNext();
}

void CImageViewer :: deleteCurrentImage() {
        QVector<QFileInfo>::Iterator It = FileInfoList.begin();
        QVector<TypeImg>::Iterator Itt = ImageTypeList.begin();
        for ( int i = 0; i < CurrentImage; ++i) { ++It; ++Itt; }
        FileInfoList.erase(It);
        ImageTypeList.erase(Itt);
        if ( CurrentImage == FileInfoList.count()) ImageTool->RefreshButton->setEnabled(false);
        setBackNext();
}

void CImageViewer :: errorImage(QString Icon, QString message) {
  ImageViewport->takeWidget();
  delete Image;
  Image = new QImage(ImageTool->size().width() - 20, ImageTool->size().width()/2 - 20, QImage::Format_ARGB32_Premultiplied);
  QFont font;
  font.setPixelSize(20);
  font.setWeight(QFont::Bold);
  QPainter painter;
  Image->fill(Qt::gray);
  QImage ErrorIcon;
  painter.begin(Image);
  painter.setFont(font);
  if ( Icon == "Error") painter.drawPixmap(10, 10, *(ImageTool->pxCache->getIcon(149)));
  if ( Icon == "Warning") painter.drawPixmap(10, 10, *(ImageTool->pxCache->getIcon(150)));
  QRect r (130, 130, Image->size().width() - 150, Image->size().height() - 150);
  painter.drawText(r, Qt::TextWordWrap, message);
  painter.end();
  ErrorImage = true;
  QBrush brush;
  QSize ImgSize = Image->size();
  brush.setTextureImage(*Image);
  ImageCanvas->setFixedSize(ImgSize);
  ImageViewport->setFixedWidth(ImageCanvas->size().width() + 6);
  ImageViewport->setFixedHeight(ImageCanvas->size().height() + 6);
  ImageTool->ScaleSlider->setValue(Scale);
  ImageTool->RightLabel->setText(QString::number(Scale) + "%");
  QPalette pal;
  pal.setBrush(ImageCanvas->backgroundRole(), brush);
  ImageCanvas->setPalette(pal);
  ImageViewport->setWidget(ImageCanvas);
  ImageViewport->move( (this->size().width() - ImageViewport->size().width())/2,
                                                (this->size().height() - ImageViewport->size().height() - ImageTool->size().height())/2 );

  setEnabledTool(false);
  setBackNext();
}

void CImageViewer :: setEnabledTool(bool enable) {
  ImageTool->RotateRightButton->setEnabled(enable);
  ImageTool->RotateLeftButton->setEnabled(enable);
  ImageTool->CloneHorizontalButton->setEnabled(enable);
  ImageTool->CloneVerticalButton->setEnabled(enable);
  ImageTool->NegativeButton->setEnabled(enable);
  ImageTool->ScaleViewportButton->setEnabled(enable);
  ImageTool->ScaleSlider->setEnabled(enable);
}

void CImageViewer :: setEnabledModifiedButtons(bool enable) {
    ImageTool->RotateLeftButton->setEnabled(enable);
    ImageTool->RotateRightButton->setEnabled(enable);
    ImageTool->CloneHorizontalButton->setEnabled(enable);
    ImageTool->CloneVerticalButton->setEnabled(enable);
    ImageTool->NegativeButton->setEnabled(enable);
}


void CImageViewer :: setCastomPalette(QPalette palette, QColor color) {
  epal = palette;
  changeBackgroundPalette(&epal, color);
  this->setPalette(palette);
  ImageViewport->horizontalScrollBar()->setPalette(epal);
  ImageViewport->verticalScrollBar()->setPalette(epal);
  ImageTool->setBackColor(darkerColor(color, 30));
  QColor fcolor = palette.color(QPalette::Text);
  ImageTool->setFontColor(fcolor);
}

CImageViewer :: ~CImageViewer() {
  delete ImageCanvas;
  delete AnimateCanvas;
  delete SvgCanvas;
  delete Image;
  delete Animate;
}
