#include "../../Include/SelectApplication.h"

CComboBox :: CComboBox(QWidget *wgt) : QComboBox(wgt) {}

void CComboBox :: keyPressEvent (QKeyEvent *ev) {
  if (ev->key() == Qt::Key_Return) {
    emit returnPressed();
    return;
  }
  QComboBox::keyPressEvent(ev);
}

CSelectApplicationWindow :: CSelectApplicationWindow (PixmapsCache *pxcache, QFileInfo fileinfo, QWidget *wgt) : QDialog(wgt) {
  pxCache = pxcache;
  FileInfo = fileinfo;
  FileName = FileInfo.absoluteFilePath();
  MimeType = pxCache->getMimeSource(FileInfo);
  GridLayout = new QGridLayout(this);
  Label1 = new QLabel("<b>" + tr("Please select the program from the dropdown list") + ":</b>");
  ComboBox = new CComboBox();
  ExcludeEvents = new CExcludeEvents(82, this);
  ComboBox->installEventFilter(ExcludeEvents);
  SelectButton = new QPushButton(*(pxCache->getIcon(194)), "");
  Label2 = new QLabel("<b>" + tr("Or from the list of available applications") + ":</b>");
  ListWidget = new QListWidget();
  ListWidget->setGridSize(QSize(32,32));
  ListWidget->setSpacing(5);
  CheckBox = new QCheckBox(tr("Associte file type with this program"));
  Notes = new QLabel();
  OkButton = new QPushButton(*(pxCache->getIcon(190)), tr("Ok"));
  CancelButton = new QPushButton(*(pxCache->getIcon(191)), tr("Cancel"));

  GridLayout->addWidget(Label1, 0, 0, 1, 2);
  GridLayout->addWidget(ComboBox, 1, 0);
  GridLayout->addWidget(SelectButton, 1, 1);
  GridLayout->addWidget(Label2, 2, 0, 1, 2);
  GridLayout->addWidget(ListWidget, 3, 0, 1, 2);
  GridLayout->addWidget(CheckBox, 4, 0, 1, 2);
  GridLayout->addWidget(Notes, 5, 0, 1, 2);
  GridLayout->addWidget(OkButton, 6, 0);
  GridLayout->addWidget(CancelButton, 6, 1);
  ComboBox->setFixedSize(CheckBox->sizeHint());
  CheckBox->setContentsMargins(5,5,5,5);

  ListWidget->setViewMode(QListView::ListMode);

  QVector<TemplateApps> Tapps = pxCache->getApplicationList();
  foreach ( TemplateApps app, Tapps) {
    QListWidgetItem *item = new QListWidgetItem(ListWidget);
    item->setIcon(*(app.iconApp));
    item->setText(app.nameApp);
    item->setToolTip(app.execApp);
    ListItem << item;
  }
  QList<QString>::Iterator It = pxCache->getUserApplicationList()->end();
  while ( It != pxCache->getUserApplicationList()->begin())  {
    --It;
    ComboBox->addItem( *It );
  }
  QString acceptAssociated = pxcache->findAssociation(MimeType);
  if ( !acceptAssociated.isEmpty()) {
    int z = ComboBox->findText(acceptAssociated);
    if ( z != -1 ) {
       ComboBox->insertItem(0, "");
       ComboBox->setCurrentIndex(z+1);
       CheckBox->setChecked(true);
    }
    else {
      ComboBox->insertItem(0, acceptAssociated);
      ComboBox->insertItem(0, "");
      ComboBox->setCurrentIndex(1);
      CheckBox->setChecked(true);
    }
  }
  else {
     ComboBox->insertItem(0, "");
     ComboBox->setCurrentIndex(0);
  }
  LastItem = 0;
  ComboBox->setEditable(true);

  foreach ( QListWidgetItem *item, ListItem) {
    if ( item->toolTip() == acceptAssociated) {
      item->setSelected(true);
      ListWidget->scrollToItem(item, QAbstractItemView::PositionAtTop);
      break;
    }
  }

  connect (ListWidget, SIGNAL(itemClicked(QListWidgetItem*)), SLOT(clickedItem(QListWidgetItem*)));
  connect (OkButton, SIGNAL(clicked()), SLOT(signalFromButton()));
  connect (CancelButton, SIGNAL(clicked()), SLOT(signalFromButton()));
  connect (SelectButton, SIGNAL(clicked()), SLOT(signalFromSelect()));
  connect (ComboBox, SIGNAL(currentIndexChanged(int)), SLOT(indexChanged(int)));
  connect (CheckBox, SIGNAL(clicked()), SLOT(signalFromCheck()));
  connect (ListWidget, SIGNAL(itemDoubleClicked(QListWidgetItem*)), SLOT(signalDoubleClicked(QListWidgetItem*)));

  connect (ComboBox, SIGNAL(returnPressed()), SLOT(signalFromCombo()));
}

void CSelectApplicationWindow :: setPalette(QPalette pal, QColor color) {
   QPalette bpal = pal;
   changeBackgroundPalette(&bpal, darkerColor(color, 30));
   QDialog::setPalette(bpal);
   ComboBox->setPalette(bpal);
   ListWidget->setPalette(pal);
   ListWidget->verticalScrollBar()->setPalette(bpal);
   ListWidget->horizontalScrollBar()->setPalette(bpal);
}

void CSelectApplicationWindow :: clickedItem(QListWidgetItem *item) {
  QString command = item->toolTip();
  int i;
  for ( i = 0; i < ComboBox->count(); ++i ) {
    if ( ComboBox->itemText(i) == command ) {
      ComboBox->setCurrentIndex(i);
      return;
    }
  }
  ComboBox->setCurrentIndex(0);
  ComboBox->setItemText(ComboBox->currentIndex(), command);
  CheckBox->setChecked(pxCache->findAssociation(MimeType, command));
}

void CSelectApplicationWindow :: signalFromButton() {
  QPushButton *SenderButton = qobject_cast<QPushButton*>(sender());
  if ( SenderButton == OkButton) done(1);
  if ( SenderButton == CancelButton) done(2);
}

void CSelectApplicationWindow :: signalFromCombo() {
  done(1);
}

QString CSelectApplicationWindow :: exec() {
  int i = QDialog::exec();
  if ( i == 1) {
    QString Command = ComboBox->currentText();
    if (Command != "") {
      pxCache->addUserApplicationItem(Command);
      return Command + "  \"" + FileName + "\"";
    }
    else return "";
  }
  if ( i == 2) return "";
  return "";
}

void CSelectApplicationWindow :: indexChanged(int index) {
  ComboBox->setCurrentIndex(index);
  CheckBox->setChecked(pxCache->findAssociation(MimeType, ComboBox->currentText()));
}

void CSelectApplicationWindow :: signalFromSelect() {
  QString Command = QFileDialog::getOpenFileName(0, "Bathyscaphe" + QString(" -- ") + tr("Select program") + ":", FileInfo.absolutePath(), "*");
  if ( Command != "" ) {
      int i;
      for ( i = 0; i < ComboBox->count(); ++i ) {
        if ( ComboBox->itemText(i) == Command ) {
          ComboBox->setCurrentIndex(i);
          return;
        }
      }
      ComboBox->setCurrentIndex(0);
      ComboBox->setItemText(ComboBox->currentIndex(), Command);
      CheckBox->setChecked(pxCache->findAssociation(MimeType, Command));
    }
}

void CSelectApplicationWindow :: signalFromCheck() {
    QString Command = ComboBox->currentText();
    if ( !Command.isEmpty())
       pxCache->addAssociationItem(MimeType, Command, CheckBox->isChecked());
}

void CSelectApplicationWindow :: closeEvent(QCloseEvent*) {
  done(2);
}

void CSelectApplicationWindow ::signalDoubleClicked(QListWidgetItem *item) {
  QString command = item->toolTip();
  int i;
  for ( i = 0; i < ComboBox->count(); ++i ) {
    if ( ComboBox->itemText(i) == command ) {
      ComboBox->setCurrentIndex(i);
      done(1);
      return;
    }
  }
  ComboBox->setCurrentIndex(0);
  ComboBox->setItemText(ComboBox->currentIndex(), command);
  done(1);
  return;
}
