#include "../../Include/WidgetManager.h"

WidgetManager :: WidgetManager ( char** argv) {
 SaveSettings = true;
 MainWgt = new MainWidget();
 MainLay = new QVBoxLayout();
 MainLay->setSpacing(0);
 MainLay->setMargin(0);
 split = new CSplitter();
 split->setOrientation(Qt::Horizontal);
 split->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
 rcache1 = new ReserveCache();
 rcache2 = new ReserveCache();
 rcache3 = new ReserveCache();
 rcache4 = new ReserveCache();
 cache = new PixmapsCache (argv, rcache1, rcache2, rcache3, rcache4);
 MainWgt->setWindowIcon(*(cache->getMainIcon(2)));
 MainWgt->setWindowTitle("Bathyscaphe -- " + tr("File manager"));
 settings = new CIconViewerSettings(cache);
 frame = new QFrame();
 frame->setContentsMargins(0,0,0,0);
 lay = new QVBoxLayout(frame);
 lay->setSpacing(0);
 lay->setMargin(0);

 control = new CIconViewerControl(cache);
 control->setBackColor(settings->getBackColor());
 control->setFont(settings->getFont());
 control->setFontSize();
 control->setFontColor(settings->getFontColor());
 panel = control->getPanel();

 firstIconWidget = new CIconViewerPanel (cache, rcache1, rcache2);
 firstIconWidget->IconViewer->ShowHidden = settings->getShowHidden();
 firstIconWidget->TableViewer->ShowHidden = settings->getShowHidden();
 firstIconWidget->setView(control->getView());
 firstIconWidget->setIconSize(settings->getIconSize());
 firstIconWidget->setFont(settings->getFont());
 firstIconWidget->setFontSize();
 firstIconWidget->setFlat(control->getFlat());

 statusBar = new CStatusBar(cache);
 statusBar->font = settings->getFont();
 statusBar->setFontSize();
 statusBar->setFontColor(settings->getFontColor());
 firstIconWidget->setExcludeDirsMime(statusBar->getExcludeDirsMimeRight());
 firstIconWidget->setExcludeDirsTemplate(statusBar->getExcludeDirsTemplateRight());
 settings->setFlat(control->getFlat());
 MainWgt->setBackColor(settings->getBackColor());
 MainWgt->setFontColor(settings->getFontColor());
 firstIconWidget->setColor(settings->getBackColor());
 if ( settings->getBackgroundColor()) {
   firstIconWidget->setBackColor(settings->getBackColor());
   statusBar->setBackColor(settings->getBackColor());
 }
 else {
   firstIconWidget->setBackColor(settings->getBackColor());
   firstIconWidget->setBackTexture(settings->getBackTexture());
   statusBar->setBackColor(settings->getBackColor());
 }
 firstIconWidget->setFontColor(settings->getFontColor());

 QObject::connect ( control, SIGNAL(changeView( QString, bool)), firstIconWidget, SLOT(changeView(QString, bool)));
 QObject::connect ( control, SIGNAL(changeView(QString, bool)), settings, SLOT(changeView(QString)));
 QObject::connect ( control, SIGNAL(openContentInOther(QString)), SLOT(openContentInOther(QString)));

 QObject::connect ( settings, SIGNAL(changeShowHidden(bool)), firstIconWidget, SLOT(setShowHidden(bool)));
 QObject::connect ( settings, SIGNAL(changeIconSize(QSize)), firstIconWidget, SLOT(setIconSize(QSize)));
 QObject::connect ( settings, SIGNAL(changeFontSize()), firstIconWidget, SLOT(setFontSize()));
 QObject::connect ( settings, SIGNAL(changeFontSize()), control, SLOT(setFontSize()));
 QObject::connect ( settings, SIGNAL(changeFontSize()), statusBar, SLOT(setFontSize()));

////////////////               setBackColor
 QObject::connect ( settings, SIGNAL(changeBackColor(QColor)), firstIconWidget, SLOT(setBackColor(QColor)));
 QObject::connect ( settings, SIGNAL(changeBackColor(QColor)), MainWgt, SLOT(setBackColor(QColor)));
 QObject::connect ( settings, SIGNAL(changeBackColor(QColor)), control, SLOT(setBackColor(QColor)));
 QObject::connect ( settings, SIGNAL(changeBackColor(QColor)), statusBar, SLOT(setBackColor(QColor)));
 QObject::connect ( settings, SIGNAL(changeBackTexture(QPixmap)), firstIconWidget, SLOT(setBackTexture(QPixmap)));

 ///////////////              setFontColor
 QObject::connect ( settings, SIGNAL(changeFontColor(QColor)), firstIconWidget, SLOT(setFontColor(QColor)));
 QObject::connect ( settings, SIGNAL(changeFontColor(QColor)), control, SLOT(setFontColor(QColor)));
 QObject::connect ( settings, SIGNAL(changeFontColor(QColor)), statusBar, SLOT(setFontColor(QColor)));
 QObject::connect ( settings, SIGNAL(changeFontColor(QColor)), MainWgt, SLOT(setFontColor(QColor)));

//////////                     setTheme
QObject::connect ( settings, SIGNAL(changeTheme()), firstIconWidget, SLOT (setTheme()));
QObject::connect ( settings, SIGNAL(changeTheme()), control, SLOT(refreshTheme()));

//////////                    setFlat
 QObject::connect (control, SIGNAL(changeFlat(bool)), firstIconWidget, SLOT(setFlat(bool)));
 QObject::connect (control, SIGNAL(changeFlat(bool)), settings, SLOT(setFlat(bool)));

 ////////////              connect from Monitor
  QObject::connect ( firstIconWidget->Monitor, SIGNAL(runningProcess()),
                                     settings, SLOT(startAnimate()));
  QObject::connect ( firstIconWidget->Monitor, SIGNAL(noRunningProcess()),
                                     settings, SLOT(stopAnimate()));
  QObject::connect ( settings, SIGNAL(clickedAnimateButton(CAnimateButton*)),
                                     firstIconWidget, SLOT(switchVisibleProgressW()));

 /////////////////            connect from StatusBar
  QObject::connect ( firstIconWidget, SIGNAL(hideFilter(bool)), statusBar, SLOT(hideRightFilter(bool)));
  QObject::connect ( firstIconWidget->IconViewer, SIGNAL(refreshWrite(QDir, bool)), statusBar, SLOT(refreshWrite(QDir, bool)));
  QObject::connect ( firstIconWidget->TableViewer, SIGNAL(refreshWrite(QDir, bool)), statusBar, SLOT(refreshWrite(QDir, bool)));
  QObject::connect ( firstIconWidget->IconViewer, SIGNAL(refreshWrite(QFileInfo, bool)), statusBar, SLOT(refreshWrite(QFileInfo, bool)));
  QObject::connect ( firstIconWidget->TableViewer, SIGNAL(refreshWrite(QFileInfo, bool)), statusBar, SLOT(refreshWrite(QFileInfo, bool)));
  QObject::connect ( firstIconWidget->IconViewer, SIGNAL(refreshWrite(int)), statusBar, SLOT(refreshWrite(int)));
  QObject::connect ( firstIconWidget->IconViewer, SIGNAL(refreshWrite()), statusBar, SLOT(refreshWrite()));
  QObject::connect ( firstIconWidget->TableViewer, SIGNAL(refreshWrite(int)), statusBar, SLOT(refreshWrite(int)));
  QObject::connect ( firstIconWidget->TableViewer, SIGNAL(refreshWrite()), statusBar, SLOT(refreshWrite()));
  QObject::connect ( firstIconWidget, SIGNAL(refreshWrite()), statusBar, SLOT(refreshWrite()));
  QObject::connect ( firstIconWidget, SIGNAL(refreshWrite(QDir, bool)), statusBar, SLOT(refreshWrite(QDir, bool)));
  QObject::connect ( firstIconWidget, SIGNAL(refreshWrite(QFileInfo, bool)), statusBar, SLOT(refreshWrite(QFileInfo, bool)));
  QObject::connect ( firstIconWidget, SIGNAL(refreshWrite(int)), statusBar, SLOT(refreshWrite(int)));

  QObject::connect ( firstIconWidget->IconViewer, SIGNAL(pressKey(QString)), statusBar, SLOT(setTextAbcRight(QString)));
  QObject::connect ( firstIconWidget->IconViewer, SIGNAL(delSymbolAbc()), statusBar, SLOT(delSymbolAbcRight()));
  QObject::connect ( firstIconWidget->TableViewer, SIGNAL(pressKey(QString)), statusBar, SLOT(setTextAbcRight(QString)));
  QObject::connect ( firstIconWidget->TableViewer,  SIGNAL(delSymbolAbc()), statusBar, SLOT(delSymbolAbcRight()));

  QObject::connect ( statusBar->abcFilterRight, SIGNAL(textChanged(QString)), firstIconWidget, SLOT(findAbc(QString)));
  QObject::connect ( statusBar, SIGNAL(changeTemplateFilterRight(QString)), firstIconWidget, SLOT(setTemplateFilter(QString)));
  QObject::connect ( statusBar, SIGNAL(changeMimeFilterRight(QString)), firstIconWidget, SLOT(setMimeFilter(QString)));
  QObject::connect ( statusBar, SIGNAL(changeExcludeDirsMimeRight(bool)), firstIconWidget, SLOT(setExcludeDirsMime(bool)));
  QObject::connect ( statusBar, SIGNAL(changeExcludeDirsTemplateRight(bool)),
                                     firstIconWidget, SLOT(setExcludeDirsTemplate(bool)));

  QObject::connect ( firstIconWidget, SIGNAL(hideStatusBar(bool)), SLOT(hideStatusBarF(bool)));
  QObject::connect( MainWgt, SIGNAL(newSize(QRect)), SLOT(setNewSplitterSize(QRect)));
  QObject::connect ( MainWgt, SIGNAL(newSize(QRect)), firstIconWidget, SLOT(resizeProgressW(QRect)));


  if ( panel == "one" ) {
    twopanel = false;
    split->addWidget(frame);
    lay->addWidget(firstIconWidget);
    MainLay->addWidget(control);
    MainLay->addWidget(split);
    MainLay->addWidget(statusBar);
    MainLay->addWidget(settings);
    MainWgt->setLayout(MainLay);
    firstIconWidget->changeView(control->getView(), true);
    statusBar->setOneFilter();
    control->setEnabledButtonOpen(false);
  }
  if ( panel == "tree" ) {
     twopanel = false;
     treeWidget = new CTreeViewerPanel(cache);
     treeWidget->setFlat(control->getFlat());

     QObject::connect ( treeWidget, SIGNAL(clickedElement(QTreeWidgetItem*, int)), firstIconWidget,
                                                             SLOT(changeDirectory(QTreeWidgetItem*)));
     QObject::connect (settings, SIGNAL(changeTheme()), treeWidget, SLOT(refreshTheme()));
     QObject::connect (control, SIGNAL(changeFlat(bool)), treeWidget, SLOT(setFlat(bool)));
     QObject::connect (treeWidget, SIGNAL(execOperation(QString,QString)), firstIconWidget, SLOT(fileOperation(QString, QString)));
     QObject::connect (treeWidget->Viewer, SIGNAL(execOperate(QList<QString>, QString, int)),
                                       firstIconWidget->Monitor, SLOT(execOperate(QList<QString>, QString, int)));

     MainLay->addWidget(control);
     lay->addWidget(firstIconWidget);
     lay->addWidget(statusBar);
     lay->addWidget(settings);
     split->addWidget(treeWidget);
     split->addWidget(frame);
     split->setStretchFactor(1,5);
     MainLay->addWidget(split);
     MainWgt->setLayout(MainLay);
     firstIconWidget->changeView(control->getView(), true);
     statusBar->setOneFilter();
     control->setEnabledButtonOpen(false);
  }
  if ( panel == "two" ) {
    twopanel = true;
    secondIconWidget = new CIconViewerPanelBase(cache, rcache3, rcache4);
    secondIconWidget->IconViewer->ShowHidden = settings->getShowHidden();
    secondIconWidget->TableViewer->ShowHidden = settings->getShowHidden();
    secondIconWidget->setView(control->getView());
    secondIconWidget->setIconSize(settings->getIconSize());
    secondIconWidget->setFont(settings->getFont());
    secondIconWidget->setFontSize();
    secondIconWidget->setFlat(control->getFlat());
    secondIconWidget->initShowWarningRemove(firstIconWidget->showWarningRemove);
    secondIconWidget->setColor(settings->getBackColor());
    if ( settings->getBackgroundColor()) {
      secondIconWidget->setBackColor(settings->getBackColor());
    }
    else {
      secondIconWidget->setBackColor(settings->getBackColor());
      secondIconWidget->setBackTexture(settings->getBackTexture());
    }
    secondIconWidget->setFontColor(settings->getFontColor());
    secondIconWidget->WebHistory->loadHistory(firstIconWidget->WebHistory);
    secondIconWidget->setExcludeDirsMime(statusBar->getExcludeDirsMimeLeft());
    secondIconWidget->setExcludeDirsTemplate(statusBar->getExcludeDirsTemplateLeft());


    QObject::connect ( control, SIGNAL(changeView( QString, bool)), secondIconWidget, SLOT(changeView(QString, bool)));
    QObject::connect ( settings, SIGNAL(changeShowHidden(bool)), secondIconWidget, SLOT(setShowHidden(bool)));
    QObject::connect ( settings, SIGNAL(changeIconSize(QSize)), secondIconWidget, SLOT(setIconSize(QSize)));
    QObject::connect ( settings, SIGNAL(changeFontSize()), secondIconWidget, SLOT(setFontSize()));
    QObject::connect ( secondIconWidget, SIGNAL(hideStatusBar(bool)), SLOT(hideStatusBarS(bool)));

    ////////////////               setBackColor
    QObject::connect ( settings, SIGNAL(changeBackColor(QColor)), secondIconWidget, SLOT(setBackColor(QColor)));
    QObject::connect ( settings, SIGNAL(changeBackTexture(QPixmap)), secondIconWidget, SLOT(setBackTexture(QPixmap)));

    ///////////////              setFontColor
    QObject::connect ( settings, SIGNAL(changeFontColor(QColor)), secondIconWidget, SLOT(setFontColor(QColor)));

    //////////                     setTheme
    QObject::connect ( settings, SIGNAL(changeTheme()), secondIconWidget, SLOT (setTheme()));

    //////////                    setFlat
    QObject::connect (control, SIGNAL(changeFlat(bool)), secondIconWidget, SLOT(setFlat(bool)));

    /////////////                    connect secondPanel
    QObject::connect ( secondIconWidget, SIGNAL(setShowWarningRemove()),
                                       firstIconWidget, SLOT(changeShowWarningRemove()));
    QObject::connect ( secondIconWidget, SIGNAL(execOperate(QList<QString>, QString, int)),
                                       firstIconWidget->Monitor, SLOT(execOperate(QList<QString>, QString, int)));

     ////////////                   connect from statusBar
  QObject::connect ( secondIconWidget, SIGNAL(hideFilter(bool)), statusBar, SLOT(hideLeftFilter(bool)));
  QObject::connect ( secondIconWidget->IconViewer, SIGNAL(refreshWrite(QDir, bool)), statusBar, SLOT(refreshWrite(QDir, bool)));
  QObject::connect ( secondIconWidget->TableViewer, SIGNAL(refreshWrite(QDir, bool)), statusBar, SLOT(refreshWrite(QDir, bool)));
  QObject::connect ( secondIconWidget->IconViewer, SIGNAL(refreshWrite(QFileInfo, bool)), statusBar, SLOT(refreshWrite(QFileInfo, bool)));
  QObject::connect ( secondIconWidget->TableViewer, SIGNAL(refreshWrite(QFileInfo, bool)), statusBar, SLOT(refreshWrite(QFileInfo, bool)));
  QObject::connect ( secondIconWidget->IconViewer, SIGNAL(refreshWrite(int)), statusBar, SLOT(refreshWrite(int)));
  QObject::connect ( secondIconWidget->IconViewer, SIGNAL(refreshWrite()), statusBar, SLOT(refreshWrite()));
  QObject::connect ( secondIconWidget->TableViewer, SIGNAL(refreshWrite(int)), statusBar, SLOT(refreshWrite(int)));
  QObject::connect ( secondIconWidget->TableViewer, SIGNAL(refreshWrite()), statusBar, SLOT(refreshWrite()));
  QObject::connect ( secondIconWidget, SIGNAL(refreshWrite()), statusBar, SLOT(refreshWrite()));
  QObject::connect ( secondIconWidget, SIGNAL(refreshWrite(QDir, bool)), statusBar, SLOT(refreshWrite(QDir, bool)));
  QObject::connect ( secondIconWidget, SIGNAL(refreshWrite(QFileInfo, bool)), statusBar, SLOT(refreshWrite(QFileInfo, bool)));
  QObject::connect ( secondIconWidget, SIGNAL(refreshWrite(int)), statusBar, SLOT(refreshWrite(int)));

  QObject::connect ( secondIconWidget->IconViewer, SIGNAL(pressKey(QString)), statusBar, SLOT(setTextAbcLeft(QString)));
  QObject::connect ( secondIconWidget->IconViewer, SIGNAL(delSymbolAbc()), statusBar, SLOT(delSymbolAbcLeft()));
  QObject::connect ( secondIconWidget->TableViewer, SIGNAL(pressKey(QString)), statusBar, SLOT(setTextAbcLeft(QString)));
  QObject::connect ( secondIconWidget->TableViewer,  SIGNAL(delSymbolAbc()), statusBar, SLOT(delSymbolAbcLeft()));

  QObject::connect ( statusBar->abcFilterLeft, SIGNAL(textChanged(QString)), secondIconWidget, SLOT(findAbc(QString)));
  QObject::connect ( statusBar, SIGNAL(changeTemplateFilterLeft(QString)), secondIconWidget, SLOT(setTemplateFilter(QString)));
  QObject::connect ( statusBar, SIGNAL(changeMimeFilterLeft(QString)), secondIconWidget, SLOT(setMimeFilter(QString)));
  QObject::connect ( statusBar, SIGNAL(changeExcludeDirsMimeLeft(bool)), secondIconWidget, SLOT(setExcludeDirsMime(bool)));
  QObject::connect ( statusBar, SIGNAL(changeExcludeDirsTemplateLeft(bool)),
                                     secondIconWidget, SLOT(setExcludeDirsTemplate(bool)));

  QObject::connect ( secondIconWidget->IconViewer, SIGNAL(resetSelection()), firstIconWidget, SLOT(resetSelection()));
  QObject::connect ( secondIconWidget->TableViewer, SIGNAL(resetSelection()), firstIconWidget, SLOT(resetSelection()));
  QObject::connect ( firstIconWidget->IconViewer, SIGNAL(resetSelection()), secondIconWidget, SLOT(resetSelection()));
  QObject::connect ( firstIconWidget->TableViewer, SIGNAL(resetSelection()), secondIconWidget, SLOT(resetSelection()));

    MainLay->addWidget(control);
    split->addWidget(secondIconWidget);
    split->addWidget(firstIconWidget);
    MainLay->addWidget(split);
    MainLay->addWidget(statusBar);
    MainLay->addWidget(settings);
    MainWgt->setLayout(MainLay);
    firstIconWidget->changeView(control->getView(), true);
    secondIconWidget->changeView(control->getView(), true);
    statusBar->setTwoFilter();
    control->setEnabledButtonOpen(true);
  }

  QObject::connect ( control, SIGNAL(changePanel(QString)), SLOT(changePanel(QString)));
  QObject::connect ( settings, SIGNAL(setNotSaveSettings()), SLOT(setNotSaveSettings()));
  firstIconWidget->Rect = MainWgt->rect();
}

void WidgetManager :: changePanel(QString newpanel) {
  if (panel == newpanel) return;
  if ( newpanel == "one" &&  panel == "two" ) {
    twopanel = false;
    secondIconWidget->hide();
    delete secondIconWidget;
    split->insertWidget(0, frame);
    lay->insertWidget(0, firstIconWidget);
    MainLay->insertWidget(1, split);
    MainLay->insertWidget(2, statusBar);
    MainLay->insertWidget(3, settings);
    statusBar->setOneFilter();
    if ( firstIconWidget->otherViewer()) statusBar->hide();
    else statusBar->show();
    control->setEnabledButtonOpen(false);

  }

  if ( newpanel == "tree" && panel == "two") {
     twopanel = false;
     secondIconWidget->hide();
     delete secondIconWidget;
     treeWidget = new CTreeViewerPanel(cache);
     treeWidget->setFlat(control->getFlat());
     treeWidget->ViewerControl->SaveSettings = SaveSettings;

     QObject::connect ( treeWidget, SIGNAL(clickedElement(QTreeWidgetItem*, int)), firstIconWidget,
                                                             SLOT(changeDirectory(QTreeWidgetItem*)));
     QObject::connect (settings, SIGNAL(changeTheme()), treeWidget, SLOT(refreshTheme()));
     QObject::connect (control, SIGNAL(changeFlat(bool)), treeWidget, SLOT(setFlat(bool)));
     QObject::connect (treeWidget, SIGNAL(execOperation(QString,QString)), firstIconWidget, SLOT(fileOperation(QString, QString)));
     QObject::connect (treeWidget->Viewer, SIGNAL(execOperate(QList<QString>, QString, int)),
                                       firstIconWidget->Monitor, SLOT(execOperate(QList<QString>, QString, int)));

     lay->insertWidget(0, firstIconWidget);
     lay->insertWidget(1, statusBar);
     lay->insertWidget(2, settings);
     split->insertWidget(0, treeWidget);
     split->insertWidget(1, frame);
     split->setStretchFactor(1,5);
     statusBar->setOneFilter();
     if ( firstIconWidget->otherViewer()) statusBar->hide();
     else statusBar->show();
     control->setEnabledButtonOpen(false);
  }

  if ( newpanel == "two" && panel == "one") {
    twopanel = true;
    secondIconWidget = new CIconViewerPanelBase(cache, rcache3, rcache4);
    secondIconWidget->IconViewer->ShowHidden = settings->getShowHidden();
    secondIconWidget->TableViewer->ShowHidden = settings->getShowHidden();
    secondIconWidget->setView(control->getView());
    secondIconWidget->setIconSize(settings->getIconSize());
    secondIconWidget->setFont(settings->getFont());
    secondIconWidget->setFontSize();
    secondIconWidget->setFlat(control->getFlat());
    secondIconWidget->initShowWarningRemove(firstIconWidget->showWarningRemove);
    secondIconWidget->setColor(settings->getBackColor());
    if ( settings->getBackgroundColor()) {
      secondIconWidget->setBackColor(settings->getBackColor());
    }
    else {
      secondIconWidget->setBackColor(settings->getBackColor());
      secondIconWidget->setBackTexture(settings->getBackTexture());
    }
    secondIconWidget->setFontColor(settings->getFontColor());
    secondIconWidget->WebHistory->loadHistory(firstIconWidget->WebHistory);
    secondIconWidget->setExcludeDirsMime(statusBar->getExcludeDirsMimeLeft());
    secondIconWidget->setExcludeDirsTemplate(statusBar->getExcludeDirsTemplateLeft());

    QObject::connect ( control, SIGNAL(changeView( QString, bool)), secondIconWidget, SLOT(changeView(QString, bool)));
    QObject::connect ( settings, SIGNAL(changeShowHidden(bool)), secondIconWidget, SLOT(setShowHidden(bool)));
    QObject::connect ( settings, SIGNAL(changeIconSize(QSize)), secondIconWidget, SLOT(setIconSize(QSize)));
    QObject::connect ( settings, SIGNAL(changeFontSize()), secondIconWidget, SLOT(setFontSize()));
    QObject::connect ( secondIconWidget, SIGNAL(hideStatusBar(bool)), SLOT(hideStatusBarS(bool)));

    ////////////////               setBackColor
    QObject::connect ( settings, SIGNAL(changeBackColor(QColor)), secondIconWidget, SLOT(setBackColor(QColor)));
    QObject::connect ( settings, SIGNAL(changeBackTexture(QPixmap)), secondIconWidget, SLOT(setBackTexture(QPixmap)));

    ///////////////              setFontColor
    QObject::connect ( settings, SIGNAL(changeFontColor(QColor)), secondIconWidget, SLOT(setFontColor(QColor)));

    //////////                     setTheme
    QObject::connect ( settings, SIGNAL(changeTheme()), secondIconWidget, SLOT (setTheme()));

    //////////                    setFlat
    QObject::connect (control, SIGNAL(changeFlat(bool)), secondIconWidget, SLOT(setFlat(bool)));

    /////////////                    connect secondPanel
    QObject::connect ( secondIconWidget, SIGNAL(setShowWarningRemove()),
                                       firstIconWidget, SLOT(changeShowWarningRemove()));
    QObject::connect ( secondIconWidget, SIGNAL(execOperate(QList<QString>, QString, int)),
                                       firstIconWidget->Monitor, SLOT(execOperate(QList<QString>, QString, int)));

     ////////////                   connect from statusBar
  QObject::connect ( secondIconWidget, SIGNAL(hideFilter(bool)), statusBar, SLOT(hideLeftFilter(bool)));
  QObject::connect ( secondIconWidget->IconViewer, SIGNAL(refreshWrite(QDir, bool)), statusBar, SLOT(refreshWrite(QDir, bool)));
  QObject::connect ( secondIconWidget->TableViewer, SIGNAL(refreshWrite(QDir, bool)), statusBar, SLOT(refreshWrite(QDir, bool)));
  QObject::connect ( secondIconWidget->IconViewer, SIGNAL(refreshWrite(QFileInfo, bool)), statusBar, SLOT(refreshWrite(QFileInfo, bool)));
  QObject::connect ( secondIconWidget->TableViewer, SIGNAL(refreshWrite(QFileInfo, bool)), statusBar, SLOT(refreshWrite(QFileInfo, bool)));
  QObject::connect ( secondIconWidget->IconViewer, SIGNAL(refreshWrite(int)), statusBar, SLOT(refreshWrite(int)));
  QObject::connect ( secondIconWidget->IconViewer, SIGNAL(refreshWrite()), statusBar, SLOT(refreshWrite()));
  QObject::connect ( secondIconWidget->TableViewer, SIGNAL(refreshWrite(int)), statusBar, SLOT(refreshWrite(int)));
  QObject::connect ( secondIconWidget->TableViewer, SIGNAL(refreshWrite()), statusBar, SLOT(refreshWrite()));
  QObject::connect ( secondIconWidget, SIGNAL(refreshWrite()), statusBar, SLOT(refreshWrite()));
  QObject::connect ( secondIconWidget, SIGNAL(refreshWrite(QDir, bool)), statusBar, SLOT(refreshWrite(QDir, bool)));
  QObject::connect ( secondIconWidget, SIGNAL(refreshWrite(QFileInfo, bool)), statusBar, SLOT(refreshWrite(QFileInfo, bool)));
  QObject::connect ( secondIconWidget, SIGNAL(refreshWrite(int)), statusBar, SLOT(refreshWrite(int)));

  QObject::connect ( secondIconWidget->IconViewer, SIGNAL(pressKey(QString)), statusBar, SLOT(setTextAbcLeft(QString)));
  QObject::connect ( secondIconWidget->IconViewer, SIGNAL(delSymbolAbc()), statusBar, SLOT(delSymbolAbcLeft()));
  QObject::connect ( secondIconWidget->TableViewer, SIGNAL(pressKey(QString)), statusBar, SLOT(setTextAbcLeft(QString)));
  QObject::connect ( secondIconWidget->TableViewer,  SIGNAL(delSymbolAbc()), statusBar, SLOT(delSymbolAbcLeft()));

  QObject::connect ( statusBar->abcFilterLeft, SIGNAL(textChanged(QString)), secondIconWidget, SLOT(findAbc(QString)));
  QObject::connect ( statusBar, SIGNAL(changeTemplateFilterLeft(QString)), secondIconWidget, SLOT(setTemplateFilter(QString)));
  QObject::connect ( statusBar, SIGNAL(changeMimeFilterLeft(QString)), secondIconWidget, SLOT(setMimeFilter(QString)));
  QObject::connect ( statusBar, SIGNAL(changeExcludeDirsMimeLeft(bool)), secondIconWidget, SLOT(setExcludeDirsMime(bool)));
  QObject::connect ( statusBar, SIGNAL(changeExcludeDirsTemplateLeft(bool)),
                                     secondIconWidget, SLOT(setExcludeDirsTemplate(bool)));

  QObject::connect ( secondIconWidget->IconViewer, SIGNAL(resetSelection()), firstIconWidget, SLOT(resetSelection()));
  QObject::connect ( secondIconWidget->TableViewer, SIGNAL(resetSelection()), firstIconWidget, SLOT(resetSelection()));
  QObject::connect ( firstIconWidget->IconViewer, SIGNAL(resetSelection()), secondIconWidget, SLOT(resetSelection()));
  QObject::connect ( firstIconWidget->TableViewer, SIGNAL(resetSelection()), secondIconWidget, SLOT(resetSelection()));

    split->insertWidget(0, secondIconWidget);
    split->insertWidget(1, firstIconWidget);
    MainLay->insertWidget(1, split);
    MainLay->insertWidget(2, statusBar);
    MainLay->insertWidget(3, settings);
    secondIconWidget->changeView(control->getView(), true);
    statusBar->setTwoFilter();
    if ( firstIconWidget->otherViewer() && secondIconWidget->otherViewer()) statusBar->hide();
    else statusBar->show();
    control->setEnabledButtonOpen(true);
    int W  = firstIconWidget->width();
    QList<int> intlst;
    intlst << W/2;
    intlst << W/2;
    split->setSizes(intlst);
  }

  if ( newpanel == "tree" && panel == "one") {
    twopanel = false;
     treeWidget = new CTreeViewerPanel(cache);
     treeWidget->setFlat(control->getFlat());
     treeWidget->ViewerControl->SaveSettings = SaveSettings;

     QObject::connect ( treeWidget, SIGNAL(clickedElement(QTreeWidgetItem*, int)), firstIconWidget,
                                                             SLOT(changeDirectory(QTreeWidgetItem*)));
     QObject::connect (settings, SIGNAL(changeTheme()), treeWidget, SLOT(refreshTheme()));
     QObject::connect (control, SIGNAL(changeFlat(bool)), treeWidget, SLOT(setFlat(bool)));
     QObject::connect (treeWidget, SIGNAL(execOperation(QString,QString)), firstIconWidget, SLOT(fileOperation(QString, QString)));
     QObject::connect (treeWidget->Viewer, SIGNAL(execOperate(QList<QString>, QString, int)),
                                       firstIconWidget->Monitor, SLOT(execOperate(QList<QString>, QString, int)));

     lay->insertWidget(0, firstIconWidget);
     lay->insertWidget(1, statusBar);
     lay->insertWidget(2, settings);
     split->insertWidget(0, treeWidget);
     split->insertWidget(1, frame);
     split->setStretchFactor(1,5);
     statusBar->setOneFilter();
     if ( firstIconWidget->otherViewer()) statusBar->hide();
     else statusBar->show();
     control->setEnabledButtonOpen(false);
  }

  if ( newpanel == "two" && panel == "tree") {
    twopanel = true;
    treeWidget->hide();
    delete treeWidget;
    secondIconWidget = new CIconViewerPanelBase(cache, rcache3, rcache4);
    secondIconWidget->IconViewer->ShowHidden = settings->getShowHidden();
    secondIconWidget->TableViewer->ShowHidden = settings->getShowHidden();
    secondIconWidget->setView(control->getView());
    secondIconWidget->setIconSize(settings->getIconSize());
    secondIconWidget->setFont(settings->getFont());
    secondIconWidget->setFontSize();
    secondIconWidget->setFlat(control->getFlat());
    secondIconWidget->initShowWarningRemove(firstIconWidget->showWarningRemove);
    secondIconWidget->setColor(settings->getBackColor());
    if ( settings->getBackgroundColor()) {
      secondIconWidget->setBackColor(settings->getBackColor());
    }
    else {
      secondIconWidget->setBackColor(settings->getBackColor());
      secondIconWidget->setBackTexture(settings->getBackTexture());
    }
    secondIconWidget->setFontColor(settings->getFontColor());
    secondIconWidget->WebHistory->loadHistory(firstIconWidget->WebHistory);
    secondIconWidget->setExcludeDirsMime(statusBar->getExcludeDirsMimeLeft());
    secondIconWidget->setExcludeDirsTemplate(statusBar->getExcludeDirsTemplateLeft());

    QObject::connect ( control, SIGNAL(changeView( QString, bool)), secondIconWidget, SLOT(changeView(QString, bool)));
    QObject::connect ( settings, SIGNAL(changeShowHidden(bool)), secondIconWidget, SLOT(setShowHidden(bool)));
    QObject::connect ( settings, SIGNAL(changeIconSize(QSize)), secondIconWidget, SLOT(setIconSize(QSize)));
    QObject::connect ( settings, SIGNAL(changeFontSize()), secondIconWidget, SLOT(setFontSize()));
    QObject::connect ( secondIconWidget, SIGNAL(hideStatusBar(bool)), SLOT(hideStatusBarS(bool)));

    ////////////////               setBackColor
    QObject::connect ( settings, SIGNAL(changeBackColor(QColor)), secondIconWidget, SLOT(setBackColor(QColor)));
    QObject::connect ( settings, SIGNAL(changeBackTexture(QPixmap)), secondIconWidget, SLOT(setBackTexture(QPixmap)));

    ///////////////              setFontColor
    QObject::connect ( settings, SIGNAL(changeFontColor(QColor)), secondIconWidget, SLOT(setFontColor(QColor)));

    //////////                     setTheme
    QObject::connect ( settings, SIGNAL(changeTheme()), secondIconWidget, SLOT (setTheme()));

    //////////                    setFlat
    QObject::connect (control, SIGNAL(changeFlat(bool)), secondIconWidget, SLOT(setFlat(bool)));

    /////////////                    connect secondPanel
    QObject::connect ( secondIconWidget, SIGNAL(setShowWarningRemove()),
                                       firstIconWidget, SLOT(changeShowWarningRemove()));
    QObject::connect ( secondIconWidget, SIGNAL(execOperate(QList<QString>, QString, int)),
                                       firstIconWidget->Monitor, SLOT(execOperate(QList<QString>, QString, int)));

     ////////////                   connect from statusBar
  QObject::connect ( secondIconWidget, SIGNAL(hideFilter(bool)), statusBar, SLOT(hideLeftFilter(bool)));
  QObject::connect ( secondIconWidget->IconViewer, SIGNAL(refreshWrite(QDir, bool)), statusBar, SLOT(refreshWrite(QDir, bool)));
  QObject::connect ( secondIconWidget->TableViewer, SIGNAL(refreshWrite(QDir, bool)), statusBar, SLOT(refreshWrite(QDir, bool)));
  QObject::connect ( secondIconWidget->IconViewer, SIGNAL(refreshWrite(QFileInfo, bool)), statusBar, SLOT(refreshWrite(QFileInfo, bool)));
  QObject::connect ( secondIconWidget->TableViewer, SIGNAL(refreshWrite(QFileInfo, bool)), statusBar, SLOT(refreshWrite(QFileInfo, bool)));
  QObject::connect ( secondIconWidget->IconViewer, SIGNAL(refreshWrite(int)), statusBar, SLOT(refreshWrite(int)));
  QObject::connect ( secondIconWidget->IconViewer, SIGNAL(refreshWrite()), statusBar, SLOT(refreshWrite()));
  QObject::connect ( secondIconWidget->TableViewer, SIGNAL(refreshWrite(int)), statusBar, SLOT(refreshWrite(int)));
  QObject::connect ( secondIconWidget->TableViewer, SIGNAL(refreshWrite()), statusBar, SLOT(refreshWrite()));
  QObject::connect ( secondIconWidget, SIGNAL(refreshWrite()), statusBar, SLOT(refreshWrite()));
  QObject::connect ( secondIconWidget, SIGNAL(refreshWrite(QDir, bool)), statusBar, SLOT(refreshWrite(QDir, bool)));
  QObject::connect ( secondIconWidget, SIGNAL(refreshWrite(QFileInfo, bool)), statusBar, SLOT(refreshWrite(QFileInfo, bool)));
  QObject::connect ( secondIconWidget, SIGNAL(refreshWrite(int)), statusBar, SLOT(refreshWrite(int)));

  QObject::connect ( secondIconWidget->IconViewer, SIGNAL(pressKey(QString)), statusBar, SLOT(setTextAbcLeft(QString)));
  QObject::connect ( secondIconWidget->IconViewer, SIGNAL(delSymbolAbc()), statusBar, SLOT(delSymbolAbcLeft()));
  QObject::connect ( secondIconWidget->TableViewer, SIGNAL(pressKey(QString)), statusBar, SLOT(setTextAbcLeft(QString)));
  QObject::connect ( secondIconWidget->TableViewer,  SIGNAL(delSymbolAbc()), statusBar, SLOT(delSymbolAbcLeft()));

  QObject::connect ( statusBar->abcFilterLeft, SIGNAL(textChanged(QString)), secondIconWidget, SLOT(findAbc(QString)));
  QObject::connect ( statusBar, SIGNAL(changeTemplateFilterLeft(QString)), secondIconWidget, SLOT(setTemplateFilter(QString)));
  QObject::connect ( statusBar, SIGNAL(changeMimeFilterLeft(QString)), secondIconWidget, SLOT(setMimeFilter(QString)));
  QObject::connect ( statusBar, SIGNAL(changeExcludeDirsMimeLeft(bool)), secondIconWidget, SLOT(setExcludeDirsMime(bool)));
  QObject::connect ( statusBar, SIGNAL(changeExcludeDirsTemplateLeft(bool)),
                                     secondIconWidget, SLOT(setExcludeDirsTemplate(bool)));

  QObject::connect ( secondIconWidget->IconViewer, SIGNAL(resetSelection()), firstIconWidget, SLOT(resetSelection()));
  QObject::connect ( secondIconWidget->TableViewer, SIGNAL(resetSelection()), firstIconWidget, SLOT(resetSelection()));
  QObject::connect ( firstIconWidget->IconViewer, SIGNAL(resetSelection()), secondIconWidget, SLOT(resetSelection()));
  QObject::connect ( firstIconWidget->TableViewer, SIGNAL(resetSelection()), secondIconWidget, SLOT(resetSelection()));

    split->insertWidget(0, secondIconWidget);
    split->insertWidget(1, firstIconWidget);
    MainLay->insertWidget(1, split);
    MainLay->insertWidget(2, statusBar);
    MainLay->insertWidget(3, settings);
    secondIconWidget->changeView(control->getView(), true);
    statusBar->setTwoFilter();
    if ( firstIconWidget->otherViewer() && secondIconWidget->otherViewer()) statusBar->hide();
    else statusBar->show();
    control->setEnabledButtonOpen(true);
    QList<int> intlst = split->sizes();
    intlst[0] = (intlst[0] + intlst[1]) /2;
    intlst[1] = intlst[0];
    split->setSizes(intlst);
  }

  if ( newpanel == "one" && panel == "tree") {
    twopanel = false;
    treeWidget->hide();
    delete treeWidget;
    split->insertWidget(0, frame);
    lay->insertWidget(0, firstIconWidget);
    MainLay->insertWidget(1, split);
    MainLay->insertWidget(2, statusBar);
    MainLay->insertWidget(3, settings);
    statusBar->setOneFilter();
    if ( firstIconWidget->otherViewer()) statusBar->hide();
    else statusBar->show();
    control->setEnabledButtonOpen(false);
  }
  panel = newpanel;
}

void WidgetManager :: hideStatusBarF(bool hd) {
  if ( hd ) {
    if ( twopanel == false) {
      statusBar->hide();
    }
    else {
      if ( secondIconWidget->otherViewer()) {
        statusBar->hide();
      }
    }
  }
  else statusBar->show();
}

void WidgetManager :: hideStatusBarS(bool hd) {
  if ( hd ) {
    if ( twopanel == false) {
      statusBar->hide();
    }
    else {
      if ( firstIconWidget->otherViewer()) {
        statusBar->hide();
      }
    }
  }
  else statusBar->show();
}

void WidgetManager :: setNotSaveSettings() {
  SaveSettings = false;
  firstIconWidget->SaveSettings = false;
  control->SaveSettings = false;
  cache->SaveSettings = false;
  MainWgt->SaveSettings = false;
  if ( panel == "tree" ) treeWidget->ViewerControl->SaveSettings = false;
}

void WidgetManager :: openContentInOther(QString task) {
  if ( task == "lefttoright" ) {
    QString location = secondIconWidget->Navigator->getLocation();
    QString targ = firstIconWidget->Navigator->getLocation();
    if ( location == targ ) return;
    firstIconWidget->changeLocation(location);
    return;
  }
  if ( task == "righttoleft" ) {
    QString location = firstIconWidget->Navigator->getLocation();
    QString targ = secondIconWidget->Navigator->getLocation();
    if ( location == targ ) return;
    secondIconWidget->changeLocation(location);
    return;
  }
  if ( task == "change" ) {
    QString leftLocation = secondIconWidget->Navigator->getLocation();
    QString rightLocation = firstIconWidget->Navigator->getLocation();
    if ( leftLocation == rightLocation) return;
    firstIconWidget->changeLocation(leftLocation);
    secondIconWidget->changeLocation(rightLocation);
  }
}

void WidgetManager :: setNewSplitterSize(QRect rect) {
  if ( panel == "two" ) {
    QList<int> intlst;
    intlst << rect.width()/2 << rect.width()/2;
    split->setSizes(intlst);
  }
}

WidgetManager :: ~WidgetManager() {
  delete control;
  delete firstIconWidget;
  if ( panel == "two") delete secondIconWidget;
  if (panel == "tree") delete treeWidget;
  delete statusBar;
  delete settings;
  delete cache;
  delete MainWgt;
}
