#include "../../Include/WindowProperites.h"
#include "../../Include/Stat.h"


//-------------------------------------                           Constructosr                              ---------------------------------------------------------------------------------

CWindowProperites :: CWindowProperites ( PixmapsCache *pxcache, QFileInfo fileinfo,
                                                                              QPalette pp, QColor color, QFont font,
                                                                              QWidget *wgt ) : QDialog(wgt) {
  pxCache = pxcache;
  FileInfo = fileinfo;
  FileTemplate = pxCache->fullTemplate(FileInfo);
  lst << tr("byte(s)") << tr("Kb") << tr("Mb") << tr("Gb") << tr("Tb");
  targetEdit = NULL;
  setAllPermission = NULL;
  labelNotChangePermissions = NULL;
  setWindowTitle(tr("Window properites") + " - Bathyscaphe");
  setWindowIcon(*(pxCache->getMainIcon(3)));
  createDefaultWidget();
  resize(600, 600);

  pal = pp;
  epal = pal;
  epal.setColor(QPalette::Background, color);
  epal.setColor(QPalette::Base, color);
  epal.setColor(QPalette::Window, color);
  setPalette(epal);
  epal.setColor(QPalette::Background, color.darker(150));
  epal.setColor(QPalette::Base, color.darker(150));
  epal.setColor(QPalette::Window, color.darker(150));
  setFont(font);
  Font = font;
  BoldFont = font;
  BoldFont.setBold(true);
  timer = new QTimer();
  threadGS = new CThreadGS();
  ExcludeEvents = new CExcludeEvents(82, this);
  connect ( timer, SIGNAL(timeout()), SLOT(refreshCountAndSize()));
  connect ( threadGS, SIGNAL(signalOfAllCounts()),
                                        SLOT(setOfAllCounts()));

  bitMask.resize(7);
  if ( !FileInfo.isSymLink() && !FileInfo.exists()) createIfNoObject();
  else   if ( FileInfo.isSymLink() && FileInfo.isDir()) createIfSymLinkAndDir();
               else if ( !FileInfo.isSymLink() && FileInfo.isDir()) createIfNotSymLinkAndDir();
                 else if ( FileInfo.isSymLink()) createIfSymLink();
                     else createIfOther();
}

CWindowProperites :: CWindowProperites ( PixmapsCache *pxcache, QList<QFileInfo> fileinfolist,
                                                                              QPalette pp, QColor color, QFont font,
                                                                              QWidget *wgt ) : QDialog(wgt) {
  pxCache = pxcache;
  FileInfoList = fileinfolist;
  setAllPermission = NULL;
  setWindowIcon(*(pxCache->getMainIcon(3)));
  lst << tr("byte(s)") << tr("Kb") << tr("Mb") << tr("Gb") << tr("Tb");

  pal = pp;
  epal = pal;
  epal.setColor(QPalette::Background, color);
  epal.setColor(QPalette::Base, color);
  epal.setColor(QPalette::Window, color);
  setPalette(epal);
  epal.setColor(QPalette::Background, color.darker(150));
  epal.setColor(QPalette::Base, color.darker(150));
  epal.setColor(QPalette::Window, color.darker(150));
  setFont(font);
  Font = font;
  BoldFont = font;
  BoldFont.setBold(true);
  timer = new QTimer();
  threadGS = new CThreadGS();
  ExcludeEvents = new CExcludeEvents(82, this);
  connect ( timer, SIGNAL(timeout()), SLOT(refreshCountAndSize()));
  connect ( threadGS, SIGNAL(signalOfAllCounts()),
                                        SLOT(setOfAllCountsFL()));
  createDefaultWidgetFL();

}

//--------------------------------------------------------------------------------------------------------------------------------------------------------------------
// ------------------------------------                         Function for one element                              ------------------------------------------------
//---------------------------------------------------------------------------------------------------------------------------------------------------------------------

void CWindowProperites :: createDefaultWidget() {
    mainLayout = new QVBoxLayout(this);
    labelHeaderInfo = new QLabel("<h4 align = center>" + tr("Properites object") + "</h4>");
    labelHeaderInfo->setPalette(epal);
    infoBox = new QGroupBox();
    infoHLayout  = new QHBoxLayout();
    permissionHLayout = new QHBoxLayout();
    labelHeaderPermissions = new QLabel("<h4 align = center>" + tr("Permissions") + "</h4>");
    permissionBox = new QGroupBox();
    moreBox = new QGroupBox();
    idBox = new QGroupBox();
    labelHeaderOwner = new QLabel("<h4 align = center>" + tr("Owner and group") + "</h4>");
    infoGridLayout = new QGridLayout();
    permissionGridLayout = new QGridLayout();
    moreGridLayout = new QGridLayout();
    idGridLayout = new QGridLayout();
    iconButton = new QPushButton();
    iconButton->setFlat(true);
    buttonsBox = new QGroupBox();
    buttonsHLayout = new QHBoxLayout();
    okButton = new QPushButton(*(pxCache->getIcon(190)), tr("Ok"));
    okButton->setEnabled(false);
    cancelButton = new QPushButton(*(pxCache->getIcon(191)), tr("Cancel"));
    applyButton = new QPushButton(*(pxCache->getIcon(196)), tr("Apply"));
    applyButton->setEnabled(false);
    refreshButton = new QPushButton(*(pxCache->getIcon(161)), tr("Refresh"));
    buttonsHLayout->addWidget(okButton);
    buttonsHLayout->addWidget(applyButton);
    buttonsHLayout->addWidget(refreshButton);
    buttonsHLayout->addWidget(cancelButton);
    buttonsBox->setLayout(buttonsHLayout);
    infoBox->setLayout(infoHLayout);
    infoHLayout->addWidget(iconButton);
    infoHLayout->addLayout(infoGridLayout);
    permissionHLayout->addWidget(permissionBox, 1);
    permissionHLayout->addWidget(moreBox);
    permissionBox->setLayout(permissionGridLayout);
    moreBox->setLayout(moreGridLayout);
    idBox->setLayout(idGridLayout);
    quint64 totalsize, freesize;
    if ( FileInfo.isSymLink())  freeSpaceFrom (FileTemplate.Path, &totalsize, &freesize);
      else freeSpaceFrom (FileTemplate.FullName, &totalsize, &freesize);
    quint64 blocksize = totalsize - freesize;
    quint64 proc = (qreal)blocksize/(qreal)totalsize*100;
    QString totalsizeS, freesizeS, blocksizeS;
    int i = 0;
    for ( i = 0; i < 5; ++i)  if ( totalsize > 1000 ) totalsize = totalsize / 1000; else break;
    totalsizeS = QString::number(totalsize) + " " + lst[i];
    for ( i = 0; i < 5; ++i)  if ( freesize > 1000 ) freesize = freesize / 1000; else break;
    freesizeS = QString::number(freesize) + " " + lst[i];
    for ( i = 0; i < 5; ++i)  if ( blocksize > 1000 ) blocksize = blocksize / 1000; else break;
    blocksizeS = QString::number(blocksize) + " " + lst[i];

    QString mp = pxCache->locateMountPoints(FileTemplate.FullName);
    if ( mp == "" ) {
      pxCache->fullMountPoints();
      mp = pxCache->locateMountPoints(FileTemplate.FullName);
    }
    labelHeaderSpace = new QLabel("<h4 align = center>" + tr("Free space") + "</h4>");
    labelFreeSpaceH = new QLabel(tr("Free") + " " + freesizeS + " " + tr("from") + " " + totalsizeS + " (" + tr("busy") + " " + QString::number(proc) + "% " + tr("on") + " " + mp + " )");
    freeSpace = new QProgressBar();
    freeSpace->setRange(0, 100);
    freeSpace->setValue(proc);

    mainLayout->addWidget(labelHeaderInfo);
    labelHeaderInfo->setPalette(epal);
    mainLayout->addWidget(infoBox);
    mainLayout->addWidget(labelHeaderPermissions);
    mainLayout->addLayout(permissionHLayout);
    mainLayout->addWidget(labelHeaderOwner);
    mainLayout->addWidget(idBox);
    mainLayout->addWidget(labelHeaderSpace);
    mainLayout->addWidget(labelFreeSpaceH);
    mainLayout->addWidget(freeSpace);
    mainLayout->addWidget(buttonsBox);

    labelTypeH = new QLabel("<b>" + tr("Type:") + "</b>");
    labelType = new QLabel();
    labelPathH = new QLabel("<b>" + tr("Path:") + "</b>");
    labelPath = new QLabel();
    labelSizeH = new QLabel("<b>" + tr("Size:") + "</b>");
    labelSize = new QLabel();
    labelCreatedH = new QLabel("<b>" + tr("Created:") + "<b>");
    labelCreated = new QLabel();
    labelModifiedH = new QLabel("<b>" + tr("Modified:") + "</b>");
    labelModified = new QLabel();
    labelLastReadH = new QLabel("<b>" + tr("Last read:") + "</b>");
    labelLastRead = new QLabel();

    connect ( refreshButton, SIGNAL(clicked()), SLOT(slotRefresh()));
    connect ( cancelButton, SIGNAL(clicked()), SLOT(slotCancel()));
    connect ( okButton, SIGNAL(clicked()), SLOT(applyChanges()));
    connect ( applyButton, SIGNAL(clicked()), SLOT(applyChanges()));
  }

void CWindowProperites :: createIfSymLinkAndDir() {
    labelNameH = new QLabel("<b>" + tr("Name:") + "</b>");
    nameEdit = new QLineEdit();
    nameEdit->installEventFilter(ExcludeEvents);
    nameEdit->setPalette(pal);
    labelTargetH = new QLabel("<b>" + tr("Target on:") + "</b>");
    targetEdit = new QLineEdit();
    targetEdit->installEventFilter(ExcludeEvents);
    targetEdit->setPalette(pal);
    labelMimeTypeH = new QLabel("<b>MIME:</b>");
    labelMimeType = new QLabel();

    infoGridLayout->addWidget(labelNameH, 0, 0);
    infoGridLayout->addWidget(nameEdit, 0, 1);
    infoGridLayout->addWidget(labelTypeH, 1, 0);
    infoGridLayout->addWidget(labelType, 1, 1);
    infoGridLayout->addWidget(labelPathH, 2, 0);
    infoGridLayout->addWidget(labelPath, 2, 1);
    infoGridLayout->addWidget(labelTargetH, 3, 0);
    infoGridLayout->addWidget(targetEdit, 3, 1);
    infoGridLayout->addWidget(labelMimeTypeH, 4, 0);
    infoGridLayout->addWidget(labelMimeType, 4, 1);
    infoGridLayout->addWidget(labelCreatedH, 5, 0);
    infoGridLayout->addWidget(labelCreated, 5, 1);
    infoGridLayout->addWidget(labelModifiedH, 6, 0);
    infoGridLayout->addWidget(labelModified, 6, 1);
    infoGridLayout->addWidget(labelLastReadH, 7, 0);
    infoGridLayout->addWidget(labelLastRead, 7, 1);
    infoGridLayout->addWidget(labelSizeH, 8, 0);
    infoGridLayout->addWidget(labelSize, 8, 1);

    nameEdit->setText(FileTemplate.FileName);
    nameEdit->setPalette(pal);
    labelType->setText(FileTemplate.Type);
    labelPath->setText(FileTemplate.Path);
    targetEdit->setText(FileTemplate.TargetLink);
    connect ( nameEdit, SIGNAL(textChanged(QString)), SLOT(changedFields()));
    connect ( targetEdit, SIGNAL(textChanged(QString)), SLOT(changedFields()));
    labelMimeType->setText(FileTemplate.Mime);
    labelCreated->setText(FileTemplate.Created);
    labelModified->setText(FileTemplate.LastModified);
    labelLastRead->setText(FileTemplate.LastRead);
    createPermissionsTable();

    iconButton->setIcon(FileTemplate.Icon);
    iconButton->setIconSize(FileTemplate.Icon.size());
    infoHLayout->itemAt(0)->setAlignment(Qt::AlignTop);

    sizeData = countData = countFolders = countFiles = 0;
    QStringList list;
    list << FileTemplate.FullName;
    threadGS->setData(list, &sizeData, &countData, NULL, NULL);
    threadGS->getSize(list);
    sd = sizeData;
    int i;
    for ( i = 0; i < 4; ++i ) if ( sd >1000 ) sd = sd / 1000; else break;
    labelSize->setText(FileTemplate.Size + ", " + QString::number(sd) + " " + lst[i]);
}

void CWindowProperites :: createIfNotSymLinkAndDir() {
    labelNameH = new QLabel("<b>" + tr("Name:") + "</b>");
    nameEdit = new QLineEdit();
    nameEdit->installEventFilter(ExcludeEvents);
    nameEdit->setPalette(pal);
    labelDescrFromContentsH = new QLabel("<b>" + tr("Contents:") + "</b>");
    labelDescrFromContents = new QLabel();
    labelMimeTypeH = new QLabel("<b>MIME:</b>");
    labelMimeType = new QLabel();
    recountButtonsHLayout = new QHBoxLayout();
    recountButton = new QPushButton(tr("Re-count"));
    stopRecountButton = new QPushButton(tr("Stop"));

    infoGridLayout->addWidget(labelNameH, 0, 0);
    infoGridLayout->addWidget(nameEdit, 0, 1);
    infoGridLayout->addWidget(labelTypeH, 1, 0);
    infoGridLayout->addWidget(labelType, 1, 1);
    infoGridLayout->addWidget(labelPathH, 2, 0);
    infoGridLayout->addWidget(labelPath, 2, 1);
    infoGridLayout->addWidget(labelMimeTypeH, 3, 0);
    infoGridLayout->addWidget(labelMimeType, 3, 1);
    infoGridLayout->addWidget(labelCreatedH, 4, 0);
    infoGridLayout->addWidget(labelCreated, 4, 1);
    infoGridLayout->addWidget(labelModifiedH, 5, 0);
    infoGridLayout->addWidget(labelModified, 5, 1);
    infoGridLayout->addWidget(labelLastReadH, 6, 0);
    infoGridLayout->addWidget(labelLastRead, 6, 1);
    infoGridLayout->addWidget(labelSizeH, 7, 0);
    infoGridLayout->addWidget(labelSize, 7, 1);
    infoGridLayout->addWidget(labelDescrFromContentsH, 8, 0);
    infoGridLayout->addWidget(labelDescrFromContents, 8, 1);
    infoGridLayout->addLayout(recountButtonsHLayout, 9, 1);
    recountButtonsHLayout->addWidget(recountButton);
    recountButtonsHLayout->addWidget(stopRecountButton);
    recountButton->setEnabled(false);
    stopRecountButton->setEnabled(true);

    nameEdit->setText(FileTemplate.FileName);
    connect ( nameEdit, SIGNAL(textChanged(QString)), SLOT(changedFields()));
    nameEdit->setPalette(pal);
    labelType->setText(FileTemplate.Type);
    labelPath->setText(FileTemplate.Path);
    labelDescrFromContents->setText(FileTemplate.DescrFromContents);
    labelMimeType->setText(FileTemplate.Mime);
    labelSize->setText(FileTemplate.Size);
    labelCreated->setText(FileTemplate.Created);
    labelModified->setText(FileTemplate.LastModified);
    labelLastRead->setText(FileTemplate.LastRead);
    createPermissionsTable();

    iconButton->setIcon(FileTemplate.Icon);
    iconButton->setIconSize(FileTemplate.Icon.size());
    infoHLayout->itemAt(0)->setAlignment(Qt::AlignTop);

    connect (recountButton, SIGNAL(clicked()), SLOT(recountSize()));
    connect (stopRecountButton, SIGNAL(clicked()), SLOT(setOfAllCounts()));
    countProcessForOne();
}

void CWindowProperites :: createIfSymLink() {
    labelNameH = new QLabel("<b>" + tr("Name:") + "</b>");
    nameEdit = new QLineEdit();
    nameEdit->installEventFilter(ExcludeEvents);
    nameEdit->setPalette(pal);
    labelDescriptionH = new QLabel("<b>" + tr("Description(from extended):") + "</b>");
    labelDescription = new QLabel();
    labelDescrFromContentsH = new QLabel("<b>" + tr("Contents:") + "</b>");
    labelDescrFromContents = new QLabel();
    labelTargetH = new QLabel("<b>" + tr("Target on:") + "</b>");
    targetEdit = new QLineEdit();
    targetEdit->installEventFilter(ExcludeEvents);
    targetEdit->setPalette(pal);
    labelMimeTypeH = new QLabel("<b>MIME:</b>");
    labelMimeType = new QLabel();
    infoBox->setFont(Font);

    infoGridLayout->addWidget(labelNameH, 0, 0);
    infoGridLayout->addWidget(nameEdit, 0, 1);
    infoGridLayout->addWidget(labelTypeH, 1, 0);
    infoGridLayout->addWidget(labelType, 1, 1);
    infoGridLayout->addWidget(labelPathH, 2, 0);
    infoGridLayout->addWidget(labelPath, 2, 1);
    infoGridLayout->addWidget(labelTargetH, 3, 0);
    infoGridLayout->addWidget(targetEdit, 3, 1);
    infoGridLayout->addWidget(labelDescriptionH, 4, 0);
    infoGridLayout->addWidget(labelDescription, 4, 1);
    infoGridLayout->addWidget(labelDescrFromContentsH, 5, 0);
    infoGridLayout->addWidget(labelDescrFromContents, 5, 1);
    infoGridLayout->addWidget(labelMimeTypeH, 6, 0);
    infoGridLayout->addWidget(labelMimeType, 6, 1);
    infoGridLayout->addWidget(labelSizeH, 7, 0);
    infoGridLayout->addWidget(labelSize, 7, 1);
    infoGridLayout->addWidget(labelCreatedH, 8, 0);
    infoGridLayout->addWidget(labelCreated, 8, 1);
    infoGridLayout->addWidget(labelModifiedH, 9, 0);
    infoGridLayout->addWidget(labelModified, 9, 1);
    infoGridLayout->addWidget(labelLastReadH, 10, 0);
    infoGridLayout->addWidget(labelLastRead, 10, 1);

    nameEdit->setText(FileTemplate.FileName);
    nameEdit->setPalette(pal);
    labelType->setText(FileTemplate.Type);
    labelPath->setText(FileTemplate.Path);
    targetEdit->setText(FileTemplate.TargetLink);
    connect ( nameEdit, SIGNAL(textChanged(QString)), SLOT(changedFields()));
    connect ( targetEdit, SIGNAL(textChanged(QString)), SLOT(changedFields()));
    labelDescription->setText(FileTemplate.Descr);
    labelDescrFromContents->setText(FileTemplate.DescrFromContents);
    labelMimeType->setText(FileTemplate.Mime);
    labelSize->setText(FileTemplate.Size);
    labelCreated->setText(FileTemplate.Created);
    labelModified->setText(FileTemplate.LastModified);
    labelLastRead->setText(FileTemplate.LastRead);
    createPermissionsTable();

    iconButton->setIcon(FileTemplate.Icon);
    iconButton->setIconSize(FileTemplate.Icon.size());
    infoHLayout->itemAt(0)->setAlignment(Qt::AlignTop);
}

void CWindowProperites :: createIfOther() {
    labelNameH = new QLabel("<b>" + tr("Name:") + "</b>");
    nameEdit = new QLineEdit();
    nameEdit->installEventFilter(ExcludeEvents);
    nameEdit->setPalette(pal);
    labelDescriptionH = new QLabel("<b>" + tr("Description(from extended):") + "</b>");
    labelDescription = new QLabel();
    labelDescrFromContentsH = new QLabel("<b>" + tr("Contents:") + "</b>");
    labelDescrFromContents = new QLabel();
    labelMimeTypeH = new QLabel("<b>MIME:</b>");
    labelMimeType = new QLabel();
    infoBox->setFont(Font);

    infoGridLayout->addWidget(labelNameH, 0, 0);
    infoGridLayout->addWidget(nameEdit, 0, 1);
    infoGridLayout->addWidget(labelTypeH, 1, 0);
    infoGridLayout->addWidget(labelType, 1, 1);
    infoGridLayout->addWidget(labelPathH, 2, 0);
    infoGridLayout->addWidget(labelPath, 2, 1);
    infoGridLayout->addWidget(labelDescriptionH, 3, 0);
    infoGridLayout->addWidget(labelDescription, 3, 1);
    infoGridLayout->addWidget(labelDescrFromContentsH, 4, 0);
    infoGridLayout->addWidget(labelDescrFromContents, 4, 1);
    infoGridLayout->addWidget(labelMimeTypeH, 5, 0);
    infoGridLayout->addWidget(labelMimeType, 5, 1);
    infoGridLayout->addWidget(labelSizeH, 6, 0);
    infoGridLayout->addWidget(labelSize, 6, 1);
    infoGridLayout->addWidget(labelCreatedH, 7, 0);
    infoGridLayout->addWidget(labelCreated, 7, 1);
    infoGridLayout->addWidget(labelModifiedH, 8, 0);
    infoGridLayout->addWidget(labelModified, 8, 1);
    infoGridLayout->addWidget(labelLastReadH, 9, 0);
    infoGridLayout->addWidget(labelLastRead, 9, 1);

    nameEdit->setText(FileTemplate.FileName);
    connect ( nameEdit, SIGNAL(textChanged(QString)), SLOT(changedFields()));
    nameEdit->setPalette(pal);
    labelType->setText(FileTemplate.Type);
    labelPath->setText(FileTemplate.Path);
    labelDescription->setText(FileTemplate.Descr);
    labelDescrFromContents->setText(FileTemplate.DescrFromContents);
    labelMimeType->setText(FileTemplate.Mime);
    labelSize->setText(FileTemplate.Size);
    labelCreated->setText(FileTemplate.Created);
    labelModified->setText(FileTemplate.LastModified);
    labelLastRead->setText(FileTemplate.LastRead);
    createPermissionsTable();

    iconButton->setIcon(FileTemplate.Icon);
    iconButton->setIconSize(FileTemplate.Icon.size());
    infoHLayout->itemAt(0)->setAlignment(Qt::AlignTop);
}

void CWindowProperites :: createPermissionsTable() {
    permissionCheck.clear();
    permissions ( FileTemplate.FullName, &Permissions, &ID_Owner, &ID_Group, &bitMask);
    for (int i = 0; i < 12; ++i ) {
      QCheckBox *chbox = new QCheckBox();
      permissionCheck << chbox;
      chbox->setChecked(Permissions.testBit(i));
      connect(chbox, SIGNAL(clicked()), SLOT(changedFields()));
    }
    permissionGridLayout->addWidget(new QLabel(tr("Read")), 0, 1);
    permissionGridLayout->addWidget(new QLabel(tr("Write")), 0, 2);
    permissionGridLayout->addWidget(new QLabel(tr("Execute")), 0, 3);
    permissionGridLayout->addWidget(new QLabel(tr("Owner")),1, 0);
    permissionGridLayout->addWidget(new QLabel(tr("Group")), 2, 0);
    permissionGridLayout->addWidget(new QLabel(tr("Other")), 3, 0);
    int z = 0;
    for ( int i = 3; i > 0; --i)
      for ( int n = 3; n > 0; --n ) {
        permissionGridLayout->addWidget(permissionCheck[z], i, n );
        ++z;
      }

    moreGridLayout->addWidget(new QLabel("<center>" + tr("More") + "</center>"), 0, 0, 1, 2);
    moreGridLayout->addWidget(new QLabel("SUID"), 1, 0);
    moreGridLayout->addWidget(new QLabel("SGID"), 2, 0);
    moreGridLayout->addWidget(new QLabel("Sticky"), 3, 0);
    moreGridLayout->addWidget(permissionCheck[11], 1, 1);
    moreGridLayout->addWidget(permissionCheck[10], 2, 1);
    moreGridLayout->addWidget(permissionCheck[9], 3, 1);

    idGridLayout->addWidget(new QLabel("<b>" + tr("Owner of file:") + "</b>"), 0, 0);
    idGridLayout->addWidget(new QLabel("<b>" + tr("Group owner of file:") + "</b>"), 1, 0);
    Uids = groupName(ID_Owner);
    Gids = groupName(ID_Group);
    idOwnerEdit = new QLineEdit(Uids);
    idGroupEdit = new QLineEdit(Gids);
    idOwnerEdit->installEventFilter(ExcludeEvents);
    idGroupEdit->installEventFilter(ExcludeEvents);
    idGridLayout->addWidget(idOwnerEdit, 0, 1);
    idGridLayout->addWidget(idGroupEdit, 1, 1);
    idOwnerEdit->setPalette(pal);
    idGroupEdit->setPalette(pal);

    if ( FileInfo.isSymLink()) {
      labelNotChangePermissions = new QLabel();
      mainLayout->insertWidget(4, labelNotChangePermissions);
      labelNotChangePermissions->setText(tr("Warning: this file is a symbolic link, and its right to match") + "<br>" + tr("with the rights of the target file"));
      for (int i = 0; i < 12; ++i) permissionCheck[i]->setEnabled(false);
    }
    else {
      if ( (int)pxCache->getOwner() != ID_Owner && (int)pxCache->getOwner() != 0) {
        labelNotChangePermissions = new QLabel();
        mainLayout->insertWidget(4, labelNotChangePermissions);
        if ( FileInfo.isFile())
          labelNotChangePermissions->setText(tr("You are not the owner of the file,") + "<br>" + tr("and can not change the law"));
        if ( FileInfo.isDir())
          labelNotChangePermissions->setText(tr("You are not the owner of the directory,") + "<br>" + tr("and can not change the law"));
        for (int i = 0; i < 12; ++i) permissionCheck[i]->setEnabled(false);
      }
      else {
        if ( FileInfo.isDir() &&!FileInfo.isSymLink()) {
          setAllPermission = new QCheckBox(tr("Apply to all attached files and directories"));
          if ( (int)pxCache->getOwner() != ID_Owner && (int)pxCache->getOwner() != 0 ) setAllPermission->setEnabled(false);
          connect(setAllPermission, SIGNAL(clicked()), SLOT(changedFields()));
          mainLayout->insertWidget(4, setAllPermission);
        }
      }
    }

    connect(idOwnerEdit, SIGNAL(textChanged(QString)), SLOT(changedFields()));
    connect(idGroupEdit, SIGNAL(textChanged(QString)), SLOT(changedFields()));
  }

void CWindowProperites :: countProcessForOne() {
  recountButton->setEnabled(false);
  stopRecountButton->setEnabled(true);
  sizeData = countData = countFolders = countFiles = 0;
  QList<QString> list;
  list << FileTemplate.FullName;
  threadGS->setData(list, &sizeData, &countData, &countFolders, &countFiles);
  timer->start(500);
  threadGS->start();
}

void CWindowProperites :: refreshCountAndSize() {
  int i;
  sd = sizeData;
  for ( i = 0; i < 5; ++i ) if ( sd >1000 ) sd = sd / 1000; else break;
  labelSize->setText(tr("Counting") + "...  " + QString::number(sd) + " " + lst[i] + "  (" + QString::number(sizeData) + " " + tr("byte(-s)"));
  labelDescrFromContents->setText(QString::number(countFolders - 1) + " " + tr("subdirectories") + ", " + QString::number(countFiles) + " " + tr("file(s)"));
}

void CWindowProperites :: setOfAllCounts() {
  timer->stop();
  threadGS->trueExit();
  while ( threadGS->isRunning()) { threadGS->trueExit(); QApplication::processEvents(); }
  recountButton->setEnabled(true);
  stopRecountButton->setEnabled(false);
  sd = sizeData;
  int i;
  for ( i = 0; i < 4; ++i ) if ( sd >1000 ) sd = sd / 1000; else break;
  if ( i != 0 )
    labelSize->setText(QString::number(sd) + " " + lst[i] + "  (" + QString::number(sizeData) + " " + tr("byte(-s)"));
  else
    labelSize->setText(QString::number(sizeData) + " " + tr("byte(-s)"));
  if ( countFolders != 0 ) countFolders = countFolders - 1;
  labelDescrFromContents->setText(QString::number(countFolders ) + " " + tr("subdirectories") + ", " + QString::number(countFiles) + " " + tr("file(s)"));
}

bool CWindowProperites :: compareParametres () {
  if ( targetEdit != NULL )
    if ( targetEdit->text() != FileTemplate.TargetLink ) return false;
  if ( nameEdit->text() != FileTemplate.FileName ) return false;
  for ( int i = 0; i < 12; ++i)  if ( Permissions.testBit(i) != permissionCheck[i]->isChecked()) return false;
  if ( idOwnerEdit->text() != Uids ) return false;
  if ( idGroupEdit->text() != Gids) return false;
  if (setAllPermission != NULL) if ( setAllPermission->isChecked()) return false;
  return true;
}

void CWindowProperites :: changedFields() {
  if ( compareParametres() == false) {
    okButton->setEnabled(true);
    applyButton->setEnabled(true);
  }
  else {
    okButton->setEnabled(false);
    applyButton->setEnabled(false);
  }
}

void CWindowProperites :: slotCancel() {
  delete this;
}

void CWindowProperites :: createIfNoObject() {
  mainLayout = new QVBoxLayout(this);
  infoHLayout = new QHBoxLayout();
  labelType = new QLabel();
  labelSize = new QLabel();
  cancelButton = new QPushButton(*(pxCache->getIcon(191)), tr("Close"));
  labelType->setPixmap(pxCache->getIcon(182)->scaled(96, 96));
  infoHLayout->addWidget(labelType);
  labelSize->setText("<h4>" + tr("File or folder") + " <br>" + FileTemplate.FullName + "<br> " + tr("does not exists") + ". </h4>");
  infoHLayout->addWidget(labelSize);
  mainLayout->addLayout(infoHLayout);
  mainLayout->addWidget(cancelButton);
  setMaximumWidth(300);
  setMaximumHeight(200);
  connect ( cancelButton, SIGNAL(clicked()), SLOT(slotCancel()));
}

void CWindowProperites :: slotRefresh() {
  timer->stop();
  threadGS->trueExit();
  while ( threadGS->isRunning()) QApplication::processEvents();

  QBitArray bb = fullBitMask(FileTemplate.FullName);
  if ( bb != bitMask ) {
    QList<QWidget*> widlist = this->findChildren<QWidget*>();
    for (int n = widlist.count() - 1 ; n >= 0; --n) { delete widlist[n]; }
    QList<QLayout*> widl = this->findChildren<QLayout*>();
    for (int n = widl.count() - 1 ; n >= 0; --n) { delete widl[n]; }

    FileInfo.setFile(FileTemplate.FullName);
    if ( !FileInfo.exists() && !FileInfo.isSymLink() ) {
      createIfNoObject();
      return;
    }
    FileTemplate = pxCache->fullTemplate(FileInfo);
    targetEdit = NULL;
    setAllPermission = NULL;
    sizeData = countData = countFolders = countFiles = 0;
    createDefaultWidget();
    if ( FileInfo.isSymLink() && FileInfo.isDir()) createIfSymLinkAndDir();
      else if ( !FileInfo.isSymLink() && FileInfo.isDir()) createIfNotSymLinkAndDir();
        else if ( FileInfo.isSymLink()) createIfSymLink();
           else createIfOther();
  }
  else {
    FileInfo.setFile(FileTemplate.FullName);
    FileTemplate = pxCache->fullTemplate(FileInfo);
    if ( FileInfo.isSymLink() && FileInfo.isDir()) refreshIfSymLinkAndDir();
      else  if (!FileInfo.isSymLink() && FileInfo.isDir()) refreshIfNotSymLinkAndDir();
         else  if ( FileInfo.isSymLink()) refreshIfSymLink();
            else refreshIfOther();
  }
  okButton->setEnabled(false);
  applyButton->setEnabled(false);
}

void CWindowProperites :: recountSize() {
  QBitArray bb = fullBitMask(FileTemplate.FullName);
  if ( bb != bitMask ) slotRefresh();
  else {
    timer->stop();
    threadGS->trueExit();
    while ( threadGS->isRunning()) QApplication::processEvents();
    countProcessForOne();
  }
}

void CWindowProperites :: refreshIfSymLinkAndDir() {
    nameEdit->setText(FileTemplate.FileName);
    labelType->setText(FileTemplate.Type);
    labelPath->setText(FileTemplate.Path);
    targetEdit->setText(FileTemplate.TargetLink);

    labelMimeType->setText(FileTemplate.Mime);
    labelCreated->setText(FileTemplate.Created);
    labelModified->setText(FileTemplate.LastModified);
    labelLastRead->setText(FileTemplate.LastRead);
    refreshPermissionsTable();
    refreshFreeSpace();

    int i;
    sizeData = countData = countFolders = countFiles = 0;
    QStringList list;
    list << FileTemplate.FullName;
    threadGS->setData(list, &sizeData, &countData, NULL, NULL);
    threadGS->getSize(list);
    sd = sizeData;
    for ( i = 0; i < 4; ++i ) if ( sd >1000 ) sd = sd / 1000; else break;
    labelSize->setText(FileTemplate.Size + ", " + QString::number(sd) + " " + lst[i]);

    iconButton->setIcon(FileTemplate.Icon);
    iconButton->setIconSize(FileTemplate.Icon.size());
}

void CWindowProperites :: refreshIfNotSymLinkAndDir() {
    nameEdit->setText(FileTemplate.FileName);
    nameEdit->setPalette(pal);
    labelType->setText(FileTemplate.Type);
    labelPath->setText(FileTemplate.Path);
    labelMimeType->setText(FileTemplate.Mime);
    labelCreated->setText(FileTemplate.Created);
    labelModified->setText(FileTemplate.LastModified);
    labelLastRead->setText(FileTemplate.LastRead);

    refreshFreeSpace();
    refreshPermissionsTable();
    iconButton->setIcon(FileTemplate.Icon);
    iconButton->setIconSize(FileTemplate.Icon.size());
}

void CWindowProperites :: refreshIfSymLink() {
    nameEdit->setText(FileTemplate.FileName);
    nameEdit->setPalette(pal);
    labelType->setText(FileTemplate.Type);
    labelPath->setText(FileTemplate.Path);
    targetEdit->setText(FileTemplate.TargetLink);
    labelDescription->setText(FileTemplate.Descr);
    labelDescrFromContents->setText(FileTemplate.DescrFromContents);
    labelMimeType->setText(FileTemplate.Mime);
    labelSize->setText(FileTemplate.Size);
    labelCreated->setText(FileTemplate.Created);
    labelModified->setText(FileTemplate.LastModified);
    labelLastRead->setText(FileTemplate.LastRead);
    refreshPermissionsTable();
    refreshFreeSpace();

    iconButton->setIcon(FileTemplate.Icon);
    iconButton->setIconSize(FileTemplate.Icon.size());
}

void CWindowProperites :: refreshIfOther() {
    nameEdit->setText(FileTemplate.FileName);
    nameEdit->setPalette(pal);
    labelType->setText(FileTemplate.Type);
    labelPath->setText(FileTemplate.Path);
    labelDescription->setText(FileTemplate.Descr);
    labelDescrFromContents->setText(FileTemplate.DescrFromContents);
    labelMimeType->setText(FileTemplate.Mime);
    labelSize->setText(FileTemplate.Size);
    labelCreated->setText(FileTemplate.Created);
    labelModified->setText(FileTemplate.LastModified);
    labelLastRead->setText(FileTemplate.LastRead);
    refreshPermissionsTable();
    refreshFreeSpace();

    iconButton->setIcon(FileTemplate.Icon);
    iconButton->setIconSize(FileTemplate.Icon.size());
}

void CWindowProperites :: refreshPermissionsTable() {
    for (int i = 0; i < 12; ++i) permissionCheck[i]->setEnabled(true);
    if ( setAllPermission != NULL ) {
      delete setAllPermission;
      setAllPermission = NULL;
    }
    if ( labelNotChangePermissions != NULL ) {
      delete labelNotChangePermissions;
      labelNotChangePermissions = NULL;
    }
    permissions ( FileTemplate.FullName, &Permissions, &ID_Owner, &ID_Group, &bitMask);
    for (int i = 0; i < 12; ++i) permissionCheck[i]->setChecked(Permissions.testBit(i));
    Uids = groupName(ID_Owner);
    Gids = groupName(ID_Group);
    idOwnerEdit->setText(Uids);
    idGroupEdit->setText(Gids);
    if ( FileInfo.isSymLink()) {
      labelNotChangePermissions = new QLabel();
      mainLayout->insertWidget(4, labelNotChangePermissions);
      labelNotChangePermissions->setText(tr("Warning: this file is a symbolic link, and its right to match") + "<br>" + tr("with the rights of the target file"));
      for (int i = 0; i < 12; ++i) permissionCheck[i]->setEnabled(false);
    }
    else {
      if ( (int)pxCache->getOwner() != ID_Owner && (int)pxCache->getOwner() != 0) {
        labelNotChangePermissions = new QLabel();
        mainLayout->insertWidget(4, labelNotChangePermissions);
        if ( FileInfo.isFile())
          labelNotChangePermissions->setText(tr("You are not the owner of the file,") + "<br>" + tr("and can not change the law"));
        if ( FileInfo.isDir())
          labelNotChangePermissions->setText(tr("You are not the owner of the directory,") + "<br>" + tr("and can not change the law"));
        for (int i = 0; i < 12; ++i) permissionCheck[i]->setEnabled(false);
      }
      else {
        if ( FileInfo.isDir() &&!FileInfo.isSymLink()) {
          setAllPermission = new QCheckBox(tr("Apply to all attached files and directories"));
          mainLayout->insertWidget(4, setAllPermission);
          if ( (int)pxCache->getOwner() != ID_Owner && (int)pxCache->getOwner() != 0 )   setAllPermission->setEnabled(false);
          connect ( setAllPermission, SIGNAL(clicked()), SLOT(changedFields()));
        }
      }
    }
}

void CWindowProperites :: applyChanges() {
  QString newname;
  QBitArray bb = fullBitMask(FileTemplate.FullName);
  if ( bb != bitMask ) {
    slotRefresh();
    return;
  }
  else {
    if ( nameEdit->text() != FileTemplate.FileName ) {
      if ( nameEdit->text() == "." || nameEdit->text() == ".." || nameEdit->text().contains("/") || nameEdit->text().contains("\\") ||
            nameEdit->text() == "" ) {
        QDialog *error = new QDialog();
        error->setWindowTitle(tr("Error: invalid characters in name"));
        error->setWindowIcon(*(pxCache->getMainIcon(3)));
        error->setModal(true);
        error->setPalette(epal);
        error->setFont(Font);
        QVBoxLayout *vlay = new QVBoxLayout(error);
        QHBoxLayout *hlay = new QHBoxLayout();
        QLabel *labicon = new QLabel();
        labicon->setPixmap(pxCache->getIcon(182)->scaled(96,96));
        QLabel *labtext = new QLabel();
        labtext->setText("<h4>" + tr("Name contains invalid characters") + ".<br>" +
                                    tr("Please enter a different name.") + "</h4>" +
                                    tr("Note: the name can not be '.' or '..'") + ",<br>" +
                                    tr("contain the characters '\\' and '/' or be empty"));
        QPushButton *close = new QPushButton(*(pxCache->getIcon(191)), tr("Close"));
        connect ( close, SIGNAL(clicked()), error, SLOT(accept()));
        vlay->addLayout(hlay);
        hlay->addWidget(labicon);
        hlay->addWidget(labtext);
        vlay->addWidget(close);
        error->exec();
        delete error;
        return;
      }
      else {
        QFileInfo nfinf(FileTemplate.Path + QDir::separator() + nameEdit->text());
        if ( nfinf.exists()) {
            QDialog *error = new QDialog();
            error->setWindowTitle(tr("Object with the same name already exists"));
            error->setWindowIcon(*(pxCache->getMainIcon(3)));
            error->setModal(true);
            error->setPalette(epal);
            error->setFont(BoldFont);
            QVBoxLayout *vlay = new QVBoxLayout(error);
            QHBoxLayout *hlay = new QHBoxLayout();
            QLabel *labname = new QLabel("<h4>" + tr("Error: Object") + " <br> " +
                                                                      nfinf.absoluteFilePath() + " <br>" + tr("aready exists"));
            QLabel *labicon = new QLabel();
            Template templ = pxCache->fullTemplate(nfinf);
            labicon->setPixmap(templ.Icon.scaled(96,96));
            QLabel *labtext = new QLabel();
            if (nfinf.isDir() && !nfinf.isSymLink())
              labtext->setText("<table><tr><td>" + tr("Type") +":</td><td>" +  templ.Type + "</td></tr><tr><td>"
                                                                  + tr("Size") + ":</td><td>" + templ.Size   + "</td></tr><tr><td>"
                                                                  + tr("Created") + ":</td><td>" + templ.Created + "</td></tr><tr><td>"
                                                                  + tr("Modified") + ":</td><td>" + templ.LastModified + "</td></tr></table>");
            else
              labtext->setText("<table><tr><td>" + tr("Type") +":</td><td>" +  templ.Type + "</td></tr><tr><td>"
                                                                  + tr("Description (from extended)") +":</td><td>" + templ.Descr + "</td></tr><tr><td>"
                                                                  + tr("Contents") + ":</td><td>" + templ.DescrFromContents + "</td></tr><tr><td>"
                                                                  + tr("Size") + ":</td><td>" + templ.Size   + "</td></tr><tr><td>"
                                                                  + tr("Created") + ":</td><td>" + templ.Created + "</td></tr><tr><td>"
                                                                  + tr("Modified") + ":</td><td>" + templ.LastModified + "</td></tr></table>");

            QPushButton *close = new QPushButton(*(pxCache->getIcon(191)), tr("Close"));
            connect ( close, SIGNAL(clicked()), error, SLOT(accept()));
            vlay->addWidget(labname);
            vlay->addLayout(hlay);
            hlay->addWidget(labicon);
            hlay->addWidget(labtext);
            vlay->addWidget(close);
            error->exec();
            delete error;
            return;
        }
        if (QFile::rename(FileTemplate.FullName, FileTemplate.Path + QDir::separator() + nameEdit->text())) {
          FileTemplate.FullName = FileTemplate.Path + QDir::separator() + nameEdit->text();
          FileTemplate = pxCache->fullTemplate(FileTemplate.FullName);
        }
      }
    }
    QBitArray nbits(12);
    int k;
    int nowner = groupId(idOwnerEdit->text());
    int ngroup = groupId(idGroupEdit->text());
    for (int i = 0; i < 12; ++ i)  nbits.setBit(i, permissionCheck[i]->isChecked());
    if ( nbits != Permissions || ID_Owner != nowner || ID_Group != ngroup) {
      int perm = bitArrayToInt(nbits);
      if ( nowner == -1 ) nowner = ID_Owner;
      if ( ngroup == -1 ) ngroup = ID_Group;
      if ( setAllPermission != NULL ) {
        if ( setAllPermission->isChecked()) {
          setRecursivePermissions(FileTemplate.FullName, perm, nowner, ngroup);
        }
        else {
          setPermissions(FileTemplate.FullName, perm, nowner, ngroup);
        }
      }
      else {
        setPermissions(FileTemplate.FullName, perm, nowner, ngroup);
      }
    }
    if ( targetEdit != NULL ) {
      if ( FileTemplate.TargetLink != targetEdit->text()) {
        unlink(FileTemplate.FullName.toLocal8Bit());
        createSymLink( targetEdit->text(), FileTemplate.FullName, &k);
      }
    }
    QPushButton *butt = qobject_cast<QPushButton*>(sender());
    if ( butt == applyButton )  slotRefresh();
    else slotCancel();
  }
}

void CWindowProperites :: refreshFreeSpace() {
    quint64 totalsize, freesize;
    if ( FileInfo.isSymLink())  freeSpaceFrom (FileTemplate.Path, &totalsize, &freesize);
      else freeSpaceFrom (FileTemplate.FullName, &totalsize, &freesize);
    quint64 blocksize = totalsize - freesize;
    quint64 proc = (qreal)blocksize/(qreal)totalsize*100;
    QString totalsizeS, freesizeS, blocksizeS;
    int i = 0;
    for ( i = 0; i < 5; ++i)  if ( totalsize > 1000 ) totalsize = totalsize / 1000; else break;
    totalsizeS = QString::number(totalsize) + " " + lst[i];
    for ( i = 0; i < 5; ++i)  if ( freesize > 1000 ) freesize = freesize / 1000; else break;
    freesizeS = QString::number(freesize) + " " + lst[i];
    for ( i = 0; i < 5; ++i)  if ( blocksize > 1000 ) blocksize = blocksize / 1000; else break;
    blocksizeS = QString::number(blocksize) + " " + lst[i];
    QString mp = pxCache->locateMountPoints(FileTemplate.FullName);
    if ( mp == "" ) {
      pxCache->fullMountPoints();
      mp = pxCache->locateMountPoints(FileTemplate.FullName);
    }
    labelFreeSpaceH->setText(tr("Free") + " " + freesizeS + " " +tr("from") + " " + totalsizeS + " (" + tr("busy") + " " + QString::number(proc) + "% " + tr("on") + "  "+ mp + " )");
    freeSpace->setRange(0, 100);
    freeSpace->setValue(proc);
}

//--------------------------------------------------------------------------------------------------------------------------------------------------------------------
// ------------------------------------                         Function for lists elements                             ------------------------------------------------
//---------------------------------------------------------------------------------------------------------------------------------------------------------------------

void CWindowProperites :: createDefaultWidgetFL() {
    setWindowTitle(tr("Properites object's") + " - Bathyscaphe");
    mainLayout = new QVBoxLayout(this);
    infoHLayout  = new QHBoxLayout();
    infoGridLayout = new QGridLayout();
    recountButtonsHLayout = new QHBoxLayout();
    buttonsHLayout = new QHBoxLayout();

    infoBox = new QGroupBox();
    buttonsBox = new QGroupBox();
    iconButton = new QPushButton();
    iconButton->setFlat(true);
    iconButton->setIcon(*(pxCache->getIcon(10)));
    iconButton->setIconSize(pxCache->getIcon(10)->size());
    infoHLayout->addWidget(iconButton);
    infoHLayout->addLayout(infoGridLayout);
    infoBox->setLayout(infoHLayout);
    infoHLayout->itemAt(0)->setAlignment(Qt::AlignTop);


    labelHeaderInfo = new QLabel("<h4 align = center>" + tr("Properites object's") + "</h4>");
    labelObjectH = new QLabel("<b>" + tr("Object's:") + "</b>");
    labelObject = new QLabel();
    labelPathH = new QLabel("<b>" + tr("Path:") + "</b>");
    labelPath = new QLabel();
    labelSizeH = new QLabel("<b>" + tr("Size:") + "</b>");
    labelSize = new QLabel();
    labelDescrFromContentsH = new QLabel("<b>" + tr("Contents:") + "</b>");
    labelDescrFromContents = new QLabel();

    recountButton = new QPushButton(tr("Re-count"));
    stopRecountButton = new QPushButton(tr("Stop"));
    recountButtonsHLayout->addWidget(recountButton);
    recountButtonsHLayout->addWidget(stopRecountButton);
    recountButton->setEnabled(false);
    stopRecountButton->setEnabled(true);

    cancelButton = new QPushButton(*(pxCache->getIcon(191)), tr("Close"));
    refreshButton = new QPushButton(*(pxCache->getIcon(161)), tr("Set permissions"));

    buttonsHLayout->addWidget(refreshButton);
    buttonsHLayout->addWidget(cancelButton);
    buttonsBox->setLayout(buttonsHLayout);
    infoGridLayout->addWidget(labelObjectH, 0, 0);
    infoGridLayout->addWidget(labelObject, 0, 1);
    infoGridLayout->addWidget(labelPathH, 1, 0);
    infoGridLayout->addWidget(labelPath, 1, 1);
    infoGridLayout->addWidget(labelSizeH, 2, 0);
    infoGridLayout->addWidget(labelSize, 2, 1);
    infoGridLayout->addWidget(labelDescrFromContentsH, 3, 0);
    infoGridLayout->addWidget(labelDescrFromContents, 3, 1);
    infoGridLayout->addLayout(recountButtonsHLayout, 4, 1);
    quint64 totalsize, freesize;
    if ( FileInfoList[0].isSymLink()) freeSpaceFrom (FileInfoList[0].absolutePath(), &totalsize, &freesize);
    else freeSpaceFrom (FileInfoList[0].absoluteFilePath(), &totalsize, &freesize);
    quint64 blocksize = totalsize - freesize;
    quint64 proc = (qreal)blocksize/(qreal)totalsize*100;
    QString totalsizeS, freesizeS, blocksizeS;
    int i = 0;
    for ( i = 0; i < 5; ++i)  if ( totalsize > 1000 ) totalsize = totalsize / 1000; else break;
    totalsizeS = QString::number(totalsize) + " " + lst[i];
    for ( i = 0; i < 5; ++i)  if ( freesize > 1000 ) freesize = freesize / 1000; else break;
    freesizeS = QString::number(freesize) + " " + lst[i];
    for ( i = 0; i < 5; ++i)  if ( blocksize > 1000 ) blocksize = blocksize / 1000; else break;
    blocksizeS = QString::number(blocksize) + " " + lst[i];
    QString mp = pxCache->locateMountPoints(FileInfoList[0].absoluteFilePath());
    if ( mp == "" ) {
      pxCache->fullMountPoints();
      mp = pxCache->locateMountPoints(FileInfoList[0].absoluteFilePath());
    }
    labelHeaderSpace = new QLabel("<h4 align = center>" + tr("Free space") + "</h4>");
    labelFreeSpaceH = new QLabel(tr("Free") + " " + freesizeS + " " + tr("from") + " " + totalsizeS + " (" + tr("busy") + " " + QString::number(proc) + "% " + tr("on") + " " + mp + " )");
    freeSpace = new QProgressBar();
    freeSpace->setRange(0, 100);
    freeSpace->setValue(proc);


    mainLayout->addWidget(labelHeaderInfo);
    mainLayout->addWidget(infoBox);
    mainLayout->addWidget(labelHeaderSpace);
    mainLayout->addWidget(labelFreeSpaceH);
    mainLayout->addWidget(freeSpace);
    mainLayout->addWidget(buttonsBox);

    countfolders = 0;
    countfiles = 0;
    foreach ( QFileInfo finf, FileInfoList) {
      if ( finf.isDir() && !finf.isSymLink()) countfolders++; else countfiles++;
    }
    labelObject->setText(QString::number(countfolders) + " " + tr("director(-y,-ies)") + ", " + QString::number(countfiles)  + tr("file(-s)") + ".");
    labelPath->setText(FileInfoList[0].absolutePath());

    connect ( refreshButton, SIGNAL(clicked()), SLOT(setPermissionsFL()));
    connect ( cancelButton, SIGNAL(clicked()), SLOT(slotCancel()));
    connect ( recountButton, SIGNAL(clicked()), SLOT(recountSizeFL()));
    connect (stopRecountButton, SIGNAL(clicked()), SLOT(setOfAllCounts()));

    countProcessForList();
}

void CWindowProperites :: countProcessForList() {
  recountButton->setEnabled(false);
  stopRecountButton->setEnabled(true);
  sizeData = countData = countFolders = countFiles = 0;
  QList<QString> list;
  foreach (QFileInfo finf, FileInfoList) {
    list << finf.absoluteFilePath();
  }
  threadGS->setData(list, &sizeData, &countData, &countFolders, &countFiles);
  timer->start(500);
  threadGS->start();
}

void CWindowProperites :: recountSizeFL() {
    timer->stop();
    threadGS->trueExit();
    while ( threadGS->isRunning()) QApplication::processEvents();
    quint64 totalsize, freesize;
    int ssize = FileInfoList.count();
    for ( int i = 0; i < ssize;) {
      if ( !FileInfoList[i].exists() && !FileInfoList[i].isSymLink()) FileInfoList.removeAt(i);
        else ++i;
    }
    if ( FileInfoList.count() == 0 ) {
      createNoObjectList();
      return;
    }
    countfolders = 0;
    countfiles = 0;
    foreach ( QFileInfo finf, FileInfoList) {
      if ( finf.isDir() && !finf.isSymLink()) countfolders++; else countfiles++;
    }
    labelObject->setText(QString::number(countfolders) + " " + tr("director(-y,-ies)") + ", " + QString::number(countfiles)  + tr("file(-s)") + ".");


    if ( FileInfoList[0].isSymLink())  freeSpaceFrom (FileInfoList[0].absolutePath(), &totalsize, &freesize);
     else  freeSpaceFrom (FileInfoList[0].absoluteFilePath(), &totalsize, &freesize);
    quint64 blocksize = totalsize - freesize;
    quint64 proc = (qreal)blocksize/(qreal)totalsize*100;
    QString totalsizeS, freesizeS, blocksizeS;
    int i = 0;
    for ( i = 0; i < 5; ++i)  if ( totalsize > 1000 ) totalsize = totalsize / 1000; else break;
    totalsizeS = QString::number(totalsize) + " " + lst[i];
    for ( i = 0; i < 5; ++i)  if ( freesize > 1000 ) freesize = freesize / 1000; else break;
    freesizeS = QString::number(freesize) + " " + lst[i];
    for ( i = 0; i < 5; ++i)  if ( blocksize > 1000 ) blocksize = blocksize / 1000; else break;
    blocksizeS = QString::number(blocksize) + " " + lst[i];
    QString mp = pxCache->locateMountPoints(FileInfoList[0].absoluteFilePath());
    if ( mp == "" ) {
      pxCache->fullMountPoints();
      mp = pxCache->locateMountPoints(FileInfoList[0].absoluteFilePath());
    }
    labelFreeSpaceH->setText(tr("Free") + " " + freesizeS + " " + tr("from") + " " + totalsizeS + " (" + tr("busy") + " " + QString::number(proc) + "% " + tr("on") + " " + mp + " )");
    freeSpace->setRange(0, 100);
    freeSpace->setValue(proc);

    labelPath->setText(FileInfoList[0].absolutePath());
    countProcessForList();
}

void CWindowProperites :: createNoObjectList() {
  QList<QWidget*> widlist = this->findChildren<QWidget*>();
  for (int n = widlist.count() - 1 ; n >= 0; --n) { delete widlist[n]; }
  QList<QLayout*> widl = this->findChildren<QLayout*>();
  for (int n = widl.count() - 1 ; n >= 0; --n) { delete widl[n]; }

  mainLayout = new QVBoxLayout(this);
  infoHLayout = new QHBoxLayout();
  labelType = new QLabel();
  labelSize = new QLabel();
  cancelButton = new QPushButton(*(pxCache->getIcon(191)), tr("Close"));
  labelType->setPixmap(pxCache->getIcon(182)->scaled(96, 96));
  infoHLayout->addWidget(labelType);
  labelSize->setText("<h4>" + tr("No objects from list") + ". </h4>");
  infoHLayout->addWidget(labelSize);
  mainLayout->addLayout(infoHLayout);
  mainLayout->addWidget(cancelButton);
  setMaximumWidth(300);
  setMaximumHeight(200);
  connect ( cancelButton, SIGNAL(clicked()), SLOT(slotCancel()));
}

void CWindowProperites :: setPermissionsFL() {
    Perm  = new QDialog(this);
    Perm->setWindowTitle(tr("Set permissions for group") + " - Bathyscaphe");
    Perm->setWindowIcon(*(pxCache->getMainIcon(3)));
    Perm->setPalette(epal);
    QVBoxLayout *permVLayout = new QVBoxLayout(Perm);
    labelHeaderPermissions = new QLabel("<h4> <center>" + tr("Permissions") + "</center></h4>");
    permVLayout->addWidget(labelHeaderPermissions);
    QHBoxLayout *permHLayout = new QHBoxLayout();
    permVLayout->addLayout(permHLayout);
    permissionBox = new QGroupBox();
    moreBox = new QGroupBox();
    idBox = new QGroupBox();
    permHLayout->addWidget(permissionBox);
    permHLayout->addWidget(moreBox);
    permissionCheck.clear();
    for (int i = 0; i < 12; ++i ) {
      QCheckBox *chbox = new QCheckBox();
      permissionCheck << chbox;
    }
    permissionGridLayout = new QGridLayout();
    permissionGridLayout->addWidget(new QLabel(tr("Read")), 0, 1);
    permissionGridLayout->addWidget(new QLabel(tr("Write")), 0, 2);
    permissionGridLayout->addWidget(new QLabel(tr("Execute")), 0, 3);
    permissionGridLayout->addWidget(new QLabel(tr("Owner")),1, 0);
    permissionGridLayout->addWidget(new QLabel(tr("Group")), 2, 0);
    permissionGridLayout->addWidget(new QLabel(tr("Other")), 3, 0);
    int z = 0;
    for ( int i = 3; i > 0; --i)
      for ( int n = 3; n > 0; --n ) {
        permissionGridLayout->addWidget(permissionCheck[z], i, n );
        ++z;
      }

    moreGridLayout = new QGridLayout();
    moreGridLayout->addWidget(new QLabel("<center>" + tr("More") + "</center>"), 0, 0, 1, 2);
    moreGridLayout->addWidget(new QLabel("SUID"), 1, 0);
    moreGridLayout->addWidget(new QLabel("SGID"), 2, 0);
    moreGridLayout->addWidget(new QLabel("Sticky"), 3, 0);
    moreGridLayout->addWidget(permissionCheck[11], 1, 1);
    moreGridLayout->addWidget(permissionCheck[10], 2, 1);
    moreGridLayout->addWidget(permissionCheck[9], 3, 1);

    permissionBox->setLayout(permissionGridLayout);
    moreBox->setLayout(moreGridLayout);
    permHLayout->addWidget(permissionBox);
    permHLayout->addWidget(moreBox);

    idGridLayout = new QGridLayout();
    idGridLayout->addWidget(new QLabel("<b>" + tr("Owner of file:") + "</b>"), 0, 0);
    idGridLayout->addWidget(new QLabel("<b>" + tr("Group owner of file:") + "</b>"), 1, 0);
    Uids = groupName(pxCache->getOwner());
    Gids = groupName(pxCache->getGroup());
    idOwnerEdit = new QLineEdit(Uids);
    idGroupEdit = new QLineEdit(Gids);
    idOwnerEdit->installEventFilter(ExcludeEvents);
    idGroupEdit->installEventFilter(ExcludeEvents);

    idGridLayout->addWidget(idOwnerEdit, 0, 1);
    idGridLayout->addWidget(idGroupEdit, 1, 1);
    idOwnerEdit->setPalette(pal);
    idGroupEdit->setPalette(pal);
    setAllPermission = new QCheckBox(tr("Apply to all attached files and directories"));
    labelHeaderOwner = new QLabel("<h4><center>" + tr("Owner and group") + "</center></h4");
    permVLayout->addWidget(setAllPermission);
    permVLayout->addWidget(labelHeaderOwner);
    idBox->setLayout(idGridLayout);
    permVLayout->addWidget(idBox);
    QHBoxLayout *addedLayout = new QHBoxLayout();
    applyButton = new QPushButton(*(pxCache->getIcon(190)), tr("Apply"));
    okButton = new QPushButton(*(pxCache->getIcon(191)), tr("Cancel"));
    addedLayout->addWidget(applyButton);
    addedLayout->addWidget(okButton);
    permVLayout->addLayout(addedLayout);

    connect(applyButton, SIGNAL(clicked()), SLOT(applyChangePermissionFL()));
    connect(okButton, SIGNAL(clicked()), Perm, SLOT(accept()));

    Perm->setModal(true);
    Perm->exec();
    delete Perm;
}

void CWindowProperites :: applyChangePermissionFL() {
    QBitArray nbits(12);
    int nowner = groupId(idOwnerEdit->text());
    int ngroup = groupId(idGroupEdit->text());
    for (int i = 0; i < 12; ++ i)  nbits.setBit(i, permissionCheck[i]->isChecked());
    int perm = bitArrayToInt(nbits);
    if ( nowner == -1 ) nowner = pxCache->getOwner();
    if ( ngroup == -1 ) ngroup = pxCache->getGroup();
    if ( setAllPermission->isChecked()) {
       foreach ( QFileInfo finf, FileInfoList) {
          setRecursivePermissions(finf.absoluteFilePath(), perm, nowner, ngroup);
        }
    }
    else {
      foreach ( QFileInfo finf, FileInfoList) {
         setPermissions(finf.absoluteFilePath(), perm, nowner, ngroup);
      }
    }
   Perm->accept();
}

void CWindowProperites :: setOfAllCountsFL() {
  timer->stop();
  threadGS->trueExit();
  while ( threadGS->isRunning()) QApplication::processEvents();
  recountButton->setEnabled(true);
  stopRecountButton->setEnabled(false);
  sd = sizeData;
  int i;
  for ( i = 0; i < 4; ++i ) if ( sd >1000 ) sd = sd / 1000; else break;
  if ( i != 0 )
    labelSize->setText(QString::number(sd) + " " + lst[i] + "  (" + QString::number(sizeData) + " " + tr("byte(-s)"));
  else
    labelSize->setText(QString::number(sizeData) + " " + tr("byte(-s)"));
  countFolders = countFolders - countfolders;
  labelDescrFromContents->setText(QString::number(countFolders ) + " " + tr("subdirectories") + ", " + QString::number(countFiles) + " " +  tr("file(s)"));
}

//----------------------------------------                                            Destructor                                     --------------------------------------------------

CWindowProperites :: ~CWindowProperites() {
  timer->stop();
  threadGS->trueExit();
  while ( threadGS->isRunning()) { threadGS->trueExit(); QApplication::processEvents(); }
  delete timer;
  delete threadGS;
}
