#include "../../Include/TableViewer.h"
#include "../../Include/Stat.h"

CTableViewer :: CTableViewer ( QWidget * wgt ) : QTreeWidget ( wgt ) {
  QList<QString> lst;
  setColumnCount(4);
  setRootIsDecorated(false);
  lst << tr("Name") << tr("Type") << tr("Size") << tr("Last modified");
  setHeaderLabels (lst );
  setAnimated(true);
  setEditTriggers(QAbstractItemView::NoEditTriggers);

  StartTimer = false;
  EditElement = false;
  w = this->width();
  if ( w < 650 ) w = 300;
    else w = w - 350;
  setColumnWidth(0,w);
  setDragEnabled(true);
  setAcceptDrops(true);

  bitIcon = NULL;
  ThreadIcon = new CThreadIcon(rCache1, fileinfolist);
  ThreadAreaIcon = new CThreadAreaIcon(rCache2, fileinfolist);

  setSelectionMode(QAbstractItemView::ContiguousSelection);

  connect ( this, SIGNAL(itemSelectionChanged()), SLOT(selectionChanged()));
  connect ( &timer, SIGNAL(timeout()), SLOT(editElement()));
  connect ( &DragTimer, SIGNAL(timeout()), this, SLOT(timeoutDragTimer()), Qt::QueuedConnection);
  connect ( &ScrollDownTimer, SIGNAL(timeout()), this, SLOT(scrollDown()), Qt::QueuedConnection);
  connect ( &ScrollUpTimer, SIGNAL(timeout()), this, SLOT(scrollUp()), Qt::QueuedConnection);
}

void CTableViewer :: showDirectory ( QString path, bool read ) {
  if ( StartTimer ) {
    timer.stop();
    StartTimer = false;
  }

   ThreadIcon->execute = false;
   while ( ThreadIcon->isRunning()) {
     QApplication::processEvents();
     ThreadIcon->execute = false;
   }
   ThreadAreaIcon->execute = false;
   while ( ThreadAreaIcon->isRunning()) {
     QApplication::processEvents();
     ThreadAreaIcon->execute = false;
   }

  if ( read ) {
     if ( TemplateFilter->count() == 0 && MimeFilter->count() == 0 ) {
          dir.setPath ( path );
          CurrentFileInfo.setFile(path);
          fileinfolist->clear();
          QFileInfoList templist, templist_hidden;
          if ( ShowHidden ) {
            templist_hidden = dir.entryInfoList ( QDir::Hidden );
            foreach ( fileinfo, templist_hidden )  if ( fileinfo.isDir()) *fileinfolist << fileinfo;
            templist = dir.entryInfoList (QDir::Dirs | QDir::NoDotAndDotDot );
            foreach ( fileinfo, templist ) if ( fileinfo.isDir()) *fileinfolist << fileinfo;
            foreach ( fileinfo, templist_hidden) if ( fileinfo.isFile() ) *fileinfolist << fileinfo;
            *fileinfolist << dir.entryInfoList(QDir::Files | QDir::System );
          }
          else {
            templist =  dir.entryInfoList ( QDir::Dirs | QDir::NoDotAndDotDot );
            foreach ( fileinfo, templist ) if ( fileinfo.isDir()) *fileinfolist << fileinfo;
            *fileinfolist << dir.entryInfoList ( QDir::Files | QDir::System);
          }
     }
     else {
       if ( TemplateFilter->count() != 0 ) {
          dir.setPath ( path );
          fileinfolist->clear();
          QFileInfoList templist, templist_hidden;
          if ( ShowHidden ) {
            if ( *ExcludeDirsTemplate ) templist_hidden = dir.entryInfoList ( QDir::Hidden );
              else  templist_hidden = dir.entryInfoList(*TemplateFilter, QDir::Hidden);
            foreach ( fileinfo, templist_hidden )  if ( fileinfo.isDir()) *fileinfolist << fileinfo;
            if ( *ExcludeDirsTemplate ) templist = dir.entryInfoList(QDir::Dirs | QDir::NoDotAndDotDot);
              else templist = dir.entryInfoList(*TemplateFilter, QDir::Dirs | QDir::NoDotAndDotDot);
            foreach ( fileinfo, templist ) if ( fileinfo.isDir()) *fileinfolist << fileinfo;
            foreach ( fileinfo, templist_hidden ) if ( fileinfo.isFile()) *fileinfolist << fileinfo;
            *fileinfolist << dir.entryInfoList(*TemplateFilter, QDir::Files | QDir::System);
          }
          else {
            if ( *ExcludeDirsTemplate ) templist =  dir.entryInfoList ( QDir::Dirs | QDir::NoDotAndDotDot );
              else templist =  dir.entryInfoList ( *TemplateFilter, QDir::Dirs | QDir::NoDotAndDotDot );
            foreach ( fileinfo, templist ) if ( fileinfo.isDir()) *fileinfolist << fileinfo;
            *fileinfolist << dir.entryInfoList ( *TemplateFilter, QDir::Files | QDir::System);
          }
       }
       if ( MimeFilter->count() != 0 ) {
          QFileInfoList templist, templist_dir_hidden, templist_dir, templist_files_hidden, templist_files,  templist_hidden;
          dir.setPath ( path );
          fileinfolist->clear();
          if ( ShowHidden ) {
            templist_hidden = dir.entryInfoList ( QDir::Hidden);
            foreach ( fileinfo, templist_hidden )  if ( fileinfo.isDir()) templist_dir_hidden << fileinfo;
            templist_dir =  dir.entryInfoList ( QDir::Dirs | QDir::NoDotAndDotDot );
            foreach ( fileinfo, templist_hidden ) if ( fileinfo.isFile()) templist_files_hidden << fileinfo;
            templist_files = dir.entryInfoList(QDir::Files | QDir::System);
          }
          else {
            templist_dir = dir.entryInfoList ( QDir::Dirs | QDir::NoDotAndDotDot );
            templist_files =  dir.entryInfoList ( QDir::Files | QDir::System);
          }
          templist.clear();
          if ( *ExcludeDirsMime ) {
            templist << templist_dir_hidden;
            templist << templist_dir;
          }
          else {
            foreach ( QFileInfo finf, templist_dir_hidden) {
              foreach ( QRegExp rexp, *MimeFilter) {
                if ( pxCache->getMimeSource(finf).contains(rexp)) { templist << finf; break; }
              }
            }
            foreach ( QFileInfo finf, templist_dir) {
              foreach ( QRegExp rexp, *MimeFilter) {
                if ( pxCache->getMimeSource(finf).contains(rexp)) { templist << finf; break; }
              }
            }
          }
          foreach ( QFileInfo finf, templist_files_hidden) {
            foreach ( QRegExp rexp, *MimeFilter) {
              if ( pxCache->getMimeSource(finf).contains(rexp)) { templist << finf; break; }
            }
          }
          foreach ( QFileInfo finf, templist_files) {
            foreach ( QRegExp rexp, *MimeFilter) {
              if ( pxCache->getMimeSource(finf).contains(rexp)) { templist << finf; break; }
            }
          }
          *fileinfolist = templist;
       }
     }
  }

  emit refreshWrite();
  itemsList.clear();
  clear();
  if ( fileinfolist->count() == 0) return;
  this->verticalScrollBar()->setSliderPosition(0);


   delete ThreadIcon;
   delete ThreadAreaIcon;
   if ( bitIcon != NULL ) delete bitIcon;

  ThreadIcon = new CThreadIcon(rCache1, fileinfolist);
  ThreadAreaIcon = new CThreadAreaIcon(rCache2, fileinfolist);

   startElement = 0;
   endElement = 0;

  bitIcon = new QBitArray(fileinfolist->count());
  ThreadAreaIcon->setData(bitIcon, 'T', &startElement, &endElement);
  ThreadIcon->setData( bitIcon, 'T');

  foreach ( fileinfo, *fileinfolist ) {
    item = new QTreeWidgetItem (this);
    item->setTextAlignment (1, Qt::AlignLeft | Qt::AlignVCenter);
    item->setTextAlignment (2, Qt::AlignRight | Qt::AlignVCenter);
    item->setTextAlignment (3, Qt::AlignRight | Qt::AlignVCenter);
    item->setText (0, fileinfo.fileName());
    item->setToolTip (0,  fileinfo.absoluteFilePath());
    item->setSizeHint(0, iconSize());
    itemsList << item;
    addTopLevelItem(item);
  }
    connect ( this->verticalScrollBar(), SIGNAL(valueChanged(int)), SLOT(initAreaIcon(int)));
    connect ( ThreadIcon, SIGNAL(pixmapElement( int, int, int, QString, QString, QString, bool)),
                                              SLOT(setPixmapElement(int, int, int, QString, QString, QString, bool)));
    connect ( ThreadAreaIcon, SIGNAL(pixmapElement(int, int, int, QString, QString, QString, bool)),
                                                      SLOT(setPixmapElement(int, int, int, QString, QString, QString, bool)));
    connect (ThreadIcon, SIGNAL(quitSubThread()), SLOT(finishSubThread()));
    ThreadIcon->start(QThread::HighPriority);
    ThreadAreaIcon->start(QThread::HighPriority);
    this->setFocus();
}

void CTableViewer :: resizeEvent (QResizeEvent * ){
  w = this->width();
  if ( w < 650 ) w = 300;
    else w = w - 350;
  setColumnWidth(0,w);
 }

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

 void CTableViewer :: editElement() {
      timer.stop();
      StartTimer = false;
      QFontMetrics fontMetric (*font);
      QRect rect = visualItemRect(item);
      EditElement = true;
      Editor = new CLineEdit(this);
      Editor->setFont(*font);
      item->setSelected(false);
      Editor->setText(item->text( 0 ));
      int q = itemsList.indexOf(item);
      QString bname = fileinfolist->value(q).baseName();
      Editor->setSelection(0, bname.length());
      Editor->setFixedWidth(w-iconSize().width()-20);
      if ( rect.height() > fontMetric.height() + 4 )  Editor->setFixedHeight ( rect.height());
      else Editor->setFixedHeight ( fontMetric.height() + 4);
      Editor->move(rect.x()+iconSize().width()+5, rect.y() + iconSize().height());
      Editor->setPalette(this->palette());
      Editor->setFocus();
      Editor->show();
      QObject::connect( Editor, SIGNAL(endOfReturn()), SLOT(textChanged()));
}

void CTableViewer :: mousePressEvent( QMouseEvent *ev){
  if ( StartTimer ) {
    timer.stop();
    StartTimer = false;
    QTreeWidget::mousePressEvent(ev);
    return;
  }
  if ( EditElement ) {
    textChanged();
    return;
  }
  item = itemAt(ev->x(), ev->y());
  if ( ev->button() == Qt::LeftButton) {
    if ( item )
      if ( isItemSelected(item) && (selectedItems().count() == 1) ) {
        mousePosX = ev->x();
        mousePosY = ev->y();
        StartTimer = true;
        timer.start(500);
        return;
      }
    QTreeWidget::mousePressEvent(ev);
    return;
  }

  if ( ev->button() == Qt::RightButton) {
    if (ev->x()  > w) {
      if ( selectedItems().count() > 0 ) {
        foreach ( QTreeWidgetItem* iitem, selectedItems()) {
          iitem->setSelected(false);
        }
      }
      emit contextMenu(ev); return;
    }
    if ( item ) {
      if ( isItemSelected(item) && (selectedItems().count() == 1)) {
        emit contextMenu( ev, item->toolTip(0)); return;
      }
      if ( isItemSelected(item) && (selectedItems().count() > 1)) {
        QList<QString> list;
        foreach ( QTreeWidgetItem *iitem, selectedItems()) {
          list << iitem->toolTip(0);
        }
        emit contextMenu( ev, list);  return;
      }
      if ( selectedItems().count() == 0 ) {
        emit contextMenu( ev, item->toolTip(0));  return;
      }
      if ( !isItemSelected(item) && (selectedItems().count() > 0)) {
        foreach ( QTreeWidgetItem* iitem, selectedItems()) {
          iitem->setSelected(false);
        }
        emit contextMenu( ev, item->toolTip(0));  return;
      }
    }
    else {
      if ( selectedItems().count() > 0) {
        foreach ( QTreeWidgetItem* iitem, selectedItems()) {
          iitem->setSelected(false);
        }
      }
      emit contextMenu(ev); return;
    }
  }
}

void CTableViewer :: mouseMoveEvent(QMouseEvent *ev) {
  if ( StartTimer ) {
    timer.stop();
    StartTimer = false;
  }
  if ( EditElement ) textChanged();
  QTreeWidget::mouseMoveEvent(ev);
}

void CTableViewer :: mouseDoubleClickEvent(QMouseEvent *ev) {
  if ( StartTimer ) {
    timer.stop();
    StartTimer = false;
  }
  if ( EditElement ) textChanged();
  QTreeWidget::mouseDoubleClickEvent(ev);
}

void CTableViewer :: keyPressEvent(QKeyEvent * ev) {
  QString ffstr;
  if ( ev->key() == Qt::Key_Backspace || ev->key() == Qt::Key_Delete) {
    emit delSymbolAbc();
    return;
  }
  if ( ev->modifiers() == Qt::ControlModifier ) {
    if ( ev->key() == Qt::Key_A ) {
      foreach ( QTreeWidgetItem *item, itemsList)  item->setSelected(!item->isSelected());
      return;
    }
    if ( ev->key() == Qt::Key_X ) {
      QList<QTreeWidgetItem*> itt = selectedItems();
      if ( itt.count() == 1 ) {  ffstr = itt[0]->toolTip(0); emit contextAction("cut", ffstr); }
      if ( itt.count() >1 ) {
        QList<QString> actlst;
        foreach ( QTreeWidgetItem* item, itt ) actlst << item->toolTip(0);
        emit contextAction("cut", actlst);
      }
      return;
    }
    if ( ev->key() == Qt::Key_C ) {
      QList<QTreeWidgetItem*> itt = selectedItems();
      if ( itt.count() == 1 )  {  ffstr = itt[0]->toolTip(0); emit contextAction("copy", ffstr); }
      if ( itt.count() >1 ) {
        QList<QString> actlst;
        foreach ( QTreeWidgetItem* item, itt ) actlst << item->toolTip(0);
        emit contextAction("copy", actlst);
      }
      return;
    }
    if ( ev->key() == Qt::Key_V && QApplication::clipboard()->mimeData()->hasFormat("text/uri-list")) {
      emit contextAction("paste");
      return;
    }
    if ( ev->key() == Qt::Key_F && CurrentFileInfo.isWritable()) {
      emit contextAction("created");
      return;
    }
    if ( ev->key() == Qt::Key_S && CurrentFileInfo.isWritable()) {
      emit contextAction("creates");
      return;
    }
    if ( ev->key() == Qt::Key_Delete ) {
      QList<QTreeWidgetItem*> itt = selectedItems();
      if ( itt.count() == 1 )  {  ffstr = itt[0]->toolTip(0); emit contextAction("del", ffstr); }
      if ( itt.count() >1 ) {
        QList<QString> actlst;
        foreach ( QTreeWidgetItem* item, itt ) actlst << item->toolTip(0);
        emit contextAction("del", actlst);
      }
      return;
    }
    if ( ev->key() == Qt::Key_Enter ) {
      QList<QTreeWidgetItem*> itt = selectedItems();
      if ( itt.count() == 0 ) emit contextAction("prop");
      if ( itt.count() == 1 )  {  ffstr = itt[0]->toolTip(0); emit contextAction("prop", ffstr); }
      if ( itt.count() >1 ) {
        QList<QString> actlst;
        foreach ( QTreeWidgetItem* item, itt ) actlst << item->toolTip(0);
        emit contextAction("prop", actlst);
      }
      return;
    }
  }
  if ( ev->key() == Qt::Key_F2 ) {
    QList<QTreeWidgetItem*> itt = selectedItems();
    if ( itt.count() == 1 )  {  ffstr = itt[0]->toolTip(0); emit contextAction("ren", ffstr); }
    return;
  }
  emit pressKey(ev->text());
}

void CTableViewer :: textChanged() {
   fileinfo.setFile(item->toolTip( 0 ));
   QString str, oldname;
   oldname = fileinfo.fileName();
   if (Editor->text() == oldname ) {
      delete Editor;
      EditElement = false;
      emit refreshWrite();
      return;
    }
    else {
      str = Editor->text();
      delete Editor;
      EditElement = false;
      emit renameItem(item->toolTip(0), str);
  }
}

void CTableViewer :: setPixmapElement(int index_item, int index, int flag,
                                                                         QString descr, QString size, QString lastm, bool link)  {
   if ( index_item >=  itemsList.count()) return;
   QTreeWidgetItem* item = itemsList.value(index_item);

   if ( link ) {
     if ( flag == 0 ) {
       pixmap = *(pxCache->getIcon(index));
       painter.begin(&pixmap);
       painter.drawPixmap(0, 64, pixmap_link);
       painter.end();
     }
     else {
       pixmap = *(pxCache->getPixmap(index));
       painter.begin(&pixmap);
       painter.drawPixmap(0, 64, pixmap_link);
       painter.end();
     }
     item->setIcon(0, pixmap);
     font->setStyle(QFont::StyleItalic);
     item->setFont(0, *font);
     item->setFont(1, *font);
     item->setFont(2, *font);
     item->setFont(3, *font);
     font->setStyle(QFont::StyleNormal);
   }
   else {
     if (flag == 0) item->setIcon(0, *(pxCache->getIcon(index)));
     else item->setIcon(0, *(pxCache->getPixmap(index)));
   }

   item->setText(1, descr);
   item->setText(2, size);
   item->setText(3, lastm);

   bitIcon->setBit(index_item, true);
}

void CTableViewer :: initAreaIcon(int value) {
          if ( fileinfolist->count() == 0 || itemsList.count() == 0 || value <=0) return;
          QSize sizeElement = itemsList.value(0)->sizeHint(0);
          QSize sizeWidget = this->viewport()->size();
          int Max = this->verticalScrollBar()->maximum();
          float step;
          step = (float) fileinfolist->count() / (float) Max;
          startElement = step * (float) value;
          endElement = startElement + sizeWidget.height() / sizeElement.height();
          if ( startElement - 15 <= 0) startElement = 0; else startElement = startElement - 15;
          if ( endElement  >= fileinfolist->count()) {
            endElement = fileinfolist->count() - 1;
            startElement = endElement - (sizeWidget.height() / sizeElement.height()) *2;
            if (startElement < 0) startElement = 0;
          }
    }

void CTableViewer :: selectionChanged() {
  QList<QTreeWidgetItem*> list = selectedItems();
  if ( list.count() == 0 ) {
    emit refreshWrite();
    emit resetSelection();
    return;
  }
  if ( list.count() == 1) {
    QFileInfo finf(list.value(0)->toolTip(0));
    emit refreshWrite(finf, true);
    emit resetSelection();
    return;
  }
  if ( list.count() > 1) {
    emit refreshWrite(list.count());
    emit resetSelection();
    return;
  }
}

void CTableViewer :: findAbc(QString str) {
  QString name;
  if (str == "" ) {
    this->setCurrentItem(0);
    return;
  }
  foreach ( QTreeWidgetItem* item, itemsList) {
    if ( item->text(0).startsWith(str, Qt::CaseInsensitive)) {
      this->blockSignals(true);
      this->setCurrentItem(item);
      this->scrollToItem(item);
      this->blockSignals(false);
      emit refreshWrite(QFileInfo(item->toolTip(0)), false);
      emit resetSelection();
      return;
    }
  }
  this->blockSignals(true);
  this->setCurrentItem(0);
  this->blockSignals(false);
}

void CTableViewer :: updateDirectory(QString path) {
  if ( StartTimer ) {
    timer.stop();
    StartTimer = false;
  }
   while ( ThreadIcon->isRunning()) { QApplication::processEvents(); }
   while ( ThreadAreaIcon->isRunning()) { QApplication::processEvents(); }

    if ( TemplateFilter->count() == 0 && MimeFilter->count() == 0 ) {
        dir.setPath ( path );
        CurrentFileInfo.setFile(path);
        fileinfolist->clear();
        QFileInfoList templist, templist_hidden;
        if ( ShowHidden ) {
          templist_hidden = dir.entryInfoList ( QDir::Hidden );
          foreach ( fileinfo, templist_hidden )  if ( fileinfo.isDir()) *fileinfolist << fileinfo;
          templist = dir.entryInfoList (QDir::Dirs | QDir::NoDotAndDotDot );
          foreach ( fileinfo, templist ) if ( fileinfo.isDir()) *fileinfolist << fileinfo;
          foreach ( fileinfo, templist_hidden) if ( fileinfo.isFile() ) *fileinfolist << fileinfo;
          *fileinfolist << dir.entryInfoList(QDir::Files | QDir::System );
        }
        else {
          templist =  dir.entryInfoList ( QDir::Dirs | QDir::NoDotAndDotDot );
          foreach ( fileinfo, templist ) if ( fileinfo.isDir()) *fileinfolist << fileinfo;
          *fileinfolist << dir.entryInfoList ( QDir::Files | QDir::System);
        }
    }
    else {
      if ( TemplateFilter->count() != 0 ) {
        dir.setPath ( path );
        fileinfolist->clear();
        QFileInfoList templist, templist_hidden;
        if ( ShowHidden ) {
          if ( *ExcludeDirsTemplate ) templist_hidden = dir.entryInfoList ( QDir::Hidden );
            else  templist_hidden = dir.entryInfoList(*TemplateFilter, QDir::Hidden);
          foreach ( fileinfo, templist_hidden )  if ( fileinfo.isDir()) *fileinfolist << fileinfo;
          if ( *ExcludeDirsTemplate ) templist = dir.entryInfoList(QDir::Dirs | QDir::NoDotAndDotDot);
            else templist = dir.entryInfoList(*TemplateFilter, QDir::Dirs | QDir::NoDotAndDotDot);
          foreach ( fileinfo, templist ) if ( fileinfo.isDir()) *fileinfolist << fileinfo;
          foreach ( fileinfo, templist_hidden ) if ( fileinfo.isFile()) *fileinfolist << fileinfo;
          *fileinfolist << dir.entryInfoList(*TemplateFilter, QDir::Files | QDir::System);
        }
        else {
          if ( *ExcludeDirsTemplate ) templist =  dir.entryInfoList ( QDir::Dirs | QDir::NoDotAndDotDot );
            else templist =  dir.entryInfoList ( *TemplateFilter, QDir::Dirs | QDir::NoDotAndDotDot );
          foreach ( fileinfo, templist ) if ( fileinfo.isDir()) *fileinfolist << fileinfo;
          *fileinfolist << dir.entryInfoList ( *TemplateFilter, QDir::Files | QDir::System);
        }
      }
      if ( MimeFilter->count() != 0 ) {
        QFileInfoList templist, templist_dir_hidden, templist_dir, templist_files_hidden, templist_files,  templist_hidden;
        dir.setPath ( path );
        fileinfolist->clear();
        if ( ShowHidden ) {
          templist_hidden = dir.entryInfoList ( QDir::Hidden);
          foreach ( fileinfo, templist_hidden )  if ( fileinfo.isDir()) templist_dir_hidden << fileinfo;
          templist_dir =  dir.entryInfoList ( QDir::Dirs | QDir::NoDotAndDotDot );
          foreach ( fileinfo, templist_hidden ) if ( fileinfo.isFile()) templist_files_hidden << fileinfo;
          templist_files = dir.entryInfoList(QDir::Files | QDir::System);
        }
        else {
          templist_dir = dir.entryInfoList ( QDir::Dirs | QDir::NoDotAndDotDot );
          templist_files =  dir.entryInfoList ( QDir::Files | QDir::System);
        }
        templist.clear();
        if ( *ExcludeDirsMime ) {
          templist << templist_dir_hidden;
          templist << templist_dir;
        }
        else {
          foreach ( QFileInfo finf, templist_dir_hidden) {
            foreach ( QRegExp rexp, *MimeFilter) {
              if ( pxCache->getMimeSource(finf).contains(rexp)) { templist << finf; break; }
            }
          }
          foreach ( QFileInfo finf, templist_dir) {
            foreach ( QRegExp rexp, *MimeFilter) {
              if ( pxCache->getMimeSource(finf).contains(rexp)) { templist << finf; break; }
            }
          }
        }
        foreach ( QFileInfo finf, templist_files_hidden) {
          foreach ( QRegExp rexp, *MimeFilter) {
            if ( pxCache->getMimeSource(finf).contains(rexp)) { templist << finf; break; }
          }
        }
        foreach ( QFileInfo finf, templist_files) {
          foreach ( QRegExp rexp, *MimeFilter) {
            if ( pxCache->getMimeSource(finf).contains(rexp)) { templist << finf; break; }
          }
        }
        *fileinfolist = templist;
      }
    }

    int i = 0, fsize, p;
    QString descr;
    QPixmap *pix;
    QDir tempdir;
    QFileInfo targetlink;
    QList<QString> itemListString;
    QList<QString> fileListString;
    QFont fonti = *font;
    fonti.setItalic(true);
    QList<QString> label;
    label << tr("byte(s)") << tr("Kb") << tr("Mb") << tr("Gb") << tr("Tb");

    foreach ( fileinfo, *fileinfolist ) fileListString << fileinfo.absoluteFilePath();
    foreach ( QTreeWidgetItem *item, itemsList) {
      if ( !fileListString.contains(item->toolTip(0))) {
        this->takeTopLevelItem(i);
        itemsList.removeOne(item);
       --i;
      }
    ++i;
    }

    struct stat sb;
    i = 0;
    foreach ( QTreeWidgetItem *item, itemsList ) itemListString << item->toolTip(0);
    foreach (fileinfo, *fileinfolist) {
      if ( itemListString.contains(fileinfo.absoluteFilePath())) {
        ++i;
        continue;
      }
      QTreeWidgetItem* newitem = new QTreeWidgetItem();
      newitem->setToolTip(0, fileinfo.absoluteFilePath());
      newitem->setText(0, fileinfo.fileName());
      newitem->setTextAlignment (1, Qt::AlignLeft | Qt::AlignVCenter);
      newitem->setTextAlignment (2, Qt::AlignRight | Qt::AlignVCenter);
      newitem->setTextAlignment (3, Qt::AlignRight | Qt::AlignVCenter);
      if ( fileinfo.isSymLink()) {
        newitem->setFont(0, fonti);
        newitem->setFont(1, fonti);
        newitem->setFont(2, fonti);
        newitem->setFont(3, fonti);
        targetlink.setFile(fileinfo.symLinkTarget());
        while ( targetlink.isSymLink()) targetlink.setFile(targetlink.symLinkTarget());
        if (targetlink.exists()) {
            if ( targetlink.isDir()) {
              pixmap = *(pxCache->getIcon(2));
              if ( targetlink.absoluteFilePath() == QDir::homePath()) pixmap = *(pxCache->getIcon(1));
              if ( fileinfo.isHidden()) pixmap = *(pxCache->getIcon(4));
              painter.begin(&pixmap);
              painter.drawPixmap(0,64, pixmap_link);
              painter.end();
              newitem->setIcon(0, pixmap);
              newitem->setText(1, tr("link to catalog"));
              tempdir.setPath(targetlink.absoluteFilePath());
              newitem->setText(2, QString::number(tempdir.count()-2) + " " + tr("element(s)"));
              newitem->setText(3, targetlink.lastModified().toString("dd.MM.yy hh:mm"));
            }
            else {
              pixmap = *(pxCache->getIcon(targetlink, &descr));
              painter.begin(&pixmap);
              painter.drawPixmap(0,64, pixmap_link);
              painter.end();
              fsize = targetlink.size();                                                 //   установить размер
              for ( p = 0; p < 5; p++ ) {
                if ( fsize < 1000 ) break;
                fsize = fsize / 1000;
              }
              newitem->setIcon(0, pixmap);
              newitem->setText(1, tr("link to") + " " + descr);
              newitem->setText(2, QString::number(fsize) + " " + label[p]);
              newitem->setText(3, targetlink.lastModified().toString("dd.MM.yy hh:mm"));
              }
           }
           else {
             pixmap = *(pxCache->getIcon(113));
             painter.begin(&pixmap);
             painter.drawPixmap(0,64, pixmap_link);
             painter.end();
             newitem->setIcon(0, pixmap);
             newitem->setText(1, tr("broken link"));
             if ( lstat(fileinfo.absoluteFilePath().toLocal8Bit(), &sb) != -1) {
                newitem->setText(2, QString::number(sb.st_size) + " " + tr("byte"));
                newitem->setText(3, QDateTime::fromTime_t(sb.st_mtime).toString("dd.MM.yy hh:mm"));
             }
             else {
               newitem->setText(2, tr("0 byte"));
               newitem->setText(3, fileinfo.lastModified().toString("dd.MM.yy hh:mm"));
             }
           }
        }
        else {
          newitem->setFont(0, *font);
          newitem->setFont(1, *font);
          newitem->setFont(2, *font);
          newitem->setFont(3, *font);
          if ( fileinfo.isDir()) {
              newitem->setIcon(0, *(pxCache->getIcon(2)));
              if ( fileinfo.absoluteFilePath() == QDir::homePath()) newitem->setIcon(0, *(pxCache->getIcon(1)));
              if ( fileinfo.isHidden()) newitem->setIcon(0, *(pxCache->getIcon(4)));
              newitem->setText(1, "catalog");
              tempdir.setPath(fileinfo.absoluteFilePath());
              newitem->setText(2, QString::number(tempdir.count()-2) + " " + tr("element(s)"));
              newitem->setText(3, fileinfo.lastModified().toString("dd.MM.yy hh:mm"));
          }
          else {
            pix = pxCache->getIcon(fileinfo, &descr);
            newitem->setIcon(0, *pix);
            fsize = fileinfo.size();                                                 //   установить размер
            for (  p = 0; p < 5; p++ ) {
              if ( fsize < 1000 ) break;
              fsize = fsize / 1000;
            }
            newitem->setText(1, descr);
            newitem->setText(2, QString::number(fsize) + " " + label[p]);
            newitem->setText(3, fileinfo.lastModified().toString("dd.MM.yy hh:mm"));
          }
        }
      this->insertTopLevelItem(i, newitem);
      itemsList.insert(i, newitem);
      ++i;
    }
    QList<QTreeWidgetItem*> lstt = this->selectedItems();
    if ( lstt.count() == 1 ) emit refreshWrite(QFileInfo(lstt[0]->toolTip(0)), false);
    if ( lstt.count() > 1 ) emit refreshWrite(lstt.count());
    if ( lstt.count() == 0 )   emit refreshWrite();
}

void CTableViewer :: startDrag(Qt::DropActions act) {
  DragListItem = this->selectedItems();
  QMimeData *DragMime = new QMimeData();
  QList<QUrl> Url;
  foreach ( item, DragListItem) Url << item->toolTip(0);
  DragMime->setUrls(Url);
  Drag = new QDrag(this);
  Drag->setMimeData(DragMime);
  if ( DragListItem.count() == 1 ) Drag->setPixmap(DragListItem[0]->icon(0).pixmap(128, 128).scaled(iconSize()));
    else Drag->setPixmap(pxCache->getIcon(5)->scaled(iconSize()));
  Drag->exec(act);
}

void CTableViewer :: dragMoveEvent( QDragMoveEvent *ev) {
  DragTimer.stop();
  DragPos = ev->pos();
  if ( DragPos.y() > this->height()-60) {
    DragTimer.stop();
    ScrollDownTimer.start(100);
    return;
  }
  else {
    if ( DragPos.y() < 10 ) {
      DragTimer.stop();
      ScrollUpTimer.start(100);
      return;
    }
    else {
      ScrollDownTimer.stop();
      ScrollUpTimer.stop();
    }
  }

  item = itemAt(DragPos);
  if ( item && DragPos.x() < w) {
    QString filename = item->toolTip(0);
    int i;
    bool found  = false;
    for ( i = 0; i < fileinfolist->count(); ++i ) {
      if ( fileinfolist->value(i).absoluteFilePath() == filename){
        found = true;
        break;
      }
    }
    if ( found ) {
      fileinfo = fileinfolist->value(i);
      if ( fileinfo.isDir() && fileinfo.isWritable()) {
        ev->acceptProposedAction();
        DragTimer.start(700);
      }
      else ev->setAccepted(false);
    }
  }
  else
    if ( CurrentFileInfo.isWritable()) ev->setAccepted(true);
      else ev->setAccepted(false);
}

void CTableViewer :: dragEnterEvent(QDragEnterEvent* ev) {
  if ( ev->mimeData()->hasFormat("text/uri-list") && CurrentFileInfo.isWritable()) ev->acceptProposedAction();
}

void CTableViewer :: dragLeaveEvent(QDragLeaveEvent*) {
  DragTimer.stop();
  ScrollUpTimer.stop();
  ScrollDownTimer.stop();
}

void CTableViewer :: dropEvent(QDropEvent* ev) {
  DragTimer.stop();
  ScrollDownTimer.stop();
  ScrollUpTimer.stop();
  ev->acceptProposedAction();
  QList<QString> DropList;
  QFileInfo dropfinf;
  bool exm = false;
  DragPos = ev->pos();
  QList<QUrl> urllist = ev->mimeData()->urls();
  foreach ( QUrl url, urllist) DropList << url.path();
  if ( ev->keyboardModifiers() == Qt::NoModifier ) {
    exm = true;
    item = itemAt(DragPos);
    if ( item && DragPos.x() < w) emit dropMenu( DropList, QCursor::pos(), item->toolTip(0), exm, 0);
      else  emit dropMenu( DropList, QCursor::pos(), "", exm, 0);
    return;
  }
  if ( ev->keyboardModifiers() == Qt::CTRL ) {
    item = itemAt(DragPos);
    if ( item && DragPos.x() < w ) emit dropMenu(DropList, QCursor::pos(), item->toolTip(0), exm, 1);
      else emit dropMenu(DropList, QCursor::pos(), "", exm, 1);
    return;
  }
  if ( ev->keyboardModifiers() == Qt::SHIFT) {
    item = itemAt(DragPos);
    if ( item && DragPos.x() < w ) emit dropMenu( DropList, QCursor::pos(), item->toolTip(0), exm, 2);
      else emit dropMenu(DropList, QCursor::pos(),  "", exm,  2);
    return;
  }
  if ( ev->keyboardModifiers() == Qt::CTRL + Qt::SHIFT) {
    item = itemAt(DragPos);
    if ( item && DragPos.x() < w ) emit dropMenu(DropList, QCursor::pos(), item->toolTip(0), exm, 4);
      else emit dropMenu(DropList, QCursor::pos(), "", exm, 4);
  }
}

void CTableViewer :: timeoutDragTimer() {
  DragTimer.stop();
  item = itemAt(DragPos);
  if ( item ) {
    QFileInfo dragtarget(item->toolTip(0));
    if (dragtarget.isDir()) emit expandedItem(item);
  }
}

void CTableViewer :: scrollUp() {
  if ( verticalScrollBar()->value() >  verticalScrollBar()->minimum()) verticalScrollBar()->setValue(verticalScrollBar()->value() - 1);
}

void CTableViewer :: scrollDown() {
  if ( verticalScrollBar()->value() < verticalScrollBar()->maximum()) verticalScrollBar()->setValue(verticalScrollBar()->value() + 1);
}

CTableViewer :: ~CTableViewer() {
  while ( ThreadIcon->isRunning() || ThreadAreaIcon->isRunning()) QApplication::processEvents();
  delete ThreadIcon;
  delete ThreadAreaIcon;
  if ( bitIcon != NULL) delete bitIcon;
}
