#include "../../Include/TextViewer.h"

CTextViewer :: CTextViewer(QWidget * wgt) : QTextEdit(wgt) {
}

void CTextViewer :: setTextViewerTool ( CTextViewerTool * text_viewer_tool ) {
  TextViewerTool = text_viewer_tool;
  setFontSize();
  connect ( this, SIGNAL(textChanged()), SLOT(ifTextChanged()));
  connect ( TextViewerTool->DontEditButton, SIGNAL(clicked()), SLOT(dontEditClicked()));
  connect ( TextViewerTool->UndoButton, SIGNAL(clicked()), SLOT(undoButtonClicked()));
  connect ( TextViewerTool->RedoButton, SIGNAL(clicked()), SLOT(redoButtonClicked()));
  connect ( TextViewerTool->RefreshButton, SIGNAL(clicked()), SLOT(reloadText()));
  connect ( TextViewerTool->ScaleButtonPlus, SIGNAL(clicked()), SLOT(scaleTextPlus()));
  connect ( TextViewerTool->ScaleButtonMinus, SIGNAL(clicked()), SLOT(scaleTextMinus()));
  connect ( TextViewerTool->SaveButton, SIGNAL(clicked()), SLOT(saveText()));
  connect ( TextViewerTool->SaveAsButton, SIGNAL(clicked()), SLOT(saveTextAs()));
}

void CTextViewer :: contextMenuEvent(QContextMenuEvent* event) {
  if ( this->isReadOnly()) return;
  QMenu * menu = createStandardContextMenu(event->pos());
  menu->setPalette(epal);
  menu->setFont(*Font);
  QList<QAction*> list = menu->findChildren<QAction*>();
  QList<QAction*> actionlist;
  QString str;
  foreach ( QAction *actt, list ) {
    str = actt->text();
    if ( actt->text().contains("&Undo"))  actionlist << actt;
    if ( actt->text().contains("&Redo"))  actionlist << actt;
    if ( actt->text().contains("Cu&t"))  actionlist << actt;
    if ( actt->text().contains("&Copy"))  actionlist << actt;
    if ( actt->text().contains("&Paste"))  actionlist << actt;
    if ( actt->text().contains("Delete"))  actionlist << actt;
    if ( actt->text().contains("Select"))  actionlist << actt;
    if ( actt->text().contains("Insert"))  actionlist << actt;
  }

  actionlist[0]->setText(tr("Undo"));
  actionlist[0]->setShortcut(QKeySequence(Qt::ControlModifier + Qt::Key_Z));
  actionlist[1]->setText(tr("Redo"));
  actionlist[1]->setShortcut(QKeySequence(Qt::ControlModifier + Qt::ShiftModifier + Qt::Key_Z));
  actionlist[2]->setText(tr("Cut"));
  actionlist[2]->setShortcut(QKeySequence(Qt::ControlModifier + Qt::Key_X));
  actionlist[3]->setText(tr("Copy"));
  actionlist[3]->setShortcut(QKeySequence(Qt::ControlModifier + Qt::Key_C));
  actionlist[4]->setText(tr("Paste"));
  actionlist[4]->setShortcut(QKeySequence(Qt::ControlModifier + Qt::Key_V));
  actionlist[5]->setText(tr("Delete"));
  actionlist[6]->setText(tr("Select all"));
  actionlist[6]->setShortcut(QKeySequence(Qt::ControlModifier + Qt::Key_A));
  actionlist[7]->setText(tr("Insert Unicode control character"));

  QList<QMenu*> menulist = menu->findChildren<QMenu*>();
  menulist[0]->setPalette(epal);
  menulist[0]->setFont(*Font);

  menu->exec(event->globalPos());
}

void CTextViewer :: ifTextChanged() {
  if ( document()->isUndoAvailable()) TextViewerTool->UndoButton->setEnabled(true);
    else TextViewerTool->UndoButton->setEnabled(false);
  if ( document()->isRedoAvailable()) TextViewerTool->RedoButton->setEnabled(true);
    else TextViewerTool->RedoButton->setEnabled(false);
  if ( document()->isModified()) TextViewerTool->StateLabel->setText(tr("Changed"));
    else TextViewerTool->StateLabel->setText("");
}

void CTextViewer :: dontEditClicked() {
  if ( TextViewerTool->DontEditButton->isChecked()) {
    TextViewerTool->RedoButton->setEnabled(false);
    TextViewerTool->UndoButton->setEnabled(false);
    setReadOnly(true);
  }
  else {
    setReadOnly(false);
    TextViewerTool->UndoButton->setEnabled(document()->isUndoAvailable());
    TextViewerTool->RedoButton->setEnabled(document()->isRedoAvailable());
  }
}

void CTextViewer :: undoButtonClicked() {
  undo();
  if ( !document()->isUndoAvailable()) TextViewerTool->UndoButton->setEnabled(false);
    else TextViewerTool->UndoButton->setEnabled(true);
  if ( !document()->isRedoAvailable()) TextViewerTool->RedoButton->setEnabled(false);
    else  TextViewerTool->RedoButton->setEnabled(true);
}

void CTextViewer :: redoButtonClicked() {
  redo();
  if ( !document()->isRedoAvailable()) TextViewerTool->RedoButton->setEnabled(false);
  if ( !document()->isUndoAvailable()) TextViewerTool->UndoButton->setEnabled(false);
}

void CTextViewer :: reloadText() {
  QFile file(FileName);
  if ( !file.open(QIODevice::ReadOnly)) {
    errorMessage(file.errorString());
    return;
  }
  QTextStream stream(&file);
  QString str = stream.readAll();
  if ( stream.status() != QTextStream::Ok ) errorMessage(tr("Error reading"));
  file.close();
  this->setPlainText(str);
  TextViewerTool->StateLabel->setText("");
}

void CTextViewer :: scaleTextPlus() {
  Scale++;
  if ( Scale >= Font->pixelSize()) TextViewerTool->ScaleButtonPlus->setEnabled(false);
  TextViewerTool->ScaleButtonMinus->setEnabled(true);
  setFontSize();
}

void CTextViewer :: scaleTextMinus() {
  Scale = Scale--;
  if ( Scale <= -Font->pixelSize() + 1 ) TextViewerTool->ScaleButtonMinus->setEnabled(false);
  TextViewerTool->ScaleButtonPlus->setEnabled(true);
  setFontSize();
}

void CTextViewer :: setFontSize() {
  QFont font = *Font;
  if ( Scale <= - font.pixelSize()) Scale = -font.pixelSize() + 1;
  qreal procent = (qreal) ((font.pixelSize() + Scale) / (qreal) ( font.pixelSize()) ) * 100.;
  int scaletxt = procent;
  TextViewerTool->ScaleText->setText(QString::number(scaletxt) + "%");
  font.setPixelSize(font.pixelSize() + Scale);
  this->setFont(font);
}

void CTextViewer :: loadText( QString fname) {
  FileName = fname;
  QFileInfo fileinfo(fname);
  dir = fileinfo.filePath();
  reloadText();
}

void CTextViewer :: saveText() {
  QFile file(FileName);
  if ( !file.open(QIODevice::WriteOnly)) {
    errorMessage(file.errorString());
    return;
  }
  QTextStream stream(&file);
  stream << this->toPlainText();
  if ( stream.status() != QTextStream::Ok ) errorMessage ( tr("Error writing"));
  file.close();
  this->document()->setModified(false);
  TextViewerTool->StateLabel->setText("");
}

void CTextViewer :: errorMessage(QString errstr) {
    Message = new CMessageBox(TextViewerTool->pxCache, this);
    Message->setPalette(epal);
    Message->setFont(*Font);
    Message->createInform(*(TextViewerTool->pxCache->getIcon(182)));
    Message->setModal(true);
    Message->inform->setText("<b>" + tr("Filed to save the file.") + " </b>");
    Message->detals->setText(tr("System message:") + " <br><br>" + QString("       ") + errstr +
                                                    "<br><br>" + tr("Check permissions and/or space from disk"));
    Message->exec();
    delete Message;
    return;
}

void CTextViewer :: saveTextAs () {
  QString str = QFileDialog::getSaveFileName(this, tr("Save file as..."), dir, ".txt");
  if (str.isEmpty()) return;
  QFile file(str);
  if ( !file.open(QIODevice::WriteOnly)) {
    errorMessage(file.errorString());
    return;
  }
  QTextStream stream(&file);
  stream << this->toPlainText();
  if ( stream.status() != QTextStream::Ok ) {
    errorMessage ( tr("Error writing"));
    file.close();
    return;
  }
  file.close();
  FileName = str;
  QFileInfo fileinfo(file);
  dir = fileinfo.filePath();
  TextViewerTool->StateLabel->setText("");
  this->document()->setModified(false);
  if ( !document()->isRedoAvailable()) TextViewerTool->RedoButton->setEnabled(false);
    else TextViewerTool->RedoButton->setEnabled(true);
  if ( !document()->isUndoAvailable()) TextViewerTool->UndoButton->setEnabled(false);
    else TextViewerTool->UndoButton->setEnabled(true);
    emit changeNewPath(FileName);
}

void CTextViewer :: setCastomPalette(QPalette palette, QColor color) {
  epal = palette;
  changeBackgroundPalette(&epal, color);
  this->setPalette(palette);
  this->verticalScrollBar()->setPalette(epal);
  this->horizontalScrollBar()->setPalette(epal);
  TextViewerTool->setBackColor(darkerColor(color, 30));
  QColor fcolor = palette.color(QPalette::Text);
  TextViewerTool->setFontColor(fcolor);
}
