#include "../../Include/ThreadOp.h"
#include "../../Include/MessageBox.h"

CThreadMonitor :: CThreadMonitor (QWidget *wgt) : QScrollArea(wgt) {
  setWindowFlags(Qt::ToolTip);
  setAutoFillBackground(true);
  layout = new QVBoxLayout();
  layout->setMargin(0);
  MainFrame = new QFrame();
  MainFrame->setLayout(layout);
  setWidget(MainFrame);
  countProcess = 0;
  Visible = false;
  bitState = new QBitArray(10);
  bitState->setBit(7, false);
  bitState->setBit(8, false);
  bitState->setBit(9, false);
  setFrameStyle(QFrame::Panel | QFrame::Sunken);
  setLineWidth(2);
  Lock = false;
}

void CThreadMonitor :: execOperate(QList<QString> source, QString target, int type) {
  CProgressW * wgt = new CProgressW(source, target, type );
  wgt->BColor = Color;
  wgt->FColor = this->palette().color(QPalette::Text);
  for ( int i = 0; i < 7; ++i ) {
    wgt->bitState->setBit(i, bitState->testBit(i));
  }

  connect ( wgt, SIGNAL(showProgressW(CProgressW*)), SLOT(showProgressW(CProgressW*)));
  connect ( wgt, SIGNAL(endExec(CProgressW*)), SLOT(deleteOperate(CProgressW*)));
  connect ( wgt, SIGNAL(existsName(QString, QString)), this, SLOT(ifExistsName(QString, QString)), Qt::BlockingQueuedConnection);
  connect ( wgt, SIGNAL(errorCriticalComplete(CThreadOp*)), this,
                             SLOT(errorCritical(CThreadOp*)), Qt::BlockingQueuedConnection);

  wgt->wtimer->start(1000);
  wgt->threadGS->start();
  wgt->CThreadOp::start();
  wgt->CThreadOp::exec();
}

void CThreadMonitor :: showProgressW(CProgressW* progr) {
  connect ( progr, SIGNAL(abortOperation(CThreadOp*)), this, SLOT(abortOperation(CThreadOp*)), Qt::BlockingQueuedConnection);
  layout->addWidget(progr->frame);
  countProcess++;
  QFont font(this->font());
  QFontMetrics metric ( font );
  int w = metric.width("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa");
  int h = metric.height();
  progr->frame->setFixedSize(w, h*6 + 60);
  progr->cancelButton->setIcon(*(pxCache->getIcon(171)));
  setOptimumSize();
  emit runningProcess();
}

void CThreadMonitor :: deleteOperate(CProgressW* progr) {
      if ( progr->errprint && progr->bufferErrorFile.count() > 0 ) {
        setLock(true);
        CMessageBox *error = new CMessageBox(pxCache);
        error->createResultOperate(pxCache->getIcon(181)->scaled(96, 96));
        error->setCastomPalette(this->palette(), Color);
        error->setFont(this->font());
        error->setWindowTitle(tr("The surgery was not too successful") + " - Bathyscaphe");
        if ( progr->type == 1 ){
           error->inform->setText("<h3>" + tr("Error:") + "</h3> <br>" + tr("When performing the copy operation error occurred."));
           error->detals->setText(tr("Check the permissions."));
	}
        if ( progr->type == 2 ) {
           error->inform->setText("<h3>" + tr("Error:") + "</h3> <br>" + tr("When performing the move operation error occurred."));
           error->detals->setText(tr("Check the permissions."));
	}
	if ( progr->type == 3 ) {
           error->inform->setText("<h3>" + tr("Error:") + "</h3> <br>" + tr("When performing the delete operation error occurred."));
           error->detals->setText(tr("Check the permissions."));
	}
        for ( int i = 0; i < progr->bufferErrorFile.count(); ++i) {
          QTreeWidgetItem *item = new QTreeWidgetItem();
          if ( progr->bufferErrorStatus[i] == 'W') item->setIcon(0, *(pxCache->getIcon(181)));
          if ( progr->bufferErrorStatus[i] == 'C') item->setIcon(0, *(pxCache->getIcon(182)));
          item->setText(0, progr->bufferErrorFile[i]);
    	  item->setText(1, progr->bufferErrorMessage[i]);
    	  item->setText(2, progr->bufferErrorSystem[i]);
          error->treewidget->addTopLevelItem(item);
        }
        error->exec();
        error->treewidget->clear();
        delete error;
        setLock(false);
      }
  if ( progr->trueW ) {
      countProcess--;
      if ( countProcess == 0 ) {
        emit noRunningProcess();
      }
      layout->removeWidget(progr->frame);
      delete progr->progressBar;
      delete progr->labelHeader;
      delete progr->countElement;
      delete progr->detals;
      delete progr->cancelButton;
      delete progr->buttonLayout;
      delete progr->layout;
      delete progr->frame;
      setOptimumSize();
      emit resizeThis();
  }
  delete progr->bitState;
  delete progr;
}


void CThreadMonitor :: abortOperation(CThreadOp * wgt) {
  setLock(true);
  CMessageBox *query = new CMessageBox(pxCache);
  query->createQuestion(pxCache->getIcon(183)->scaled(64, 64));
  query->setPalette(this->palette());
  query->setFont(this->font());
  query->setWindowTitle(tr("Abort operation") + " - Bathyscaphe");
  query->inform->setText(tr("Are you sure you want to abort the operation?"));
  if ( query->exec() == 1 ) {
    delete query;
    wgt->bitState->setBit(10, false);
    wgt->bitState->setBit(9, true);
  }
  else {
    delete query;
    wgt->bitState->setBit(10, false);
  }
  setLock(false);
}

void CThreadMonitor :: errorCritical(CThreadOp * opp) {
  setLock(true);
  CMessageBox *error = new CMessageBox(pxCache);
  error->createResultOperate(pxCache->getIcon(182)->scaled(96, 96));
  error->setCastomPalette(this->palette(), Color);
  error->setFont(this->font());
  error->setWindowTitle(tr("Critical error") + " - Bathyscaphe");
  if ( opp->type == 1)
     error->inform->setText("<h3>" + tr("Error:")  + "</h3> <br>" + tr("When performing copy operations critical error has occurred.") + "<br>" + tr("The operation will be aborted."));
  if ( opp->type == 2 )
     error->inform->setText("<h3>" + tr("Error:") + " </h3> <br>" + tr("When performing move operations critical error has occurred.") + "<br>" + tr("The operation will be aborted."));
  error->detals->setText(tr("Check the permissions and / or  free space disk"));

  for ( int i = 0; i < opp->bufferErrorFile.count(); ++i) {
    QTreeWidgetItem *item = new QTreeWidgetItem();
    if ( opp->bufferErrorStatus[i] == 'W') item->setIcon(0, *(pxCache->getIcon(181)));
    if ( opp->bufferErrorStatus[i] == 'C') item->setIcon(0, *(pxCache->getIcon(182)));
    item->setText(0, opp->bufferErrorFile[i]);
    item->setText(1, opp->bufferErrorMessage[i]);
    item->setText(2, opp->bufferErrorSystem[i]);
    error->treewidget->addTopLevelItem(item);
  }
  error->exec();
  error->treewidget->clear();
  delete error;
  opp->bitState->setBit(10, false);
  opp->bitState->setBit(9, true);
  setLock(false);
}



void CThreadMonitor :: setOptimumSize() {
  int i = layout->count();
  if ( i > 0) {
    OptimumSize.setWidth(layout->itemAt(0)->widget()->width());
    OptimumSize.setHeight(layout->itemAt(0)->widget()->height() * i + 10*i);
    MainFrame->setFixedSize(OptimumSize);
  }
  else {
    OptimumSize.setWidth(0);
    OptimumSize.setHeight(0);
    MainFrame->setFixedSize(OptimumSize);
  }
}

void CThreadMonitor :: ifExistsName(QString source, QString target) {
  CProgressW * wgt = qobject_cast<CProgressW*>(this->sender());
  QFileInfo fileinfo_source(source);
  QFileInfo fileinfo_target(target);
  int i;
  Template templ1, templ2;
  templ1 = pxCache->fullTemplate(fileinfo_source);
  QString string;
  string = target;
  QFile efile;
    while(true) {
  if ( fileinfo_source.absoluteFilePath() == fileinfo_target.absoluteFilePath() ) {

      if ( wgt->bitState->testBit(11)) {
        wgt->bitState->setBit(7, true);
        wgt->bitState->setBit(8, true);
        return;
      }
      CMessageBox *warning = new CMessageBox(pxCache);
      setLock(true);
      warning->createWarningCopyOnIt(pxCache->getIcon(181)->scaled(96, 96));
      warning->setCastomPalette(this->palette(), Color);
      warning->setFont(this->font());
      warning->setWindowTitle(tr("This name already exists") + " - Bathyscaphe");
      warning->inform->setText("<h3>" + tr("Warning!") + " </h3> <br>" + tr("This item can only be recorded under another name."));
      warning->setTextIntoWrite(templ1.FileName);
      warning->lineedit1->setSelection(0, templ1.BaseName.length());
      warning->lineedit1->setFocus();
      warning->labelname1->setText("<h3>" + templ1.FullName + "</h3>");
      warning->labelicon1->setPixmap(templ1.Icon.scaled(96,96));
      if (fileinfo_source.isDir() && !fileinfo_source.isSymLink())
        warning->labeldetals1->setText("<table><tr><td>" + tr("Type") +":</td><td>" +  templ1.Type + "</td></tr><tr><td>"
                                                            + tr("Size") + ":</td><td>" + templ1.Size   + "</td></tr><tr><td>"
                                                            + tr("Created") + ":</td><td>" + templ1.Created + "</td></tr><tr><td>"
                                                            + tr("Modified") + ":</td><td>" + templ1.LastModified + "</td></tr></table>");
      else
        warning->labeldetals1->setText("<table><tr><td>" + tr("Type") +":</td><td>" +  templ1.Type + "</td></tr><tr><td>"
                                                            + tr("Description (from extended)") +":</td><td>" + templ1.Descr + "</td></tr><tr><td>"
                                                            + tr("Contents") + ":</td><td>" + templ1.DescrFromContents + "</td></tr><tr><td>"
                                                            + tr("Size") + ":</td><td>" + templ1.Size   + "</td></tr><tr><td>"
                                                            + tr("Created") + ":</td><td>" + templ1.Created + "</td></tr><tr><td>"
                                                            + tr("Modified") + ":</td><td>" + templ1.LastModified + "</td></tr></table>");

      i = warning->execInputString(&string);
      delete warning;
      if ( i == 5 ) {
        wgt->bitState->setBit(11, true);
        wgt->bitState->setBit(7, true);
        wgt->bitState->setBit(8, true);
        setLock(false);
        return;
      }
      if ( i == 4 ) {
        wgt->bitState->setBit(7, true);
        wgt->bitState->setBit(8, true);
        setLock(false);
        return;
      }
      if ( i == 1 ) {
        if ( string == "" || string == "." || string == ".." || string.contains("/") || string.contains("\\")) {
          CMessageBox *error = new CMessageBox(pxCache);
          error->createInform(pxCache->getIcon(182)->scaled(96, 96));
          error->setCastomPalette(this->palette(), Color);
          error->setFont(this->font());
          error->setWindowTitle(tr("Not valid name") + " - Bathyscaphe");
          error->inform->setText("<h3>" + tr("Error:") + " </h3>  <br>"  + tr("The name of the file or directory can not be '.' or'..', <br>contain the characters '/' or '\\' or empty.."));
          error->detals->setText(tr("Please try again."));
          error->exec();
          delete error;
          continue;
        }
        fileinfo_target.setFile(fileinfo_target.absolutePath() + QDir::separator() + string);
        if ( fileinfo_target.exists() || fileinfo_target.isSymLink()) {
          continue;
        }
        else {
          wgt->newName = string;
          wgt->bitState->setBit(8, true);
          setLock(false);
          return;
        }
      }
      if ( i == 2 ) {
       CMessageBox *query = new CMessageBox(pxCache);
       query->createQuestion(pxCache->getIcon(183)->scaled(64, 64));
       query->setCastomPalette(this->palette(), Color);
       query->setFont(this->font());
       query->setWindowTitle(tr("Aborting operation") + " - Bathyscaphe");
       query->inform->setText(tr("Are you sure you want to abort the operation?"));
       if ( query->exec() == 1 ) {
         delete query;
         wgt->bitState->setBit(9, true);
         setLock(false);
         return;
       }
       else delete query;
     }
    setLock(false);
    }
//  }

//  while ( true ) {
    if ( (fileinfo_source.isFile() || fileinfo_source.isSymLink()) && (fileinfo_target.isFile() || fileinfo_target.isSymLink())) {      //1
       wgt->newName = fileinfo_target.fileName();
       if ( wgt->bitState->testBit(1)) {
         wgt->bitState->setBit(7, true);
         wgt->bitState->setBit(8, true);
         return;
       }
       if ( wgt->bitState->testBit(0)) {
	     efile.setFileName(target);
         if ( !efile.remove()) {
           wgt->bufferErrorFile << target;
	       wgt->bufferErrorMessage << tr("Could not overwrite the file");
	       wgt->bufferErrorSystem << efile.errorString();
           wgt->bufferErrorStatus << 'C';
	       wgt->errprint = false;
	       errorCritical(wgt);
	       //wgt->bitState->setBit(10, true);
	       wgt->bitState->setBit(9, true);
           wgt->bitState->setBit(7, true);
           wgt->bitState->setBit(8,true);
           return;
         }
         else {
           wgt->bitState->setBit(8, true);
           return;
         }
       }
       CMessageBox *warning = new CMessageBox(pxCache);
       setLock(true);
       warning->createWarningCopy(pxCache->getIcon(181)->scaled(96, 96));
       i = showWarningCopy(warning, fileinfo_source, fileinfo_target, templ1, templ2, &string);
       if ( i == 1) {
          if ( string == "" || string == "." || string == ".." || string.contains("/") || string.contains("\\")) {
            CMessageBox *error = new CMessageBox(pxCache);
            error->createInform(pxCache->getIcon(182)->scaled(96, 96));
            error->setCastomPalette(this->palette(), Color);
            error->setFont(this->font());
            error->setWindowTitle(tr("Not valid name") + " - Bathyscaphe");
            error->inform->setText("<h3>" + tr("Error:") + " </h3>  <br>"  + tr("The name of the file or directory can not be '.' or'..', <br>contain the characters '/' or '\\' or empty.."));
            error->detals->setText("Please try again.");
            error->exec();
            delete error;
            continue;
          }
          fileinfo_target.setFile(fileinfo_target.absolutePath() + QDir::separator() + string);
          if ( fileinfo_target.exists()) {
            continue;
          }
          else {
            wgt->newName = string;
            wgt->bitState->setBit(8, true);
            setLock(false);
            return;
          }
       }
       if ( i ==  4 ) {
         wgt->bitState->setBit(7, true);
         wgt->bitState->setBit(8, true);
         setLock(false);
         return;
       }
       if ( i == 5 ) {
         wgt->bitState->setBit(7, true);
         wgt->bitState->setBit(1, true);
         wgt->bitState->setBit(8, true);
         setLock(false);
         return;
       }
       if ( i == 6 ) {
	 efile.setFileName(target);
         if ( !efile.remove()) {
           wgt->bufferErrorFile << target;
	       wgt->bufferErrorMessage << tr("Could not overwrite the file");
	       wgt->bufferErrorSystem << efile.errorString();
           wgt->bufferErrorStatus << 'C';
	       wgt->errprint = false;
	       errorCritical(wgt);
  	       //wgt->bitState->setBit(10, true);
	       wgt->bitState->setBit(9, true);
           wgt->bitState->setBit(7, true);
           wgt->bitState->setBit(8, true);
           setLock(false);
           return;
         }
         else {
              wgt->bitState->setBit(8, true);
              setLock(false);
              return;
            }
       }
        if ( i == 7 ) {
         wgt->bitState->setBit(0, true);
	 efile.setFileName(target);
         if ( !efile.remove()) {
           wgt->bufferErrorFile << target;
           wgt->bufferErrorMessage << tr("Could not overwrite the file");
	       wgt->bufferErrorSystem << efile.errorString();
           wgt->bufferErrorStatus << 'C';
	       wgt->errprint = false;
	       errorCritical(wgt);
	       //wgt->bitState->setBit(10, true);
	       wgt->bitState->setBit(9, true);
           wgt->bitState->setBit(7, true);
           wgt->bitState->setBit(8, true);
           setLock(false);
           return;
         }
         else {
              wgt->bitState->setBit(8, true);
              setLock(false);
              return;
            }
       }
       if ( i == 2 ) {
       CMessageBox *query = new CMessageBox(pxCache);
       query->createQuestion(pxCache->getIcon(183)->scaled(64, 64));
       query->setCastomPalette(this->palette(), Color);
       query->setFont(this->font());
       query->setWindowTitle(tr("Aborting operation") + " - Bathyscaphe");
       query->inform->setText(tr("Are you sure you want to abort the operation?"));
       if ( query->exec() == 1 ) {
         delete query;
         wgt->bitState->setBit(9, true);
         setLock(false);
         return;
       }
       else delete query;
     }
    setLock(false);
    }

    if ( (fileinfo_source.isSymLink() || fileinfo_source.isFile())  && (fileinfo_target.isDir() && !fileinfo_target.isSymLink())) {  //2
       wgt->newName = fileinfo_target.fileName();
       if ( wgt->bitState->testBit(2)) {
         wgt->bitState->setBit(7, true);
         wgt->bitState->setBit(8, true);
         return;
       }
       CMessageBox *warning = new CMessageBox(pxCache);
       setLock(true);
       warning->createWarningCopy(pxCache->getIcon(181)->scaled(96, 96), false);
       i = showWarningCopy(warning, fileinfo_source, fileinfo_target, templ1, templ2, &string);
       if ( i == 1) {
          if ( string == "" || string == "." || string == ".." || string.contains("/") || string.contains("\\")) {
            CMessageBox *error = new CMessageBox(pxCache);
            error->createInform(pxCache->getIcon(182)->scaled(96, 96));
            error->setCastomPalette(this->palette(), Color);
            error->setFont(this->font());
            error->setWindowTitle(tr("Not valid name") + " - Bathyscaphe");
            error->inform->setText("<h3>" + tr("Error:") + " </h3>  <br>"  + tr("The name of the file or directory can not be '.' or'..', <br>contain the characters '/' or '\\' or empty.."));
            error->detals->setText(tr("Please try again."));
            error->exec();
            delete error;
            continue;
          }
          fileinfo_target.setFile(fileinfo_target.absolutePath() + QDir::separator() + string);
          if ( fileinfo_target.exists()) {
            continue;
          }
          else {
            wgt->newName = string;
            wgt->bitState->setBit(8, true);
            setLock(false);
            return;
         }
       }
       if ( i ==  4 ) {
         wgt->bitState->setBit(7, true);
         wgt->bitState->setBit(8, true);
         setLock(false);
         return;
       }
       if ( i == 5 ) {
         wgt->bitState->setBit(7, true);
         wgt->bitState->setBit(2, true);
         wgt->bitState->setBit(8, true);
         setLock(false);
         return;
       }
       if ( i == 2 ) {
       CMessageBox *query = new CMessageBox(pxCache);
       query->createQuestion(pxCache->getIcon(183)->scaled(64, 64));
       query->setCastomPalette(this->palette(), Color);
       query->setFont(this->font());
       query->setWindowTitle(tr("Aborting operation") + " - Bathyscaphe");
       query->inform->setText(tr("Are you sure you want to abort the operation?"));
       if ( query->exec() == 1 ) {
         delete query;
         wgt->bitState->setBit(9, true);
         setLock(false);
         return;
       }
       else delete query;
     }
    setLock(false);
   }

    if ((fileinfo_source.isDir() && !fileinfo_source.isSymLink()) && (fileinfo_target.isFile() || fileinfo_target.isSymLink())) {   //3
       wgt->newName = fileinfo_target.fileName();
       if ( wgt->bitState->testBit(4)) {
         wgt->bitState->setBit(7, true);
         wgt->bitState->setBit(8, true);
         return;
       }
       if ( wgt->bitState->testBit(3)) {
	 efile.setFileName(target);
         if ( !efile.remove()) {
           wgt->bufferErrorFile << target;
	       wgt->bufferErrorMessage << tr("Could not overwrite the file");
	       wgt->bufferErrorSystem << efile.errorString();
           wgt->bufferErrorStatus << 'C';
	       wgt->errprint = false;
	       errorCritical(wgt);
	       //wgt->bitState->setBit(10, true);
	       wgt->bitState->setBit(9, true);
           wgt->bitState->setBit(7, true);
           wgt->bitState->setBit(8, true);
           setLock(false);
           return;
         }
         else {
           wgt->bitState->setBit(8, true);
           return;
         }
       }
      CMessageBox *warning = new CMessageBox(pxCache);
      setLock(true);
       warning->createWarningCopy(pxCache->getIcon(181)->scaled(96, 96));
       i = showWarningCopy(warning, fileinfo_source, fileinfo_target, templ1, templ2, &string);
       if ( i == 1) {
          if ( string == "" || string == "." || string == ".." || string.contains("/") || string.contains("\\")) {
            CMessageBox *error = new CMessageBox(pxCache);
            error->createInform(pxCache->getIcon(182)->scaled(96, 96));
            error->setCastomPalette(this->palette(), Color);
            error->setFont(this->font());
            error->setWindowTitle(tr("Not valid name") + " - Bathyscaphe");
            error->inform->setText("<h3>" + tr("Error:") + " </h3>  <br>"  + tr("The name of the file or directory can not be '.' or'..', <br>contain the characters '/' or '\\' or empty.."));
            error->detals->setText(tr("Please try again."));
            error->exec();
            delete error;
            continue;
          }
          fileinfo_target.setFile(fileinfo_target.absolutePath() + QDir::separator() + string);
          if ( fileinfo_target.exists()) {
            continue;
          }
          else {
            wgt->newName = string;
            wgt->bitState->setBit(8, true);
            setLock(false);
            return;
          }
       }
       if ( i ==  4 ) {
         wgt->bitState->setBit(7, true);
         wgt->bitState->setBit(8, true);
         setLock(false);
         return;
       }
       if ( i == 5 ) {
         wgt->bitState->setBit(7, true);
         wgt->bitState->setBit(4, true);
         wgt->bitState->setBit(8, true);
         setLock(false);
         return;
       }
       if ( i == 6 ) {
	 efile.setFileName(target);
         if ( !efile.remove()) {
           wgt->bufferErrorFile << target;
	   wgt->bufferErrorMessage << tr("Could not overwrite the file");
	   wgt->bufferErrorSystem << efile.errorString();
           wgt->bufferErrorStatus << 'C';
	   wgt->errprint = false;
	   errorCritical(wgt);
	   //wgt->bitState->setBit(10, true);
	   wgt->bitState->setBit(9, true);
           wgt->bitState->setBit(7, true);
           wgt->bitState->setBit(8, true);
           setLock(false);
           return;
         }
         else {
              wgt->bitState->setBit(8, true);
              setLock(false);
              return;
            }
        }
        if ( i == 7 ) {
         wgt->bitState->setBit(3, true);
	 efile.setFileName(target);
         if ( !efile.remove()) {
           wgt->bufferErrorFile << target;
	   wgt->bufferErrorMessage << tr("Could not overwrite the file");
	   wgt->bufferErrorSystem << efile.errorString();
           wgt->bufferErrorStatus << 'C';
	   wgt->errprint = false;
	   errorCritical(wgt);
	   //wgt->bitState->setBit(10, true);
	   wgt->bitState->setBit(9, true);
           wgt->bitState->setBit(7, true);
           wgt->bitState->setBit(8, true);
           setLock(false);
           return;
         }
         else {
              wgt->bitState->setBit(8, true);
              setLock(false);
              return;
         }
       }
       if ( i == 2 ) {
       CMessageBox *query = new CMessageBox(pxCache);
       query->createQuestion(pxCache->getIcon(183)->scaled(64, 64));
       query->setCastomPalette(this->palette(), Color);
       query->setFont(this->font());
       query->setWindowTitle(tr("Aborting operation") + " - Bathyscaphe");
       query->inform->setText(tr("Are you sure you want to abort the operation?"));
       if ( query->exec() == 1 ) {
         delete query;
         wgt->bitState->setBit(9, true);
         setLock(false);
         return;
        }
       else delete query;
      }
    setLock(false);
    }

    if ( (fileinfo_source.isDir() && !fileinfo_source.isSymLink()) && ( fileinfo_target.isDir() && !fileinfo_target.isSymLink())) {   //4
       wgt->newName = fileinfo_target.fileName();
       if ( wgt->bitState->testBit(6)) {
         wgt->bitState->setBit(7, true);
         wgt->bitState->setBit(8, true);
         return;
       }
       if ( wgt->bitState->testBit(5)) {
         wgt->bitState->setBit(8, true);
         return;
       }
       CMessageBox *warning = new CMessageBox(pxCache, this);
       setLock(true);
       warning->createWarningCopy(pxCache->getIcon(181)->scaled(96, 96));
       i = showWarningCopy(warning, fileinfo_source, fileinfo_target, templ1, templ2, &string);
       if ( i == 1) {
          if ( string == "" || string == "." || string == ".." || string.contains("/") || string.contains("\\")) {
            CMessageBox *error = new CMessageBox(pxCache);
            error->createInform(pxCache->getIcon(182)->scaled(96, 96));
            error->setFont(this->font());
            error->setCastomPalette(this->palette(), Color);
            error->setWindowTitle(tr("Not valid name") + " - Bathyscaphe");
            error->inform->setText("<h3>" + tr("Error:") + " </h3>  <br>"  + tr("The name of the file or directory can not be '.' or'..', <br>contain the characters '/' or '\\' or empty.."));
            error->detals->setText(tr("Please try again."));
            error->exec();
            delete error;
            continue;
          }
          fileinfo_target.setFile(fileinfo_target.absolutePath() + QDir::separator() + string);
          if ( fileinfo_target.exists()) {
            continue;
          }
          else {
            QDir dir;
            if ( dir.mkdir(fileinfo_target.absolutePath() + QDir::separator() + string)) {
               wgt->newName = string;
               wgt->bitState->setBit(8, true);
               setLock(false);
               return;
            }
            else {
	          wgt->bufferErrorFile << fileinfo_target.absolutePath() + QDir::separator() + string;
	          wgt->bufferErrorMessage << tr("Could not create directory");
	          wgt->bufferErrorSystem << " --- ";
	          wgt->bufferErrorStatus << 'C';
	          wgt->errprint = false;
	          errorCritical(wgt);
	          //wgt->bitState->setBit(10, true);
	          wgt->bitState->setBit(9, true);
              wgt->bitState->setBit(7, true);
              wgt->bitState->setBit(8, true);
              setLock(false);
              return;
            }
          }
       }
       if ( i ==  4 ) {
         wgt->bitState->setBit(7, true);
         wgt->bitState->setBit(8, true);
         setLock(false);
         return;
       }
       if ( i == 5 ) {
         wgt->bitState->setBit(7, true);
         wgt->bitState->setBit(6, true);
         wgt->bitState->setBit(8, true);
         setLock(false);
         return;
       }
       if ( i == 6 ) {
          wgt->bitState->setBit(8, true);
          setLock(false);
          return;
        }
       if ( i == 7 ) {
         wgt->bitState->setBit(5, true);
         wgt->bitState->setBit(8, true);
         setLock(false);
         return;
       }
       if ( i == 2 ) {
       CMessageBox *query = new CMessageBox(pxCache);
       query->createQuestion(pxCache->getIcon(183)->scaled(64, 64));
       query->setCastomPalette(this->palette(), Color);
       query->setFont(this->font());
       query->setWindowTitle(tr("Aborting operation") + " - Bathyscaphe");
       query->inform->setText(tr("Are you sure you want to abort the operation?"));
       if ( query->exec() == 1 ) {
         delete query;
         wgt->bitState->setBit(9, true);
         setLock(false);
         return;
        }
       else delete query;
      }
     setLock(false);
    }

    if ( !fileinfo_source.exists() && !fileinfo_source.isSymLink()) {
      wgt->bitState->setBit(9, true);
      wgt->bitState->setBit(8, true);
      setLock(false);
      return;
    }
  }
}

int CThreadMonitor :: showWarningCopy (CMessageBox *warning, QFileInfo &fileinfo_source, QFileInfo &fileinfo_target,
                                                                          Template &templ1, Template &templ2, QString * string) {
    warning->setCastomPalette(this->palette(), Color);
    warning->setFont(this->font());
    warning->setWindowTitle(tr("This name already exists") + " - Bathyscaphe");
    templ2 = pxCache->fullTemplate(fileinfo_target);
    warning->inform->setText("<h3>" + tr("Warning:") + "</h3> <br> " + tr("When copying in the target directory is found stacks element name"));
    warning->setTextIntoWrite(templ2.FileName);
    warning->lineedit1->setSelection(0, templ1.BaseName.length());
    warning->lineedit1->setFocus();
    warning->labelname1->setText("<h3>" + templ1.FullName + "</h3>");
    warning->labelicon1->setPixmap(templ1.Icon.scaled(96,96));
    if (fileinfo_source.isDir() && !fileinfo_source.isSymLink())
      warning->labeldetals1->setText("<table><tr><td>" + tr("Type") +":</td><td>" +  templ1.Type + "</td></tr><tr><td>"
                                                          + tr("Size") + ":</td><td>" + templ1.Size   + "</td></tr><tr><td>"
                                                          + tr("Created") + ":</td><td>" + templ1.Created + "</td></tr><tr><td>"
                                                          + tr("Modified") + ":</td><td>" + templ1.LastModified + "</td></tr></table>");
    else
      warning->labeldetals1->setText("<table><tr><td>" + tr("Type") +":</td><td>" +  templ1.Type + "</td></tr><tr><td>"
                                                          + tr("Description (from extended)") +":</td><td>" + templ1.Descr + "</td></tr><tr><td>"
                                                          + tr("Contents") + ":</td><td>" + templ1.DescrFromContents + "</td></tr><tr><td>"
                                                          + tr("Size") + ":</td><td>" + templ1.Size   + "</td></tr><tr><td>"
                                                          + tr("Created") + ":</td><td>" + templ1.Created + "</td></tr><tr><td>"
                                                          + tr("Modified") + ":</td><td>" + templ1.LastModified + "</td></tr></table>");


    warning->labelname2->setText("<h3>" + templ2.FullName + "</h3>");
    warning->labelicon2->setPixmap(templ2.Icon.scaled(96,96));
    if (fileinfo_target.isDir() && !fileinfo_target.isSymLink())
      warning->labeldetals2->setText("<table><tr><td>" + tr("Type") +":</td><td>" +  templ2.Type + "</td></tr><tr><td>"
                                                          + tr("Size") + ":</td><td>" + templ2.Size   + "</td></tr><tr><td>"
                                                          + tr("Created") + ":</td><td>" + templ2.Created + "</td></tr><tr><td>"
                                                          + tr("Modified") + ":</td><td>" + templ2.LastModified + "</td></tr></table>");
    else
      warning->labeldetals2->setText("<table><tr><td>" + tr("Type") +":</td><td>" +  templ2.Type + "</td></tr><tr><td>"
                                                          + tr("Description (from extended)") +":</td><td>" + templ2.Descr + "</td></tr><tr><td>"
                                                          + tr("Contents") + ":</td><td>" + templ2.DescrFromContents + "</td></tr><tr><td>"
                                                          + tr("Size") + ":</td><td>" + templ2.Size   + "</td></tr><tr><td>"
                                                          + tr("Created") + ":</td><td>" + templ2.Created + "</td></tr><tr><td>"
                                                          + tr("Modified") + ":</td><td>" + templ2.LastModified + "</td></tr></table>");

       int i = warning->execInputString(string);
       delete warning;
       return i;
}

void CThreadMonitor :: setPalette(QPalette pal, QColor color ) {
  QScrollArea::setPalette(pal);
  Color = color;
}

void CThreadMonitor :: setFont(QFont font) {
  font.setBold(true);
  int i = layout->count();
  QFontMetrics metric ( font );
  int w = metric.width("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa");
  int h = metric.height()*6 + 60;
  for ( int n = 0; n < i; ++n )    layout->itemAt(n)->widget()->setFixedSize(w, h);

  if ( i >0) {
    OptimumSize.setWidth(layout->itemAt(0)->widget()->width());
    OptimumSize.setHeight(layout->itemAt(0)->widget()->height() * i + i*10);
    MainFrame->setFixedSize(OptimumSize);
  }
  else {
    OptimumSize.setWidth(0);
    OptimumSize.setHeight(0);
    MainFrame->setFixedSize(OptimumSize);
  }
  QScrollArea::setFont(font);
  emit resizeThis();
}

 bool CThreadMonitor :: event(QEvent *ev) {
   QRect rectange;
   rectange = this->geometry();
   QScrollArea::event(ev);
   if ( ev->type() == QEvent::MouseButtonPress ||  ev->type() == QEvent::MouseMove || ev->type() == QEvent::Leave) {
      if (ev->type() == QEvent::Leave ) {
        grabMouse();
        setMouseTracking(true);
        return true;
      }
      if ( ev->type() == QEvent::MouseMove) {
        QMouseEvent *me = static_cast<QMouseEvent*>(ev);
        if ( rectange.contains(me->globalPos())) {
          releaseMouse();
        }
        else {
          grabMouse();
          setMouseTracking(true);
        }
        return true;
      }
    if ( ev->type() == QEvent::MouseButtonPress) {
      QMouseEvent *me = static_cast<QMouseEvent*>(ev);
      if ( !rectange.contains(me->globalPos())) {
         releaseMouse();
         this->Visible = false;
         this->hide();
         return true;
      }
    }
   }
   return false;
 }

void CThreadMonitor :: setLock(bool lock) {
  Lock = lock;
  if (Lock) {
    Visible = false;
    hide();
    releaseMouse();
  }
}





