#include "../../Include/ThreadOp.h"
#include "../../Include/Stat.h"

//=============================                 CThreadGS                       ============================================

CThreadGS :: CThreadGS(QObject *parent) : QThread(parent) { }

void CThreadGS :: setData( QList<QString> list, quint64 *sz, quint64 *sd, quint64 *fd, quint64 *fl) {
  listObj = list;
  sizeData = sz;
  countData = sd;
  countFolders = fd;
  countFiles = fl;
  Exit = false;
}

CThreadGS :: CThreadGS ( QList<QString> list, quint64 *sz, quint64 *sd, QObject *parent) : QThread(parent){
  listObj = list;
  sizeData = sz;
  countData = sd;
  countFolders = NULL;
  countFiles = NULL;
  Exit = false;
}

void CThreadGS :: run() {
  if ( countFolders == NULL && countFiles == NULL) {
    getSize(listObj);
    if ( !Exit ) emit signalOfValueSizeAndCount(*sizeData, *countData);
  }
  else {
    getSizeAndCount(listObj);
    if ( !Exit ) emit signalOfAllCounts();
  }
}

void CThreadGS :: getSize (QList<QString> list) {
  QFileInfo fileinfo;
  foreach ( QString str, list ){
    if ( Exit ) return;
    ++  *(countData);
    fileinfo.setFile ( str );
    *sizeData = *sizeData + fileinfo.size();
    if ( fileinfo.isFile() || fileinfo.isSymLink()) { }
    else getSizeDirectory(fileinfo.absoluteFilePath());
  }
}

void CThreadGS :: getSizeDirectory(QString directory) {
  QFileInfoList fileinfolist;
  getFileInfoList( directory, &fileinfolist);
  foreach ( QFileInfo finf, fileinfolist ) {
    if ( Exit ) return;
    ++  *(countData);
    *sizeData = *sizeData + finf.size();
    if ( finf.isFile() || finf.isSymLink())  { }
    else {
      getSizeDirectory(finf.absoluteFilePath());
    }
  }
}

void CThreadGS :: getSizeAndCount(QList<QString> list) {
  QFileInfo fileinfo;
  foreach ( QString str, list ){
    if ( Exit ) return;
    ++  *(countData);
    fileinfo.setFile ( str );
    *sizeData = *sizeData + fileinfo.size();
    if ( fileinfo.isFile() || fileinfo.isSymLink()) {
      ++ *(countFiles);
    }
    else {
      ++ *(countFolders);
      getSizeDirectoryAndCount(fileinfo.absoluteFilePath());
    }
  }
}

void CThreadGS :: getSizeDirectoryAndCount(QString directory) {
  QFileInfoList fileinfolist;
  getFileInfoList( directory, &fileinfolist);
  foreach ( QFileInfo finf, fileinfolist ) {
    if ( Exit ) return;
    ++  *(countData);
    *sizeData = *sizeData + finf.size();
    if ( finf.isFile() || finf.isSymLink())  {
      ++ *countFiles;
    }
    else {
      ++ *(countFolders);
      getSizeDirectoryAndCount(finf.absoluteFilePath());
    }
  }
}

//========================                                  CThreadOp                              =======================================

CThreadOp :: CThreadOp ( QList<QString> sour, QString targ, int tp, QObject *parent ) : QThread(parent) {
  Errn = &errno;
  source = sour;
  target = targ;
  type = tp;
  sizeData = 1;
  sizeComplete = 0;
  countData = 1;
  countComplete = 0;
  bitState = new QBitArray(12);
  bitState->setBit(11, false);
  bitState->setBit(10, false);
  timer = new QTimer();
  connect ( timer, SIGNAL(timeout()), SLOT ( getFromTimer()));
  timer->start(500);
  errprint = true;
}

//===================                         Вспомогательные функции                     ==========================================

void CThreadOp :: deleteDirectory (QString directory) {
      QDir dir(directory);
      QFile efile;
      QFileInfoList fileinfolistall = dir.entryInfoList(QDir::Dirs | QDir::Files | QDir::Hidden | QDir::System | QDir::NoDotAndDotDot);
      QFileInfoList directorylist;
      if ( fileinfolistall.isEmpty()) {
        if ( !dir.rmdir(directory)) {
          bufferErrorFile << dir.absolutePath();
	      bufferErrorMessage << tr("Unable to delete directory.");
	      bufferErrorSystem << " --- ";
          bufferErrorStatus << 'W';
        }
        countComplete++;
        return;
      }
      foreach ( QFileInfo fileinfo, fileinfolistall ) {
        if ( bitState->testBit(10))  emit abortOperation(this);
        if ( bitState->testBit(9)) return;
        if ( fileinfo.isSymLink() || fileinfo.isFile()) {
	      efile.setFileName(fileinfo.absoluteFilePath());
          if ( !efile.remove()) {
            bufferErrorFile << fileinfo.absoluteFilePath();
	        bufferErrorMessage << tr("Unable to delete file.");
	        bufferErrorSystem << efile.errorString();
            bufferErrorStatus << 'W';
          }
          countComplete++;
        }
        else directorylist << fileinfo;
      }

      if ( directorylist.isEmpty()) {
        if ( !dir.rmdir(directory)) {
          bufferErrorFile << dir.absolutePath();
	      bufferErrorMessage << tr("Unable to delete directory.");
	      bufferErrorSystem << " --- ";
          bufferErrorStatus << 'W';
        }
        countComplete++;
        return;
      }
      else {
        foreach ( QFileInfo fileinfo, directorylist ) {
          deleteDirectory (fileinfo.absoluteFilePath());
        }
      }
      if ( !dir.rmdir(directory)) {
          bufferErrorFile << dir.absolutePath();
	      bufferErrorMessage << tr("Unable to delete directory.");
	      bufferErrorSystem << " --- ";
          bufferErrorStatus << 'W';
        }
      countComplete++;
}

void CThreadOp :: del(QList<QString> list ) {
  QFile efile;
  QFileInfo fileinfo;
  foreach ( QString str, list ) {
    if ( bitState->testBit(10))  emit abortOperation(this);
    if ( bitState->testBit(9)) return;
    fileinfo.setFile(str);
    if ( fileinfo.isFile() || fileinfo.isSymLink()) {
      countComplete++;
      efile.setFileName(fileinfo.absoluteFilePath());
      if ( !efile.remove()) {
        bufferErrorFile << str;
        bufferErrorMessage << tr("Unable to delete file.");
        bufferErrorSystem << efile.errorString();
        bufferErrorStatus << 'W';
      }
    }
    else deleteDirectory(str);
  }
}

bool CThreadOp :: copyFile ( QString file, QString targ) {
  QFileInfo fileinfo(file);
  if ( fileinfo.isSymLink()) {
    QString errorString;
    bool result = createSymLinkFrom(file, targ, Errn);
    if ( !result ) {
      setErrorString(file, targ, &errorString, Errn);
      bufferErrorFile << file + "-->\n" + targ;
      bufferErrorMessage << tr("Failed to copy a file");
      bufferErrorSystem << errorString;
      bufferErrorStatus << 'C';
      errprint = false;
      emit errorCriticalComplete(this);
      return false;
    }
    else {
      return true;
    }
  }

  int blsize;
  QFile file1(file);
  if ( !file1.open(QIODevice::ReadOnly ))  {
     bufferErrorFile << file;
     bufferErrorMessage << tr("Failed to open file for reading.");
     bufferErrorSystem << file1.errorString();
     bufferErrorStatus << 'C';
     errprint = false;
     emit errorCriticalComplete(this);
     return false;
  }
  QFile file2(targ);
  if ( !file2.open(QIODevice::WriteOnly))  {
    bufferErrorFile << targ;
    bufferErrorMessage << tr("Failed to open file for writing.");
    bufferErrorSystem << file2.errorString();
    bufferErrorStatus << 'C';
    errprint = false;
    file1.close();
    emit errorCriticalComplete(this);
    file1.close();
    return false;
  }
  char a[102400];
  while ( !file1.atEnd()) {
    if ( bitState->testBit(10))  emit abortOperation(this);
    if ( bitState->testBit(9)) {
      file1.close();
      file2.close();
      return false;
    }
    blsize = file1.read(a, sizeof(a));
    if (file2.write ( a, blsize ) != blsize ) {
      bufferErrorFile << targ;
      bufferErrorMessage << tr("Failed to copy a file.");
      bufferErrorSystem << file2.errorString();
      bufferErrorStatus << 'C';
      file1.close();
      file2.close();
      errprint = false;
      emit errorCriticalComplete(this);
      return false;
    }
    sizeComplete = sizeComplete + blsize;
  }
  file1.close();
  file2.close();
  return true;
}

//..............................................................................................................................................................................................................

void CThreadOp :: copy (QList<QString> list, QString targ) {
  QFileInfo fileinfo_source, fileinfo_target, finf;
  QString filename;
  QDir dir;
  foreach ( QString str, list ) {
    if ( bitState->testBit(10))  emit abortOperation(this);
    if ( bitState->testBit(9)) return;
    fileinfo_source.setFile(str);
    filename = fileinfo_source.fileName();
    fileinfo_target.setFile(targ + QDir::separator() + filename);
    if ( fileinfo_target.exists() || fileinfo_target.isSymLink()) {
      emit existsName( str, targ + QDir::separator() + filename );
      if ( bitState->testBit(9)) return;
      if ( bitState->testBit(7)) {
            bitState->setBit(7, false);
            bitState->setBit(8, false);
            continue;
      }
      filename = newName;
    }
    bitState->setBit(7, false);
    bitState->setBit(8, false);

    if ( fileinfo_source.isSymLink() || fileinfo_source.isFile()) {
      copyFile ( fileinfo_source.absoluteFilePath(), targ + QDir::separator() + filename);
      continue;
    }
    if ( fileinfo_source.isDir()) {
      QList<QString> listt;
      QList<QString> newlist;
      dir.setPath(fileinfo_source.absoluteFilePath());
      listt = dir.entryList(QDir::Dirs | QDir::Files | QDir::Hidden | QDir::System | QDir::NoDotAndDotDot);
      foreach (QString str, listt) {
        newlist << fileinfo_source.absoluteFilePath() + QDir::separator() + str;
      }
      finf.setFile(targ + QDir::separator() + filename);
      if ( !finf.exists()) {
        if ( !dir.mkdir(targ + QDir::separator() + filename)) {
          bufferErrorFile << str + "-->\n" + targ + QDir::separator() + filename;
	      bufferErrorMessage << tr("Unable to create directory.");
	      bufferErrorSystem << " --- ";
          bufferErrorStatus << 'C';
          errprint = false;
          emit errorCriticalComplete(this);
          return;
        }
      }
      sizeComplete = sizeComplete + finf.size();
      copy (newlist, targ + QDir::separator() + filename);
    }
  }
}

void CThreadOp :: move ( QList<QString> list, QString targ ) {
  QList<QString> deleteList;
  QFileInfo fileinfo_source, fileinfo_target;
  bool rezult;
  QDir dir;
  QChar sep = QDir::separator();
  QString filename;

  foreach ( QString str, list ) {
    if ( bitState->testBit(10)) emit abortOperation(this);
    if ( bitState->testBit(9)) return;
    fileinfo_source.setFile(str);
    filename = fileinfo_source.fileName();
    fileinfo_target.setFile(targ + sep + filename);
    if ( fileinfo_target.exists() || fileinfo_target.isSymLink()) {
        emit existsName( str, fileinfo_target.absoluteFilePath() );
        if ( bitState->testBit(9)) return;
        if ( bitState->testBit(7)) {
          bitState->setBit(7, false);
          bitState->setBit(8, false);
          continue;
        }
        filename = newName;
    }

    bitState->setBit(7, false);
    bitState->setBit(8, false);
    if ( fileinfo_source.isSymLink()) {
      copyFile(str, targ + sep + filename);
      deleteList << str;
      continue;
    }
    if ( fileinfo_source.isFile()) {
      rezult = QFile::rename(str, targ + sep + filename);
      if ( !rezult) {
        copyFile(str, targ + sep + filename);
        deleteList << str;
      }
      continue;
    }
    if ( fileinfo_source.isDir()) {
      rezult = dir.rename(str, targ + sep + filename);
      if ( !rezult) {
        QStringList lst;
        lst << str;
        copy (lst, targ);
        deleteList << str;
      }
    }
  }

   if ( bitState->testBit(10)) emit abortOperation(this);
   if ( bitState->testBit(9)) return;
   del(deleteList);
}

void CThreadOp :: createLinks(QList<QString> source, QString target) {
  QFileInfo fileinfo_source, fileinfo_target;
  QDir dir;
  QChar sep = QDir::separator();
  QString filename, errorString;
  int errn;
  foreach ( QString str, source) {
    if ( bitState->testBit(10)) emit abortOperation(this);
    if ( bitState->testBit(9)) return;
    fileinfo_source.setFile(str);
    filename = fileinfo_source.fileName();
    fileinfo_target.setFile(target + sep + filename);
    if ( fileinfo_target.exists() || fileinfo_target.isSymLink()) {
        emit existsName( str, fileinfo_target.absoluteFilePath() );
        if ( bitState->testBit(9)) return;
        if ( bitState->testBit(7)) {
          bitState->setBit(7, false);
          bitState->setBit(8, false);
          continue;
        }
        filename = newName;
    }

    if (!createSymLink(fileinfo_source.absoluteFilePath(), target + QDir::separator() + filename, &errn)) {
      setErrorString(fileinfo_source.absoluteFilePath(), target + QDir::separator() + filename, &errorString, &errn);
      bufferErrorFile << fileinfo_source.absoluteFilePath() + " -> \n" +target + QDir::separator() + filename;
      bufferErrorMessage << tr("Unable to create symbolic link.");
      bufferErrorSystem << errorString;
      bufferErrorStatus << 'W';
    }
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void CThreadOp :: run (){
  if ( type == 1 ) copy ( source, target );
  if ( type ==  2 ) move ( source, target );
  if ( type ==  3 ) del ( source );
  if ( type == 4 ) createLinks(source, target);

  timer->stop();
  delete timer;
  emit complete();
}

void CThreadOp :: getFromTimer() {
  if ( type == 1 || type == 2 ) {
    int n = sizeData/100;
    if ( n == 0 ) emit execComplete (0);
      else
      emit execComplete( sizeComplete  /  n );
  }
  if ( type == 3 ) {
    int n = countComplete/100;
    if ( n == 0 ) emit execComplete(0);
      else
      emit execComplete ( countComplete /  n);
  }
}
