#include "../../Include/ToolBar.h"
#include "../../Include/Button.h"

CToolBar :: CToolBar ( QWidget *wgt ) : QFrame(wgt){
   Flat = false;
   Text = true;
   IconSize.setWidth(16);
   IconSize.setHeight(16);
   Font = new QFont();
   setFrameStyle(QFrame::Panel | QFrame::Sunken);
   setContentsMargins(2, 2, 2, 2);
   setLineWidth(1);
   setMidLineWidth(1);
   setAutoFillBackground(true);
   setMaximumHeight(IconSize.height() + 16);
   Layout.setMargin(2);
   Layout.setSpacing(2);
   Layout.setAlignment(Qt::AlignLeft);
   this->setLayout(&Layout);
   QPalette pal = this->palette();
   QColor color = pal.color(this->backgroundRole());
   setBackColor(color);
   color = pal.color(QPalette::ButtonText);
   setFontColor(color);
}

CButton* CToolBar :: addAction(const QIcon &icon, const QString &text, const QString &toolTip){
  QFontMetrics fm(*Font);
  CButton *button = new CButton();
  button->setFlat(Flat);
  button->setIcon(icon);
  button->setToolTip(toolTip);
  button->setIconSize(IconSize);
  button->setFont(*Font);
  button->setMaximumHeight(IconSize.height() + 8);
  button->setMinimumHeight(IconSize.height() + 8);
  if ( !Text ) {
    button->setMaximumWidth(IconSize.width() + 8);
    button->setMinimumWidth(IconSize.width() + 8);
  }
  else {
       if ( !text.isEmpty()){
       button->setMaximumWidth(IconSize.width() + fm.width(text) + 14);
       button->setMinimumWidth(IconSize.width() + fm.width(text) + 14);
       button->setText(text);
       }
       else {
            button->setMaximumWidth(IconSize.width() + 8);
            button->setMinimumWidth(IconSize.width() + 8);
       }
     }
  ListButton << button;
  ListLabel << text;
  Layout.addWidget(button);
  QObject::connect ( button, SIGNAL(clicked()), SLOT(clickButton()));
  return button;
}

CButton* CToolBar :: addAction (const QIcon &icon, const QString &toolTip){
  CButton *button = new CButton();
  button->setFlat(Flat);
  button->setIcon(icon);
  button->setToolTip(toolTip);
  button->setIconSize(IconSize);
  button->setMaximumWidth(IconSize.width() + 8);
  button->setMaximumHeight(IconSize.height() +8);
  button->setMinimumWidth(IconSize.width() + 8);
  button->setMinimumHeight(IconSize.height() +8);
  ListButton << button;
  ListLabel << "";
  Layout.addWidget(button);
  QObject::connect ( button, SIGNAL(clicked()), SLOT(clickButton()));
  return button;
}

CButton* CToolBar :: addAction (const QString &text, const QString &toolTip){
  QFontMetrics fm(*Font);
  CButton *button = new CButton();
  button->setFlat(Flat);
  button->setToolTip(toolTip);
  button->setFont(*Font);
  button->setMaximumWidth(fm.width(text) + 10);
  button->setMinimumWidth(fm.width(text) + 10);
  button->setMinimumHeight(IconSize.height() + 8);
  button->setMaximumHeight(IconSize.height() + 8);
  button->setText(text);
  ListButton << button;
  ListLabel << text;
  Layout.addWidget(button);
  QObject::connect ( button, SIGNAL(clicked()), SLOT(clickButton()));
  return button;
}

CAnimateButton* CToolBar :: addAction ( QString filename) {
  CAnimateButton* button = new CAnimateButton(filename);
  ListAnimateButton << button;
  Layout.addWidget(button);
  QObject::connect( button, SIGNAL(clicked()), SLOT(clickAnimateButton()));
  return button;
}

void CToolBar :: setIconSize(QSize size){
  IconSize = size;
  QString text;
  QFontMetrics fm(*Font);
  setMaximumHeight(IconSize.height() + 16);
  foreach ( CButton* button, ListButton ) {
    button->setIconSize(size);
    button->setMaximumHeight(IconSize.height() + 8);
    button->setMinimumHeight(IconSize.height() + 8);
    if ( !Text ){
      button->setMaximumWidth(IconSize.width() + 8);
      button->setMinimumWidth(IconSize.width() + 8);
    }
      else {
        text = button->text();
        if ( !text.isEmpty()) {
           button->setMaximumWidth(IconSize.width() + fm.width(text) + 14);
           button->setMinimumWidth(IconSize.width() + fm.width(text) + 14);
        }
        else {
            button->setMaximumWidth(IconSize.width() + 8);
            button->setMinimumWidth(IconSize.width() + 8);
        }
      }
  }
}

void CToolBar :: setShowText ( bool show ){
  if ( show != Text ) {
    Text = show;
    QFontMetrics fm(*Font);
    if ( Text ){
      QList<QString>::Iterator It = ListLabel.begin();
      foreach ( CButton* button, ListButton ){
        if ( (*It).isEmpty()) {
              button->setText("");
              button->setMaximumHeight(IconSize.height() + 8);
              button->setMinimumHeight(IconSize.height() + 8);
              button->setMaximumWidth(IconSize.width() + 8);
              button->setMinimumWidth(IconSize.width() + 8);
              ++It;
        }
        else{
          button->setText(*It);
          button->setMaximumHeight(IconSize.height() + 8);
          button->setMinimumHeight(IconSize.height() + 8);
          if (!(button->icon().isNull())) {
              button->setMaximumWidth(IconSize.width() + fm.width(*It) +14);
              button->setMinimumWidth(IconSize.width() + fm.width(*It) +14);
          }
          else {
           button->setMaximumWidth(fm.width(*It) +10);
           button->setMinimumWidth(fm.width(*It) +10);
          }
          ++It;
        }
      }
    }
    else {
      foreach ( CButton* button, ListButton ) {
        if ( !(button->icon().isNull())) {
          button->setText("");
          button->setMaximumHeight(IconSize.height() + 8);
          button->setMinimumHeight(IconSize.height() + 8);
          button->setMaximumWidth(IconSize.width() + 8);
          button->setMinimumWidth(IconSize.width() + 8);
        }
      }
    }
  }
}

void CToolBar :: setFontSize(){
  QList<QString>::Iterator It = ListLabel.begin();
  QFontMetrics fm(*Font);
  if ( Text ){
      foreach ( CButton* button, ListButton ){
        button->setFont(*Font);
        if ( (*It).isEmpty()) {
              button->setMaximumHeight(IconSize.height() + 8);
              button->setMinimumHeight(IconSize.height() + 8);
              button->setMaximumWidth(IconSize.width() + 8);
              button->setMinimumWidth(IconSize.width() + 8);
              ++It;
        }
        else{
          button->setMaximumHeight(IconSize.height() + 8);
          button->setMinimumHeight(IconSize.height() + 8);
          if (!(button->icon().isNull())) {
              button->setMaximumWidth(IconSize.width() + fm.width(*It) +14);
              button->setMinimumWidth(IconSize.width() + fm.width(*It) +14);
          }
          else {
           button->setMaximumWidth(fm.width(*It) +10);
           button->setMinimumWidth(fm.width(*It) +10);
          }
          ++It;
        }
      }
    }
    else {
      foreach ( CButton* button, ListButton ) {
        button->setFont(*Font);
        if ( !(button->icon().isNull())) {
          button->setMaximumHeight(IconSize.height() + 8);
          button->setMinimumHeight(IconSize.height() + 8);
          button->setMaximumWidth(IconSize.width() + 8);
          button->setMinimumWidth(IconSize.width() + 8);
        }
        else {
          button->setMaximumHeight(IconSize.height() + 8);
          button->setMinimumHeight(IconSize.height() + 8);
          button->setMaximumWidth(fm.width(*It) + 10);
          button->setMinimumWidth(fm.width(*It) + 10);
        }
        ++It;
      }
    }
  }

void CToolBar :: setFlat(bool flat){
  Flat = flat;
  foreach (CButton* button, ListButton) {
    button->setFlat(Flat);
  }
}

void CToolBar :: clear(){
  QLayoutItem * child;
  while ((child = Layout.takeAt(0)) != NULL) delete child;
  ListSeparator.clear();
  ListButton.clear();
  ListLabel.clear();
  ListPointSeparator.clear();
  ListAnimateButton.clear();
}

void CToolBar :: setBackColor (QColor color){
  QPalette pal = this->palette();
  QPalette pall = pal;
  changeBackgroundPalette(&pal, color);
  changeBackgroundPalette(&pall, darkerColor(color, 20));
  this->setPalette(pal);
  foreach (CButton *button, ListButton) {
     if (button->existsMenu()) button->getMenu()->setPalette(pall);
  }
}

void CToolBar :: setFontColor (QColor color) {
  QPalette pal = this->palette();
  QPalette pall = pal;
  QColor  cl = pall.color(QPalette::Base);
  changeTextPalette(&pal, color);
  changeBackgroundPalette(&pall, darkerColor(cl, 20));
  changeTextPalette(&pall, darkerColor(color, 20));
  this->setPalette(pal);
  foreach (CButton *button, ListButton) {
     if (button->existsMenu()) button->getMenu()->setPalette(pall);
  }
}

void CToolBar :: changeFlat(){
  if ( Flat ) {
    Flat = false;
    setFlat(Flat);
  }
  else{
    Flat = true;
    setFlat(Flat);
  }
}

void CToolBar :: addSeparator(){
   Separator = new QWidget();
   Separator->setWindowFlags(Qt::Window | Qt::FramelessWindowHint);
   Separator->setAutoFillBackground(true);
   QPalette pp = this->palette();
   pp.setColor(Separator->backgroundRole(), Qt::red);
   Separator->setPalette(pp);
   Separator->setMaximumWidth(2);
   Separator->setMinimumWidth(2);
   Separator->setMaximumHeight(IconSize.height() + 8);
   Separator->setMinimumHeight(IconSize.height() + 8);
   ListSeparator << Separator;
   Layout.addWidget(Separator);
}

void CToolBar :: addPoint(){
   CPointSeparator *Point = new CPointSeparator();
   Point->setMaximumHeight(IconSize.height() + 8);
   Point->setMinimumHeight(IconSize.height() + 8);
   Point->setMinimumWidth(4);
   Point->setMaximumWidth(4);
   QPalette pp = this->palette();
   Point->setPalette(pp);
   ListPointSeparator << Point;
   Layout.addWidget(Point);
}

void CToolBar :: clickButton(){
  CButton* button = qobject_cast<CButton*>(sender());
  emit clickedButton(button);
}

void CToolBar :: clickAnimateButton() {
  CAnimateButton* button = qobject_cast<CAnimateButton*>(sender());
  emit clickedAnimateButton(button);
}









