#include "../../Include/TreeViewer.h"

CTreeViewer :: CTreeViewer(PixmapsCache *cache, QWidget *wgt ) : QTreeWidget (wgt) {
  pxCache = cache;
  setContentsMargins(0,0,0,0);
  lst << tr("Folders and drives");
  setHeaderLabels ( lst );
  home_icon = *(pxCache->getIcon(1));
  root_icon = *(pxCache->getIcon(0));
  drive_icon = *(pxCache->getIcon(3));
  default_icon = *(pxCache->getIcon(2));
  hidden_icon = *(pxCache->getIcon(4));
  symlink_icon = pxCache->getIcon(111)->scaled(64, 64);

  item = new QTreeWidgetItem ( this );
  item->setIcon ( 0, home_icon );
  font.setBold (true );
  item->setFont ( 0,  font );
  item->setText ( 0, "Home Directoty" );
  item->setToolTip ( 0, QDir::homePath());
  if ( ! watcher.directories().contains ( item->toolTip ( 0 ))) watcher.addPath ( item->toolTip ( 0 ));
  item = new QTreeWidgetItem ( this );
  item->setIcon ( 0, root_icon );
  item->setFont ( 0,  font );
  item->setText ( 0, "Root Directory" );
  item->setToolTip ( 0, QDir::rootPath() );
  if ( ! watcher.directories().contains ( item->toolTip ( 0 ))) watcher.addPath ( item->toolTip ( 0 ));
  font.setBold ( false );
  font.setStyle ( QFont::StyleItalic );
  QFileInfo targetlink;
  QPixmap pix;
  QPainter painter;

   list_drv = QDir::drives();
   foreach ( finf, list_drv ) {
    if ( finf.absoluteFilePath() == QDir::rootPath() ) continue;
    item = new QTreeWidgetItem (this);
    item->setIcon ( 0, drive_icon );
    lst.clear();
    lst << "Local Drive" << finf.absoluteFilePath();
    item-> setText ( 0, lst.join ( " " ));
    item->setToolTip ( 0, finf.absoluteFilePath() );
    if ( ! watcher.directories().contains ( item->toolTip ( 0 ))) watcher.addPath ( item->toolTip ( 0 ));
  }

  int index = topLevelItemCount();
  for ( int i = 0; i < index; ++i ) {
    item = topLevelItem ( i );
    tmpdir = QDir ( item->toolTip ( 0 ));
    lst.clear();
    if ( ShowHidden ) {
      list_drv = tmpdir.entryInfoList ( QDir::Hidden );
      foreach ( finf, list_drv )  if ( finf.isDir() ) lst << finf.fileName();
    }
    lst << tmpdir.entryList ( QDir::Dirs | QDir::NoDotAndDotDot );
    if ( !lst.empty()) {
      foreach ( QString str, lst ) {
        finf.setFile ( tmpdir.absoluteFilePath ( str ));
        if ( !finf.isDir()) continue;
        QTreeWidgetItem * newitem = new QTreeWidgetItem ( item );
        if ( finf.isSymLink() ) {
          targetlink.setFile(finf.symLinkTarget());
          while ( targetlink.isSymLink()) targetlink.setFile(targetlink.symLinkTarget());
          pix = default_icon;
          if ( targetlink.isRoot() ) pix = root_icon;
          if ( targetlink.isHidden()) pix = hidden_icon;
          if ( list_drv.contains(targetlink)) pix = drive_icon;
          if ( targetlink.absoluteFilePath() == QDir::homePath()) pix = home_icon;
          painter.begin(&pix);
          painter.drawPixmap(0, 64, symlink_icon);
          painter.end();
          newitem->setFont ( 0, font );
          newitem->setText ( 0, str );
          newitem->setIcon (0, pix );
          newitem->setToolTip ( 0, tmpdir.absoluteFilePath ( str ));
          continue;
          }
          else {
             if ( finf.isHidden() ) {
               newitem->setIcon ( 0, hidden_icon );
               newitem->setText ( 0, str );
               newitem->setToolTip ( 0, tmpdir.absoluteFilePath ( str ));
               continue;
             }
             if ( finf.absoluteFilePath() == QDir::homePath()) {
             newitem->setIcon ( 0, home_icon );
             newitem->setText ( 0, str );
             newitem->setToolTip ( 0, tmpdir.absoluteFilePath ( str ));
             continue;
            }
            newitem->setIcon ( 0, default_icon );
            newitem->setText ( 0, str );
            newitem->setToolTip ( 0, tmpdir.absoluteFilePath ( str ));
            continue;
         }
      }
    }
  }

 setDragEnabled(true);
 setAcceptDrops(true);

 connect ( this, SIGNAL ( itemExpanded ( QTreeWidgetItem * )), SLOT ( ifExpanded ( QTreeWidgetItem * )));
 connect ( this, SIGNAL ( itemCollapsed ( QTreeWidgetItem * )), SLOT ( ifCollapsed ( QTreeWidgetItem * )));
 connect ( &watcher, SIGNAL ( directoryChanged ( QString  )), SLOT ( ifWatched ( QString  )));
 connect ( this, SIGNAL(execContextMenu(QMouseEvent*, QTreeWidgetItem*)),
                             SLOT(contextMenu(QMouseEvent*, QTreeWidgetItem*)));
  connect ( &DragTimer, SIGNAL(timeout()), this, SLOT(timeoutDragTimer()), Qt::QueuedConnection);
  connect ( &ScrollDownTimer, SIGNAL(timeout()), this, SLOT(scrollDown()), Qt::QueuedConnection);
  connect ( &ScrollUpTimer, SIGNAL(timeout()), this, SLOT(scrollUp()), Qt::QueuedConnection);
  connect ( this, SIGNAL(execDrag()), this, SLOT(slotDrag()), Qt::QueuedConnection);
  connect ( this, SIGNAL(dropMenu(QList<QString>, QPoint, QString, bool, int)),
                              SLOT(execDropMenu(QList<QString>, QPoint, QString, bool, int)));

}

void CTreeViewer::ifExpanded ( QTreeWidgetItem * element ) {
      int count = element->childCount();
      QFileInfo targetlink;
      QPixmap pix;
      QPainter painter;
      for ( int i = 0; i < count; ++i ) {
          tmpdir = QDir ( element->child(i)->toolTip ( 0 ));
          finf.setFile (tmpdir.absolutePath ());
          if ( finf.isSymLink()) { Link << tmpdir.absolutePath(); }

          if ( ! watcher.directories().contains ( element->child(i)->toolTip ( 0 )))
                 watcher.addPath ( element->child(i)->toolTip ( 0 ));

          lst.clear();
          if ( ShowHidden ) {
            list_drv = tmpdir.entryInfoList ( QDir::Hidden );
            foreach ( finf, list_drv )  if ( finf.isDir() ) lst << finf.fileName();
          }
          lst << tmpdir.entryList ( QDir::Dirs | QDir::NoDotAndDotDot );
          foreach  ( QString str, lst ) {
            finf.setFile ( tmpdir.absoluteFilePath ( str ));
            if ( !finf.isDir()) continue;
            item = new QTreeWidgetItem ( element->child(i));
            if ( finf.isSymLink() ) {
              targetlink.setFile(finf.symLinkTarget());
              while ( targetlink.isSymLink()) targetlink.setFile(targetlink.symLinkTarget());
              pix = default_icon;
              if ( targetlink.isRoot() ) pix = root_icon;
              if ( targetlink.isHidden()) pix = hidden_icon;
              if ( QDir::drives().contains(targetlink)) pix = drive_icon;
              if ( targetlink.absoluteFilePath() == QDir::homePath()) pix = home_icon;
              painter.begin(&pix);
              painter.drawPixmap(0, 64, symlink_icon);
              painter.end();
              item->setFont ( 0, font );
              item->setText ( 0, str );
              item->setIcon (0, pix );
              item->setToolTip ( 0, tmpdir.absoluteFilePath ( str ));
              continue;
              }
          else {
             if ( finf.isHidden() ) {
               item->setIcon ( 0, hidden_icon );
               item->setText ( 0, str );
               item->setToolTip ( 0, tmpdir.absoluteFilePath ( str ));
               continue;
             }
             if ( finf.absoluteFilePath() == QDir::homePath()) {
             item->setIcon ( 0, home_icon );
             item->setText ( 0, str );
             item->setToolTip ( 0, tmpdir.absoluteFilePath ( str ));
             continue;
            }
            item->setIcon ( 0, default_icon );
            item->setText ( 0, str );
            item->setToolTip ( 0, tmpdir.absoluteFilePath ( str ));
            continue;
         }
    }
  }
  Link.removeDuplicates();
  }

void CTreeViewer::ifCollapsed ( QTreeWidgetItem * element ) {
    QList<QTreeWidgetItem*> listchild = element->takeChildren ();
    QList<QTreeWidgetItem*>::Iterator It;
    for ( It = listchild.begin(); It != listchild.end(); ++It ) {
      tmpdir = QDir ( (*It)->toolTip ( 0 ));
      item = new QTreeWidgetItem ( element );
      item->setToolTip ( 0, (*It)->toolTip (0 ));
      item->setIcon ( 0, (*It)->icon ( 0 ) );
      item->setText ( 0, (*It)->text ( 0 ));
      item->setFont(0, (*It)->font(0));
      }

     lst.clear ();
     listchild.clear();
     int index = topLevelItemCount();
     for ( int i = 0; i < index; ++i ) {
       item = topLevelItem (i);
       if ( item->isExpanded() ) {
         listchild << item;
         lst << item->toolTip (0);
        }
       else lst << item->toolTip (0);
     }
     if ( !listchild.empty()) createListDeleteWatcher ( listchild );
     QStringList listwatcher = watcher.directories ();
     foreach ( QString str, listwatcher ) {
       if ( ! lst.contains (str)) watcher.removePath (str);
     }
     QStringList proxilist;
     foreach ( QString str, Link ) {
       if (  lst.contains (str)) proxilist << str;
     }
     Link.clear();
     foreach ( QString str, proxilist ) {
       Link << str;
     }
}

void CTreeViewer::ifWatched ( QString dir ) {
  QFileInfo targetlink;
  QPixmap pix;
  QPainter painter;
   tmpdir.setPath ( dir );
   if ( tmpdir.exists()) {
        QTreeWidgetItemIterator It ( this, QTreeWidgetItemIterator::All );
        while (*It) {
          if ( (*It)->toolTip ( 0 ) == dir ) {
            if ((*It)->parent())
              if ( !(*It)->parent()->isExpanded()) {
                It++;
                continue;
              }
              lst.clear ();
              if ( ShowHidden ) {
                list_drv = tmpdir.entryInfoList ( QDir::Hidden );
                foreach ( finf, list_drv )  if ( finf.isDir() ) lst << finf.fileName();
              }
              list_drv = tmpdir.entryInfoList(QDir::Dirs | QDir::NoDotAndDotDot);
              foreach ( finf, list_drv ) if ( finf.isDir()) lst << finf.fileName();

              QList<QTreeWidgetItem*> listchild ;
              QStringList listitem;
              bool rflag = false;
              if ( (*It)->isExpanded() ) {
                  int count = (*It)->childCount();
                  for ( int i = 0; i < count; ++i ) {
                    listchild << (*It)->child ( i );
                  }

                  foreach ( item, listchild ) {
                    if ( ! lst.contains ( item->text ( 0 )) ) {
                        (*It)->removeChild ( item );
                        rflag = true;
                    }
                    else  listitem << item->text ( 0 );
                  }
              }
              else {
                listchild = (*It)->takeChildren();
                foreach ( item, listchild ) {
                  if ( lst.contains ( item->text (0))) {
                    listitem << item->text(0);
                    (*It)->addChild ( item);
                  }
                  else rflag = true;
              }
          }
              if ( rflag ) {
                    QStringList dirlist = watcher.directories();
                    foreach ( QString str, dirlist )
                      if ( str.contains ( dir + QDir::separator()) && !QFileInfo(str).exists())  watcher.removePath ( str );
                }

              foreach ( QString str, lst ) {
                  if ( ! listitem.contains ( str )) {
                      item = new QTreeWidgetItem ( );
                      item->setText ( 0, str );
                      item->setToolTip ( 0, tmpdir.absoluteFilePath ( str ));
                      finf.setFile ( tmpdir.absoluteFilePath ( str ));
                      if ( finf.isSymLink()) {
                        targetlink.setFile(finf.symLinkTarget());
                        while ( targetlink.isSymLink()) targetlink.setFile(targetlink.symLinkTarget());
                        pix = default_icon;
                        if ( targetlink.isRoot() ) pix = root_icon;
                        if ( targetlink.isHidden()) pix = hidden_icon;
                        if ( QDir::drives().contains(targetlink)) pix = drive_icon;
                        if ( targetlink.absoluteFilePath() == QDir::homePath()) pix = home_icon;
                        painter.begin(&pix);
                        painter.drawPixmap(0, 64, symlink_icon);
                        painter.end();
                        item->setFont ( 0, font );
                        item->setText ( 0, str );
                        item->setIcon (0, pix );
                        (*It)->addChild ( item );
                        if ( ! watcher.directories().contains ( item->toolTip ( 0 ))) watcher.addPath ( item->toolTip ( 0 ));
                        continue;
                        }
                    else {
                      if ( finf.isHidden() ) {
                      item->setIcon ( 0, hidden_icon );
                      item->setText ( 0, str );
                      (*It)->addChild ( item );
                      if ( ! watcher.directories().contains ( item->toolTip ( 0 ))) watcher.addPath ( item->toolTip ( 0 ));
                      continue;
                      }
                      if ( finf.absoluteFilePath() == QDir::homePath()) {
                      item->setIcon ( 0, home_icon );
                      item->setText ( 0, str );
                      (*It)->addChild ( item );
                      if ( ! watcher.directories().contains ( item->toolTip ( 0 ))) watcher.addPath ( item->toolTip ( 0 ));
                      continue;
                      }
                      item->setIcon ( 0, default_icon );
                      item->setText ( 0, str );
                      (*It)->addChild ( item );
                      if ( ! watcher.directories().contains ( item->toolTip ( 0 ))) watcher.addPath ( item->toolTip ( 0 ));
                      continue;
                  }
               }
           }
        }
       It++;
      }
   }

   if ( ! Link.empty() ) {
     foreach ( QString str, Link ) {
     QTreeWidgetItemIterator Itt ( this, QTreeWidgetItemIterator::All );
       while ( *Itt ) {
         if ((*Itt)->toolTip(0) == str) {
            if ((*Itt)->parent())
              if ( !(*Itt)->parent()->isExpanded()) {
                Itt++;
                continue;
              }
           linkWatched ( *Itt );
         }
         Itt++;
       }
     }
   }
}

void CTreeViewer::linkWatched ( QTreeWidgetItem *It ) {
  tmpdir.setPath ( It->toolTip(0));
  if ( tmpdir.exists() ) {
              lst.clear ();
              if ( ShowHidden ) {
                list_drv = tmpdir.entryInfoList ( QDir::Hidden );
                foreach ( finf, list_drv )  if ( finf.isDir() ) lst << finf.fileName();
              }
              list_drv = tmpdir.entryInfoList(QDir::Dirs | QDir::NoDotAndDotDot);
              foreach ( finf, list_drv ) if ( finf.isDir()) lst << finf.fileName();

              QList<QTreeWidgetItem*> listchild ;
              QStringList listitem;
              bool rflag = false;
              if ( It->isExpanded() ) {
                  int count = It->childCount();
                  for ( int i = 0; i < count; ++i ) {
                    listchild << It->child ( i );
                  }

                  foreach ( item, listchild ) {
                    if ( ! lst.contains ( item->text ( 0 )) ) {
                        It->removeChild ( item );
                        rflag = true;
                    }
                    else  listitem << item->text ( 0 );
                  }
              }
              else {
                listchild = It->takeChildren();
                foreach ( item, listchild ) {
                  if ( lst.contains ( item->text (0))) {
                    listitem << item->text(0);
                    It->addChild ( item);
                  }
                  else rflag = true;
              }
          }
              if ( rflag ) {
                    QStringList dirlist = watcher.directories();
                    foreach ( QString str, dirlist )
                      if ( str.contains ( It->toolTip(0) + QDir::separator()) && !QFileInfo(str).exists())  watcher.removePath ( str );
                }

              foreach ( QString str, lst ) {
                  if ( ! listitem.contains ( str )) {
                      item = new QTreeWidgetItem ( );
                      item->setText ( 0, str );
                      item->setToolTip ( 0, tmpdir.absoluteFilePath ( str ));
                      finf.setFile ( tmpdir.absoluteFilePath ( str ));
                      if ( finf.isHidden ()) item->setIcon ( 0,  hidden_icon );
                        else item->setIcon ( 0,  default_icon );
                      It->addChild ( item );
                      if ( ! watcher.directories().contains ( item->toolTip ( 0 ))) watcher.addPath ( item->toolTip ( 0 ));
                  }
           }
        }
        if (It->isExpanded()) {
          int count = It->childCount();
          for ( int i = 0; i < count; ++i ) {
            item = It->child (i);
            linkWatched ( item );
          }
        }
    }

void CTreeViewer::createListDeleteWatcher ( QList<QTreeWidgetItem*> List ) {
  QList<QTreeWidgetItem*> newList;
  foreach ( QTreeWidgetItem * item, List ) {
      int index =  item->childCount ();
      for ( int i = 0; i < index; ++i ) {
        QTreeWidgetItem *newitem = item->child (i);
        if ( newitem->isExpanded ()) {
          newList << newitem;
          lst << newitem->toolTip (0);
         }
        else lst << newitem->toolTip (0);
      }
    }
   if ( !newList.empty() ) createListDeleteWatcher ( newList );
}

void CTreeViewer::setShowHidden (bool showhide) {
  ShowHidden = showhide;
  if ( ShowHidden ) {
      QTreeWidgetItemIterator It ( this, QTreeWidgetItemIterator::All );
      while (*It) {
          int i=0;
          tmpdir.setPath  ((*It)->toolTip (0));
          lst.clear();
          list_drv.clear();
          if ( (*It)->isExpanded()) {
            list_drv = tmpdir.entryInfoList (QDir::Hidden);
            foreach ( finf, list_drv ) {
              if ( finf.isDir()) {
                item = new QTreeWidgetItem ();
                item->setText ( 0, finf.fileName());
                item->setToolTip ( 0, finf.absoluteFilePath());
                if ( finf.isSymLink() ) {
                  item->setFont ( 0, font );
                  item->setIcon ( 0, symlink_icon );
                  Link << finf.absoluteFilePath();
                }
                else item->setIcon ( 0, hidden_icon );
                (*It)->insertChild (i, item);
                i++;
                if ( !watcher.directories().contains(item->toolTip (0))) watcher.addPath ( item->toolTip(0));
                tmpdir.setPath ( item->toolTip(0));
                QFileInfoList list_drv2 = tmpdir.entryInfoList ( QDir::Hidden );
                list_drv2 << tmpdir.entryInfoList ( QDir::Dirs | QDir::NoDotAndDotDot );
                foreach ( QFileInfo finf2, list_drv2 ) {
                  if ( finf2.isDir()) {
                    QTreeWidgetItem * newitem = new QTreeWidgetItem ();
                    newitem->setText ( 0, finf2.fileName());
                    newitem->setToolTip ( 0, finf2.absoluteFilePath());
                    if ( finf2.isSymLink()) {
                      newitem->setFont ( 0, font );
                      newitem->setIcon ( 0, symlink_icon );
                    }
                    else
                      if ( finf2.isHidden() ) newitem->setIcon ( 0, hidden_icon );
                         else newitem->setIcon ( 0, default_icon );
                    item->addChild (newitem);
                  }
                }
              }
            }
           }
            else if ( (*It)->parent()) if ((*It)->parent()->isExpanded()) {
                list_drv = tmpdir.entryInfoList (QDir::Hidden);
                int i = 0;
                foreach ( finf, list_drv ) {
                  if ( finf.isDir()) {
                    item = new QTreeWidgetItem ();
                    item->setText ( 0, finf.fileName());
                    item->setToolTip ( 0, finf.absoluteFilePath());
                    if ( finf.isSymLink() ) {
                      item->setFont ( 0, font );
                      item->setIcon ( 0, symlink_icon );
                      Link << finf.absoluteFilePath();
                    }
                    else item->setIcon ( 0, hidden_icon );
                    (*It)->insertChild (i, item);
                    i++;
                  } else{}
                }
            } else {}
            else {
                list_drv = tmpdir.entryInfoList (QDir::Hidden);
                int i = 0;
                foreach ( finf, list_drv ) {
                  if ( finf.isDir()) {
                    item = new QTreeWidgetItem ();
                    item->setText ( 0, finf.fileName());
                    item->setToolTip ( 0, finf.absoluteFilePath());
                    if ( finf.isSymLink() ) {
                      item->setFont ( 0, font );
                      item->setIcon ( 0, symlink_icon );
                      Link << finf.absoluteFilePath();
                    }
                    else item->setIcon ( 0, hidden_icon );
                    (*It)->insertChild (i, item);
                    i++;
                  }
                }
            }
It++;
      }
    }
    else {
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
      QTreeWidgetItemIterator It ( this, QTreeWidgetItemIterator::All );
      while ( *It ) {
        finf.setFile ( (*It)->toolTip (0));
        if ( finf.isHidden()) {
          if ( (*It)->parent()) {
            item = (*It)->parent();
            item->removeChild(*It);
          }
            else this->removeItemWidget (*It, 0);
        }
     else     It++;
      }
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
     QList<QTreeWidgetItem*> listchild;
     lst.clear ();
     listchild.clear();
     int index = topLevelItemCount();
     for ( int i = 0; i < index; ++i ) {
       item = topLevelItem (i);
       if ( item->isExpanded() ) {
         listchild << item;
         lst << item->toolTip (0);
        }
       else lst << item->toolTip (0);
     }
     if ( !listchild.empty()) createListDeleteWatcher ( listchild );
     QStringList listwatcher = watcher.directories ();
     foreach ( QString str, listwatcher ) {
       if ( ! lst.contains (str)) watcher.removePath (str);
     }
     QStringList proxilist;
     foreach ( QString str, Link ) {
       if (  lst.contains (str)) proxilist << str;
     }
     Link.clear();
     foreach ( QString str, proxilist ) {
       Link << str;
     }
    }
 }

void CTreeViewer :: refreshTheme() {
  home_icon = (*(pxCache->getIcon(1)));
  root_icon = (*(pxCache->getIcon(0)));
  drive_icon = (*(pxCache->getIcon(3)));
  default_icon = (*(pxCache->getIcon(2)));
  hidden_icon = (*(pxCache->getIcon(4)));
  symlink_icon = pxCache->getIcon(111)->scaled(64, 64);

  QTreeWidgetItemIterator It (this, QTreeWidgetItemIterator::All);
    while(*It) {
      finf.setFile((*It)->toolTip(0));
      if ( !finf.isSymLink()) {
        if (finf.isRoot()) {
          (*It)->setIcon(0, root_icon);
          ++It;
          continue;
        }
        if (finf.absoluteFilePath() == QDir::homePath()) {
          (*It)->setIcon(0, home_icon);
          ++It;
          continue;
        }
        if (finf.isHidden()) {
          (*It)->setIcon(0, hidden_icon);
          ++It;
          continue;
        }
        (*It)->setIcon(0, default_icon);
        ++It;
        continue;
      }
      else {
        (*It)->setIcon(0, symlink_icon);
        ++It;
        continue;
      }
    }
 }

void CTreeViewer :: mousePressEvent(QMouseEvent* event) {
  if ( event->button() == Qt::RightButton) {
    item = itemAt(event->x(), event->y());
    if ( item ) {
      QList<QTreeWidgetItem*> listit = selectedItems();
      foreach (QTreeWidgetItem* iit, listit) iit->setSelected(false);
      item->setSelected(true);
      emit execContextMenu(event, item);
    }
    return;
  }
  else QTreeWidget::mousePressEvent(event);
}

void CTreeViewer :: contextMenu(QMouseEvent* ev, QTreeWidgetItem *itt) {
  finf.setFile(itt->toolTip(0));
  stritem = itt->toolTip(0);
  QMenu *Menu = new QMenu(this);
  QAction *act1, *act2, *act3, *act4;
  act1 = Menu->addAction( *(pxCache->getIcon(82)), tr("Cut"));
  act1->setToolTip("cut");
  act2 = Menu->addAction (*(pxCache->getIcon(83)), tr("Copy"));
  act2->setToolTip("copy");
  Menu->addSeparator();
  act3 = Menu->addAction (*(pxCache->getIcon(84)), tr("Paste into this directory"));
  act3->setToolTip("paste");
  if ( !finf.isReadable())  act1->setEnabled(false);
  if ( !finf.isReadable()) act2->setEnabled(false);
  if ( !finf.isWritable()) act1->setEnabled(false);
  Menu->addSeparator();
  act1 = Menu->addAction(*(pxCache->getIcon(80)), tr("Create new directory into this") + " ..." );
  act1->setToolTip("create");
  if ( !finf.isWritable()) {
    act1->setEnabled(false);
  }
  Menu->addSeparator();
  act3 = Menu->addAction (*(pxCache->getIcon(85)), tr("Rename") + " ..." );
  act3->setToolTip("ren");
  act4 = Menu->addAction (*(pxCache->getIcon(86)), tr("Delete"));
  act4->setToolTip("del");
  if ( !finf.isWritable() || finf.absoluteFilePath() == QDir::rootPath()) {
    act3->setEnabled(false);
    act4->setEnabled(false);
  }
  Menu->addSeparator();
  act3 = Menu->addAction(*(pxCache->getIcon(89)), tr("Properites") + " ..." );
  act3->setToolTip("prop");
  QPalette pal = this->palette();
  pal.setColor(QPalette::Window, Color);
  Menu->setPalette(pal);
  QAction *act = Menu->exec(ev->globalPos());
  if ( act ) {
    strop = act->toolTip();
    delete Menu;
    contextMenuSlot();
    return;
  }
  else {
    delete Menu;
    return;
  }
}

void CTreeViewer :: contextMenuSlot() {
  emit execOperation(stritem, strop);
}

void CTreeViewer :: startDrag(Qt::DropActions act) {
  DragListItem = this->selectedItems();
  QMimeData *DragMime = new QMimeData();
  QList<QUrl> Url;
  foreach ( item, DragListItem) Url << item->toolTip(0);
  DragMime->setUrls(Url);
  Drag = new QDrag(this);
  Drag->setMimeData(DragMime);
  if ( DragListItem.count() == 1 ) Drag->setPixmap(DragListItem[0]->icon(0).pixmap(128, 128).scaled(iconSize()));
    else Drag->setPixmap(pxCache->getIcon(5)->scaled(iconSize()));
  Drag->exec(act);
}

void CTreeViewer :: dragMoveEvent( QDragMoveEvent *ev) {
  DragTimer.stop();
  DragPos = ev->pos();
  if ( DragPos.y() > this->height()-60) {
    DragTimer.stop();
    ScrollDownTimer.start(100);
    return;
  }
  else {
    if ( DragPos.y() < 10 ) {
      DragTimer.stop();
      ScrollUpTimer.start(100);
      return;
    }
    else {
      ScrollDownTimer.stop();
      ScrollUpTimer.stop();
    }
  }

  item = itemAt(DragPos);
  if ( item ) {
    QFileInfo dropfinf (item->toolTip(0));
    if ( dropfinf.isWritable()) {
      ev->acceptProposedAction();
      DragTimer.start(700);
    }
    else ev->setAccepted(false);
  }
  else ev->setAccepted(false);
}

void CTreeViewer :: dragEnterEvent(QDragEnterEvent* ev) {
  if ( ev->mimeData()->hasFormat("text/uri-list")) ev->acceptProposedAction();
}

void CTreeViewer :: dragLeaveEvent(QDragLeaveEvent*) {
  DragTimer.stop();
  ScrollUpTimer.stop();
  ScrollDownTimer.stop();
}

void CTreeViewer :: dropEvent(QDropEvent* ev) {
  DragTimer.stop();
  ScrollDownTimer.stop();
  ScrollUpTimer.stop();
  ev->acceptProposedAction();
  QList<QString> DropList;
  QFileInfo dropfinf;
  bool exm = false;
  DragPos = ev->pos();
  QList<QUrl> urllist = ev->mimeData()->urls();
  foreach ( QUrl url, urllist) DropList << url.path();
  if ( ev->keyboardModifiers() == Qt::NoModifier ) {
    exm = true;
    item = itemAt(DragPos);
    if ( item ) emit dropMenu( DropList, QCursor::pos(), item->toolTip(0), exm, 0);
      else  emit dropMenu( DropList, QCursor::pos(), "", exm, 0);
    return;
  }
  if ( ev->keyboardModifiers() == Qt::CTRL ) {
    item = itemAt(DragPos);
    if ( item ) emit dropMenu(DropList, QCursor::pos(), item->toolTip(0), exm, 1);
      else emit dropMenu(DropList, QCursor::pos(), "", exm, 1);
    return;
  }
  if ( ev->keyboardModifiers() == Qt::SHIFT) {
    item = itemAt(DragPos);
    if ( item ) emit dropMenu( DropList, QCursor::pos(), item->toolTip(0), exm, 2);
      else emit dropMenu(DropList, QCursor::pos(), "", exm,  2);
    return;
  }
  if ( ev->keyboardModifiers() == Qt::CTRL + Qt::SHIFT) {
    item = itemAt(DragPos);
    if ( item ) emit dropMenu(DropList, QCursor::pos(), item->toolTip(0), exm, 4);
      else emit dropMenu(DropList, QCursor::pos(), "", exm, 4);
  }
}

void CTreeViewer :: timeoutDragTimer() {
  DragTimer.stop();
  item = itemAt(DragPos);
  if ( item ) item->setExpanded(!(item->isExpanded()));
}

void CTreeViewer :: scrollUp() {
  if ( verticalScrollBar()->value() >  verticalScrollBar()->minimum()) verticalScrollBar()->setValue(verticalScrollBar()->value() - 1);
}

void CTreeViewer :: scrollDown() {
  if ( verticalScrollBar()->value() < verticalScrollBar()->maximum()) verticalScrollBar()->setValue(verticalScrollBar()->value() + 1);
}

void CTreeViewer :: execDropMenu(QList<QString> droplist, QPoint point, QString str, bool exm, int op) {
  DropList = droplist;
  DropTarget = str;
  bool mvp = false;
  if ( droplist.count() > 0) {
    QFileInfo fileinfo(droplist[0]);
    if ( fileinfo.absolutePath() == str) mvp = true;
  }
  if ( exm ) {
    QMenu *DropMenu = new QMenu(this);
    QAction *act1, *act2, *act3;
    act1 = DropMenu->addAction(*(pxCache->getIcon(83)), tr("Copy here"));
    act1->setShortcut(QKeySequence(Qt::ControlModifier));
    act1->setToolTip("copy");
    if ( !mvp ) {
      act2 = DropMenu->addAction(*(pxCache->getIcon(84)), tr("Move here"));
      act2->setShortcut(QKeySequence(Qt::ShiftModifier));
      act2->setToolTip("move");
    }
    act3 = DropMenu->addAction(*(pxCache->getIcon(81)), tr("Create symlink"));
    act3->setShortcut(QKeySequence(Qt::ControlModifier + Qt::ShiftModifier));
    act3->setToolTip("sym");

    DropMenu->addSeparator();
    act3 = DropMenu->addAction(*(pxCache->getIcon(191)), tr("Cancel"));
    act3->setToolTip("can");
    QPalette pal = this->palette();
    pal.setColor(QPalette::Window, Color.darker(120));
    DropMenu->setPalette(pal);
    QFont fn;
    fn = this->font;
    fn.setItalic(false);
    DropMenu->setFont(fn);
    QAction *act = DropMenu->exec(point);
    if ( act ) {
      QString strop = act->toolTip();
      if ( strop == "can" ) {
        delete DropMenu;
        return;
      }
      if (strop == "copy") DropOp = 1;
      if (strop == "move") DropOp = 2;
      if (strop == "sym") DropOp = 4;
      delete DropMenu;
      emit execDrag();
      return;
    }
    else {
      delete DropMenu;
      return;
    }
  }
  else {
    DropOp = op;
    if ( op == 2 && mvp ) DropOp = 1;
    emit execDrag();
    return;
  }
}

void CTreeViewer :: slotDrag() {
  emit execOperate(DropList, DropTarget, DropOp);
}










