#include "../../Include/TreeViewerPanel.h"
#include "../../Include/TreeViewerSettings.h"
#include "../../Include/TreeViewer.h"


CTreeViewerPanel::CTreeViewerPanel (PixmapsCache *Cache, QWidget * wgt):QFrame(wgt) {
  pxCache = Cache;
  Viewer = new CTreeViewer(pxCache);
  ViewerControl = new CTreeViewerSettings(pxCache);

  QPalette pal = this->palette();
  QPalette vpal = pal;
  QColor color = ViewerControl->getBackColor();
  changeBackgroundPalette(&pal, color);
  this->setPalette(pal);
  if (!ViewerControl->getBackgroundColor())  pal.setBrush(QPalette::Base, ViewerControl->getBackTexture());
  changeTextPalette(&pal, ViewerControl->getFontColor());
  Viewer->setPalette(pal);
  setIconSize(ViewerControl->getIconSize());
  Viewer->setFont(*(ViewerControl->getFont()));
  Viewer->Color = ViewerControl->getBackColor();
  ViewerControl->setBackColor(darkerColor(ViewerControl->getBackColor(), 50));
  ViewerControl->setFontColor(ViewerControl->getFontColor());
  Viewer->setShowHidden(ViewerControl->getShowHidden());

  Layout = new QVBoxLayout (this);
  Layout->setSpacing (0);
  Layout->setMargin(0);
  Layout->addWidget ( Viewer );
  Layout->addWidget ( ViewerControl );

  connect ( ViewerControl, SIGNAL ( changeShowHidden(bool)), SLOT ( setShowHidden(bool)));
  connect ( ViewerControl, SIGNAL ( changeBackColor(QColor)), SLOT (setBackgroundColor(QColor)));
  connect ( ViewerControl, SIGNAL ( changeBackTexture(QPixmap)), SLOT ( setBackgroundTexture (QPixmap)));
  connect ( ViewerControl, SIGNAL ( changeFontColor(QColor)), SLOT ( setFontColor (QColor)));
  connect ( ViewerControl, SIGNAL ( changeIconSize(QSize)), SLOT ( setIconSize(QSize)));
  connect ( ViewerControl, SIGNAL ( changeFontSize(int)), SLOT (setFontSize(int)));

  connect ( Viewer, SIGNAL(itemClicked(QTreeWidgetItem*, int)),
                                          SIGNAL ( clickedElement ( QTreeWidgetItem*,int)));
  connect ( Viewer, SIGNAL(execOperation(QString, QString)), this, SIGNAL(execOperation(QString, QString)));

}



void CTreeViewerPanel :: setBackgroundColor (QColor color) {
        QPalette pal;
        changeBackgroundPalette(&pal, color);
        this->setPalette(pal);
        Viewer->setPalette(pal);
        Viewer->verticalScrollBar()->setPalette(pal);
        Viewer->horizontalScrollBar()->setPalette(pal);
        ViewerControl->setBackColor(darkerColor(ViewerControl->getBackColor(), 50));
        Viewer->Color = ViewerControl->getBackColor();
}

void CTreeViewerPanel :: setBackgroundTexture (QPixmap pix) {
        QPalette pal, epal;
        pal = Viewer->palette();
        epal = pal;
        changeBackgroundPalette(&epal, ViewerControl->getBackColor());
        pal.setBrush (QPalette::Base, pix);
        pal.setBrush(QPalette::Window, pix);
        Viewer->setPalette(pal);
        Viewer->verticalScrollBar()->setPalette(epal);
        Viewer->horizontalScrollBar()->setPalette(epal);
        this->setPalette(epal);
}

void CTreeViewerPanel :: setFontColor (QColor color) {
        QPalette pal;
        pal = Viewer->palette();
        changeTextPalette(&pal, color);
        Viewer->setPalette(pal);
        changeBackgroundPalette(&pal, ViewerControl->getBackColor());
        Viewer->verticalScrollBar()->setPalette(pal);
        Viewer->horizontalScrollBar()->setPalette(pal);
        pal = ViewerControl->palette();
        changeTextPalette(&pal, color);
        ViewerControl->setPalette(pal);
        this->setPalette(pal);
}

void CTreeViewerPanel :: setShowHidden(bool showhide) {
  Viewer->setShowHidden (showhide);
}

void CTreeViewerPanel :: setFontSize (int size) {
  QFont font;
  font.setPixelSize(size);
  Viewer->setFont(font);
}

void CTreeViewerPanel :: setIconSize(QSize size) {
  Viewer->setIconSize(size);
}

void CTreeViewerPanel :: setFlat(bool flat) {
  ViewerControl->setFlat(flat);
}

void CTreeViewerPanel :: refreshTheme() {
  ViewerControl->refreshTheme();
  Viewer->refreshTheme();
}
