#include "../../Include/TreeViewerSettings.h"

CTreeViewerSettings :: CTreeViewerSettings(PixmapsCache *pxcache, QWidget *wgt) : CToolBar(wgt){
  readSettings();
  pxCache = pxcache;
  ShowHide = addAction ( *(pxCache->getIcon(60)), tr("Show/Hide hidden files"));
  ShowHide->setCheckable(true);
  if ( showHidden ) ShowHide->setChecked(true); else ShowHide->setChecked(false);
  addSeparator();
  BackColor = addAction ( *(pxCache->getIcon(61)), tr("Set background color"));
  BackTexture = addAction ( *(pxCache->getIcon(62)), tr("Set background texture"));
  FontColor = addAction ( *(pxCache->getIcon(63)), tr("Set font color"));
  FontSize = addAction ( *(pxCache->getIcon(64)), tr("Set font size"));
  IconSize = addAction ( *(pxCache->getIcon(65)), tr("Set icon size"));

//-------------------------------------------------------------------------- create Menu
  QAction *actt;
  back_texture = new QMenu(this);
  QList<QPixmap> TexturePix = settingsCache.getTexturePixmaps();
  QList<QString> TextureName = settingsCache.getTextureName();
  QList<QPixmap>::Iterator It = TexturePix.begin();
  QList<QString>::Iterator Itt = TextureName.begin();
  for ( ; It != TexturePix.end(); ++It ){
    actt = back_texture->addAction ( *It,  *Itt );
    if ( *Itt == currentBackTextureName ) {
      actt->setEnabled(false);
      back_texture->setDefaultAction(actt);
      currentBackTexture = *It;
    }
    ++Itt;
  }
  BackTexture->includeMenu (back_texture);

  font_size = new QMenu(this);
  QString sizeitem;
  for ( int sz = 6; sz != 21; ) {
    sizeitem = QString::number(sz);
    actt = font_size->addAction(sizeitem);
    if (sz == currentFontSize) {
      actt->setEnabled(false);
      font_size->setDefaultAction(actt);
    }
    sz = sz + 1;
  }
  FontSize->includeMenu (font_size);

  icon_size = new QMenu(this);
  QList<int> sizeicon;
  sizeicon << 8 << 16 << 24 << 32 << 48 << 64 << 96 <<128;
  foreach ( int sz, sizeicon ){
    actt = icon_size->addAction ( QString::number(sz) + " x " + QString::number(sz));
    if ( currentIconSize == QSize(sz,sz)) {
      icon_size->setDefaultAction(actt);
      actt->setEnabled(false);
    }
  }
  IconSize->includeMenu ( icon_size );

  connect ( back_texture, SIGNAL(triggered(QAction*)), SLOT(clickedBackTexture(QAction*)));
  connect ( font_size, SIGNAL(triggered(QAction*)), SLOT(clickedFontSize(QAction*)));
  connect ( icon_size, SIGNAL(triggered(QAction*)), SLOT(clickedIconSize(QAction*)));
  connect ( ShowHide, SIGNAL(clicked()), SLOT(clickedShowHidden()));
  connect ( BackColor, SIGNAL(clicked()), SLOT(clickedBackColor()));
  connect ( FontColor, SIGNAL(clicked()), SLOT(clickedFontColor()));

  SaveSettings = true;
}

void CTreeViewerSettings :: clickedShowHidden(){
    if ( showHidden ) {
      ShowHide->setChecked(false);
      showHidden = false;
      emit changeShowHidden(false);
    }
    else {
      ShowHide->setChecked(true);
      showHidden = true;
      emit changeShowHidden(true);
    }
}

void CTreeViewerSettings::clickedBackColor(){                                    //action
  QColor color = QColorDialog::getColor ( currentBackColor );
  if ( color.isValid()) {
    backgroundColor = true;
    currentBackColor = color;
    QAction *actt;
    actt = back_texture->defaultAction();
    if ( actt ) actt->setEnabled(true);
    setBackColor(color.darker(150));
    emit changeBackColor(color);
  }
}

void CTreeViewerSettings::clickedFontColor(){                                       //action
  QColor color = QColorDialog::getColor(currentFontColor);
  if ( color.isValid()) {
    currentFontColor = color;
    setFontColor(color);
    emit changeFontColor(color);
  }
}

void CTreeViewerSettings::clickedBackTexture(QAction* action){         //menu
  backgroundColor = false;
  QAction *actt = back_texture->defaultAction();
  if ( actt ) actt->setEnabled(true);
  back_texture->setDefaultAction(action);
  action->setEnabled(false);
  currentBackTextureName = action->text();
  currentBackTexture = action->icon().pixmap(128,128);
  emit changeBackTexture(currentBackTexture); //(action->icon().pixmap(100,100));
}

void CTreeViewerSettings::clickedFontSize(QAction* action) {            //menu
  QAction *actt = font_size->defaultAction();
  if ( actt ) actt->setEnabled(true);
  font_size->setDefaultAction(action);
  action->setEnabled(false);
  currentFontSize = action->text().toInt();
  font.setPixelSize(currentFontSize);
  emit changeFontSize(currentFontSize );
}

void CTreeViewerSettings::clickedIconSize(QAction* action){                   //menu
  QAction *actt = icon_size->defaultAction();
  if ( actt ) actt->setEnabled(true);
  icon_size->setDefaultAction(action);
  action->setEnabled(false);
  QStringList lst = action->text().split(" x ");
  int sz = lst[0].toInt();
  currentIconSize = QSize(sz,sz);
  emit changeIconSize(QSize(sz, sz));
}

void CTreeViewerSettings :: readSettings(){
  settings.beginGroup("/Bathyscaphe/TreeViewer");
  showHidden = settings.value ("/ShowHide", false).toBool();
  backgroundColor = settings.value("/bbackColor", true).toBool();
  currentBackColor = settings.value("/BackColor", QColor(255,255,255)).value<QColor>();
  if ( !backgroundColor ) {
    currentBackTexture = settings.value("/BackTexture", 0).value<QPixmap>();
    currentBackTextureName = settings.value("/TextureName", "none").toString();
  }
  currentFontColor = settings.value("/FontColor", QColor(0,0,0)).value<QColor>();
  currentIconSize = settings.value("/IconSize", QSize(24,24)).value<QSize>();
  currentFontSize = settings.value("/FontSize", 0).toInt();
  if ( currentFontSize  ) {
    font.setPixelSize(currentFontSize);
  }
  settings.endGroup();
}

void CTreeViewerSettings :: writeSettings(){
  if ( SaveSettings ) {
    settings.beginGroup("/Bathyscaphe/TreeViewer");
    settings.setValue ("/ShowHide", showHidden);
    settings.setValue("/bbackColor", backgroundColor);
    settings.setValue("/BackColor", currentBackColor);
    if ( !backgroundColor ) {
      settings.setValue ("/BackTexture", currentBackTexture);
      settings.setValue("/TextureName", currentBackTextureName);
    }
    settings.setValue("/FontColor", currentFontColor);
    settings.setValue ("/IconSize", currentIconSize);
    settings.setValue("/FontSize", currentFontSize);
    settings.endGroup();
  }
}

void CTreeViewerSettings :: setFlat(bool flat){
  CToolBar::setFlat(flat);
}

void CTreeViewerSettings :: refreshTheme() {
  ShowHide->setIcon( *(pxCache->getIcon(60)));
  BackColor->setIcon( *(pxCache->getIcon(61)));
  BackTexture->setIcon( *(pxCache->getIcon(62)));
  FontColor->setIcon( *(pxCache->getIcon(63)));
  FontSize->setIcon( *(pxCache->getIcon(64)));
  IconSize->setIcon( *(pxCache->getIcon(65)));
}

CTreeViewerSettings :: ~CTreeViewerSettings(){
  writeSettings();
}
