#include "../../Include/WebHistory.h"

CWebHistory :: CWebHistory() {
  MaximumSize = 0;
  setMaximumSize();
}

void CWebHistory :: addItem (CWebViewer * page) {
  QList<QString>::Iterator ItLink = LinkList.begin();
  QList<QString>::Iterator ItTitle = TitleList.begin();
  QList<QIcon>::Iterator ItIcon = IconList.begin();
  foreach ( QString str, LinkList ) {
    if ( str == page->url().toString()) {
      IconList.erase(ItIcon);
      TitleList.erase(ItTitle);
      LinkList.erase(ItLink);
      break;
    }
    ++ItTitle;
    ++ItIcon;
    ++ItLink;
  }

  IconList << page->icon();
  if (page->title().isEmpty()) TitleList << QString(tr("Untitled") + "   --|--    " + page->url().toString());
    else TitleList << QString(page->title() + "   --|--    " + page->url().toString());
  LinkList << page->url().toString();

  if ( count() > MaximumSize ) {
    IconList.pop_front();
    TitleList.pop_front();
    LinkList.pop_front();
  }
  CurrentItem = count() - 1;
}

void CWebHistory :: setMaximumSize(int size) {
  if ( size == MaximumSize ) return;
  if ( size > MaximumSize ) {
    MaximumSize = size;
    return;
  }
  if ( size < MaximumSize ) {
    while ( MaximumSize > size ) {
      IconList.pop_front();
      TitleList.pop_front();
      LinkList.pop_front();
      MaximumSize--;
    }
  }
}

void CWebHistory :: loadHistory() {
  QSettings settings;
  settings.beginGroup("/Bathyscaphe/IconViewer");
  QBuffer buffer;
  MaximumSize = settings.value("/MaximumSize", 20).toInt();
  QByteArray arr;
  arr = settings.value("WebHistoryElements", QByteArray()).toByteArray();
  QDataStream stream(arr);
  stream >> IconList;
  stream >> TitleList;
  stream >> LinkList;
  settings.endGroup();
  CurrentItem = count() - 1;
}

void CWebHistory :: loadHistory(CWebHistory* history) {
  clear();
  int i = history->count();
  MaximumSize = history->maximumSize();
  for ( int n = 0; n < i; ++n ) {
    IconList << history->icon();
    TitleList << history->title();
    LinkList << history->link();
  }
  CurrentItem = i - 1;
}

void CWebHistory :: saveHistory() {
  QSettings settings;
  settings.beginGroup("/Bathyscaphe/IconViewer");
  QByteArray arr;
  QDataStream stream(&arr, QIODevice::WriteOnly);
  stream << IconList;
  stream << TitleList;
  stream << LinkList;
  settings.setValue("/WebHistoryElements", arr);
  settings.setValue("/MaximumSize", MaximumSize);
  settings.endGroup();
}

void CWebHistory :: clear() {
  IconList.clear();
  TitleList.clear();
  LinkList.clear();
  CurrentItem = 0;
}
